@@ -1,8 +1,13 @@
 (load "setenv")
-					;(trace unix:unix-exit :break t)
+;;(trace unix:unix-exit :break t)
 (in-package "USER")
 
-(setf *debugger-hook* #'(lambda (&rest stuff) (ext:quit t))
+#+nil
+(setf *debugger-hook* #'(lambda (&rest stuff) 
+                          (format t "~&~%~%Quiting because of: ~A~%" stuff)
+			  #+nil
+			  (ext:quit t)
+			  (unix:unix-exit 1))
       *interactive* nil			; try to ask about errors
       *gc-verbose* nil			; give GC messages
       *compile-verbose* nil		; be chatty
