@@ -13,29 +13,13 @@
     (load "/etc/lisp-config.lisp")
   (format t "~%;;; Hey: there is no /etc/lisp-config.lisp file, please run cmuclconfig"))
 
-;;; We would appreciate it if each site establishes a local maintainer who can
-;;; filter bug reports from novice users to make sure that they really have
-;;; found a bug.  Fill in the maintainer's address here..
-(rplaca
- (cdr (member :bugs *herald-items*))
- '("Send bug reports and questions to your local CMU CL maintainer, " terpri
-   "or to " 
-   "pvaneynd@debian.org" terpri
-   "or to " 
-   "cmucl-help@cons.org. (prefered)" terpri terpri
-   "type (help) for help, (quit) to exit, and (demo) to see the demos" terpri
-   terpri
-   "Loaded subsystems:" terpri))
-
 ;;; If you have sources installed on your system, un-comment the following form
 ;;; and change it to point to the source location.  This will allow the Hemlock
 ;;; "Edit Definition" command and the debugger to find sources for functions in
 ;;; the core.
 (setf (ext:search-list "target:")
       '(
-        "/usr/src/cmucl/cmucl/x86/"      ; object dir
-; place stuff there, get if from below
-        "/usr/src/cmucl/cmucl/src/"    ; x86 port
+        "/usr/share/common-lisp/source/cmucl/"      ; object dir
         ))
 
 (setf (ext:search-list "library:") '("/usr/lib/cmucl/"))
@@ -49,11 +33,11 @@
 
 (defun read-hyperspec-index ()
   (with-open-file (table
-		   "/usr/doc/cmucl/HyperSpec/HyperSpec/Data/Symbol-Table.text"
+                   "/usr/share/doc/hyperspec/Data/Symbol-Table.text"
 		   :direction :input
 		   :if-does-not-exist nil)
     (if (not table)
-	(format t "I can't find the symbol table of the Hyperspec,~%Are you sure it's installed?~%")
+	(format t "I can't find the symbol table of the Hyperspec,~%Are you sure you've installed the hyperspec package?~%")
 	(progn
 	  (loop
 	   (let* ((symbol (read table nil :einde))
@@ -67,7 +51,7 @@
 (defun lookup-in-hyperspec (key)
   (if (not (gethash 'nil hs-index))
       (read-hyperspec-index))
-  (concatenate 'string "/usr/doc/cmucl/HyperSpec/HyperSpec/Data/"
+  (concatenate 'string "/usr/share/doc/hyperspec/Data/"
 	       (gethash key hs-index)))
 
 (defun show-hs (symbol)
@@ -105,12 +89,12 @@
 ;;; also modify the *herald-items* above
 
 #-hemlock
-(defun ed ()
-  (format t "Sorry, you haven't loaded the hemlock package.~%~
-Configure it in by executing /usr/lib/cmucl/config~
-, or do: ~%(load ~Clibrary:subsystems/hemlock-library~C)~%~
-Note that this library depends on CLX, so load it too. (load ~Csubsystems/clx-library~C)~%"
- #\" #\" #\" #\"))
+(defun ed (&rest rest)
+  (multiple-value-bind (return errorp)
+    (ignore-errors (require :cmucl-hemlock))
+   (if errorp
+     (error "Sorry, cannot find hemlock, please install and use ilisp. Reason: ~S" errorp)
+    (apply #'ed rest))))
 
 (defun help ()
   (format t "~
@@ -120,10 +104,10 @@ If you aren't running this with ilisp in emacs,
 or aren't intending to use hemlock then you
 deserve to lose. :-)
 
-Read the documentation in /usr/doc/cmucl.
+Read the documentation in /usr/share/doc/cmucl.
 
 Create your own personal image with
-/usr/lib/cmucl/config
+/usr/sbin/cmuclconfig
 
 (quit) is to quit.
 (ed) starts hemlock (if installed)
@@ -139,45 +123,48 @@ normal package.
 
 General demos:
 CLX demos:
- if you have loaded clx you can do:
- (load \"/usr/src/cmucl/cmucl/src/clx/demo/hello\")
+ if you have installed cmucl-source you can do:
+ (requre :cmucl-clx)
+ (load \"cl-library:cmucl-clx;demo;hello\")
  (xlib::hello-world \"\")
- (load \"/usr/src/cmucl/cmucl/src/clx/demo/menu\")
- (xlib::just-say-lisp \"slartibartfast.org\")
- (xlib::pop-up \"g\"
+ (load \"cl-library:;cmucl-clx;demo;menu\")
+ (xlib::just-say-lisp \"\")
+ (xlib::pop-up \"\"
 	       '(\"Linux\" \"FreeBSD\" \"OpenBSD\"))
   exit by pressing control+C followed by a keypress.
 
 Clue demos:
- if you have clue loaded you can do: (after uncompressing the demo file)
- (load \"/usr/doc/clue/examples/menu\")
+ if you have installed the clue package you can do:
+ (require :clue)
+ (load \"cl-library:;clue;examples;menu\")
  (clue-examples::beatlemenuia \"\")
  (clue-examples::pick-one \"\"
 			  \"One\"
 			  \"Two\"
 			  \"Three\")
  (clue-examples::just-say-lisp \"\")
-
-Clio demos:
- if you have clio loaded you can do:
- (load \"/usr/doc/clio/examples/defsystem\")
- (load-clio-examples :host nil
-		     :directory \"/usr/doc/clio/examples/defsystem\"
-		     :compile-p nil)
- This doesn't work 100% :-(
-
-(clio-examples::sketch :host  \"\")
+ 
+ or you can use the Clio demos:
+ (require :clio)
+ (require :clio-examples)
+ (clio-examples::sketch :host  "")
 
 Pictures demos:
- don't work with me.
-
-Tk demos:
- if you have lisp-tk loaded you can do:
- (tk::tkconnect)
- (load \"/usr/src/tk/demo/load-all\")
- the gc monitor doesn't work yet.
+ (require :pictures)
+ (load \"cl-library:;pictures;examples;road-demo\")
+ (pictures::road-demo)
+ press control-a to animate
 
 "))
 
-;(setf *batch-mode* t)
+(load "/usr/share/common-lisp/source/common-lisp-controller/common-lisp-controller.lisp")
+
+(common-lisp-controller:init-common-lisp-controller 
+ (cond
+   ((member :high-security *features*)
+    "/usr/lib/common-lisp/cmucl-safe/")
+   ((member :small *features*)
+    "/usr/lib/common-lisp/cmucl-small/")
+   (t
+    "/usr/lib/common-lisp/cmucl-normal/")))
 
