@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/assembly/sparc/arith.lisp,v 1.14 1999/11/11 16:40:11 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/assembly/sparc/arith.lisp,v 1.17 2001/10/05 15:13:51 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -36,17 +36,17 @@
 			  (:temp lra descriptor-reg lra-offset)
 			  (:temp nargs any-reg nargs-offset)
 			  (:temp ocfp any-reg ocfp-offset))
-  (inst andcc zero-tn x 3)
+  (inst andcc zero-tn x fixnum-tag-mask)
   (inst b :ne DO-STATIC-FUN)
-  (inst andcc zero-tn y 3)
+  (inst andcc zero-tn y fixnum-tag-mask)
   (inst b :ne DO-STATIC-FUN)
   (inst nop)
   (inst addcc temp x y)
   (inst b :vc done)
   (inst nop)
 
-  (inst sra temp x 2)
-  (inst sra temp2 y 2)
+  (inst sra temp x fixnum-tag-bits)
+  (inst sra temp2 y fixnum-tag-bits)
   (inst add temp2 temp)
   (with-fixed-allocation (res temp bignum-type (1+ bignum-digits-offset))
     (storew temp2 res bignum-digits-offset other-pointer-type))
@@ -80,17 +80,17 @@
 			  (:temp lra descriptor-reg lra-offset)
 			  (:temp nargs any-reg nargs-offset)
 			  (:temp ocfp any-reg ocfp-offset))
-  (inst andcc zero-tn x 3)
+  (inst andcc zero-tn x fixnum-tag-mask)
   (inst b :ne DO-STATIC-FUN)
-  (inst andcc zero-tn y 3)
+  (inst andcc zero-tn y fixnum-tag-mask)
   (inst b :ne DO-STATIC-FUN)
   (inst nop)
   (inst subcc temp x y)
   (inst b :vc done)
   (inst nop)
 
-  (inst sra temp x 2)
-  (inst sra temp2 y 2)
+  (inst sra temp x fixnum-tag-bits)
+  (inst sra temp2 y fixnum-tag-bits)
   (inst sub temp2 temp temp2)
   (with-fixed-allocation (res temp bignum-type (1+ bignum-digits-offset))
     (storew temp2 res bignum-digits-offset other-pointer-type))
@@ -130,28 +130,43 @@
 			  (:temp nargs any-reg nargs-offset)
 			  (:temp ocfp any-reg ocfp-offset))
   ;; If either arg is not a fixnum, call the static function.
-  (inst andcc zero-tn x 3)
+  (inst andcc zero-tn x fixnum-tag-mask)
   (inst b :ne DO-STATIC-FUN)
-  (inst andcc zero-tn y 3)
+  (inst andcc zero-tn y fixnum-tag-mask)
   (inst b :ne DO-STATIC-FUN)
   (inst nop)
 
   ;; Remove the tag from one arg so that the result will have the correct
   ;; fixnum tag.
-  (inst sra temp x 2)
-  (inst wry temp)
-  (inst andcc hi zero-tn)
-  (inst nop)
-  (inst nop)
-  (dotimes (i 32)
-    (inst mulscc hi y))
-  (inst mulscc hi zero-tn)
-  (inst cmp x)
-  (inst b :ge MULTIPLIER-POSITIVE)
-  (inst nop)
-  (inst sub hi y)
-  MULTIPLIER-POSITIVE
-  (inst rdy lo)
+  (inst sra temp x fixnum-tag-bits)
+  ;; Compute the produce temp * y and return the double-word product
+  ;; in hi:lo.
+  (cond ((backend-featurep :sparc-64)
+	 ;; Sign extend y to a full 64-bits.  temp was already
+	 ;; sign-extended by the sra instruction above.
+	 (inst sra y 0)
+	 (inst mulx hi temp y)
+	 (inst move lo hi)
+	 (inst srax hi 32))
+	((or (backend-featurep :sparc-v8)
+	     (backend-featurep :sparc-v9))
+	 (inst smul lo temp y)
+	 (inst rdy hi))
+	(t
+	 (let ((MULTIPLIER-POSITIVE (gen-label)))
+	   (inst wry temp)
+	   (inst andcc hi zero-tn)
+	   (inst nop)
+	   (inst nop)
+	   (dotimes (i 32)
+	     (inst mulscc hi y))
+	   (inst mulscc hi zero-tn)
+	   (inst cmp x)
+	   (inst b :ge MULTIPLIER-POSITIVE)
+	   (inst nop)
+	   (inst sub hi y)
+	   (emit-label MULTIPLIER-POSITIVE)
+	   (inst rdy lo))))
 
   ;; Check to see if the result will fit in a fixnum.  (I.e. the high word
   ;; is just 32 copies of the sign bit of the low word).
@@ -160,21 +175,22 @@
   (inst b :eq LOW-FITS-IN-FIXNUM)
   ;; Shift the double word hi:lo down two bits to get rid of the fixnum tag.
   (inst sll temp hi 30)
-  (inst srl lo 2)
+  (inst srl lo fixnum-tag-bits)
   (inst or lo temp)
-  (inst sra hi 2)
-  ;; Allocate a BIGNUM for the result.
-  (pseudo-atomic (:extra (pad-data-block (1+ bignum-digits-offset)))
+  (inst sra hi fixnum-tag-bits)
+  ;; Allocate a BIGNUM for the result. We always allocate 2 words for
+  ;; the bignum result, even if we only need one.  The copying GC will
+  ;; take care of the extra word if it isn't needed.
+  (with-fixed-allocation
+      (res temp bignum-type (+ 2 bignum-digits-offset))
     (let ((one-word (gen-label)))
-      (inst or res alloc-tn other-pointer-type)
       ;; We start out assuming that we need one word.  Is that correct?
       (inst sra temp lo 31)
       (inst xorcc temp hi)
       (inst b :eq one-word)
       (inst li temp (logior (ash 1 type-bits) bignum-type))
-      ;; Nope, we need two, so allocate the addition space.
-      (inst add alloc-tn (- (pad-data-block (+ 2 bignum-digits-offset))
-			    (pad-data-block (1+ bignum-digits-offset))))
+      ;; Need 2 words.  Set the header appropriately, and save the
+      ;; high and low parts.
       (inst li temp (logior (ash 2 type-bits) bignum-type))
       (storew hi res (1+ bignum-digits-offset) other-pointer-type)
       (emit-label one-word)
@@ -209,14 +225,23 @@
 				  (:temp temp ,sc nl2-offset))
 	  ,@(when (eq type 'tagged-num)
 	      `((inst sra x 2)))
-	  (inst wry x)
-	  (inst andcc temp zero-tn)
-	  (inst nop)
-	  (inst nop)
-	  (dotimes (i 32)
-	    (inst mulscc temp y))
-	  (inst mulscc temp zero-tn)
-	  (inst rdy res))))
+	 (cond ((backend-featurep :sparc-64)
+		;; Sign extend, then multiply
+		(inst sra x 0)
+		(inst sra y 0)
+		(inst mulx res x y))
+	       ((or (backend-featurep :sparc-v8)
+		    (backend-featurep :sparc-v9))
+		(inst smul res x y))
+	       (t
+		(inst wry x)
+		(inst andcc temp zero-tn)
+		(inst nop)
+		(inst nop)
+		(dotimes (i 32)
+		  (inst mulscc temp y))
+		(inst mulscc temp zero-tn)
+		(inst rdy res))))))
   (frob unsigned-* "unsigned *" 40 unsigned-num unsigned-reg)
   (frob signed-* "unsigned *" 41 signed-num signed-reg)
   (frob fixnum-* "fixnum *" 30 tagged-num any-reg))
@@ -389,9 +414,9 @@
 				  
 				  (:temp nargs any-reg nargs-offset)
 				  (:temp ocfp any-reg ocfp-offset))
-	  (inst andcc zero-tn x 3)
+	  (inst andcc zero-tn x fixnum-tag-mask)
 	  (inst b :ne DO-STATIC-FN)
-	  (inst andcc zero-tn y 3)
+	  (inst andcc zero-tn y fixnum-tag-mask)
 	  (inst b :eq DO-COMPARE)
 	  (inst cmp x y)
 	  
@@ -431,9 +456,9 @@
 			  (:temp ocfp any-reg ocfp-offset))
   (inst cmp x y)
   (inst b :eq RETURN-T)
-  (inst andcc zero-tn x 3)
+  (inst andcc zero-tn x fixnum-tag-mask)
   (inst b :eq RETURN-NIL)
-  (inst andcc zero-tn y 3)
+  (inst andcc zero-tn y fixnum-tag-mask)
   (inst b :ne DO-STATIC-FN)
   (inst nop)
 
@@ -466,9 +491,9 @@
 			  (:temp lra descriptor-reg lra-offset)
 			  (:temp nargs any-reg nargs-offset)
 			  (:temp ocfp any-reg ocfp-offset))
-  (inst andcc zero-tn x 3)
+  (inst andcc zero-tn x fixnum-tag-mask)
   (inst b :ne DO-STATIC-FN)
-  (inst andcc zero-tn y 3)
+  (inst andcc zero-tn y fixnum-tag-mask)
   (inst b :ne DO-STATIC-FN)
   (inst cmp x y)
   (inst b :eq RETURN-T)
@@ -504,9 +529,9 @@
 			  (:temp ocfp any-reg ocfp-offset))
   (inst cmp x y)
   (inst b :eq RETURN-NIL)
-  (inst andcc zero-tn x 3)
+  (inst andcc zero-tn x fixnum-tag-mask)
   (inst b :ne DO-STATIC-FN)
-  (inst andcc zero-tn y 3)
+  (inst andcc zero-tn y fixnum-tag-mask)
   (inst b :ne DO-STATIC-FN)
   (inst nop)
 
