@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/alieneval.lisp,v 1.48 1999/09/15 15:14:45 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/alieneval.lisp,v 1.51 2001/06/01 12:49:39 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -1543,8 +1543,21 @@
 (declaim (inline %make-alien))
 (defun %make-alien (bits)
   (declare (type kernel:index bits) (optimize-interface (safety 2)))
-  (alien-funcall (extern-alien "malloc" (function system-area-pointer unsigned))
-		 (ash (the kernel:index (+ bits 7)) -3)))
+  (let ((sap (alien-funcall (extern-alien "malloc" (function system-area-pointer unsigned))
+		 (ash (the kernel:index (+ bits 7)) -3))))
+   ;; Now for some strange reason the newest glibc fails
+   ;; if you don't touch all of the allocated memory. So let us
+   ;; just do that won't we? [Peter Van Eynde 2001-06-03]
+   (unless (<= #x08000000 (SYSTEM:SAP-INT sap) #x10000000) 
+     (error "Allocated memory outside normal range: ~S < ~S ~S < ~S" 
+       #x08000000 (SYSTEM:SAP-INT sap) sap  #x10000000))
+   (loop for offset fixnum from 0  below
+		 (ash (the kernel:index (+ bits 7)) -3)
+		 by
+		 1024
+	do
+	(KERNEL:%SET-SAP-REF-8 sap offset 0))
+   sap))
 
 ;;; FREE-ALIEN -- public
 ;;;
@@ -1777,7 +1790,7 @@
   (declare (ignore info))
   (setf (deref alien) value))
 
-(define-setf-method local-alien (&whole whole info alien)
+(define-setf-expander local-alien (&whole whole info alien)
   (let ((value (gensym))
 	(info (if (and (consp info)
 		       (eq (car info) 'quote))
