@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/array.lisp,v 1.29 1998/07/24 17:17:49 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/array.lisp,v 1.32 2001/11/21 22:45:36 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -232,6 +232,11 @@
 		 (when (or initial-element-p initial-contents)
 		   (error "Neither :initial-element nor :initial-contents ~
 		   can be specified along with :displaced-to"))
+		 (unless (subtypep element-type
+				   (array-element-type displaced-to))
+		   (error "One can't displace an array of type ~S into ~
+                           another of type ~S."
+			  element-type (array-element-type displaced-to)))
 		 (let ((offset (or displaced-index-offset 0)))
 		   (when (> (+ offset total-size)
 			    (array-total-size displaced-to))
@@ -351,7 +356,8 @@
 				    `(,atype
 				      (data-vector-set (the ,atype vector)
 						       index
-						       (the ,type new-value)))))
+						       (the ,type new-value))
+				      new-value)))
 			      stuff))))
       (dispatch
        t
@@ -583,7 +589,10 @@
   "Returns values of :displaced-to and :displaced-index-offset options to
    make-array, or the defaults nil and 0 if not a displaced array."
   (declare (array array))
-  (values (%array-data-vector array) (%array-displacement array)))
+  (if (and (array-header-p array) (%array-displaced-p array))
+      (values (%array-data-vector array)
+            (truly-the fixnum (%array-displacement array)))
+      (values nil 0)))
 
 (defun adjustable-array-p (array)
   "Returns T if (adjust-array array...) would return an array identical
@@ -742,17 +751,23 @@
 	       (declare (fixnum old-length new-length))
 	       (with-array-data ((old-data array) (old-start)
 				 (old-end old-length))
-		 (cond ((or (%array-displaced-p array)
-			    (< old-length new-length))
-			(setf new-data
-			      (data-vector-from-inits
-			       dimensions new-length element-type
-			       initial-contents initial-element
-			       initial-element-p))
-			(replace new-data old-data
-				 :start2 old-start :end2 old-end))
-		       (t (setf new-data
-				(shrink-vector old-data new-length))))
+		 (cond
+		   ((and (adjustable-array-p array)
+			 (not (%array-displaced-p array))
+			 (<= new-length old-length))
+		    ;; Shrink underlying vector in-place.  We don't do this
+		    ;; for non-adjustable arrays, since that might confuse
+		    ;; user expectations about adjust-array consing a fresh
+		    ;; array in that case.
+		    (setf new-data (shrink-vector old-data new-length)))
+		   (t
+		    (setf new-data
+			  (data-vector-from-inits
+			   dimensions new-length element-type
+			   initial-contents initial-element
+			   initial-element-p))
+		    (replace new-data old-data
+			     :start2 old-start :end2 old-end)))
 		 (if (adjustable-array-p array)
 		     (set-array-header array new-data new-length
 				       (get-new-fill-pointer array new-length
