@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/class.lisp,v 1.42 2000/01/10 14:43:54 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/class.lisp,v 1.47 2001/12/06 23:24:42 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -34,7 +34,7 @@
 		 built-in-class-direct-superclasses
 		 find-class-cell class-cell-name class-cell-class
 		 make-layout make-undefined-class insured-find-class
-		 redefine-layout-warning))
+		 redefine-layout-warning std-compute-class-precedence-list))
 
 (in-package "LISP")
 (export '(class structure-class standard-class class-name find-class class-of
@@ -174,6 +174,52 @@
 	    (1+ (random layout-hash-max seed)))))
   layout)
 
+;;; order-layout-inherits  --  Interface
+;;;
+;;; Arrange the inherited layouts to appear at their expected depth, ensuring
+;;; that hierarchical type tests succeed. Layouts with a specific depth are
+;;; placed first, then the non-hierarchical layouts fill remaining elements.
+;;; Any empty elements are filled with layout copies ensuring that all
+;;; elements have a valid layout. This re-ordering may destroy CPL ordering so
+;;; the inherits should not be read as being in CPL order, and further
+;;; duplicates may be introduced.
+;;;
+(defun order-layout-inherits (layouts)
+  (declare (simple-vector layouts))
+  (let ((length (length layouts))
+	(max-depth -1))
+    (dotimes (i length)
+      (let ((depth (layout-inheritance-depth (svref layouts i))))
+	(when (> depth max-depth)
+	  (setf max-depth depth))))
+    (let* ((new-length (max (1+ max-depth) length))
+	   (inherits (make-array new-length)))
+      (dotimes (i length)
+	(let* ((layout (svref layouts i))
+	       (depth (layout-inheritance-depth layout)))
+	  (unless (eql depth -1)
+	    (let ((old-layout (svref inherits depth)))
+	      (unless (or (eql old-layout 0) (eq old-layout layout))
+		(error "Layout depth confict: ~S~%" layouts)))
+	    (setf (svref inherits depth) layout))))
+      (do ((i 0 (1+ i))
+	   (j 0))
+	  ((>= i length))
+	(declare (type index i j))
+	(let* ((layout (svref layouts i))
+	       (depth (layout-inheritance-depth layout)))
+	  (when (eql depth -1)
+	    (loop (when (eql (svref inherits j) 0)
+		    (return))
+		  (incf j))
+	    (setf (svref inherits j) layout))))
+      (do ((i (1- new-length) (1- i)))
+	  ((< i 0))
+	(declare (type fixnum i))
+	(when (eql (svref inherits i) 0)
+	  (setf (svref inherits i) (svref inherits (1+ i)))))
+      inherits)))
+
 
 ;;; The CLASS structure is a supertype of all CLASS types.  A CLASS is also a
 ;;; CTYPE structure as recognized by the type system.
@@ -321,30 +367,44 @@
 	res
 	(error "Class not yet defined:~%  ~S" name))))
 ;;;
-(defun (setf find-class) (new-value name)
-  (declare (type class new-value))
-  (ecase (info type kind name)
-    ((nil))
-    (:instance
-     (let ((old (class-of (find-class name)))
-	   (new (class-of new-value)))
-       (unless (eq old new)
-	 (warn "Changing meta-class of ~S from ~S to ~S."
-	       name (class-name old) (class-name new)))))
-    (:primitive
-     (error "Illegal to redefine standard type ~S." name))
-    (:defined
-     (warn "Redefining DEFTYPE type to be a class: ~S."
-	   name)
-     (setf (info type expander name) nil)))
-
-  (remhash name *forward-referenced-layouts*)
-  (%note-type-defined name)
-  (setf (info type kind name) :instance)
-  (setf (class-cell-class (find-class-cell name)) new-value)
-  (unless (eq (info type compiler-layout name)
-	      (class-layout new-value))
-    (setf (info type compiler-layout name) (class-layout new-value)))
+(defun (setf find-class) (new-value name &optional (errorp t) environment)
+  (declare (type (or null class) new-value) (ignore errorp environment))
+  (cond
+    ((null new-value)
+     ;; Clear info db if name names a class
+     (ecase (info type kind name)
+       (:primitive
+	(error "Illegal to redefine standard type ~S." name))
+       (:defined)
+       ((nil))
+       (:instance
+	(setf (info type kind name) nil
+	      (info type class name) nil
+	      (info type documentation name) nil
+	      (info type compiler-layout name) nil))))
+    (t
+     (ecase (info type kind name)
+       ((nil))
+       (:instance
+	(let ((old (class-of (find-class name)))
+	      (new (class-of new-value)))
+	  (unless (eq old new)
+	    (warn "Changing meta-class of ~S from ~S to ~S."
+		  name (class-name old) (class-name new)))))
+       (:primitive
+	(error "Illegal to redefine standard type ~S." name))
+       (xdefined
+	(warn "Redefining DEFTYPE type to be a class: ~S."
+	      name)
+	(setf (info type expander name) nil)))
+     
+     (remhash name *forward-referenced-layouts*)
+     (%note-type-defined name)
+     (setf (info type kind name) :instance)
+     (setf (class-cell-class (find-class-cell name)) new-value)
+     (unless (eq (info type compiler-layout name)
+		 (class-layout new-value))
+       (setf (info type compiler-layout name) (class-layout new-value)))))
   new-value)
 
 
@@ -750,7 +810,7 @@
 	  (list :translation (or cons (member nil))
 		:inherits (sequence mutable-sequence mutable-collection
 			   generic-sequence collection))
-	  (cons :codes (#.vm:list-pointer-type)
+	  (cons :translation cons :codes (#.vm:list-pointer-type)
 		:inherits (list sequence mutable-sequence mutable-collection
 			   generic-sequence collection))
 	  (null :translation (member nil)
@@ -758,8 +818,7 @@
 			   generic-sequence collection symbol)
 		:direct-superclasses (list symbol))
 
-	  (stream :hierarchical nil :state :read-only
-		  :inherits (instance t)))))
+	  (stream :state :read-only :depth 3 :inherits (instance)))))
 
 ;;; See also type-init.lisp where we finish setting up the translations for
 ;;; built-in types.
@@ -767,7 +826,7 @@
 (cold-load-init
   (dolist (x built-in-classes)
     (destructuring-bind (name &key (translation nil trans-p) inherits codes
-			      enumerable state (hierarchical t)
+			      enumerable state (hierarchical t) depth
 			      (direct-superclasses
 			       (if inherits (list (car inherits)) '(t))))
 			x
@@ -787,12 +846,11 @@
 	(setf (class-cell-class (find-class-cell name)) class)
 	(unless trans-p
 	  (setf (info type builtin name) class))
-	(let* ((inheritance-depth (if hierarchical (length inherits) -1))
+	(let* ((inheritance-depth (if hierarchical
+				      (or depth (length inherits))
+				      -1))
 	       (inherit-layouts
-		(#-high-security
-		 map
-		 #+high-security
-		 lisp::map-without-errorchecking
+		(map
 		     'vector
 		     #'(lambda (x)
 			 (let ((super-layout (class-layout (find-class x))))
@@ -808,24 +866,21 @@
 ;;; correctly and the lisp layout replaced by a PCL wrapper after PCL
 ;;; is loaded and the class defined.
 (cold-load-init
-  (dolist (x '((fundamental-stream (t instance stream))))
+  (dolist (x '((fundamental-stream (t instance stream stream))))
     (let* ((name (first x))
 	   (inherits (second x))
-	   (class (lisp::make-standard-class :name name))
-	   (class-cell (lisp::find-class-cell name)))
-      (setf (lisp::class-cell-class class-cell) class)
+	   (class (make-standard-class :name name))
+	   (class-cell (find-class-cell name)))
+      (setf (class-cell-class class-cell) class)
       (setf (info type class name) class-cell)
       (setf (info type kind name) :instance)
       (let ((inherit-layouts
-	     (#-high-security
-	      map
-	      #+high-security
-	      lisp::map-without-errorchecking
+	     (map
 	      'vector #'(lambda (x)
-			      (lisp::class-layout (lisp:find-class x)))
+			      (class-layout (lisp:find-class x)))
 		  inherits)))
-	(lisp::register-layout (lisp::find-layout name 0 inherit-layouts -1)
-			       :invalidate nil)))))
+	(register-layout (find-layout name 0 inherit-layouts -1)
+			 :invalidate nil)))))
 
 ;;; Now that we have set up the class heterarchy, seal the sealed classes.
 ;;; This must be done after the subclasses have been set up.
@@ -1086,6 +1141,90 @@
 	       old)))
 	  (t old))))
 
+
+;;; Class precedence lists
+
+;;; topological-sort  --  Public.
+;;;
+;;; Topologically sort the list of objects to meet a set of ordering
+;;; constraints given by pairs (A . B) constraining A to precede B.  When
+;;; there are multiple objects to choose, the tie-breaker function is called
+;;; with both the list of object to choose from and the reverse ordering built
+;;; so far.
+;;;
+(defun topological-sort (objects constraints tie-breaker)
+  (declare (list objects constraints)
+	   (function tie-breaker))
+  (let ((obj-info (make-hash-table :size (length objects)))
+	(free-objs nil)
+	(result nil))
+    (dolist (constraint constraints)
+      (let ((obj1 (car constraint))
+	    (obj2 (cdr constraint)))
+	(let ((info2 (gethash obj2 obj-info)))
+	  (if info2
+	      (incf (first info2))
+	      (setf (gethash obj2 obj-info) (list 1))))
+	(let ((info1 (gethash obj1 obj-info)))
+	  (if info1
+	      (push obj2 (rest info1))
+	      (setf (gethash obj1 obj-info) (list 0 obj2))))))
+    (dolist (obj objects)
+      (let ((info (gethash obj obj-info)))
+	(when (or (not info) (zerop (first info)))
+	  (push obj free-objs))))
+    (loop
+     (flet ((next-result (obj)
+	      (push obj result)
+	      (dolist (successor (rest (gethash obj obj-info)))
+		(let* ((successor-info (gethash successor obj-info))
+		       (count (1- (first successor-info))))
+		  (setf (first successor-info) count)
+		  (when (zerop count)
+		    (push successor free-objs))))))
+       (cond ((endp free-objs)
+	      (do-hash (obj info obj-info)
+		(unless (zerop (first info))
+		  (error "Topological sort failed due to constrain on ~S."
+			 obj)))
+	      (return (nreverse result)))
+	     ((endp (rest free-objs))
+	      (next-result (pop free-objs)))
+	     (t
+	      (let ((obj (funcall tie-breaker free-objs result)))
+		(setf free-objs (remove obj free-objs))
+		(next-result obj))))))))
+
+
+;;; std-compute-class-precedence-list  --  Internal.
+;;;
+;;; Standard class precedence list computation.
+;;;
+(defun std-compute-class-precedence-list (class)
+  (let ((classes nil)
+	(constraints nil))
+    (labels ((note-class (class)
+	       (unless (member class classes)
+		 (push class classes)
+		 (let ((superclasses (class-direct-superclasses class)))
+		   (do ((prev class)
+			(rest superclasses (rest rest)))
+		       ((endp rest))
+		     (let ((next (first rest)))
+		       (push (cons prev next) constraints)
+		       (setf prev next)))
+		   (dolist (class superclasses)
+		     (note-class class)))))
+	     (std-cpl-tie-breaker (free-classes rev-cpl)
+	       (dolist (class rev-cpl (first free-classes))
+		 (let* ((superclasses (class-direct-superclasses class))
+			(intersection (intersection free-classes
+						    superclasses)))
+		   (when intersection
+		     (return (first intersection)))))))
+      (note-class class)
+      (topological-sort classes constraints #'std-cpl-tie-breaker))))
+
 
 ;;; PCL stuff:
 
