@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/clx-ext.lisp,v 1.14 1997/08/06 12:25:07 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/code/clx-ext.lisp,v 1.15 2001/03/04 20:12:30 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -227,7 +227,7 @@
 
 (defvar *object-set-event-handler-print* nil)
 
-(proclaim '(declaration values))
+(declaim (declaration values))
 
 (defun object-set-event-handler (display)
   "This display event handler uses object sets to map event windows cross
