@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/commandline.lisp,v 1.6 1996/05/08 02:03:08 ram Exp $")
+  "$Header: /home/CVS-cmucl/src/code/commandline.lisp,v 1.12 2001/12/08 18:42:39 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -155,7 +155,8 @@
       (cond (demon (funcall demon switch))
 	    ((or (member name *legal-cmd-line-switches* :test #'string-equal)
 		 (not *complain-about-illegal-switches*)))
-	    (t (warn "~S is an illegal switch" switch))))))
+	    (t (warn "~S is an illegal switch" switch)))
+      (lisp::finish-standard-output-streams))))
 
 (defmacro defswitch (name &optional function)
   "Associates function with the switch name in *command-switch-demons*.  Name
@@ -175,18 +176,32 @@
 
 
 (defun eval-switch-demon (switch)
-  (eval (read-from-string (cmd-switch-arg switch))))
+  (let ((cmds (cmd-switch-arg switch)))
+    (do ((length (length cmds))
+	 (start 0))
+	((>= start length))
+      (multiple-value-bind (form next)
+	  (read-from-string cmds nil nil :start start)
+	(eval form)
+	(lisp::finish-standard-output-streams)
+	(setf start next)))))
 (defswitch "eval" #'eval-switch-demon)
 
 (defun load-switch-demon (switch)
   (load (cmd-switch-arg switch)))
 (defswitch "load" #'load-switch-demon)
 
-(defun batch-switch-demon (switch)
-  (setq *batch-mode* t)
-  (setf (cmd-switch-value switch) t))
-(defswitch "batch" #'batch-switch-demon)
-
+(defun quiet-switch-demon (switch)
+  (declare (ignore switch))
+  (setq *load-verbose* nil
+        *compile-verbose* nil
+        *compile-print* nil
+        *compile-progress* nil
+        *require-verbose* nil
+        *gc-verbose* nil
+        *herald-items* nil))
+(defswitch "quiet" #'quiet-switch-demon)
+        
 (defun cmd-switch-arg (switch)
   (or (cmd-switch-value switch)
       (car (cmd-switch-words switch))
@@ -195,6 +210,9 @@
 (defswitch "core")
 (defswitch "init")
 (defswitch "noinit")
+(defswitch "nositeinit")
 (defswitch "hinit")
 (defswitch "lazy")
-
+(defswitch "nolazy")
+(defswitch "batch")
+(defswitch "dynamic-space-size")
