@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/error.lisp,v 1.54 1999/01/09 11:12:17 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/error.lisp,v 1.60 2001/04/19 16:40:47 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -420,10 +420,10 @@
 				parent-types)))))
 	 (cond-layout (info type compiler-layout 'condition))
 	 (olayout (info type compiler-layout name))
-	 (new-inherits 
-	  (concatenate 'simple-vector
-		       (layout-inherits cond-layout)
-		       (mapcar #'class-layout cpl))))
+	 (new-inherits
+	  (order-layout-inherits (concatenate 'simple-vector
+					      (layout-inherits cond-layout)
+					      (mapcar #'class-layout cpl)))))
     (if (and olayout
 	     (not (mismatch (layout-inherits olayout) new-inherits)))
 	olayout
@@ -437,9 +437,6 @@
 
 ;;;; Condition reporting:
 
-;;; 7/13/98 BUG? CPL is not sorted and results here depend on order of
-;;; superclasses in define-condition call!
-
 (defun %print-condition (s stream d)
   (declare (ignore d))
   (if *print-escape*
@@ -501,6 +498,8 @@
       (if (eq val *empty-slot*)
 	  (let ((actual-initargs (condition-actual-initargs condition))
 		(slot (find-slot (condition-class-cpl class) name)))
+	    (unless slot
+	      (error "Slot ~S of ~S missing." name condition))
 	    (dolist (initarg (condition-slot-initargs slot))
 	      (let ((val (getf actual-initargs initarg *empty-slot*)))
 		(unless (eq val *empty-slot*)
@@ -583,18 +582,10 @@
 
     (setf (find-class name) class)
     ;;
-    ;; Initialize CPL slot from layout.
-    (collect ((cpl))
-      (cpl class)
-      (let ((inherits (layout-inherits layout)))
-	(do ((i (1- (length inherits)) (1- i)))
-	    ((minusp i))
-	  (let ((super (find-class
-			(class-name
-			 (layout-class (svref inherits i))))))
-	    (when (typep super 'condition-class)
-	      (cpl super)))))
-      (setf (condition-class-cpl class) (cpl))))
+    ;; Initialize CPL slot.
+    (setf (condition-class-cpl class)
+	  (remove-if-not #'condition-class-p 
+			 (std-compute-class-precedence-list class))))
   (undefined-value))
 
 ); eval-when (compile load eval)
@@ -824,7 +815,7 @@
 				bindings))
 		*handler-clusters*)))
      (multiple-value-prog1
-      ,@forms
+      (progn ,@forms)
       ;; Wait for any float exceptions
       #+x86 (float-wait))))
 
@@ -978,6 +969,10 @@
 
 ;;;; HANDLER-CASE and IGNORE-ERRORS.
 
+#|
+;;; Version used in older versions of CMUCL which had lossage closing over
+;;; tags.
+;;;
 (defmacro handler-case (form &rest cases)
   "(HANDLER-CASE form
    { (type ([var]) body) }* )
@@ -1005,7 +1000,7 @@
 	  `(let ((,outer-tag (cons nil nil))
 		 (,inner-tag (cons nil nil))
 		 ,var ,tag-var)
-	     ,var			;ignoreable
+	     (declare (ignorable ,var))
 	     (catch ,outer-tag
 	       (catch ,inner-tag
 		 (throw ,outer-tag
@@ -1031,10 +1026,11 @@
 					     ,@body))
 					 body))))
 			   annotated-cases))))))))
-#|
-This macro doesn't work in our system due to lossage in closing over tags.
-The previous version sets up unique run-time tags.
+|#
 
+;;; This macro doesn't work in older version of CMUCL system due to lossage
+;;; in closing over tags.  The previous version sets up unique run-time tags.
+;;;
 (defmacro handler-case (form &rest cases)
   "(HANDLER-CASE form
    { (type ([var]) body) }* )
@@ -1059,7 +1055,7 @@ The previous version sets up unique run-time tags.
 				       cases)))
 	  `(block ,tag
 	     (let ((,var nil))
-	       ,var				;ignorable
+	       (declare (ignorable ,var))
 	       (tagbody
 		 (handler-bind
 		  ,(mapcar #'(lambda (annotated-case)
@@ -1070,7 +1066,11 @@ The previous version sets up unique run-time tags.
 					       '(declare (ignore temp)))
 					  (go ,(car annotated-case)))))
 			   annotated-cases)
-			       (return-from ,tag ,form))
+			       (return-from ,tag
+				 #-x86 ,form
+				 #+x86 (multiple-value-prog1 ,form
+					 ;; Need to catch FP errors here!
+					 (kernel::float-wait))))
 		 ,@(mapcan
 		    #'(lambda (annotated-case)
 			(list (car annotated-case)
@@ -1086,7 +1086,6 @@ The previous version sets up unique run-time tags.
 					 (t
 					  `(progn ,@body)))))))
 			   annotated-cases))))))))
-|#
 
 (defmacro ignore-errors (&rest forms)
   "Executes forms after establishing a handler for all error conditions that
