@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/eval.lisp,v 1.30 1999/03/04 12:32:45 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/code/eval.lisp,v 1.33 2001/12/13 01:04:20 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -50,7 +50,8 @@
 (export '(parse-body find-if-in-closure))
 
 (in-package "EXTENSIONS")
-(export '(*top-level-auto-declare*))
+(export '(*top-level-auto-declare*
+	  compiler-macroexpand-1 compiler-macroexpand))
 
 (in-package "KERNEL")
 (export '(invoke-macroexpand-hook))
@@ -127,12 +128,6 @@
 ;;;; EVAL and friends.
 (in-package "LISP")
 
-;;;
-;;; This flag is used by EVAL-WHEN to keep track of when code has already been
-;;; evaluated so that it can avoid multiple evaluation of nested EVAL-WHEN
-;;; (COMPILE)s.
-(defvar *already-evaled-this* nil)
-
 ;;;
 ;;; This needs to be initialized in the cold load, since the top-level catcher
 ;;; will always restore the initial value.
@@ -151,9 +146,7 @@
 
 ;;; EVAL  --  Public
 ;;;
-;;;    Pick off a few easy cases, and call INTERNAL-EVAL for the rest.  If
-;;; *ALREADY-EVALED-THIS* is true, then we bind it to NIL before doing a call
-;;; so that the effect is confined to the lexical scope of the EVAL-WHEN.
+;;;    Pick off a few easy cases, and call INTERNAL-EVAL for the rest.
 ;;;
 (defun eval (original-exp)
   "Evaluates its single arg in a null lexical environment, returns the
@@ -167,7 +160,7 @@
 	  (values (info variable constant-value exp)))
 	 ((:special :global)
 	  (symbol-value exp))
-	 (:alien
+	 ((:alien :macro)
 	  (eval:internal-eval original-exp))))
       (list
        (let ((name (first exp))
@@ -249,10 +242,7 @@
 		(collect ((args))
 		  (dolist (arg (rest exp))
 		    (args (eval arg)))
-		  (if *already-evaled-this*
-		      (let ((*already-evaled-this* nil))
-			(apply (symbol-function name) (args)))
-		      (apply (symbol-function name) (args))))
+		  (apply (symbol-function name) (args)))
 		(eval:internal-eval original-exp))))))
       (t
        exp))))
@@ -402,10 +392,13 @@
 	((symbolp form)
 	 (let* ((venv (when env (c::lexenv-variables env)))
 		(local-def (cdr (assoc form venv))))
-	   (if (and (consp local-def)
-		    (eq (car local-def) 'macro))
-	       (values (cdr local-def) t)
-	       (values form nil))))
+	   (cond ((and (consp local-def)
+		       (eq (car local-def) 'macro))
+		  (values (cdr local-def) t))
+		 ((eq (info variable kind form) :macro)
+		  (values (info variable macro-expansion form) t))
+		 (t
+		  (values form nil)))))
 	(t
 	 (values form nil))))
 
@@ -447,8 +440,9 @@
   (setf (info function compiler-macro-function name) function)
   function)
 
-#|These seem to have been dropped from the spec, and we don't use them
-internally...
+;;; While these have been dropped from the spec, and we don't use them
+;;; internally, we implement them anyway, for the benefit of a user
+;;; trying to debug his compiler macros.
 
 (defun compiler-macroexpand-1 (form &optional env)
   "If FORM is a function call for which a compiler-macro has been defined,
@@ -472,7 +466,6 @@ internally...
 		   (frob new-form t)
 		   (values new-form expanded)))))
     (frob form env)))
-|#
 
 (defun constantp (object &optional environment)
   "True of any Lisp object that has a constant value: types that eval to
