@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/exports.lisp,v 1.167 2000/01/10 14:45:14 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/exports.lisp,v 1.187 2001/12/13 01:04:21 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -68,6 +68,10 @@
 (if (find-package "SGI")
     (rename-package "SGI" "SGI" '("VM" "OLD-SGI"))
     (make-package "SGI" :nicknames '("VM" "OLD-SGI") :use nil))
+#+ppc
+(if (find-package "PPC")
+    (rename-package "PPC" "PPC" '("VM" "OLD-PPC"))
+    (make-package "PPC" :nicknames '("VM" "OLD-PPC")))
 (if (find-package "CONDITIONS")
     (rename-package "CONDITIONS" "CONDITIONS" 'nil)
     (make-package "CONDITIONS" :nicknames 'nil :use nil))
@@ -215,17 +219,17 @@
              "UNIX-FSYNC" "UNIX-FTRUNCATE" "UNIX-GETDTABLESIZE" "UNIX-GETEGID"
              "UNIX-GETGID" "UNIX-GETHOSTID" "UNIX-GETHOSTNAME"
              "UNIX-GETPAGESIZE"  "UNIX-GETPEERNAME" "UNIX-GETPGRP"
-	     "UNIX-GETPID" "UNIX-GETPPID" "UNIX-GETRUSAGE"
-	     "UNIX-GETSOCKNAME" "UNIX-GETTIMEOFDAY" "UNIX-GETUID" "UNIX-GID"
+	     "UNIX-GETPID" "UNIX-GETPPID" "UNIX-GETRUSAGE" "UNIX-GETSOCKNAME"
+	     "UNIX-GETSOCKOPT" "UNIX-GETTIMEOFDAY" "UNIX-GETUID" "UNIX-GID"
              "UNIX-IOCTL" "UNIX-ISATTY" "UNIX-LINK" "UNIX-LISTEN" "UNIX-LSEEK"
              "UNIX-LSTAT" "UNIX-MKDIR" "UNIX-OPEN" "UNIX-PATHNAME" "UNIX-PID"
              "UNIX-PIPE" "UNIX-READ" "UNIX-READLINK" "UNIX-RECV" "UNIX-RENAME"
-             "UNIX-RMDIR" "UNIX-SCHED-YIELD" "UNIX-SELECT" "UNIX-SEND"
-	     "UNIX-SETPGID" "UNIX-SETPGRP"
-             "UNIX-SETREGID" "UNIX-SETREUID" "UNIX-SOCKET" "UNIX-STAT"
-             "UNIX-SYMLINK" "UNIX-SYNC"
-	     "UNIX-TIMES" "UNIX-TRUNCATE" "UNIX-TTYNAME"
-             "UNIX-UID" "UNIX-UNLINK" "UNIX-UTIMES" "UNIX-WRITE" "WINSIZE"
+             "UNIX-RMDIR" "UNIX-SCHED-YIELD" "UNIX-SELECT"
+	     "UNIX-SEND" "UNIX-SETPGID" "UNIX-SETPGRP"
+             "UNIX-SETREGID" "UNIX-SETREUID" "UNIX-SETSOCKOPT" "UNIX-SOCKET"
+	     "UNIX-STAT" "UNIX-SYMLINK" "UNIX-SYNC"
+	     "UNIX-TIMES" "UNIX-TRUNCATE" "UNIX-TTYNAME" "UNIX-UID"
+	     "UNIX-UNAME" "UNIX-UNLINK" "UNIX-UTIMES" "UNIX-WRITE" "WINSIZE"
              "WRITEGRP" "WRITEOTH" "WRITEOWN" "WS-COL" "WS-ROW" "WS-XPIXEL"
              "WS-YPIXEL" "W_OK" "X_OK" "SIGSYS" "TCSETPGRP" "FD-ZERO"
              "SIGEMSG" "SIGQUIT" "SIGCHLD" "SIGSEGV" "FD-CLR" "SIGUSR2"
@@ -255,7 +259,7 @@
              "SIGTTOU" "EEXIST" "SIGPROF" "SIGSTOP" "ENETRESET" "SIGURG"
              "ENOBUFS" "EPROCLIM" "EROFS" "ETOOMANYREFS" "UNIX-FILE-KIND"
              "ELOCAL" "UNIX-SIGSETMASK" "EREMOTE" "ESOCKTNOSUPPORT"
-	     "TIOCSIGSEND" "SIGWAITING"
+	     "TIOCSIGSEND" "SIGWAITING" "SIGABRT"
 	     "C-IFLAG" "UNIX-TCGETATTR" "C-LFLAG" "C-OFLAG" "C-CFLAG"
 	     "TCSAFLUSH" "C-CC" "C-ISPEED" "C-OSPEED" "SIOCSPGRP" "TERMIOS"
 	     "UNIX-TCSETATTR" "O_NDELAY" "O_NOCTTY"
@@ -276,7 +280,7 @@
              "TTY-CSIZE" "TTY-CS5" "TTY-CS6" "TTY-CS7" "TTY-CS8" "VMIN" "VTIME"
              "VSUSP" "VSTART" "VSTOP" "VDSUSP" "UNIX-TCSENDBREAK"
              "UNIX-TCDRAIN" "UNIX-TCFLUSH" "UNIX-TCFLOW"
-	     #+(or svr4 freebsd linux) "O_NDELAY")
+	     #+(or svr4 bsd linux) "O_NDELAY")
             #+(or svr4 linux)
             (:export "EADDRINUSE" "EADDRNOTAVAIL" "EADV" "EAFNOSUPPORT"
 	     "EALREADY" "EBADE" "EBADFD" "EBADMSG" "EBADR" "EBADRQC"
@@ -295,7 +299,8 @@
 	     "EPROTONOSUPPORT" "EPROTOTYPE" "EREMCHG" "EREMOTE"
 	     #+linux "EREMOTEIO" "ERESTART" "ESHUTDOWN" "ESOCKTNOSUPPORT"
 	     "ESRMNT" "ESTALE" "ESTRPIPE" "ETIME" "ETIMEDOUT" "ETOOMANYREFS"
-	     #+linux "EUCLEAN" "EUNATCH" "EUSERS" "EWOULDBLOCK" "EXFULL"))
+	     #+linux "EUCLEAN" "EUNATCH" "EUSERS" "EWOULDBLOCK" "EXFULL"
+	     "UTSNAME"))
   
 (defpackage "FORMAT")
 (defpackage "COMMON-LISP"
@@ -357,8 +362,8 @@
              "COMPILATION-SPEED" "COMPILE" "COMPILE-FILE"
 	     "COMPILE-FILE-PATHNAME"
              "COMPILED-FUNCTION" "COMPILED-FUNCTION-P"
-	     "COMPILER-MACRO" "COMPILER-MACRO-FUNCTION" "COMPILER-MACROEXPAND"
-	     "COMPILER-MACROEXPAND-1" "COMPLEMENT" "COMPLEX"
+	     "COMPILER-MACRO" "COMPILER-MACRO-FUNCTION"
+	     "COMPLEMENT" "COMPLEX"
 	     "COMPLEXP" "COMPUTE-RESTARTS" "CONCATENATE"
 	     "CONCATENATED-STREAM" "CONCATENATED-STREAM-STREAMS"
 	     "COND" "CONDITION"
@@ -370,7 +375,8 @@
              "COUNT-IF-NOT" "CTYPECASE" "DEBUG" "DECF" "DECLAIM"
              "DECLARATION" "DECLARE" "DECODE-FLOAT" "DECODE-UNIVERSAL-TIME"
              "DEFCONSTANT" "DEFINE-COMPILER-MACRO" "DEFINE-CONDITION"
-             "DEFINE-MODIFY-MACRO" "DEFINE-SETF-METHOD" "DEFMACRO" "DEFPACKAGE"
+             "DEFINE-MODIFY-MACRO" "DEFINE-SYMBOL-MACRO"
+	     "DEFMACRO" "DEFPACKAGE"
              "DEFPARAMETER" "DEFSETF" "DEFSTRUCT" "DEFTYPE" "DEFUN" "DEFVAR"
              "DELETE" "DELETE-DUPLICATES" "DELETE-FILE" "DELETE-IF"
              "DELETE-IF-NOT" "DELETE-PACKAGE"
@@ -406,7 +412,6 @@
              "GENTEMP" "GET" "GET-DECODED-TIME" "GET-DISPATCH-MACRO-CHARACTER"
              "GET-INTERNAL-REAL-TIME" "GET-INTERNAL-RUN-TIME"
              "GET-MACRO-CHARACTER" "GET-OUTPUT-STREAM-STRING" "GET-PROPERTIES"
-             "GET-SETF-METHOD" "GET-SETF-METHOD-MULTIPLE-VALUE"
              "GET-UNIVERSAL-TIME" "GETF" "GETHASH" "GO" "GRAPHIC-CHAR-P"
              "HANDLER-BIND" "HANDLER-CASE" "HASH-TABLE" "HASH-TABLE-COUNT"
              "HASH-TABLE-P" "HASH-TABLE-REHASH-THRESHOLD"
@@ -437,9 +442,7 @@
              "LIST-LENGTH" "LISTEN" "LISTP" "LOAD"
 	     "LOAD-LOGICAL-PATHNAME-TRANSLATIONS" "LOAD-TIME-VALUE" "LOCALLY"
              "LOG" "LOGAND" "LOGANDC1" "LOGANDC2" "LOGBITP" "LOGCOUNT" "LOGEQV"
-	     "LOGICAL-PATHNAME"
-             "LOGICAL-PATHNAME-P"
-	     "LOGICAL-PATHNAME-TRANSLATIONS"
+	     "LOGICAL-PATHNAME" "LOGICAL-PATHNAME-TRANSLATIONS"
 	     "LOGIOR" "LOGNAND" "LOGNOR" "LOGNOT" "LOGORC1" "LOGORC2" "LOGTEST"
              "LOGXOR" "LONG-FLOAT" "LONG-FLOAT-EPSILON"
              "LONG-FLOAT-NEGATIVE-EPSILON" "LONG-SITE-NAME" "LOOP"
@@ -511,7 +514,7 @@
              "SIMPLE-BIT-VECTOR-P" "SIMPLE-CONDITION"
              "SIMPLE-CONDITION-FORMAT-ARGUMENTS"
 	     "SIMPLE-CONDITION-FORMAT-CONTROL"
-             "SIMPLE-CONDITION-FORMAT-STRING" "SIMPLE-ERROR"
+	     "SIMPLE-ERROR"
              "SIMPLE-STRING" "SIMPLE-STRING-P" "SIMPLE-TYPE-ERROR"
              "SIMPLE-VECTOR" "SIMPLE-VECTOR-P" "SIMPLE-WARNING" "SIN"
              "SINGLE-FLOAT" "SINGLE-FLOAT-EPSILON"
@@ -564,8 +567,8 @@
              "COMPUTE-APPLICABLE-METHODS" "DEFCLASS" "DEFGENERIC"
              "DEFINE-METHOD-COMBINATION" "DEFMETHOD" "DESCRIBE-OBJECT"
 	     "ENSURE-GENERIC-FUNCTION"
-	     "FIND-METHOD" "FUNCTION-KEYWORDS" "GENERIC-FLET"
-             "GENERIC-LABELS" "INITIALIZE-INSTANCE" "INVALID-METHOD-ERROR"
+	     "FIND-METHOD" "FUNCTION-KEYWORDS"
+	     "INITIALIZE-INSTANCE" "INVALID-METHOD-ERROR"
              "MAKE-INSTANCE" "MAKE-INSTANCES-OBSOLETE"
              "METHOD-COMBINATION-ERROR" "METHOD-QUALIFIERS" "NEXT-METHOD-P"
              "NO-APPLICABLE-METHOD" "NO-NEXT-METHOD" "PRINT-OBJECT"
@@ -574,11 +577,9 @@
              "SLOT-UNBOUND" "SLOT-VALUE" "STANDARD" "STANDARD-CLASS"
              "STANDARD-GENERIC-FUNCTION" "STANDARD-METHOD" "STANDARD-OBJECT"
 	     "UPDATE-INSTANCE-FOR-DIFFERENT-CLASS"
-             "UPDATE-INSTANCE-FOR-REDEFINED-CLASS" "WITH-ACCESSORS"
-             "WITH-ADDED-METHODS" "WITH-SLOTS"
+             "UPDATE-INSTANCE-FOR-REDEFINED-CLASS" "WITH-ACCESSORS" "WITH-SLOTS"
 	     "MAKE-METHOD" "UNBOUND-SLOT" "UNBOUND-SLOT-INSTANCE"
              "CELL-ERROR-NAME"
-	     "HASH-TABLE-WEAK-P"
              "GET-SETF-EXPANSION" "STYLE-WARNING" 
              "PRINT-NOT-READABLE-OBJECT"
              "DEFINE-SETF-EXPANDER" "PRINT-NOT-READABLE" "METHOD"
@@ -616,11 +617,11 @@
        "DOUBLE-FLOAT-P" "LONG-FLOAT-P" "SIMPLE-ARRAY-P" "SINGLE-FLOAT-P"))
   (intern name "KERNEL"))
 (defpackage #+pmax "PMAX" #+sparc "SPARC" #+ibmrt "RT"
-            #+x86 "X86" #+hppa "HPPA" #+alpha "ALPHA" #+sgi "SGI"
+            #+x86 "X86" #+hppa "HPPA" #+alpha "ALPHA" #+sgi "SGI" #+ppc "PPC"
             (:nicknames "VM" #+(or pmax sgi) "MIPS"
 			#+pmax "OLD-MIPS" #+sparc "OLD-SPARC" #+ibmrt "OLD-RT"
 			#+x86 "OLD-X86" #+hppa "HPPA" #+alpha "OLD-ALPHA"
-			#+sgi "OLD-SGI")
+			#+sgi "OLD-SGI" #+ppc "OLD-PPC")
             (:import-from "LISP" "%ARRAY-TYPEP" "%ASET" "%BITSET" "%CHARSET"
              "%PUT" "%RPLACA" "%RPLACD" "%SBITSET" "%SCHARSET"
              "%SET-FDEFINITION" "%SET-FILL-POINTER"
@@ -694,10 +695,7 @@
 	   "FUNCALLABLE-INSTANCE-HEADER-TYPE" 
 	   "FUNCALLABLE-INSTANCE-INFO-OFFSET"
 	   "FUNCTION-ARGLIST-SLOT" "FUNCTION-CODE-OFFSET"
-	   "FUNCTION-END-BREAKPOINT-TRAP" "FUNCTION-HEADER-ARGLIST-SLOT"
-	   "FUNCTION-HEADER-CODE-OFFSET" "FUNCTION-HEADER-NAME-SLOT"
-	   "FUNCTION-HEADER-NEXT-SLOT" "FUNCTION-HEADER-SELF-SLOT"
-	   "FUNCTION-HEADER-TYPE" "FUNCTION-HEADER-TYPE-SLOT"
+	   "FUNCTION-END-BREAKPOINT-TRAP" "FUNCTION-HEADER-TYPE"
 	   "FUNCTION-NAME-SLOT" "FUNCTION-NEXT-SLOT" "FUNCTION-POINTER-TYPE"
 	   "FUNCTION-SELF-SLOT" "FUNCTION-TYPE-SLOT"
 	   "FUNCALLABLE-INSTANCE-LAYOUT-SLOT"
@@ -792,6 +790,10 @@
 	   "WEAK-POINTER-SIZE" "WEAK-POINTER-TYPE" "WEAK-POINTER-VALUE-SLOT"
 	   "WORD-BITS" "WORD-BYTES" "WORD-REG-SC-NUMBER" "WORD-SHIFT"
 	   "ZERO-SC-NUMBER"
+	   #+sparc "POSITIVE-FIXNUM-BITS"
+	   #+sparc "FIXNUM-TAG-BITS"
+	   #+sparc "FIXNUM-TAG-MASK"
+	   #+sparc "PSEUDO-ATOMIC-TRAP"
 	   ))
 
 (defpackage "CONDITIONS")
@@ -853,7 +855,8 @@
 	   "ADD-OFFS-NOTE-HOOK"
 	   "ADD-OFFS-COMMENT-HOOK"
 	   "DSTATE-CUR-ADDR"
-	   "DSTATE-NEXT-ADDR"	     
+	   "DSTATE-NEXT-ADDR"
+	   #+sparc "MAYBE-NOTE-STATIC-FUNCTION"
 	   ))
 (dolist
     (name
@@ -877,9 +880,13 @@
 
 (intern "CHAR" "LISP")
 (defpackage "EXTENSIONS"
-	    (:nicknames "EXTENSIONS")
-            (:export "*AFTER-GC-HOOKS*" "*AFTER-SAVE-INITIALIZATIONS*"
-             "*ALL-MODIFIER-NAMES*" "*BACKUP-EXTENSION*" "*BEFORE-GC-HOOKS*"
+  (:nicknames "EXTENSIONS")
+  (:import-from "LISP" "GET-SETF-METHOD")
+  (:import-from "KERNEL" "*ANSI-DEFSTRUCT-OPTIONS-P*")
+  (:export   "*AFTER-GC-HOOKS*" "*AFTER-SAVE-INITIALIZATIONS*"
+	     "*ALL-MODIFIER-NAMES*" "*ANSI-DEFSTRUCT-OPTIONS-P*"
+	     "*AUTOLOAD-TRANSLATIONS*"
+	     "*BACKUP-EXTENSION*" "*BEFORE-GC-HOOKS*"
              "*BEFORE-SAVE-INITIALIZATIONS*" "*BLOCK-COMPILE-DEFAULT*"
              "*BYTES-CONSED-BETWEEN-GCS*" "*CHAR" "*CLX-FDS-TO-DISPLAYS*"
              "*COMMAND-LINE-STRINGS*" "*COMMAND-LINE-SWITCHES*"
@@ -894,6 +901,7 @@
              "*ENCLOSING-SOURCE-CUTOFF*" "*ENVIRONMENT-LIST*"
              "*ERROR-PRINT-LENGTH*" "*ERROR-PRINT-LEVEL*" "*GC-INHIBIT-HOOK*"
              "*GC-NOTIFY-AFTER*" "*GC-NOTIFY-BEFORE*" "*GC-VERBOSE*"
+	     "GET-SETF-METHOD" ; temp until uses are converted to g-s-expansion
              "*HEMLOCK-VERSION*" "*IGNORE-FLOATING-POINT-UNDERFLOW*"
              "*INFO-ENVIRONMENT*" "*INTEXP-MAXIMUM-EXPONENT*"
              "*KEYWORD-PACKAGE*" "*LISP-PACKAGE*" "*LOAD-IF-SOURCE-NEWER*"
@@ -902,6 +910,7 @@
 	     "*PROMPT*" "*REQUIRE-VERBOSE*"
 	     "*SETF-FDEFINITION-HOOK*"
              "*TOP-LEVEL-AUTO-DECLARE*"
+	     "COMPILER-MACROEXPAND-1" "COMPILER-MACROEXPAND"
              "*UNDEFINED-WARNING-LIMIT*" "ACCEPT-TCP-CONNECTION"
              "ADD-OOB-HANDLER" "AMBIGUOUS-FILES" "ARGUMENT-LIST" "ASSQ"
              "BASIC-DEFINITION" "BIGNUMP" "BITP" "CACHE-HASH-EQ"
@@ -924,7 +933,7 @@
              "DELETEF" "DELQ" "DISABLE-CLX-EVENT-HANDLING"
              "DO-ALPHA-KEY-EVENTS" "DO-ANONYMOUS" "DO-INFO"
              "DOUBLE-FLOAT-NEGATIVE-INFINITY" "DOUBLE-FLOAT-POSITIVE-INFINITY"
-             "DOUBLE-FLOATP" "DOVECTOR" "E" "ENABLE-CLX-EVENT-HANDLING"
+             "DOUBLE-FLOATP" "DOVECTOR" "ENABLE-CLX-EVENT-HANDLING"
              "ENCAPSULATE" "ENCAPSULATED-DEFINITION" "ENCAPSULATED-P"
              "END-BLOCK" "ENUMERATE-SEARCH-LIST" "FILE-COMMENT" "FILE-WRITABLE"
              "FINALIZE" "FIXNUMP" "FLOAT-DENORMALIZED-P" "FLOAT-INFINITY-P"
@@ -951,7 +960,8 @@
 	     "MAKE-STREAM-COMMAND" "MAKE-WEAK-POINTER"
              "MAYBE-INLINE" "MEMQ" "NAME-KEYSYM" "NTOHL" "NTOHS"
              "OBJECT-SET-EVENT-HANDLER" "OLD-TRACE" "OLD-UNTRACE" "ONCE-ONLY"
-             "OPEN-CLX-DISPLAY" "OPTIMIZE-INTERFACE" "PARSE-TIME"
+             "OPEN-CLX-DISPLAY" "OPTIMIZE-INTERFACE"
+	     "PACKAGE-CHILDREN" "PACKAGE-PARENT" "PARSE-TIME"
              "PRINT-DIRECTORY" "PRINT-HERALD" "PRINT-PRETTY-KEY"
              "PRINT-PRETTY-KEY-EVENT" "PROCESS-ALIVE-P" "PROCESS-CLOSE"
              "PROCESS-CORE-DUMPED" "PROCESS-ERROR" "PROCESS-EXIT-CODE"
@@ -1023,7 +1033,8 @@
 	     "STREAM-LISTEN" "STREAM-PEEK-CHAR" "STREAM-READ-BYTE"
 	     "STREAM-READ-CHAR" "STREAM-READ-CHAR-NO-HANG" "STREAM-READ-LINE"
 	     "STREAM-START-LINE-P" "STREAM-TERPRI" "STREAM-UNREAD-CHAR"
-	     "STREAM-WRITE-BYTE" "STREAM-WRITE-CHAR" "STREAM-WRITE-STRING"))
+	     "STREAM-WRITE-BYTE" "STREAM-WRITE-CHAR" "STREAM-WRITE-STRING"
+	     "STREAM-READ-SEQUENCE" "STREAM-WRITE-SEQUENCE"))
 
 (defpackage "LOOP")
 (dolist
@@ -1159,6 +1170,7 @@
 	   "NON-DESCRIPTOR-STACK" "NOTE-ENVIRONMENT-START"
 	   "NOTE-THIS-LOCATION" "OPTIMIZER"
 	   "PACK-TRACE-TABLE" "PMAX-FASL-FILE-IMPLEMENTATION" "POLICY"
+	   "PPC-FASL-FILE-IMPLEMENTATION"
 	   "PREDICATE" "PRIMITIVE-TYPE" "PRIMITIVE-TYPE-OF"
 	   "PRIMITIVE-TYPE-OR-LOSE" "PRIMITIVE-TYPE-VOP" "PUSH-VALUES"
 	   "READ-PACKED-BIT-VECTOR" "READ-VAR-INTEGER" "READ-VAR-STRING"
@@ -1435,14 +1447,14 @@
 (defpackage "KERNEL"
   (:import-from "LISP" "BOOLEAN")
   (:import-from "C-CALL" "VOID")
-  (:export "%ACOS" "%ACOSH" "%ARRAY-AVAILABLE-ELEMENTS"
+  (:export "*ANSI-DEFSTRUCT-OPTIONS-P*"
+	   "%ACOS" "%ACOSH" "%ARRAY-AVAILABLE-ELEMENTS"
 	   "%ARRAY-DATA-VECTOR" "%ARRAY-DIMENSION" "%ARRAY-DISPLACED-P"
 	   "%ARRAY-DISPLACEMENT" "%ARRAY-FILL-POINTER"
 	   "%ARRAY-FILL-POINTER-P" "%ASIN" "%ASINH" "%ATAN" "%ATAN2" "%ATANH"
 	   "%CALLER-FRAME-AND-PC" "%CHECK-BOUND" "%CLOSURE-FUNCTION"
 	   "%CLOSURE-INDEX-REF" "%COS" "%COS-QUICK" "%COSH" "%DEPOSIT-FIELD"
-	   "%DOUBLE-FLOAT" "%DPB" "%EXP" "%EXPM1" "%FUNCTION-HEADER-ARGLIST"
-	   "%FUNCTION-HEADER-NAME" "%FUNCTION-HEADER-TYPE" "%HYPOT" "%LDB"
+	   "%DOUBLE-FLOAT" "%DPB" "%EXP" "%EXPM1" "%HYPOT" "%LDB"
 	   "%LOG" "%LOGB" "%LOG10" "%LOG1P" "%LONG-FLOAT"
 	   "%MAKE-COMPLEX" "%MAKE-FUNCALLABLE-INSTANCE" "%MAKE-RATIO"
 	   "%MASK-FIELD" "%NEGATE" "%POW"
@@ -1488,7 +1500,9 @@
 	   "CODE-INSTRUCTIONS" "COMPLEX-DOUBLE-FLOAT-P" "COMPLEX-FLOAT-P"
 	   "COMPLEX-LONG-FLOAT-P" "COMPLEX-RATIONAL-P" "COMPLEX-SINGLE-FLOAT-P"
 	   "CONSED-SEQUENCE" "CONSTANT-TYPE"
-	   "CONSTANT-TYPE-P" "CONSTANT-TYPE-TYPE" "CONTAINING-INTEGER-TYPE"
+	   "CONSTANT-TYPE-P" "CONSTANT-TYPE-TYPE"
+	   "CONS-TYPE" "CONS-TYPE-CAR-TYPE" "CONS-TYPE-CDR-TYPE" "CONS-TYPE-P"
+	   "CONTAINING-INTEGER-TYPE"
 	   "CONTROL-STACK-POINTER-SAP" "COPY-FROM-SYSTEM-AREA"
 	   "COPY-NUMERIC-TYPE" "COPY-TO-SYSTEM-AREA" "CSUBTYPEP" "CTYPE"
 	   "CTYPE-OF" "CTYPE-P" "CTYPEP" "CURRENT-FP" "CURRENT-SP"
@@ -1519,7 +1533,8 @@
 	   "LONG-FLOAT-LOW-BITS" "LONG-FLOAT-MID-BITS" "LONG-FLOAT-P"
 	   "LRA" "LRA-CODE-HEADER" "LRA-P"
 	   "MAKE-ALIEN-TYPE-TYPE" "MAKE-ARGS-TYPE"
-	   "MAKE-ARRAY-HEADER" "MAKE-DOUBLE-FLOAT" "MAKE-FUNCTION-TYPE"
+	   "MAKE-ARRAY-HEADER"  "MAKE-CONS-TYPE"
+	   "MAKE-DOUBLE-FLOAT" "MAKE-FUNCTION-TYPE"
 	   "MAKE-KEY-INFO" "MAKE-LISP-OBJ" "MAKE-LONG-FLOAT"
 	   "MAKE-MEMBER-TYPE"
 	   "MAKE-NAMED-TYPE" "MAKE-NUMERIC-TYPE" "MAKE-SINGLE-FLOAT"
@@ -1570,7 +1585,8 @@
 	   "OBJECT-NOT-INSTANCE-ERROR" "OBJECT-NOT-SYMBOL-ERROR"
 	   "OBJECT-NOT-TYPE-ERROR" "OBJECT-NOT-UNSIGNED-BYTE-32-ERROR"
 	   "OBJECT-NOT-VECTOR-ERROR" "OBJECT-NOT-WEAK-POINTER-ERROR"
-	   "ODD-KEYWORD-ARGUMENTS-ERROR" "OUTPUT-OBJECT" "OUTPUT-UGLY-OBJECT"
+	   "ODD-KEYWORD-ARGUMENTS-ERROR" "ORDER-LAYOUT-INHERITS"
+	   "OUTPUT-OBJECT" "OUTPUT-UGLY-OBJECT"
 	   "PARSE-LAMBDA-LIST" "PARSE-UNKNOWN-TYPE"
 	   "PARSE-UNKNOWN-TYPE-SPECIFIER" "PATHNAME-DEVICE"
 	   "PATHNAME-DIRECTORY" "PATHNAME-HOST" "PATHNAME-NAME"
@@ -1590,6 +1606,7 @@
 	   "SIMPLE-ARRAY-SIGNED-BYTE-32-P" "SIMPLE-ARRAY-SIGNED-BYTE-8-P" 
 	   "SIMPLE-UNBOXED-ARRAY" "SINGLE-FLOAT-BITS" "SINGLE-FLOAT-EXPONENT"
 	   "SINGLE-FLOAT-P" "SINGLE-VALUE-TYPE" "SPECIFIER-TYPE" "STACK-REF"
+	   "STD-COMPUTE-CLASS-PRECEDENCE-LIST"
 	   "STREAMLIKE" "STRINGABLE" "STRINGLIKE"
 	   "%INSTANCE-LENGTH"
 	   "%INSTANCE-REF" "%INSTANCE-SET"
@@ -1619,7 +1636,7 @@
 	   "INFINITE-ERROR-PROTECT"
 	   "FIND-CALLER-NAME" "OLD-IN-PACKAGE"
 	   "%SET-SYMBOL-VALUE" "%SET-SYMBOL-PACKAGE" "OUTPUT-SYMBOL-NAME"
-	   "FSET" "RAW-DEFINITION" "%IN-PACKAGE" 
+	   "FSET" "RAW-DEFINITION" "%IN-PACKAGE" "%DEFPACKAGE"
 	   "INVOKE-MACROEXPAND-HOOK" "TYPEDEF-INIT" "DEFAULT-STRUCTURE-PRINT"
 	   "LAYOUT" "LAYOUT-LENGTH" "RATIOP" "FIXNUMP"
 	   "LAMBDA-WITH-ENVIRONMENT" "LAYOUT-PURE" "DSD-RAW-TYPE"
