@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/fd-stream.lisp,v 1.50 1999/12/04 16:02:34 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/fd-stream.lisp,v 1.57 2001/07/08 17:37:52 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -48,7 +48,7 @@
 ;;;
 ;;; Returns the next available buffer, creating one if necessary.
 ;;;
-(proclaim '(inline next-available-buffer))
+(declaim (inline next-available-buffer))
 ;;;
 (defun next-available-buffer ()
   (if *available-buffers*
@@ -207,7 +207,7 @@
 	)
       (let ((length (- end start)))
 	(multiple-value-bind
-	    (count errno)
+	      (count errno)
 	    (unix:unix-write (fd-stream-fd stream) base start length)
 	  (cond ((not count)
 		 (if (= errno unix:ewouldblock)
@@ -399,7 +399,7 @@
 (defun fd-sout (stream thing start end)
   (let ((start (or start 0))
 	(end (or end (length (the vector thing)))))
-    (declare (fixnum start end))
+    (declare (type index start end))
     (if (stringp thing)
 	(let ((last-newline (and (find #\newline (the simple-string thing)
 				       :start start :end end)
@@ -1023,6 +1023,12 @@
     (:charpos
      (fd-stream-char-pos stream))
     (:file-length
+     (unless (fd-stream-file stream)
+       (error 'simple-type-error
+	      :datum stream
+	      :expected-type 'file-stream
+	      :format-control "~s is not a stream associated with a file."
+	      :format-arguments (list stream)))
      (multiple-value-bind
 	 (okay dev ino mode nlink uid gid rdev size
 	       atime mtime ctime blksize blocks)
@@ -1033,9 +1039,9 @@
 	 (error "Error fstating ~S: ~A"
 		stream
 		(unix:get-unix-error-msg dev)))
-       (if (zerop (the index mode))
+       (if (zerop mode)
 	   nil
-	   (truncate (the index size) (fd-stream-element-size stream)))))
+	   (truncate size (fd-stream-element-size stream)))))
     (:file-position
      (fd-stream-file-position stream arg1))))
 
@@ -1044,16 +1050,15 @@
 ;;;
 (defun fd-stream-file-position (stream &optional newpos)
   (declare (type fd-stream stream)
-	   (type (or index (member nil :start :end)) newpos))
+	   (type (or (integer 0) (member nil :start :end)) newpos))
   (if (null newpos)
       (system:without-interrupts
-	;; First, find the position of the UNIX file descriptor in the
-	;; file.
+	;; First, find the position of the UNIX file descriptor in the file.
 	(multiple-value-bind
-	    (posn errno)
+	      (posn errno)
 	    (unix:unix-lseek (fd-stream-fd stream) 0 unix:l_incr)
-	  (declare (type (or index null) posn))
-	  (cond ((fixnump posn)
+	  (declare (type (or (integer 0) null) posn))
+	  (cond (posn
 		 ;; Adjust for buffered output:
 		 ;;  If there is any output buffered, the *real* file position
 		 ;; will be larger than reported by lseek because lseek
@@ -1081,8 +1086,9 @@
 		   (error "Error lseek'ing ~S: ~A"
 			  stream
 			  (unix:get-unix-error-msg errno)))))))
-      (let ((offset 0) origin)
-	(declare (type index offset))
+      (let ((offset 0)
+	    origin)
+	(declare (type (integer 0) offset))
 	;; Make sure we don't have any output pending, because if we move the
 	;; file pointer before writing this stuff, it will be written in the
 	;; wrong location.
@@ -1095,14 +1101,16 @@
 	(setf (fd-stream-unread stream) nil)
 	(setf (fd-stream-ibuf-head stream) 0)
 	(setf (fd-stream-ibuf-tail stream) 0)
-	;; Trash cashed value for listen, so that we check next time.
+	;; Trash cached value for listen, so that we check next time.
 	(setf (fd-stream-listen stream) nil)
 	;; Now move it.
 	(cond ((eq newpos :start)
-	       (setf offset 0 origin unix:l_set))
+	       (setf offset 0
+		     origin unix:l_set))
 	      ((eq newpos :end)
-	       (setf offset 0 origin unix:l_xtnd))
-	      ((typep newpos 'index)
+	       (setf offset 0
+		     origin unix:l_xtnd))
+	      ((typep newpos '(integer 0))
 	       (setf offset (* newpos (fd-stream-element-size stream))
 		     origin unix:l_set))
 	      (t
@@ -1110,7 +1118,7 @@
 	(multiple-value-bind
 	    (posn errno)
 	    (unix:unix-lseek (fd-stream-fd stream) offset origin)
-	  (cond ((typep posn 'fixnum)
+	  (cond (posn
 		 t)
 		((eq errno unix:espipe)
 		 nil)
@@ -1235,6 +1243,16 @@
 		   (unix:get-unix-error-msg err))
 	   nil))))
 
+;;; RETURN-STREAM -- internal
+;;;
+;;; (this is just to save having to reindent the code in OPEN...move it there)
+;;;
+(defmacro return-stream (class &body body)
+  (let ((stream (gensym)))
+    `(let ((,stream (progn ,@body)))
+       (return (if ,class
+                  (make-instance ,class :lisp-stream ,stream)
+                  ,stream)))))
 
 ;;; OPEN -- public
 ;;;
@@ -1247,6 +1265,7 @@
 	     (if-exists nil if-exists-given)
 	     (if-does-not-exist nil if-does-not-exist-given)
 	     (external-format :default)
+	     class
 	     &aux ; Squelch assignment warning.
 	     (direction direction)
 	     (if-does-not-exist if-does-not-exist)
@@ -1373,7 +1392,7 @@
 		  (unix:unix-open namestring mask mode)
 		  (values nil unix:enoent))
 	    (cond ((numberp fd)
-		   (return
+		   (return-stream class
 		    (case direction
 		      ((:input :output :io)
 		       (make-fd-stream fd
@@ -1451,10 +1470,7 @@
   (setf *terminal-io* (make-synonym-stream '*tty*))
   (setf *standard-output* (make-synonym-stream '*stdout*))
   (setf *standard-input*
-	(#-high-security
-	 make-two-way-stream
-	 #+high-security
-	 %make-two-way-stream (make-synonym-stream '*stdin*)
+	(make-two-way-stream (make-synonym-stream '*stdin*)
 			     *standard-output*))
   (setf *error-output* (make-synonym-stream '*stderr*))
   (setf *query-io* (make-synonym-stream '*terminal-io*))
@@ -1474,12 +1490,13 @@
 	(make-fd-stream 1 :name "Standard Output" :output t :buffering :line))
   (setf *stderr*
 	(make-fd-stream 2 :name "Standard Error" :output t :buffering :line))
-  (let ((tty (unix:unix-open "/dev/tty" unix:o_rdwr #o666)))
-    (if tty
-	(setf *tty*
+  (let ((tty (and (not *batch-mode*)
+		  (unix:unix-open "/dev/tty" unix:o_rdwr #o666))))
+    (setf *tty*
+	  (if tty
 	      (make-fd-stream tty :name "the Terminal" :input t :output t
-			      :buffering :line :auto-close t))
-	(setf *tty* (make-two-way-stream *stdin* *stdout*))))
+			      :buffering :line :auto-close t)
+	      (make-two-way-stream *stdin* *stdout*))))
   nil)
 
 
