@@ -6,7 +6,7 @@
 ;;; Scott Fahlman or slisp-group@cs.cmu.edu.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/filesys.lisp,v 1.56 1999/06/03 15:55:49 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/code/filesys.lisp,v 1.65 2001/12/13 13:45:08 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -38,9 +38,9 @@
 ;;; search-list := [^:/]*:
 ;;; file := [^/]*
 ;;; type := "." [^/.]*
-;;; version := "." ([0-9]+ | "*")
+;;; version := ".*" | ".~" ([0-9]+ | "*") "~"
 ;;;
-;;; Note: this grammer is ambiguous.  The string foo.bar.5 can be parsed
+;;; Note: this grammer is ambiguous.  The string foo.bar.~5~ can be parsed
 ;;; as either just the file specified or as specifying the file, type, and
 ;;; version.  Therefore, we use the following rules when confronted with
 ;;; an ambiguous file.type.version string:
@@ -170,44 +170,67 @@
 	      (t
 	       (make-pattern (pattern)))))))
 
+;;; extract-name-type-and-version  --  Internal.
+;;;
 (defun extract-name-type-and-version (namestr start end)
   (declare (type simple-base-string namestr)
 	   (type index start end))
-  (let* ((last-dot (position #\. namestr :start (1+ start) :end end
-			     :from-end t))
-	 (second-to-last-dot (and last-dot
-				  (position #\. namestr :start (1+ start)
-					    :end last-dot :from-end t)))
-	 (version :newest))
-    ;; If there is a second-to-last dot, check to see if there is a valid
-    ;; version after the last dot.
-    (when second-to-last-dot
-      (cond ((and (= (+ last-dot 2) end)
-		  (char= (schar namestr (1+ last-dot)) #\*))
-	     (setf version :wild))
-	    ((and (< (1+ last-dot) end)
-		  (do ((index (1+ last-dot) (1+ index)))
-		      ((= index end) t)
-		    (unless (char<= #\0 (schar namestr index) #\9)
-		      (return nil))))
-	     (setf version
-		   (parse-integer namestr :start (1+ last-dot) :end end)))
-	    (t
-	     (setf second-to-last-dot nil))))
-    (cond (second-to-last-dot
-	   (values (maybe-make-pattern namestr start second-to-last-dot)
-		   (maybe-make-pattern namestr
-				       (1+ second-to-last-dot)
-				       last-dot)
-		   version))
-	  (last-dot
-	   (values (maybe-make-pattern namestr start last-dot)
-		   (maybe-make-pattern namestr (1+ last-dot) end)
-		   version))
-	  (t
-	   (values (maybe-make-pattern namestr start end)
-		   nil
-		   version)))))
+  (labels
+      ((explicit-version (namestr start end)
+	 (cond ((or (< (- end start) 5)
+		    (char/= (schar namestr (1- end)) #\~))
+		(values :newest end))
+	       ((and (char= (schar namestr (- end 2)) #\*)
+		     (char= (schar namestr (- end 3)) #\~)
+		     (char= (schar namestr (- end 4)) #\.))
+		(values :wild (- end 4)))
+	       (t
+		(do ((i (- end 2) (1- i)))
+		    ((< i (+ start 2)) (values :newest end))
+		  (let ((char (schar namestr i)))
+		    (when (eql char #\~)
+		      (return (if (char= (schar namestr (1- i)) #\.)
+				  (values (parse-integer namestr :start (1+ i)
+							 :end (1- end))
+					  (1- i))
+				  (values :newest end))))
+		    (unless (char<= #\0 char #\9)
+		      (return (values :newest end))))))))
+       (any-version (namestr start end)
+	 ;; process end of string looking for a version candidate.
+	 (multiple-value-bind (version where)
+	   (explicit-version namestr start end)
+	   (cond ((not (eq version :newest))
+		  (values version where))
+		 ((and (>= (- end 2) start)
+		       (char= (schar namestr (- end 1)) #\*)
+		       (char= (schar namestr (- end 2)) #\.)
+		       (find #\. namestr
+			     :start (min (1+ start) (- end 2))
+			     :end (- end 2)))
+		  (values :wild (- end 2)))
+		 (t (values version where)))))
+       (any-type (namestr start end)
+	 ;; Process end of string looking for a type. A leading "."
+	 ;; is part of the name.
+	 (let ((where (position #\. namestr
+				:start (min (1+ start) end)
+				:end end :from-end t)))
+	   (when where
+	     (values where end))))
+       (any-name (namestr start end)
+	 (declare (ignore namestr))
+	 (values start end)))
+    (multiple-value-bind
+	(version vstart)(any-version namestr start end)
+      (multiple-value-bind
+	  (tstart tend)(any-type namestr start vstart)
+	(multiple-value-bind
+	    (nstart nend)(any-name namestr start (or tstart vstart))
+	  (values
+	   (maybe-make-pattern namestr nstart nend)
+	   (and tstart (maybe-make-pattern namestr (1+ tstart) tend))
+	   version))))))
 
 ;;; Take a string and return a list of cons cells that mark the char
 ;;; separated subseq. The first value t if absolute directories location.
@@ -390,13 +413,14 @@
 (defun unparse-unix-directory (pathname)
   (declare (type pathname pathname))
   (unparse-unix-directory-list (%pathname-directory pathname)))
-  
+
 (defun unparse-unix-file (pathname)
   (declare (type pathname pathname))
   (collect ((strings))
     (let* ((name (%pathname-name pathname))
 	   (type (%pathname-type pathname))
 	   (type-supplied (not (or (null type) (eq type :unspecific))))
+	   (logical-p (logical-pathname-p pathname))
 	   (version (%pathname-version pathname))
 	   (version-supplied (not (or (null version) (eq version :newest)))))
       (when name
@@ -407,11 +431,10 @@
 	(strings ".")
 	(strings (unparse-unix-piece type)))
       (when version-supplied
-	(unless type-supplied
-	  (error "Cannot specify the version without a type: ~S" pathname))
 	(strings (if (eq version :wild)
-		     ".*"
-		     (format nil ".~D" version)))))
+		     (if logical-p ".*" ".~*~")
+		     (format nil (if logical-p ".~D" ".~~~D~~")
+			     version)))))
     (and (strings) (apply #'concatenate 'simple-string (strings)))))
 
 (defun unparse-unix-namestring (pathname)
@@ -469,9 +492,9 @@
 	(when version-needed
 	  (typecase pathname-version
 	    ((member :wild)
-	     (strings ".*"))
+	     (strings ".~*~"))
 	    (integer
-	     (strings (format nil ".~D" pathname-version)))
+	     (strings (format nil ".~~~D~~" pathname-version)))
 	    (t
 	     (lose)))))
       (apply #'concatenate 'simple-string (strings)))))
@@ -499,125 +522,116 @@
 ;;;; Wildcard matching stuff.
 
 (defmacro enumerate-matches ((var pathname &optional result
-				  &key (verify-existance t))
+				  &key (verify-existance t) (follow-links t))
 			     &body body)
   (let ((body-name (gensym)))
     `(block nil
        (flet ((,body-name (,var)
 		,@body))
 	 (%enumerate-matches (pathname ,pathname)
-			     ,verify-existance
+			     ,verify-existance ,follow-links
 			     #',body-name)
 	 ,result))))
 
-(defun %enumerate-matches (pathname verify-existance function)
+(defun %enumerate-matches (pathname verify-existance follow-links function)
   (when (pathname-type pathname)
     (unless (pathname-name pathname)
       (error "Cannot supply a type without a name:~%  ~S" pathname)))
-  (when (and (integerp (pathname-version pathname))
-	     (member (pathname-type pathname) '(nil :unspecific)))
-    (error "Cannot supply a version without a type:~%  ~S" pathname))
   (let ((directory (pathname-directory pathname)))
     (if directory
 	(ecase (car directory)
 	  (:absolute
 	   (%enumerate-directories "/" (cdr directory) pathname
-				   verify-existance function))
+				   verify-existance follow-links
+				   nil function))
 	  (:relative
 	   (%enumerate-directories "" (cdr directory) pathname
-				   verify-existance function)))
+				   verify-existance follow-links
+				   nil function)))
 	(%enumerate-files "" pathname verify-existance function))))
 
-;;; This has been altered quite substantially.  
-
-;;; 1. :wild and :wild-inferiors were not patterns, and so are now
-;;; treated separately;
-
-;;; 2. Links to directories are no longer followed when
-;;; :wild-inferiors is being processed, due to the danger on systems
-;;; with linux and /proc filesystems infested with links to the root
-;;; directory. (Infinite loops: bad)
-
-(defun %enumerate-directories (head tail pathname verify-existance function)
+;;; %enumerate-directories  --   Internal
+;;;
+;;; The directory node and device numbers are maintained for the current path
+;;; during the search for the detection of paths loops upon :wild-inferiors.
+;;;
+(defun %enumerate-directories (head tail pathname verify-existance
+			       follow-links nodes function)
   (declare (simple-string head))
-  (if tail
-      (let ((piece (car tail)))
-	(etypecase piece
-	  (simple-string
-	   (%enumerate-directories (concatenate 'string head piece "/")
-				   (cdr tail) pathname verify-existance
-				   function))
-	  ((member :wild)
-	   (let ((dir (unix:open-dir head)))
-	     (when dir
-	       (unwind-protect
-		   (loop
-		     (let ((name (unix:read-dir dir)))
-		       (cond ((null name)
-			      (return))
-			     ((string= name "."))
-			     ((string= name ".."))
-			     (t
-			      (let ((subdir (concatenate 'string
-							 head name)))
-				;; see below for health warning
-				(when (eq (unix:unix-file-kind subdir t)
-					  :directory)
-				  (%enumerate-directories
-				   (concatenate 'string subdir "/")
-				   (cdr tail) pathname verify-existance
-				   function)))))))
-		 (unix:close-dir dir)))))	   
-	  ((member :wild-inferiors)
-	   (let ((dir (unix:open-dir head)))
-	     (when dir
-	       (unwind-protect
-		   (loop
-		     (let ((name (unix:read-dir dir)))
-		       (cond ((null name)
-			      (return))
-			     ((string= name "."))
-			     ((string= name ".."))
-			     (t ; always match 
-			      ;; warning - this depends
-			      ;; on lstat behaving in the same way on
-			      ;; different platforms. Basically,
-			      ;; lstat("foo") -> :link, lstat("foo/")
-			      ;; -> directory on linux 2.2.
-			      (let ((subdir (concatenate 'string
-							 head name)))
-				(when (eq (unix:unix-file-kind subdir t)
-					  :directory)
-				  (%enumerate-directories
-				   (concatenate 'string subdir "/")
-				   tail pathname verify-existance
-				   function)))))))
-		 (unix:close-dir dir))
-	       (%enumerate-files head pathname verify-existance	function))))
-	  (pattern
-	   (let ((dir (unix:open-dir head)))
-	     (when dir
-	       (unwind-protect
-		   (loop
-		     (let ((name (unix:read-dir dir)))
-		       (cond ((null name)
-			      (return))
-			     ((string= name "."))
-			     ((string= name ".."))
-			     ((pattern-matches piece name)
-			      (let ((subdir (concatenate 'string
-							 head name "/")))
-				(when (eq (unix:unix-file-kind subdir)
-					  :directory)
-				  (%enumerate-directories
-				   subdir (cdr tail) pathname verify-existance
-				   function)))))))
-		 (unix:close-dir dir)))))
-	  ((member :up)
-	   (%enumerate-directories (concatenate 'string head "../")
-				   (cdr tail) pathname verify-existance
-				   function))))
-      (%enumerate-files head pathname verify-existance function)))
+  (macrolet ((unix-xstat (name)
+	       `(if follow-links
+		    (unix:unix-stat ,name)
+		    (unix:unix-lstat ,name)))
+	     (with-directory-node-noted ((head) &body body)
+	       `(multiple-value-bind (res dev ino mode)
+		    (unix-xstat ,head)
+		  (when (and res (eql (logand mode unix:s-ifmt) unix:s-ifdir))
+		    (let ((nodes (cons (cons dev ino) nodes)))
+		      ,@body))))
+	     (do-directory-entries ((name directory) &body body)
+	       `(let ((dir (unix:open-dir ,directory)))
+		  (when dir
+		    (unwind-protect
+			 (loop
+			  (let ((,name (unix:read-dir dir)))
+			    (cond ((null ,name)
+				   (return))
+				  ((string= ,name "."))
+				  ((string= ,name ".."))
+				  (t
+				   ,@body))))
+		      (unix:close-dir dir))))))
+    (if tail
+	(let ((piece (car tail)))
+	  (etypecase piece
+	    (simple-string
+	     (let ((head (concatenate 'string head piece)))
+	       (with-directory-node-noted (head)
+		 (%enumerate-directories (concatenate 'string head "/")
+					 (cdr tail) pathname
+					 verify-existance follow-links
+					 nodes function))))
+	    ((member :wild-inferiors)
+	     (%enumerate-directories head (rest tail) pathname
+				     verify-existance follow-links
+				     nodes function)
+	     (do-directory-entries (name head)
+	       (let ((subdir (concatenate 'string head name)))
+		 (multiple-value-bind (res dev ino mode)
+		     (unix-xstat subdir)
+		   (declare (type (or fixnum null) mode))
+		   (when (and res (eql (logand mode unix:s-ifmt) unix:s-ifdir))
+		     (unless (dolist (dir nodes nil)
+			       (when (and (eql (car dir) dev)
+					  (eql (cdr dir) ino))
+				 (return t)))
+		       (let ((nodes (cons (cons dev ino) nodes))
+			     (subdir (concatenate 'string subdir "/")))
+			 (%enumerate-directories subdir tail pathname
+						 verify-existance follow-links
+						 nodes function))))))))
+	    ((or pattern (member :wild))
+	     (do-directory-entries (name head)
+	       (when (or (eq piece :wild) (pattern-matches piece name))
+		 (let ((subdir (concatenate 'string head name)))
+		   (multiple-value-bind (res dev ino mode)
+		       (unix-xstat subdir)
+		     (declare (type (or fixnum null) mode))
+		     (when (and res
+				(eql (logand mode unix:s-ifmt) unix:s-ifdir))
+		       (let ((nodes (cons (cons dev ino) nodes))
+			     (subdir (concatenate 'string subdir "/")))
+			 (%enumerate-directories subdir (rest tail) pathname
+						 verify-existance follow-links
+						 nodes function))))))))
+	    ((member :up)
+	     (let ((head (concatenate 'string head "..")))
+	       (with-directory-node-noted (head)
+		 (%enumerate-directories (concatenate 'string head "/")
+					 (rest tail) pathname
+					 verify-existance follow-links
+					 nodes function))))))
+	(%enumerate-files head pathname verify-existance function))))
 
 (defun %enumerate-files (directory pathname verify-existance function)
   (declare (simple-string directory))
@@ -708,7 +722,8 @@
   (enumerate-search-list
       (pathname path)
     (collect ((names))
-      (enumerate-matches (name pathname nil :verify-existance for-input)
+      (enumerate-matches (name pathname nil :verify-existance for-input
+			       :follow-links t)
 	(when (or (not executable-only)
 		  (and (eq (unix:unix-file-kind name) :file)
 		       (unix:unix-access name unix:x_ok)))
@@ -873,217 +888,38 @@
 
 ;;; DIRECTORY  --  public.
 ;;; 
-
-;;; modified 13/Apr/1999 csr21@cam.ac.uk Christophe Rhodes
-(defun directory (pathname &key (all t) (check-for-subdirs nil) (delete-duplicates nil) (follow-links nil) (resolve-links nil) (sort nil))
-  "Returns a list of pathnames, one for each file that matches the
-given pathname. Keyword arguments:
-
-all: If false, will ignore files with names beginning with a #\\.  
-
-check-for-subdirs: If true, will check the file to see if it is
-actually a directory, and if so the resultant pathname will reflect
-such.
-
-delete-duplicates: If true, will delete duplicate pathnames in the
-result list.
-
-follow-links: if true, will treat links linked to directories as
-directories. Warning: this may cause infinite loops if a symlink
-points to an ancestor (/proc/ on Linux is a spectacular example of
-this).
-
-resolve-links: if true, will attempt to resolve links in the resultant
-list (via truename). Duplicates resulting from the resolving process
-will be removed if delete-duplicates is non-nil.
-
-sort: if true, will attempt to sort the results alphabetically."
-  ;; I don't think that this can result in any
-  ;; duplicates. Let me know if you think otherwise.
-  (let ((answer 
-	 (dir (merge-pathnames pathname 
-			       (make-pathname :name :wild
-					      :type :wild))
-	      :all all
-	      :check-for-subdirs check-for-subdirs
-	      :follow-links follow-links
-	      :sort sort)))
-    ;; If the above assertion is correct, then this one is not needed.
-    (when delete-duplicates
-      (setf answer (delete-duplicates answer :test #'string=)))
-    (setf answer
-	  (if resolve-links
-	      (mapcar #'(lambda (x)
-			  (let ((*ignore-wildcards* t))
-			    (truename (parse-namestring x))))
-		      answer)
-	    (mapcar #'(lambda (x)
-			(let ((*ignore-wildcards* t))
-			  (parse-namestring x)))
-		    answer)))
-    (when delete-duplicates
-      (setf answer (delete-duplicates answer :test #'pathname=)))
-    answer))
-
-
-(defun dir (pathname &key all check-for-subdirs follow-links sort)
-  "Returns namestrings (suitable for parse-namestring) for all files
-matching the specification of pathname."
-  (let ((directory (pathname-directory pathname)))
-    (let ((answer (loop for d in (matching-directories directory '("") :all all :follow-links follow-links :sort sort)
-			append (matching-files d pathname :all all :sort sort))))
-      (if check-for-subdirs
-	  (mapcar #'(lambda (filename) 
-		      (if (eq :directory (unix:unix-file-kind filename (not follow-links)))
-			  (concatenate 'string filename "/")
-			filename))
-		  answer)
-	answer))))
-
-(defun directory-contents (dirstring &key (all t) (ignore-dots t) sort)
-  "Return a list of all files returned by unix:read-dir in the
-directory named by dirstring. If :ignore-dots is t (the default), then
-\".\" and \"..\" are not included. If all is nil, filenames beginning
-with #\\. are removed from the results."
-  (let ((dir (unix:open-dir dirstring)))
-    (let ((result (when dir
-		    (let ((contents (unwind-protect
-					(loop when (unix:read-dir dir)
-					      collect it
-					      else do (loop-finish))
-				      (unix:close-dir dir))))
-		      (unless all
-			(setf contents 
-			      (remove-if #'(lambda (filename) 
-					     (char= (aref filename 0) #\.))
-					 contents)))
-		      (if ignore-dots
-			  (remove-if #'(lambda (f) (or (equal f ".")
-						       (equal f "..")))
-				     contents)
-			contents)))))
-      (when sort
-	(setf result (sort result #'string<)))
-      result)))
-
-;;; The logic behind some of these cases is slightly contorted, but, I
-;;; believe, correct.
-(defun matching-directories (directory &optional (result '("")) &key all follow-links sort)
-  "Return a list of strings naming all directories matching the
-directory spec. If directory is a string, pathname-directory is called
-on it."
-  (labels ((string-add (string &key endp) 
-	    #'(lambda (s) (if endp
-			      (concatenate 'string s string)
-			    (concatenate 'string string s)))))
-    (when (typep directory 'string)
-      (setf directory (pathname-directory directory)))
-    (typecase (car directory)
-      ;; no more directory -- therefore, result contains all matching
-      ;; directories.
-      (null result)
-      ;; :absolute and :relative should reset the result list.
-      ((member :absolute) (matching-directories (cdr directory) '("/")
-						:all all
-						:follow-links follow-links
-						:sort sort))
-      ((member :relative) (matching-directories (cdr directory) '("")
-						:all all
-						:follow-links follow-links
-						:sort sort))
-      ;; this one's not too hard either, since .. is always valid, and
-      ;; I don't think we should protect people against (directory
-      ;; "/../") even if it weren't -- it's their problem.
-      ((member :up) (matching-directories (cdr directory) (mapcar (string-add "../" :endp t) result) :all all :follow-links follow-links :sort sort))
-      ;; We want all the directories such that each directory
-      ;; currently in the results list is checked for subdirectories
-      ;; that match the pattern.
-      ((satisfies pattern-p)
-       (let ((directories
-	      (remove-if-not 
-	       #'(lambda (file) (eq :directory (unix:unix-file-kind file (not follow-links))))
-	       (apply #'append 
-		      (mapcar #'(lambda (d) 
-				  (mapcar (string-add d) 
-					  (remove-if-not
-					   #'(lambda (str)
-					       (pattern-matches 
-						(car directory)
-						str))
-					      (directory-contents d :all all :sort sort))))
-			      result)))))
-       (matching-directories (cdr directory) (mapcar (string-add "/" :endp t) directories) :follow-links follow-links :all all :sort sort)))
-      ;; If the first directory entry is :wild, then we want all
-      ;; directories (remove-if-not-directory) contained in all of the
-      ;; directories currently in result.
-      ((member :wild)
-       (let ((directories 
-	      (remove-if-not 
-	       #'(lambda (file) (eq :directory (unix:unix-file-kind file (not follow-links))))
-	       (apply #'append 
-		      (mapcar #'(lambda (d) 
-				  (mapcar (string-add d) (directory-contents d :all all :sort sort))) 
-			      result)))))
-	 (when directories
-	   (matching-directories (cdr directory)
-				 (mapcar (string-add "/" :endp t) directories)
-				 :follow-links follow-links :all all :sort sort))))
-      ;; (:wild-inferiors ,@foo) is equivalent to (:wild
-      ;; :wild-inferiors ,@foo) and (,@foo).  The slowness resulting
-      ;; is due to the fact that each level requires traversal of all
-      ;; previous levels. I can't think of a nice way to do this -- it
-      ;; may have to be special-cased. However, it is tolerable now, I
-      ;; believe.  Sorting has to be done here too, as we are doing a
-      ;; recursion rightwards but sorting from the left.
-      ((member :wild-inferiors)
-       (let ((temp (append (matching-directories (cons :wild directory) result :follow-links follow-links :all all :sort sort)
-			   (matching-directories (cdr directory) result :follow-links follow-links :all all :sort sort))))
-	 (when sort
-	   (setf temp (sort temp #'string<)))
-	 temp))
-      (string (matching-directories 
-	       (cdr directory)
-	       (mapcar (string-add "/" :endp t)
-		       (remove-if-not
-			#'(lambda (file) (eq :directory (unix:unix-file-kind file (not follow-links))))
-			(mapcar (string-add (car directory) :endp t) result)))
-	       :follow-links follow-links :all all :sort sort)))))
-				    
-(defun matching-files (directory pathname &key all sort)
-  "Returns a list of strings naming all files in directory matching
-the name, type and version fields of pathname."
-  (let ((name (pathname-name pathname))
-	(type (pathname-type pathname))
-	(version (pathname-version pathname)))
-    (cond
-     ((member name '(nil :unspecific)) ())
-     ;; there's still a nasty interaction here on versioned
-     ;; files. (pathname-version "/home/csr21/*") => :newest.
-     ((or (pattern-p name)
-	  (pattern-p type)
-	  (eq name :wild)
-	  (eq type :wild))
-       (loop for file in (directory-contents directory :all all :sort sort)
-	     if (multiple-value-bind
-		    (file-name file-type file-version)
-		    (let ((*ignore-wildcards* t))
-		      (extract-name-type-and-version file 0 (length file)))
-		  (and (components-match file-name name)
-		       (components-match file-type type)
-		       (components-match file-version version)))
-	     collect (concatenate 'string directory file)))
-     ;; name is a string
-     (t 
-      (let ((file (concatenate 'string directory name)))
-	(unless (or (null type) (eq type :unspecific))
-	  (setf file (concatenate 'string file "." type)))
-	(unless (member version '(nil :newest :wild))
-	  (setf file (concatenate 'string file "."
-				  (quick-integer-to-string version))))
-	;; return nil if the file doesn't exist
-	(when (unix:unix-file-kind file) (list file))))
-      )))
-		  
+(defun directory (pathname &key (all t) (check-for-subdirs t)
+			   (truenamep t) (follow-links t))
+  "Returns a list of pathnames, one for each file that matches the given
+   pathname.  Supplying :ALL as nil causes this to ignore Unix dot files.  This
+   never includes Unix dot and dot-dot in the result.  If :TRUENAMEP is NIL,
+   then symblolic links in the result are not expanded which is not the
+   default because TRUENAME does follow links, and the result pathnames are
+   defined to be the TRUENAME of the pathname (the truename of a link may well
+   be in another directory.) If FOLLOW-LINKS is NIL then symbolic links are
+   not followed."
+  (let ((results nil))
+    (enumerate-search-list
+	(pathname (merge-pathnames pathname
+				   (make-pathname :name :wild
+						  :type :wild
+						  :version :wild)))
+      (enumerate-matches (name pathname nil :follow-links follow-links)
+	(when (or all
+		  (let ((slash (position #\/ name :from-end t)))
+		    (or (null slash)
+			(= (1+ slash) (length name))
+			(char/= (schar name (1+ slash)) #\.))))
+	  (push name results))))
+    (let ((*ignore-wildcards* t))
+      (mapcar #'(lambda (name)
+		  (let ((name (if (and check-for-subdirs
+				       (eq (unix:unix-file-kind name)
+					   :directory))
+				  (concatenate 'string name "/")
+				  name)))
+		    (if truenamep (truename name) (pathname name))))
+	      (sort (delete-duplicates results :test #'string=) #'string<)))))
 
 
 ;;;; Printing directories.
@@ -1104,7 +940,7 @@ the name, type and version fields of pathname."
 
 (defun print-directory-verbose (pathname all return-list)
   (let ((contents (directory pathname :all all :check-for-subdirs nil
-			     :follow-links nil))
+			     :truenamep nil))
 	(result nil))
     (format t "Directory of ~A :~%" (namestring pathname))
     (dolist (file contents)
@@ -1179,7 +1015,7 @@ the name, type and version fields of pathname."
 	(names ())
 	(cnt 0)
 	(max-len 0)
-	(result (directory pathname :all all :follow-links nil)))
+	(result (directory pathname :all all :truenamep nil)))
     (declare (list names) (fixnum max-len cnt))
     ;;
     ;; Get the data.
@@ -1308,7 +1144,7 @@ the name, type and version fields of pathname."
 			       ignore-types)
   (let ((files (directory (complete-file-directory-arg pathname defaults)
 			  :check-for-subdirs nil
-			  :follow-links nil)))
+			  :truenamep nil)))
     (cond ((null files)
 	   (values nil nil))
 	  ((null (cdr files))
@@ -1376,7 +1212,7 @@ the name, type and version fields of pathname."
    We look in the directory specified by Defaults as well as looking down
    the search list."
   (directory (complete-file-directory-arg pathname defaults)
-	     :follow-links nil
+	     :truenamep nil
 	     :check-for-subdirs nil))
 
 
