@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/float-trap.lisp,v 1.15 1999/12/05 16:22:17 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/float-trap.lisp,v 1.17 2001/12/06 19:15:40 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -156,7 +156,7 @@
 ;;;    Signal the appropriate condition when we get a floating-point error.
 ;;;
 
-#+FreeBSD
+#+BSD
 (define-condition floating-point-exception (arithmetic-error)
   ((flags :initarg :traps
 	  :reader floating-point-exception-traps))
@@ -189,7 +189,7 @@
 	   (error 'floating-point-underflow))
 	  ((not (zerop (logand float-inexact-trap-bit traps)))
 	   (error 'ext:floating-point-inexact))
-	  #+FreeBSD
+	  #+BSD
 	  ((zerop (ldb float-exceptions-byte modes))
 	   ;; I can't tell what caused the exception!!
 	   (error 'floating-point-exception
@@ -211,15 +211,16 @@
 	(trap-mask (dpb (lognot (float-trap-mask traps))
 			float-traps-byte #xffffffff))
 	(exception-mask (dpb (lognot (vm::float-trap-mask traps))
-			     float-sticky-bits #xffffffff)))
-    `(let ((orig-modes (floating-point-modes)))
+			     float-sticky-bits #xffffffff))
+	(orig-modes (gensym)))
+    `(let ((,orig-modes (floating-point-modes)))
       (unwind-protect
 	   (progn
 	     (setf (floating-point-modes)
-		   (logand orig-modes ,(logand trap-mask exception-mask)))
+		   (logand ,orig-modes ,(logand trap-mask exception-mask)))
 	     ,@body)
 	;; Restore the original traps and exceptions.
 	(setf (floating-point-modes)
-	      (logior (logand orig-modes ,(logior traps exceptions))
+	      (logior (logand ,orig-modes ,(logior traps exceptions))
 		      (logand (floating-point-modes)
 			      ,(logand trap-mask exception-mask))))))))
