@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/float.lisp,v 1.20 1999/11/19 15:12:36 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/float.lisp,v 1.24 2001/06/17 19:08:35 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -189,16 +189,26 @@
 (defconstant single-float-negative-epsilon
   (single-from-bits 0 (- vm:single-float-bias vm:single-float-digits) 1))
 (defconstant short-float-negative-epsilon single-float-negative-epsilon)
+#-(and long-float x86)
 (defconstant double-float-epsilon
   (double-from-bits 0 (- vm:double-float-bias (1- vm:double-float-digits)) 1))
+#+(and long-float x86)
+(defconstant double-float-epsilon
+  (double-from-bits 0 (- vm:double-float-bias (1- vm:double-float-digits))
+		    (expt 2 42)))
 #-long-float
 (defconstant long-float-epsilon double-float-epsilon)
 #+(and long-float x86)
 (defconstant long-float-epsilon
   (long-from-bits 0 (- vm:long-float-bias (1- vm:long-float-digits))
 		  (+ 1 (ash vm:long-float-hidden-bit 32))))
+#-(and long-float x86)
 (defconstant double-float-negative-epsilon
   (double-from-bits 0 (- vm:double-float-bias vm:double-float-digits) 1))
+#+(and long-float x86)
+(defconstant double-float-negative-epsilon
+  (double-from-bits 0 (- vm:double-float-bias vm:double-float-digits)
+		    (expt 2 42)))
 #-long-float
 (defconstant long-float-negative-epsilon double-float-negative-epsilon)
 #+(and long-float x86)
@@ -209,7 +219,7 @@
 
 ;;;; Float predicates and environment query:
 
-(proclaim '(maybe-inline float-denormalized-p float-infinity-p float-nan-p
+(declaim (maybe-inline float-denormalized-p float-infinity-p float-nan-p
 			 float-trapping-nan-p))
 
 ;;; FLOAT-DENORMALIZED-P  --  Public
@@ -288,7 +298,7 @@
 ;;; actual exponent (and hence how denormalized it is), otherwise we just
 ;;; return the number of digits or 0.
 ;;;
-(proclaim '(maybe-inline float-precision))
+(declaim (maybe-inline float-precision))
 (defun float-precision (f)
   "Returns a non-negative number of significant digits in it's float argument.
   Will be less than FLOAT-DIGITS if denormalized or zero."
@@ -336,7 +346,7 @@
     #+long-float
     (long-float vm:long-float-digits)))
 
-(proclaim '(inline float-digits float-radix))
+(declaim (inline float-digits float-radix))
 
 (defun float-digits (f)
   "Returns a non-negative number of radix-b digits used in the
@@ -351,18 +361,15 @@
 (defun float-radix (f)
   "Returns (as an integer) the radix b of its floating-point
    argument."
-  #-high-security
-  (declare (ignore f))
-  #+high-security
-  (check-type f float)
+  (declare (float f)(ignorable f))
   2)
 
 
 
 ;;;; INTEGER-DECODE-FLOAT and DECODE-FLOAT:
 
-(proclaim '(maybe-inline integer-decode-single-float
-			 integer-decode-double-float))
+(declaim (maybe-inline integer-decode-single-float
+		       integer-decode-double-float))
 
 ;;; INTEGER-DECODE-SINGLE-DENORM  --  Internal
 ;;;
@@ -552,7 +559,7 @@
      (integer-decode-long-float x))))
 
 
-(proclaim '(maybe-inline decode-single-float decode-double-float))
+(declaim (maybe-inline decode-single-float decode-double-float))
 
 ;;; DECODE-SINGLE-DENORM  --  Internal
 ;;;
@@ -700,7 +707,7 @@
 
 ;;;; SCALE-FLOAT:
 
-(proclaim '(maybe-inline scale-single-float scale-double-float))
+(declaim (maybe-inline scale-single-float scale-double-float))
 
 ;;; SCALE-FLOAT-MAYBE-UNDERFLOW  --  Internal
 ;;;
@@ -1075,6 +1082,7 @@ rounding modes & do ieee round-to-integer.
     ((rational) x)))
 
 
+#+nil
 (defun rationalize (x)
   "Converts any REAL to a RATIONAL.  Floats are converted to a simple rational
   representation exploiting the assumption that floats are only accurate to
@@ -1114,3 +1122,107 @@ rounding modes & do ieee round-to-integer.
 		   (integer-/-integer num den))
 		(declare ((dispatch-type x) xx)))))))
     ((rational) x)))
+
+;;; RATIONALIZE  --  Public
+;;;
+;;; The algorithm here is the method described in CLISP.  Bruno Haible has
+;;; graciously given permission to use this algorithm.  He says, "You can use
+;;; it, if you present the following explanation of the algorithm."
+;;;
+;;; Algorithm (recursively presented):
+;;;   If x is a rational number, return x.
+;;;   If x = 0.0, return 0.
+;;;   If x < 0.0, return (- (rationalize (- x))).
+;;;   If x > 0.0:
+;;;     Call (integer-decode-float x). It returns a m,e,s=1 (mantissa,
+;;;     exponent, sign).
+;;;     If m = 0 or e >= 0: return x = m*2^e.
+;;;     Search a rational number between a = (m-1/2)*2^e and b = (m+1/2)*2^e
+;;;     with smallest possible numerator and denominator.
+;;;     Note 1: If m is a power of 2, we ought to take a = (m-1/4)*2^e.
+;;;       But in this case the result will be x itself anyway, regardless of
+;;;       the choice of a. Therefore we can simply ignore this case.
+;;;     Note 2: At first, we need to consider the closed interval [a,b].
+;;;       but since a and b have the denominator 2^(|e|+1) whereas x itself
+;;;       has a denominator <= 2^|e|, we can restrict the seach to the open
+;;;       interval (a,b).
+;;;     So, for given a and b (0 < a < b) we are searching a rational number
+;;;     y with a <= y <= b.
+;;;     Recursive algorithm fraction_between(a,b):
+;;;       c := (ceiling a)
+;;;       if c < b
+;;;         then return c       ; because a <= c < b, c integer
+;;;         else
+;;;           ; a is not integer (otherwise we would have had c = a < b)
+;;;           k := c-1          ; k = floor(a), k < a < b <= k+1
+;;;           return y = k + 1/fraction_between(1/(b-k), 1/(a-k))
+;;;                             ; note 1 <= 1/(b-k) < 1/(a-k)
+;;;
+;;; You can see that we are actually computing a continued fraction expansion.
+;;;
+;;; Algorithm (iterative):
+;;;   If x is rational, return x.
+;;;   Call (integer-decode-float x). It returns a m,e,s (mantissa,
+;;;     exponent, sign).
+;;;   If m = 0 or e >= 0, return m*2^e*s. (This includes the case x = 0.0.)
+;;;   Create rational numbers a := (2*m-1)*2^(e-1) and b := (2*m+1)*2^(e-1)
+;;;   (positive and already in lowest terms because the denominator is a
+;;;   power of two and the numerator is odd).
+;;;   Start a continued fraction expansion
+;;;     p[-1] := 0, p[0] := 1, q[-1] := 1, q[0] := 0, i := 0.
+;;;   Loop
+;;;     c := (ceiling a)
+;;;     if c >= b
+;;;       then k := c-1, partial_quotient(k), (a,b) := (1/(b-k),1/(a-k)),
+;;;            goto Loop
+;;;   finally partial_quotient(c).
+;;;   Here partial_quotient(c) denotes the iteration
+;;;     i := i+1, p[i] := c*p[i-1]+p[i-2], q[i] := c*q[i-1]+q[i-2].
+;;;   At the end, return s * (p[i]/q[i]).
+;;;   This rational number is already in lowest terms because
+;;;   p[i]*q[i-1]-p[i-1]*q[i] = (-1)^i.
+;;;
+(defun rationalize (x)
+  "Converts any REAL to a RATIONAL.  Floats are converted to a simple rational
+  representation exploiting the assumption that floats are only accurate to
+  their precision.  RATIONALIZE (and also RATIONAL) preserve the invariant:
+      (= x (float (rationalize x) x))"
+  (number-dispatch ((x real))
+    (((foreach single-float double-float #+long-float long-float))
+     ;; This is a fairly straigtforward implementation of the iterative
+     ;; algorithm above.
+     (multiple-value-bind (frac expo sign)
+	 (integer-decode-float x)
+       (cond ((or (zerop frac) (>= expo 0))
+	      (if (minusp sign)
+		  (- (ash frac expo))
+		  (ash frac expo)))
+	     (t
+	      ;; expo < 0 and (2*m-1) and (2*m+1) are coprime to 2^(1-e),
+	      ;; so build the fraction up immediately, without having to do
+	      ;; a gcd.
+	      (let ((a (build-ratio (- (* 2 frac) 1) (ash 1 (- 1 expo))))
+		    (b (build-ratio (+ (* 2 frac) 1) (ash 1 (- 1 expo))))
+		    (p0 0)
+		    (q0 1)
+		    (p1 1)
+		    (q1 0))
+		(do ((c (ceiling a) (ceiling a)))
+		    ((< c b)
+		     (let ((top (+ (* c p1) p0))
+			   (bot (+ (* c q1) q0)))
+		       (build-ratio (if (minusp sign)
+					(- top)
+					top)
+				    bot)))
+		  (let* ((k (- c 1))
+			 (p2 (+ (* k p1) p0))
+			 (q2 (+ (* k q1) q0)))
+		    (psetf a (/ (- b k))
+			   b (/ (- a k)))
+		    (setf p0 p1
+			  q0 q1
+			  p1 p2
+			  q1 q2))))))))
+    ((rational) x)))
+
