@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/format-time.lisp,v 1.6 2000/01/30 13:58:27 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/format-time.lisp,v 1.8 2001/12/06 23:39:35 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 
@@ -68,11 +68,13 @@
    destination which can be accepted by the Format function.  The
    timezone keyword is an integer specifying hours west of Greenwich.
    The style keyword can be :short (numeric date), :long (months and
-   weekdays expressed as words), :abbreviated (like :long but words are
-   abbreviated), or :government (of the form \"XX Mon XX XX:XX:XX\")
-   The keyword date-first, if nil, will print the time first instead
-   of the date (the default).  The print- keywords, if nil, inhibit
-   the printing of the obvious part of the time/date."
+   weekdays expressed as words), :abbreviated (like :long but words
+   are abbreviated), :rfc1123 (conforming to RFC 1123), :government
+   (of the form \"XX Mon XX XX:XX:XX\"), or :iso8601 (conforming to
+   ISO 8601), which is the recommended way of printing date and time.
+   The keyword date-first, if nil, will print the time first instead of
+   the date (the default).  The print- keywords, if nil, inhibit the
+   printing of the obvious part of the time/date."
   (unless (valid-destination-p destination)
     (error "~A: Not a valid format destination." destination))
   (unless (integerp universal-time)
@@ -93,11 +95,15 @@
 	   (case style
 	     (:short "~D/~D/~2,'0D")			;;  MM/DD/YY
 	     ((:abbreviated :long) "~A ~D, ~D")		;;  Month DD, YYYY
+	     (:rfc1123 "~2,'0D ~A ~4,'0D")		;;  DD Mon YYYY
 	     (:government "~2,'0D ~:@(~A~) ~2,'0D")	;;  DD MON YY
+	     (:iso8601 "~4,'0D-~2,'0D-~2,'0D")          ;;  YYYY-MM-DD
 	     (t
 	      (error "~A: Unrecognized :style keyword value." style))))
 	  (time-args
-	   (list mins (max (mod hours 12) (1+ (mod (1- hours) 12)))))
+	   (case style
+	     ((:rfc1123 :iso8601) (list mins hours))
+	     (t (list mins (max (mod hours 12) (1+ (mod (1- hours) 12)))))))
 	  (date-args (case style
 		       (:short
 			(list month day (mod year 100)))
@@ -105,15 +111,23 @@
 			(list (svref abbrev-month-table (1- month)) day year))
 		       (:long
 			(list (svref long-month-table (1- month)) day year))
+		       (:rfc1123
+			(list day (svref abbrev-month-table (1- month)) year))
 		       (:government
 			(list day (svref abbrev-month-table (1- month))
-			      (mod year 100))))))
-      (declare (simple-string time-string date-string))
+			      (mod year 100)))
+		       (:iso8601
+			(list year month day))))
+	  (timezone-name (case style
+			   (:rfc1123 (timezone-rfc1123-name dst tz))
+			   (:iso8601 (timezone-iso8601-name dst tz))
+			   (t (timezone-name dst tz)))))
+      (declare (simple-string time-string date-string timezone-name))
       (when print-weekday
 	(push (case style
 		((:short :long) (svref long-weekday-table dow))
-		(:abbreviated (svref abbrev-weekday-table dow))
-		(:government (svref abbrev-weekday-table dow)))
+		((:abbreviated :rfc1123 :government :iso8601)
+		 (svref abbrev-weekday-table dow)))
 	      date-args)
 	(setq date-string
 	      (concatenate 'simple-string "~A, " date-string)))
@@ -121,32 +135,35 @@
 	(push secs time-args)
 	(setq time-string
 	      (concatenate 'simple-string time-string ":~2,'0D")))
-      (when print-meridian
+      (when (and print-meridian (not (member style '(:rfc1123 :iso8601))))
 	(push (signum (floor hours 12)) time-args)
 	(setq time-string
 	      (concatenate 'simple-string time-string " ~[am~;pm~]")))
       (apply #'format destination
-	     (if date-first
+	     (if (or date-first (eq style :iso8601))
 		 (concatenate 'simple-string date-string " " time-string
-			      (if print-timezone " ~A"))
+			      (when print-timezone
+				(if (eq style :iso8601)
+				    "~A"
+				    " ~A")))
 		 (concatenate 'simple-string time-string " " date-string
 			      (if print-timezone " ~A")))
-	     (if date-first
+	     (if (or date-first (eq style :iso8601))
 		 (nconc date-args (nreverse time-args)
 			(if print-timezone
-			    (list (timezone-name dst tz))))
+			    (list timezone-name)))
 		 (nconc (nreverse time-args) date-args
 			(if print-timezone
-			    (list (timezone-name dst tz)))))))))
+			    (list timezone-name))))))))
 
 (defun timezone-name (dst tz)
   (if (and (integerp tz)
-	   (or (and dst (= tz 0))
+	   (or (and (not dst) (= tz 0))
 	       (<= 5 tz 8)))
       (svref (if dst daylight-table timezone-table) tz)
       (multiple-value-bind
 	  (rest seconds)
-	  (truncate (* tz 60 60) 60)
+	  (truncate (* (if dst (1- tz) tz) 60 60) 60)
 	(multiple-value-bind
 	    (hours minutes)
 	    (truncate rest 60)
@@ -158,6 +175,36 @@
 		  (not (zerop seconds))
 		  (abs seconds))))))
 
+;;; RFC 1123 style timezone: GMT, +1000, -1000.
+;;; Timezone is the negative of the CL timezone.
+;;;
+(defun timezone-rfc1123-name (dst tz)
+  (let ((tz (- tz)))
+    (if (and (integerp tz)
+	     (or (and (not dst) (= tz 0))
+		 (<= 5 tz 8)))
+	(svref (if dst daylight-table timezone-table) tz)
+	(multiple-value-bind
+	      (hours minutes)
+	    (truncate (if dst (1+ tz) tz))
+	  (format nil "~C~2,'0D~2,'0D"
+		  (if (minusp tz) #\- #\+)
+		  (abs hours)
+		  (abs (truncate (* minutes 60))))))))
+
+;;; ISO 8601 style timezone: Z, +1000, -1000.
+;;; Timezone is the negative of the CL timezone.
+;;;
+(defun timezone-iso8601-name (dst tz)
+  (let ((tz (- tz)))
+    (if (and (not dst) (= tz 0))
+	"Z"
+	(multiple-value-bind (hours minutes)
+	    (truncate (if dst (1+ tz) tz))
+	  (format nil "~C~2,'0D:~2,'0D"
+		  (if (minusp tz) #\- #\+)
+		  (abs hours)
+		  (abs (truncate (* minutes 60))))))))
 
 
 ;;; Format-Decoded-Time - External.
