@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/generic-site.lisp,v 1.13 1998/05/04 00:05:54 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/generic-site.lisp,v 1.14 2000/08/20 14:42:46 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -23,8 +23,8 @@
 ;;; found a bug.  Fill in the maintainer's address here..
 (rplaca
  (cdr (member :bugs *herald-items*))
- '("Send questions and bug reports to your local CMU CL maintainer, "
-   "or to cmucl-help@cons.org. and cmucl-imp@cons.org. respectively." terpri
+ '("Send questions and bug reports to your local CMU CL maintainer, " terpri
+   "or see http://www.cons.org/cmucl/support.html ." terpri
    "Loaded subsystems:" terpri))
 
 ;;; If you have sources installed on your system, un-comment the following form
