@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/hash-new.lisp,v 1.12 2000/01/16 20:10:39 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/hash-new.lisp,v 1.17 2001/03/30 11:01:56 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -230,7 +230,7 @@
 	     (length (if (<= scaled-size 37) 37 (almost-primify scaled-size))))
 	(declare (type index size+1 scaled-size length))
 	(when weak-p
-	  (format t "* Creating unsupported weak-p hash table~%"))
+	  (format *debug-io* "* Creating unsupported weak-p hash table~%"))
 	(let* ((index-vector
 		(make-array length :element-type '(unsigned-byte 32)
 			    :initial-element 0))
@@ -313,7 +313,7 @@
 	      (fixnum
 	       (+ rehash-size old-size))
 	      (float
-	       (the index (round (* rehash-size old-size)))))))
+	       (the (values index t) (round (* rehash-size old-size)))))))
 	 (new-kv-vector (make-array (* 2 new-size) :initial-element :empty))
 	 (new-next-vector (make-array new-size
 				      :element-type '(unsigned-byte 32)
@@ -862,14 +862,21 @@
 (defmacro sxhash-string (sequence)
   (let ((data (gensym))
 	(start (gensym))
-	(end (gensym)))
-    `(with-array-data ((,data (the string ,sequence))
-		       (,start)
-		       (,end))
-       (if (zerop ,start)
-	   (%sxhash-simple-substring ,data ,end)
-	   (sxhash-simple-string (coerce (the string ,sequence)
-					 'simple-string))))))
+	(end (gensym))
+	(fill-end (gensym)))
+    (once-only ((n-sequence sequence))
+      `(let ((,fill-end (if (array-has-fill-pointer-p ,n-sequence)
+			    (fill-pointer ,n-sequence)
+			    nil)))
+	 (with-array-data ((,data (the (values string &rest t) ,n-sequence))
+			   (,start)
+			   (,end ,fill-end))
+	   (if (zerop ,start)
+	       (%sxhash-simple-substring ,data ,end)
+	       (sxhash-simple-string (coerce (the (values string &rest t)
+					       ,n-sequence)
+					     'simple-string))))))))
+
 
 (defmacro sxhash-list (sequence depth &key (equalp nil))
   `(if (= ,depth sxhash-max-depth)
