@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/hash.lisp,v 1.36 2000/01/16 20:10:39 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/hash.lisp,v 1.39 2000/07/06 18:36:37 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -350,7 +350,7 @@
 		  (fixnum
 		   (+ rehash-size old-length))
 		  (float
-		   (the index (round (* rehash-size old-length))))))
+		   (the (values index t) (round (* rehash-size old-length))))))
 	      old-length))
 	 (new-vector (make-array new-length :initial-element nil))
 	 #-gengc (weak-p (hash-table-weak-p table)))
@@ -456,8 +456,7 @@
 		       (weak-pointer-value (hash-table-bucket-key bucket))
 		     (assert valid)
 		     (when (eq key bucket-key)
-		       (return (values (hash-table-bucket-value bucket)
-				       t))))))
+		       (return (values (hash-table-bucket-value bucket) t))))))
 	       (do ((bucket (svref vector index)
 			    (hash-table-bucket-next bucket)))
 		   ((null bucket) (values default nil))
@@ -832,12 +831,13 @@
   (let ((data (gensym))
 	(start (gensym))
 	(end (gensym)))
-    `(with-array-data ((,data (the string ,sequence))
+    `(with-array-data ((,data (the (values string &rest t) ,sequence))
 		       (,start)
 		       (,end))
        (if (zerop ,start)
 	   (%sxhash-simple-substring ,data ,end)
-	   (sxhash-simple-string (coerce (the string ,sequence)
+	   (sxhash-simple-string (coerce (the (values string &rest t)
+					      ,sequence)
 					 'simple-string))))))
 
 (defmacro sxhash-list (sequence depth &key (equalp nil))
