@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/list.lisp,v 1.22 1998/05/09 22:16:45 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/code/list.lisp,v 1.26 2001/03/04 23:37:32 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -39,19 +39,19 @@
 	  rassoc rassoc-if rassoc-if-not
 	  complement constantly))
 
-(proclaim '(maybe-inline
-	    tree-equal list-length nth %setnth nthcdr last make-list append
-	    copy-list copy-alist copy-tree revappend nconc nreconc butlast
-	    nbutlast ldiff member member-if member-if-not tailp adjoin union
-	    nunion intersection nintersection set-difference nset-difference
-	    set-exclusive-or nset-exclusive-or subsetp acons pairlis assoc
-	    assoc-if assoc-if-not rassoc rassoc-if rassoc-if-not subst subst-if
-	    subst-if-not nsubst nsubst-if nsubst-if-not sublis nsublis))
+(declaim (maybe-inline
+	  tree-equal list-length nth %setnth nthcdr last make-list append
+	  copy-list copy-alist copy-tree revappend nconc nreconc butlast
+	  nbutlast ldiff member member-if member-if-not tailp adjoin union
+	  nunion intersection nintersection set-difference nset-difference
+	  set-exclusive-or nset-exclusive-or subsetp acons pairlis assoc
+	  assoc-if assoc-if-not rassoc rassoc-if rassoc-if-not subst subst-if
+	  subst-if-not nsubst nsubst-if nsubst-if-not sublis nsublis))
 
 
 (in-package "EXTENSIONS")
 (export '(assq memq delq))
-(proclaim '(maybe-inline delq))
+(declaim (maybe-inline delq))
 (in-package "LISP")
 
 
@@ -91,7 +91,7 @@
                       (cons se1 se2))
 
 
-(proclaim '(maybe-inline tree-equal-test tree-equal-test-not))
+(declaim (maybe-inline tree-equal-test tree-equal-test-not))
 
 (defun tree-equal-test-not (x y test-not)
   (cond ((consp x)
@@ -359,34 +359,36 @@
   "Returns a new list the same as List without the last N conses.
    List must not be circular."
   (declare (list list) (type index n))
-  (let ((length (do ((list list (cdr list))
-		     (i 0 (1+ i)))
-		    ((atom list) (1- i)))))
-    (declare (type index length))
-    (unless (< length n)
-      (do* ((top (cdr list) (cdr top))
-	    (result (list (car list)))
-	    (splice result)
-	    (count length (1- count)))
-	   ((= count n) result)
-	(declare (type index count))
-	(setq splice (cdr (rplacd splice (list (car top)))))))))
+  (unless (null list)
+    (let ((length (do ((list list (cdr list))
+		       (i 0 (1+ i)))
+		      ((atom list) (1- i)))))
+      (declare (type index length))
+      (unless (< length n)
+	(do* ((top (cdr list) (cdr top))
+	      (result (list (car list)))
+	      (splice result)
+	      (count length (1- count)))
+	     ((= count n) result)
+	  (declare (type index count))
+	  (setq splice (cdr (rplacd splice (list (car top))))))))))
 
 (defun nbutlast (list &optional (n 1))
   "Modifies List to remove the last N conses. List must not be circular."
   (declare (list list) (type index n))
-  (let ((length (do ((list list (cdr list))
-		     (i 0 (1+ i)))
-		    ((atom list) (1- i)))))
-    (declare (type index length))
-    (unless (< length n)
-      (do ((1st (cdr list) (cdr 1st))
-	   (2nd list 1st)
-	   (count length (1- count)))
-	  ((= count n)
-	   (rplacd 2nd ())
-	   list)
-	(declare (type index count))))))
+  (unless (null list)
+    (let ((length (do ((list list (cdr list))
+		       (i 0 (1+ i)))
+		      ((atom list) (1- i)))))
+      (declare (type index length))
+      (unless (< length n)
+	(do ((1st (cdr list) (cdr 1st))
+	     (2nd list 1st)
+	     (count length (1- count)))
+	    ((= count n)
+	     (rplacd 2nd ())
+	     list)
+	  (declare (type index count)))))))
 
 (defun ldiff (list object)
   "Returns a new list, whose elements are those of List that appear before
@@ -859,6 +861,7 @@
 (defmacro assoc-guts (test-guy)
   `(do ((alist alist (cdr alist)))
        ((endp alist))
+     (declare (optimize (inhibit-warnings 3)))
      (if (car alist)
 	 (if ,test-guy (return (car alist))))))
 
@@ -985,7 +988,8 @@
 
 (defun memq (item list)
   "Returns tail of list beginning with first element eq to item"
-  (declare (inline member))
+  (declare (inline member)
+	   (optimize (inhibit-warnings 3))) ; from MEMBER optimizations
   (member item list :test #'eq))
 
 (defun assq (item alist)
