@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/load.lisp,v 1.74 1999/02/25 13:02:58 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/code/load.lisp,v 1.81 2001/10/16 19:19:54 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -43,7 +43,13 @@
     "fasl")
   "A list of the object file types recognized by LOAD.")
 
-(declaim (list *load-source-types* *load-object-types*))
+(defvar *load-lp-object-types*
+  '(#.(string-upcase (c:backend-fasl-file-type c:*backend*))
+    #.(string-upcase (c:backend-byte-fasl-file-type c:*backend*))
+    "FASL")
+  "A list of the object file types recognized by LOAD for logical pathnames.")
+
+(declaim (list *load-source-types* *load-object-types* *load-lp-object-types*))
 
 (defvar *load-verbose* t
   "The default for the :VERBOSE argument to Load.")
@@ -66,7 +72,7 @@
   "Count of the number of recursive loads.")
 (declaim (type index *load-depth*))
 (defvar *fasl-file*)
-(declaim (type lisp-stream fasl-file))
+(declaim (type lisp-stream *fasl-file*))
 
 
 ;;; LOAD-FRESH-LINE -- internal.
@@ -233,7 +239,7 @@
 
 ;;;; Utilities for reading from the fasl file.
 
-(proclaim '(inline read-byte))
+(declaim (inline read-byte))
 
 ;;; Fast-Read-U-Integer  --  Internal
 ;;;
@@ -464,7 +470,8 @@
 ;;;
 (defun load (filename &key (verbose nil verbose-p) (print nil print-p)
 		      (if-source-newer nil if-source-newer-p)
-		      (if-does-not-exist :error) contents)
+		      (if-does-not-exist :error) contents
+		      (external-format :default))
   "Loads the file named by Filename into the Lisp environment.  The file type
    (a.k.a extension) is defaulted if missing.  These options are defined:
 
@@ -496,10 +503,11 @@
    The variables *LOAD-VERBOSE*, *LOAD-PRINT* and EXT:*LOAD-IF-SOURCE-NEWER*
    determine the defaults for the corresponding keyword arguments.  These
    variables are also bound to the specified argument values, so specifying a
-   keyword affects nested loads.  The variables EXT:*LOAD-SOURCE-TYPES* and
-   EXT:*LOAD-OBJECT-TYPES* determine the file types that we use for defaulting
-   when none is specified."
-  (declare (type (or null (member :source :binary)) contents))
+   keyword affects nested loads.  The variables EXT:*LOAD-SOURCE-TYPES*,
+   EXT:*LOAD-OBJECT-TYPES*, and EXT:*LOAD-LP-OBJECT-TYPES* determine the file
+   types that we use for defaulting when none is specified."
+  (declare (type (or null (member :source :binary)) contents)
+	   (ignore external-format))
   (collect ((vars)
 	    (vals))
     (macrolet ((frob (wot)
@@ -528,7 +536,7 @@
 		   (fasload filename)
 		   (sloload filename))
 	       (let ((pn (merge-pathnames (pathname filename)
-					  *default-pathname-defaults*)))
+					  *default-pathname-defaults* nil)))
 		 (if (wild-pathname-p pn)
 		     (let ((files (directory pn)))
 		       #+high-security
@@ -599,18 +607,18 @@
 
 ;;; TRY-DEFAULT-TYPES  --  Internal
 ;;;
-(defun try-default-types (pathname types lp-type)
-  ;; Modified 18-Jan-97/pw for logical-pathname support.
+(defun try-default-types (pathname types lp-types)
   (flet ((frob (pathname type)
 	   (let* ((pn (make-pathname :type type :defaults pathname))
 		  (tn (probe-file pn)))
 	     (values pn tn))))
-    (if (logical-pathname-p pathname)
-	(frob pathname lp-type)
-	(dolist (type types (values nil nil))
-	  (multiple-value-bind (pn tn)(frob pathname type)
-	    (when tn
-	      (return (values pn tn))))))))
+    (dolist (type (if (logical-pathname-p pathname)
+		      lp-types types)
+	     (values nil nil))
+      (multiple-value-bind (pn tn)
+	  (frob pathname type)
+	(when tn
+	  (return (values pn tn)))))))
 
 ;;; INTERNAL-LOAD-DEFAULT-TYPE  --  Internal
 ;;;
@@ -619,10 +627,10 @@
 (defun internal-load-default-type (pathname if-does-not-exist)
   (multiple-value-bind
       (src-pn src-tn)
-      (try-default-types pathname *load-source-types* "LISP")
+      (try-default-types pathname *load-source-types* '("LISP"))
     (multiple-value-bind
 	(obj-pn obj-tn)
-	(try-default-types pathname *load-object-types* "FASL")
+	(try-default-types pathname *load-object-types* *load-lp-object-types*)
       (cond
        ((and obj-tn src-tn
 	     (> (file-write-date src-tn) (file-write-date obj-tn)))
@@ -1379,4 +1387,4 @@
     code-object))
 
 
-(proclaim '(maybe-inline read-byte))
+(declaim (maybe-inline read-byte))
