@@ -1,4 +1,4 @@
-;;;   -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Lowercase:T -*-
+;;;   -*- Mode: LISP; Package: ANSI-LOOP; Syntax: Common-lisp; Base: 10; Lowercase:T -*-
 ;;;>
 ;;;> Portions of LOOP are Copyright (c) 1986 by the Massachusetts Institute of Technology.
 ;;;> All Rights Reserved.
@@ -49,7 +49,7 @@
 
 #+cmu
 (ext:file-comment
- "$Header: /home/CVS-cmucl/src/code/loop.lisp,v 1.8 1999/03/19 13:45:50 dtc Exp $")
+ "$Header: /home/CVS-cmucl/src/code/loop.lisp,v 1.10 2001/10/04 13:21:45 toy Exp $")
 
 ;;;; LOOP Iteration Macro
 
@@ -348,12 +348,12 @@ constructed.
 	    (long-float		most-positive-long-float	most-negative-long-float))
 	;; CMUCL has infinities so let's use them.
 	#+CMU
-	  '((fixnum             most-positive-fixnum            most-negative-fixnum)
-	    (short-float        single-float-positive-infinity  single-float-negative-infinity)
-	    (single-float       single-float-positive-infinity  single-float-negative-infinity)
-	    (double-float       double-float-positive-infinity  double-float-negative-infinity)
-	    (long-float         long-float-positive-infinity    long-float-negative-infinity))
-	;;If we don't know, then we cannot provide "infinite" initial values for any of the
+	  '((fixnum		most-positive-fixnum			most-negative-fixnum)
+	    (short-float	ext:single-float-positive-infinity	ext:single-float-negative-infinity)
+	    (single-float	ext:single-float-positive-infinity	ext:single-float-negative-infinity)
+	    (double-float	ext:double-float-positive-infinity	ext:double-float-negative-infinity)
+	    (long-float		ext:long-float-positive-infinity	ext:long-float-negative-infinity))
+	;; If we don't know, then we cannot provide "infinite" initial values for any of the
 	;; types but FIXNUM:
 	#-(or Genera CLOE-Runtime Minima CMU)
 	  '((fixnum   		most-positive-fixnum		most-negative-fixnum))
@@ -985,6 +985,12 @@ collected result will be returned as the value of the LOOP."
 
 
 
+(defun loop-build-destructuring-bindings (crocks forms)
+  (if crocks
+      `((destructuring-bind ,(car crocks) ,(cadr crocks)
+        ,@(loop-build-destructuring-bindings (cddr crocks) forms)))
+      forms))
+
 (defun loop-translate (*loop-source-code* *loop-macro-environment* *loop-universe*)
   (let ((*loop-original-source-code* *loop-source-code*)
 	(*loop-source-context* nil)
@@ -1035,10 +1041,7 @@ collected result will be returned as the value of the LOOP."
 				    (*loop-destructuring-hooks* (first *loop-destructuring-hooks*))
 				    (t 'let))
 			     ,vars
-			     ,@(if crocks
-				   `((destructuring-bind ,@crocks
-					 ,@forms))
-				 forms)))))))
+			     ,@(loop-build-destructuring-bindings crocks forms)))))))
       answer)))
 
 
