@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/macros.lisp,v 1.60 1999/09/15 15:12:41 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/macros.lisp,v 1.71 2001/12/04 22:27:42 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -24,9 +24,7 @@
 	  get-setf-expansion define-setf-expander
           define-modify-macro destructuring-bind nth-value
           otherwise ; Sacred to CASE and related macros.
-	  define-compiler-macro
-	  ;; CLtL1 versions:
-	  define-setf-method get-setf-method get-setf-method-multiple-value))
+	  define-compiler-macro))
 
 (in-package "EXTENSIONS")
 (export '(do-anonymous collect iterate))
@@ -57,7 +55,9 @@
       (let ((form (car tail)))
 	(cond ((and (stringp form) (cdr tail))
 	       (if doc-string-allowed
-		   (setq doc form)
+		   (setq doc form
+			 ;; Only one doc string is allowed.
+			 doc-string-allowed nil)
 		   (return (values tail (nreverse decls) doc))))
 	      ((not (and (consp form) (symbolp (car form))))
 	       (return (values tail (nreverse decls) doc)))
@@ -85,7 +85,11 @@
 		    ,@local-decs
 		    (block ,name
 		      ,body))))
-	`(c::%defmacro ',name #',def ',lambda-list ,doc)))))
+	`(progn
+	   (eval-when (:compile-toplevel)
+	     (c::do-macro-compile-time ',name #',def))
+	   (eval-when (:load-toplevel :execute)
+	     (c::%defmacro ',name #',def ',lambda-list ,doc)))))))
 
 
 ;;; %Defmacro, %%Defmacro  --  Internal
@@ -128,7 +132,12 @@
 		    ,@local-decs
 		    (block ,name
 		      ,body))))
-	`(c::%define-compiler-macro ',name #',def ',lambda-list ,doc)))))
+	`(progn
+	   (eval-when (:compile-toplevel)
+	     (c::do-compiler-macro-compile-time ',name #',def))
+	   (eval-when (:load-toplevel :execute)
+	     (c::%define-compiler-macro ',name #',def ',lambda-list ,doc)))))))
+
 
 (defun c::%define-compiler-macro (name definition lambda-list doc)
   (assert (eval:interpreted-function-p definition))
@@ -144,6 +153,34 @@
   name)
 
 
+
+;;;; DEFINE-SYMBOL-MACRO
+
+;;; define-symbol-macro  --  Public
+;;;
+(defmacro define-symbol-macro (name expansion)
+  `(eval-when (compile load eval)
+     (%define-symbol-macro ',name ',expansion)))
+;;;
+(defun %define-symbol-macro (name expansion)
+  (unless (symbolp name)
+    (error 'simple-type-error :datum name :expected-type 'symbol
+	   :format-control "Symbol macro name is not a symbol: ~S."
+	   :format-arguments (list name)))
+  (ecase (info variable kind name)
+    ((:macro :global nil)
+     (setf (info variable kind name) :macro)
+     (setf (info variable macro-expansion name) expansion))
+    (:special
+     (error 'simple-program-error
+	    :format-control "Symbol macro name already declared special: ~S."
+	    :format-arguments (list name)))
+    (:constant
+     (error 'simple-program-error
+	    :format-control "Symbol macro name already declared constant: ~S."
+	    :format-arguments (list name))))
+  name)
+    
 
 ;;; DEFTYPE is a lot like DEFMACRO.
 
@@ -193,7 +230,7 @@
 (defparameter defsetf-error-string "Setf expander for ~S cannot be called with ~S args.")
 
 (defmacro define-setf-expander (access-fn lambda-list &body body)
-  "Syntax like DEFMACRO, but creates a Setf-Method generator.  The body
+  "Syntax like DEFMACRO, but creates a Setf-Expansion generator.  The body
   must be a form that returns the five magical values."
   (unless (symbolp access-fn)
     (error "~S -- Access-function name not a symbol in DEFINE-SETF-EXPANDER."
@@ -337,7 +374,7 @@
   value, the old value is not clobbered.  The third argument is an optional
   documentation string for the variable."
   `(progn
-    (proclaim '(special ,var))
+    (declaim (special ,var))
      ,@(when valp
 	 `((unless (boundp ',var)
 	     (setq ,var ,val))))
@@ -351,7 +388,7 @@
   variable special and sets its value to VAL.  The third argument is
   an optional documentation string for the parameter."
   `(progn
-    (proclaim '(special ,var))
+    (declaim (special ,var))
     (setq ,var ,val)
     ,@(when docp
 	`((setf (documentation ',var 'variable) ',doc)))
@@ -497,11 +534,12 @@
            (declare (ignore ,@dummy-list))
            ,keeper))
       (once-only ((n n))
-	`(case (the fixnum ,n)
+	`(case (the (values fixnum &rest t) ,n)
 	   (0 (nth-value 0 ,form))
 	   (1 (nth-value 1 ,form))
 	   (2 (nth-value 2 ,form))
-	   (T (nth (the fixnum ,n) (multiple-value-list ,form)))))))
+	   (T (nth (the (values fixnum &rest t) ,n)
+			(multiple-value-list ,form)))))))
 
 
 ;;;; SETF and friends.
@@ -1146,12 +1184,6 @@
 ;;; RESTART-CASE allowing keyform to be set and retested.
 ;;;
 (defun case-body (name keyform cases multi-p test errorp proceedp)
-  (when (null cases)
-    (error 'simple-type-error
-	   :datum cases
-	   :expected-type 'list
-	   :format-control "~S was called without any clauses."
-	   :format-arguments (list name)))
   (let ((keyform-value (gensym))
 	(clauses ())
 	(keys ()))
@@ -1463,7 +1495,7 @@
   (cond ((numberp count)
          `(do ((,var 0 (1+ ,var)))
               ((>= ,var ,count) ,result)
-	    (declare (type unsigned-byte ,var))
+	    (declare (type (integer 0 ,count) ,var))
             ,@body))
         (t (let ((v1 (gensym)))
              `(do ((,var 0 (1+ ,var)) (,v1 ,count))
@@ -1519,10 +1551,6 @@
 
 ;;;; Miscellaneous macros:
 
-(defmacro locally (&rest forms)
-  "A form providing a container for locally-scoped variables."
-  `(let () ,@forms))
-
 (defmacro psetq (&rest pairs)
   (do ((lets nil)
        (setqs nil)
