@@ -5,7 +5,7 @@
 ;;; the Public domain, and is provided 'as is'.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/multi-proc.lisp,v 1.37 1999/09/25 15:39:08 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/multi-proc.lisp,v 1.39 2000/08/19 17:48:19 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -999,6 +999,10 @@
    (setf (process-wait-timeout process) nil)
    (setf (process-wait-return-value process) nil)
    (setf (process-interrupts process) nil)
+   (setf (process-scheduled-real-time process) (get-real-time))
+   (setf (process-scheduled-run-time process) (get-run-time))
+   (setf (process-%real-time process) 0d0)
+   (setf (process-%run-time process) 0d0)
    (setf (process-state process) :active)
    (push process *all-processes*))
   process)
@@ -1481,32 +1485,32 @@
 
 	(ecase direction
 	  (:input
-	   (unless (fd-usable-for-input)
-	     ;; Wait until input possible.
-	     (sys:with-fd-handler (fd :input
-				   #'(lambda (fd)
-				       (declare (ignore fd)
-						(optimize (speed 3)
-							  (safety 0)))
-				       (mp:process-yield)))
-	       (if timeout
-		   (mp:process-wait-with-timeout "Input Wait" timeout
-						 #'fd-usable-for-input)
-		   (mp:process-wait "Input Wait" #'fd-usable-for-input)))))
+	   (or (fd-usable-for-input)
+	       ;; Wait until input possible.
+	       (sys:with-fd-handler (fd :input
+					#'(lambda (fd)
+					    (declare (ignore fd)
+						     (optimize (speed 3)
+							       (safety 0)))
+					    (mp:process-yield)))
+		 (if timeout
+		     (mp:process-wait-with-timeout "Input Wait" timeout
+						   #'fd-usable-for-input)
+		     (mp:process-wait "Input Wait" #'fd-usable-for-input)))))
 	  (:output
-	   (unless (fd-usable-for-output)
-	     ;; Wait until output possible.
-	     (sys:with-fd-handler (fd :output
-				   #'(lambda (fd)
-				       (declare (ignore fd)
-						(optimize (speed 3)
-							  (safety 0)))
-				       (mp:process-yield)))
-	       (if timeout
-		   (mp:process-wait-with-timeout "Output Wait" timeout
-						 #'fd-usable-for-output)
-		   (mp:process-wait "Output Wait"
-				    #'fd-usable-for-output)))))))))
+	   (or (fd-usable-for-output)
+	       ;; Wait until output possible.
+	       (sys:with-fd-handler (fd :output
+					#'(lambda (fd)
+					    (declare (ignore fd)
+						     (optimize (speed 3)
+							       (safety 0)))
+					    (mp:process-yield)))
+		 (if timeout
+		     (mp:process-wait-with-timeout "Output Wait" timeout
+						   #'fd-usable-for-output)
+		     (mp:process-wait "Output Wait"
+				      #'fd-usable-for-output)))))))))
 
 
 ;;; Sleep  --  Public
