@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/numbers.lisp,v 1.35 1999/11/11 16:34:41 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/numbers.lisp,v 1.39 2001/09/28 14:56:28 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -217,7 +217,7 @@
 ;;; used when we know that realpart and imagpart are the same type, but
 ;;; rational canonicalization might still need to be done.
 ;;;
-(proclaim '(inline canonical-complex))
+(declaim (inline canonical-complex))
 (defun canonical-complex (realpart imagpart)
   (if (eql imagpart 0)
       realpart
@@ -241,7 +241,7 @@
 ;;; a canonical rational.  We make the denominator positive, and check whether
 ;;; it is 1.
 ;;;
-(proclaim '(inline build-ratio))
+(declaim (inline build-ratio))
 (defun build-ratio (num den)
   (multiple-value-bind (num den)
 		       (if (minusp den)
@@ -256,7 +256,7 @@
 ;;;
 ;;;    Truncate X and Y, but bum the case where Y is 1.
 ;;;
-(proclaim '(inline maybe-truncate))
+(declaim (inline maybe-truncate))
 (defun maybe-truncate (x y)
   (if (eql y 1)
       x
@@ -843,11 +843,16 @@
     ((double-float single-float)
      (,op x (coerce y 'double-float)))
     (((foreach single-float double-float #+long-float long-float) rational)
-     (if (eql y 0)
+     ;; Comparing infinity against any rational produces the same
+     ;; answer as comparing infinity against 0.  Comparison against
+     ;; zero is quite common, so add a special case for that.
+     (if (or (eql y 0) (float-infinity-p x))
 	 (,op x (coerce 0 '(dispatch-type x)))
 	 (,op (rational x) y)))
     (((foreach bignum fixnum ratio) float)
-     (,op x (rational y)))))
+     (if (or (eql x 0) (float-infinity-p y))
+	 (,op (coerce 0 '(dispatch-type y)) y)
+	 (,op x (rational y))))))
 
 
 (defmacro two-arg-</> (name op ratio-arg1 ratio-arg2 &rest cases)
