@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/reader.lisp,v 1.27 1998/05/15 01:01:02 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/reader.lisp,v 1.31 2001/07/16 13:21:07 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -158,7 +158,7 @@
 ;;; In case we get an error trying to parse a symbol, we want to rebind the
 ;;; above stuff so it's cool.
 
-(proclaim '(special *package* *keyword-package* *read-base*))
+(declaim (special *package* *keyword-package* *read-base*))
 
 
 
@@ -197,7 +197,7 @@
 
 ;;; Make this a function, since other people want to use it.
 ;;;
-(proclaim '(inline whitespacep))
+(declaim (inline whitespacep))
 (defun whitespacep (char &optional (rt *readtable*))
   (test-attribute char whitespace rt))
 
@@ -277,6 +277,8 @@
 	  (mapcar #'(lambda (pair) (cons (car pair)
 					 (copy-seq (cdr pair))))
 		  (dispatch-tables from-readtable)))
+    (setf (readtable-case to-readtable)
+	  (readtable-case from-readtable))
     to-readtable))
 
 (defun set-syntax-from-char (to-char from-char &optional
@@ -312,11 +314,11 @@
   (set-cmt-entry char function rt)
   T)
 
-(defun get-macro-character (char &optional rt)
+(defun get-macro-character (char &optional (rt *readtable*))
   "Returns the function associated with the specified char which is a macro
   character.  The optional readtable argument defaults to the current
   readtable."
-  (let ((rt (or rt *readtable*)))
+  (let ((rt (or rt std-lisp-readtable)))
     ;; Check macro syntax, return associated function if it's there.
     ;; Returns a value for all constituents.
     (cond ((constituentp char)
@@ -470,7 +472,7 @@
 ;;
 (defvar *sharp-equal-alist* ())
 
-(proclaim '(special *standard-input*))
+(declaim (special *standard-input*))
  
 ;;; READ-PRESERVING-WHITESPACE behaves just like read only it makes sure
 ;;; to leave terminating whitespace in the stream.
@@ -635,53 +637,61 @@
 
 ;;; INTERNAL-READ-EXTENDED-TOKEN  --  Internal
 ;;;
-;;; Read from the stream up to the next delimiter.  Leaves resulting token in
-;;; read-buffer, returns two values:
+;;; Read from the stream up to the next delimiter.  If escape-firstchar is
+;;; true then the firstchar is assumed to be escaped.  Leaves resulting token
+;;; in read-buffer, returns two values:
 ;;; -- a list of the escaped character positions, and
 ;;; -- The position of the first package delimiter (or NIL).
 ;;;
-(defun internal-read-extended-token (stream firstchar)
+(defun internal-read-extended-token (stream firstchar escape-firstchar)
   (reset-read-buffer)
-  (do ((char firstchar (read-char stream nil eof-object))
-       (escapes ())
-       (colon nil))
-      ((cond ((eofp char) t)
-	     ((token-delimiterp char)
-	      (unread-char char stream)
-	      t)
-	     (t nil))
-       (values escapes colon))
-    (cond ((escapep char)
-	   ;;it can't be a number, even if it's 1\23.
-	   ;;read next char here, so it won't be casified.
-	   (push ouch-ptr escapes)
-	   (let ((nextchar (read-char stream nil eof-object)))
-	     (if (eofp nextchar)
-		 (reader-eof-error stream "after escape character")
-		 (ouch-read-buffer nextchar))))
-	  ((multiple-escape-p char)
-	   ;; read to next multiple-escape, escaping single chars along
-	   ;; the way
-	   (loop
-	     (let ((ch (read-char stream nil eof-object)))
-	       (cond
-		((eofp ch)
-		 (reader-eof-error stream "inside extended token"))
-		((multiple-escape-p ch) (return))
-		((escapep ch)
-		 (let ((nextchar (read-char stream nil eof-object)))
-		   (if (eofp nextchar)
-		       (reader-eof-error stream "after escape character")
-		       (ouch-read-buffer nextchar))))
-		(t
-		 (push ouch-ptr escapes)
-		 (ouch-read-buffer ch))))))
-	  (t
-	   (when (and (constituentp char)
-		      (eql (get-secondary-attribute char) #.package-delimiter)
-		      (not colon))
-	     (setq colon ouch-ptr))
-	   (ouch-read-buffer char)))))
+  (let ((escapes ()))
+    (when escape-firstchar
+      (push ouch-ptr escapes)
+      (ouch-read-buffer firstchar)
+      (setq firstchar (read-char stream nil eof-object)))
+    (do ((char firstchar (read-char stream nil eof-object))
+	 (colon nil))
+	((cond ((eofp char) t)
+	       ((token-delimiterp char)
+		(unread-char char stream)
+		t)
+	       (t nil))
+	 (values escapes colon))
+      (cond ((escapep char)
+	     ;; It can't be a number, even if it's 1\23.
+	     ;; Read next char here, so it won't be casified.
+	     (push ouch-ptr escapes)
+	     (let ((nextchar (read-char stream nil eof-object)))
+	       (if (eofp nextchar)
+		   (reader-eof-error stream "after escape character")
+		   (ouch-read-buffer nextchar))))
+	    ((multiple-escape-p char)
+	     ;; Read to next multiple-escape, escaping single chars along the
+	     ;; way.
+	     (loop
+	      (let ((ch (read-char stream nil eof-object)))
+		(cond
+		  ((eofp ch)
+		   (reader-eof-error stream "inside extended token"))
+		  ((multiple-escape-p ch) (return))
+		  ((escapep ch)
+		   (let ((nextchar (read-char stream nil eof-object)))
+		     (cond ((eofp nextchar)
+			    (reader-eof-error stream "after escape character"))
+			   (t
+			    (push ouch-ptr escapes)
+			    (ouch-read-buffer nextchar)))))
+		  (t
+		   (push ouch-ptr escapes)
+		   (ouch-read-buffer ch))))))
+	    (t
+	     (when (and (constituentp char)
+			(eql (get-secondary-attribute char)
+			     #.package-delimiter)
+			(not colon))
+	       (setq colon ouch-ptr))
+	     (ouch-read-buffer char))))))
 
 
 ;;;; Character classes.
@@ -798,7 +808,7 @@
   ;;"unqualified-token".  Leave the result in the READ-BUFFER.
   ;;Return next char after token (last char read).
   (when *read-suppress*
-    (internal-read-extended-token stream firstchar)
+    (internal-read-extended-token stream firstchar nil)
     (return-from read-token nil))
   (let ((attribute-table (character-attribute-table *readtable*))
 	(package nil)
@@ -1111,12 +1121,24 @@
   (let ((firstch (read-char stream nil nil t)))
     (cond (firstch
 	   (multiple-value-bind (escapes colon)
-				(internal-read-extended-token stream firstch)
+	       (internal-read-extended-token stream firstch nil)
 	     (casify-read-buffer escapes)
 	     (values (read-buffer-to-string) (not (null escapes)) colon)))
 	  (t
 	   (values "" nil nil)))))
 
+(defun read-extended-token-escaped (stream &optional (*readtable* *readtable*))
+  "For semi-external use: read an extended token with the first character
+  escaped.  Returns the string for the token."
+  (let ((firstch (read-char stream nil nil)))
+    (cond (firstch
+	   (let ((escapes (internal-read-extended-token stream firstch t)))
+	     (casify-read-buffer escapes)
+	     (read-buffer-to-string)))
+	  (t
+	   (reader-eof-error stream "after escape")))))
+
+
 
 ;;;; Number reading functions.
 
@@ -1368,12 +1390,13 @@
 	      (coerce function 'function))
 	(error "~S is not a dispatch char." disp-char))))
 
-(defun get-dispatch-macro-character (disp-char sub-char &optional rt)
+(defun get-dispatch-macro-character
+       (disp-char sub-char &optional (rt *readtable*))
   "Returns the macro character function for sub-char under disp-char
    or nil if there is no associated function."
   (unless (digit-char-p sub-char)
     (let* ((sub-char (char-upcase sub-char))
-	   (rt (or rt *readtable*))
+	   (rt (or rt std-lisp-readtable))
 	   (dpair (find disp-char (dispatch-tables rt)
 			:test #'char= :key #'car)))
       (if dpair
