@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/save.lisp,v 1.36 1998/06/24 20:30:36 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/save.lisp,v 1.42 2001/12/08 18:42:39 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -91,9 +91,8 @@
 
   (setf (search-list "library:")
 	(or (parse-unix-search-list :cmucllib)
-	    '(#+mach  "/usr/misc/.cmucl/lib/"
-	      #+linux "/usr/lib/cmucl/"
-	      #-(or mach linux) "/usr/local/lib/cmucl/lib/"))))
+	    '("/usr/lib/cmucl/")))
+  (setf (search-list "modules:") (ext:unix-namestring "library:subsystems/")))
 
 
 
@@ -156,52 +155,60 @@
 	      :environment-name environment-name)
       #-gencgc (gc) #+gencgc (gc :full t))
   (dolist (f *before-save-initializations*) (funcall f))
-  (flet
-      ((restart-lisp ()
+  (labels
+      ((%restart-lisp ()
+	 (with-simple-restart (abort "Skip remaining initializations.")
+	   (catch 'top-level-catcher
+	     (reinit)
+	     (environment-init)
+	     (dolist (f *after-save-initializations*) (funcall f))
+	     (when process-command-line
+	       (ext::process-command-strings))
+	     (setf *editor-lisp-p* nil)
+	     (macrolet ((find-switch (name)
+			  `(find ,name *command-line-switches*
+				 :key #'cmd-switch-name
+				 :test #'(lambda (x y)
+					   (declare (simple-string x y))
+					   (string-equal x y)))))
+	       (when (and site-init
+			  (not (and process-command-line
+				    (find-switch "nositeinit"))))
+		 (load site-init :if-does-not-exist nil :verbose nil))
+	       (when (and process-command-line (find-switch "edit"))
+		 (setf *editor-lisp-p* t))
+	       (when (and load-init-file
+			  (not (and process-command-line
+				    (find-switch "noinit"))))
+		 (let* ((cl-switch (find-switch "init"))
+			(name (and cl-switch
+				   (or (cmd-switch-value cl-switch)
+				       (car (cmd-switch-words cl-switch))))))
+		   (if name
+		       (load (merge-pathnames name #p"home:")
+			     :if-does-not-exist nil)
+		       (or (load "home:init" :if-does-not-exist nil)
+			   (load "home:.cmucl-init"
+				 :if-does-not-exist nil))))))
+	     (when process-command-line
+	       (ext::invoke-switch-demons *command-line-switches*
+					  *command-switch-demons*))
+	     (when print-herald
+	       (print-herald))))
+	 (funcall init-function))
+       (restart-lisp ()
 	 (unix:unix-exit
 	  (catch '%end-of-the-world
-	    (with-simple-restart (abort "Skip remaining initializations.")
-	      (catch 'top-level-catcher
-		(reinit)
-		(environment-init)
-		(dolist (f *after-save-initializations*) (funcall f))
-		(when process-command-line
-		  (ext::process-command-strings))
-		(setf *editor-lisp-p* nil)
-		(macrolet ((find-switch (name)
-			     `(find ,name *command-line-switches*
-				    :key #'cmd-switch-name
-				    :test #'(lambda (x y)
-					      (declare (simple-string x y))
-					      (string-equal x y)))))
-		  (when (and site-init
-			     (not (and process-command-line
-				       (find-switch "noinit"))))
-		    (load site-init :if-does-not-exist nil :verbose nil))
-		  (when (and process-command-line (find-switch "edit"))
-		    (setf *editor-lisp-p* t))
-		  (when (and load-init-file
-			     (not (and process-command-line
-				       (find-switch "noinit"))))
-		    (let* ((cl-switch (find-switch "init"))
-			   (name (and cl-switch
-				      (or (cmd-switch-value cl-switch)
-					  (car (cmd-switch-words
-						cl-switch))))))
-		      (if name
-			  (load (merge-pathnames name #p"home:")
-				:if-does-not-exist nil)
-			  (or (load "home:init" :if-does-not-exist nil)
-			      (load "home:.cmucl-init"
-				    :if-does-not-exist nil))))))
-		(when process-command-line
-		  (ext::invoke-switch-demons *command-line-switches*
-					     *command-switch-demons*))
-		(when print-herald
-		  (print-herald))))
-	    (funcall (if (and *batch-mode* (eq init-function #'%top-level))
-			 #'%handled-top-level
-			 init-function))))))
+	    (unwind-protect
+		(if *batch-mode*
+		    (handler-case
+			(%restart-lisp)
+		      (error (cond)
+			(format *error-output* "Error in batch processing:~%~A~%"
+				cond)
+			(throw '%end-of-the-world 1)))
+		    (%restart-lisp))
+	      (finish-standard-output-streams))))))
 
     (let ((initial-function (get-lisp-obj-address #'restart-lisp)))
       (without-gcing
@@ -230,8 +237,12 @@
 	,#'(lambda (stream) (write-string (machine-instance) stream))))
 
 (setf (getf *herald-items* :bugs)
-      '("Send questions to cmucl-help@cons.org. and bug reports to cmucl-imp@cons.org."
-	terpri
+      '("For support see http://www.cons.org/cmucl/support.html Send bug reports to the debian BTS."
+        terpri
+	"or to "
+	"pvaneynd@debian.org" terpri
+       "type (help) for help, (quit) to exit, and (demo) to see the demos" terpri
+        terpri
 	"Loaded subsystems:"))
 
 ;;; PRINT-HERALD  --  Public
