@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/sharpm.lisp,v 1.20 1999/12/08 18:36:51 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/sharpm.lisp,v 1.22 2001/03/04 20:12:42 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -20,7 +20,7 @@
 
 ;;; declared in READ.LISP
 
-(proclaim '(special *read-suppress* std-lisp-readtable *bq-vector-flag*))
+(declaim (special *read-suppress* std-lisp-readtable *bq-vector-flag*))
 
 (defun ignore-numarg (sub-char numarg)
   (when numarg
@@ -28,9 +28,7 @@
 
 (defun sharp-backslash (stream backslash numarg)
   (ignore-numarg backslash numarg)
-  (unread-char backslash stream)
-  (let* ((*readtable* std-lisp-readtable)
-	 (charstring (read-extended-token stream)))
+  (let ((charstring (read-extended-token-escaped stream)))
     (declare (simple-string charstring))
     (cond (*read-suppress* nil)
 	  ((= (the fixnum (length charstring)) 1)
