@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/signal.lisp,v 1.28 1999/12/04 16:02:36 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/signal.lisp,v 1.32 2001/04/10 13:42:45 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -35,7 +35,7 @@
 (defconstant sig_dfl 0)
 (defconstant sig_ign 1)
 
-(proclaim '(special lisp::lisp-command-line-list))
+(declaim (special lisp::lisp-command-line-list))
 
 
 
@@ -99,7 +99,8 @@
 (def-unix-signal :SIGQUIT 3 "Quit")
 (def-unix-signal :SIGILL 4 "Illegal instruction")
 (def-unix-signal :SIGTRAP 5 "Trace trap")
-(def-unix-signal :SIGIOT 6 "Iot instruction")
+(def-unix-signal :SIGIOT 6 "Iot instruction") ; Compatibility
+(def-unix-signal :SIGABRT 6 "C abort()")
 #-linux
 (def-unix-signal :SIGEMT 7 "Emt instruction")
 (def-unix-signal :SIGFPE 8 "Floating point exception")
@@ -168,7 +169,7 @@
 
 ;;;; System calls that deal with signals.
 
-(proclaim '(inline real-unix-kill))
+(declaim (inline real-unix-kill))
 
 (alien:def-alien-routine ("kill" real-unix-kill) c-call:int
   (pid c-call:int)
@@ -178,10 +179,11 @@
   "Unix-kill sends the signal signal to the process with process 
    id pid.  Signal should be a valid signal number or a keyword of the
    standard UNIX signal name."
-  (real-unix-kill pid (unix-signal-number signal)))
+  (if (minusp (real-unix-kill pid (unix-signal-number signal)))
+      (values nil unix-errno)
+      t))
 
-
-(proclaim '(inline real-unix-killpg))
+(declaim (inline real-unix-killpg))
 
 (alien:def-alien-routine ("killpg" real-unix-killpg) c-call:int
   (pgrp c-call:int)
@@ -191,8 +193,9 @@
   "Unix-killpg sends the signal signal to the all the process in process
   group PGRP.  Signal should be a valid signal number or a keyword of
   the standard UNIX signal name."
-  (real-unix-killpg pgrp (unix-signal-number signal)))
-
+  (if (minusp (real-unix-killpg pgrp (unix-signal-number signal)))
+      (values nil unix-errno)
+      t))
 
 (alien:def-alien-routine ("sigblock" unix-sigblock) c-call:unsigned-long
   "Unix-sigblock cause the signals specified in mask to be
@@ -256,7 +259,8 @@
 (defmacro define-signal-handler (name what &optional (function 'error))
   `(defun ,name (signal code scp)
      (declare (ignore signal code)
-	      (type system-area-pointer scp))
+	      (type system-area-pointer scp)
+	      (optimize (inhibit-warnings 3)))
      (system:without-hemlock
       (,function ,(concatenate 'simple-string what " at #x~x.")
 		 (with-alien ((scp (* sigcontext) scp))
@@ -265,7 +269,7 @@
 (define-signal-handler sigint-handler "Interrupted" break)
 (define-signal-handler sigill-handler "Illegal Instruction")
 (define-signal-handler sigtrap-handler "Breakpoint/Trap")
-(define-signal-handler sigiot-handler "SIGIOT")
+(define-signal-handler sigabrt-handler "SIGABRT")
 #-linux
 (define-signal-handler sigemt-handler "SIGEMT")
 (define-signal-handler sigbus-handler "Bus Error")
@@ -286,7 +290,7 @@
   (enable-interrupt :sigquit #'sigquit-handler)
   (enable-interrupt :sigill #'sigill-handler)
   (enable-interrupt :sigtrap #'sigtrap-handler)
-  (enable-interrupt :sigiot #'sigiot-handler)
+  (enable-interrupt :sigabrt #'sigabrt-handler)
   #-linux
   (enable-interrupt :sigemt #'sigemt-handler)
   (enable-interrupt :sigfpe #'vm:sigfpe-handler)
