@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/stream.lisp,v 1.39 1999/02/11 12:17:58 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/stream.lisp,v 1.48 2001/09/20 14:23:05 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -182,10 +182,10 @@
   "Returns non-nil if the given Stream can perform input operations."
   (declare (type stream stream))
 
-  #+high-security
-  (when (synonym-stream-p stream)
-    (setf stream (symbol-value
-		  (synonym-stream-symbol stream))))
+  ;;#+high-security
+  ;;  (when (synonym-stream-p stream)
+  ;;    (setf stream (symbol-value
+  ;;		  (synonym-stream-symbol stream))))
   
   (and (lisp-stream-p stream)
        (not (eq (lisp-stream-in stream) #'closed-flame))
@@ -196,10 +196,10 @@
   "Returns non-nil if the given Stream can perform output operations."
   (declare (type stream stream))
 
-  #+high-security
-  (when (synonym-stream-p stream)
-    (setf stream (symbol-value
-		  (synonym-stream-symbol stream))))
+  ;; #+high-security
+  ;;  (when (synonym-stream-p stream)
+  ;;    (setf stream (symbol-value
+  ;;		  (synonym-stream-symbol stream))))
   
   (and (lisp-stream-p stream)
        (not (eq (lisp-stream-in stream) #'closed-flame))
@@ -285,9 +285,9 @@
   "This function returns the length of the file that File-Stream is open to."
   (declare (type (or file-stream synonym-stream) stream))
 
-  #+high-security
-  (check-type-var stream '(satisfies stream-associated-with-file)
-		  "A stream associated with a file")
+;;  #+high-security
+;;  (check-type-var stream '(satisfies stream-associated-with-file)
+;;		  "A stream associated with a file")
   
   (funcall (lisp-stream-misc stream) stream :file-length))
 
@@ -337,7 +337,7 @@
 ;;; We proclaim them inline here, then proclaim them notinline at EOF,
 ;;; so, except in this file, they are not inline by default, but they can be.
 ;;;
-(proclaim '(inline read-char unread-char read-byte listen))
+(declaim (inline read-char unread-char read-byte listen))
 (defun read-char (&optional (stream *standard-input*) (eof-errorp t) eof-value
 			    recursive-p)
   "Inputs a character from Stream and returns it."
@@ -375,8 +375,8 @@
 (defun peek-char (&optional (peek-type nil) (stream *standard-input*)
 			    (eof-errorp t) eof-value recursive-p)
   "Peeks at the next character in the input Stream.  See manual for details."
-  (let ((stream (in-synonym-of stream))
-	(eof-errorp (or eof-errorp recursive-p)))
+  (declare (ignore recursive-p))
+  (let ((stream (in-synonym-of stream)))
     (if (lisp-stream-p stream)
 	(let ((char (read-char stream eof-errorp eof-value)))
 	  (cond ((eq char eof-value) char)
@@ -725,17 +725,22 @@
 				       (sout #'broadcast-sout)
 				       (misc #'broadcast-misc))
 			     (:print-function %print-broadcast-stream)
-			     (:constructor #-high-security-support
+			     (:constructor ;#-high-security-support
 					   make-broadcast-stream
-					   #+high-security-support
-					   %make-broadcast-stream (&rest streams)))
+					   ;#+high-security-support
+					   ;%make-broadcast-stream 
+					   (&rest streams)))
   ;; This is a list of all the streams we broadcast to.
   (streams () :type list :read-only t))
 
-#-high-security-support
+;#-high-security-support
 (setf (documentation 'make-broadcast-stream 'function)
- "Returns an ouput stream which sends its output to all of the given streams.")
-#+high-security-support
+ "Returns an output stream which sends its output to all of the given streams.")
+
+(defun %make-broadcast-stream (&rest streams)
+  (apply #'make-broadcast-stream streams))
+
+#+nil ;#+high-security-support
 (defun make-broadcast-stream (&rest streams)
   "Returns an ouput stream which sends its output to all of the given streams."
   (dolist (stream streams)    
@@ -861,6 +866,8 @@
 	  (:listen (or (/= (the fixnum (lisp-stream-in-index syn))
 			   in-buffer-length)
 		       (funcall (lisp-stream-misc syn) syn :listen)))
+	  (:clear-input (clear-input syn))
+	  (:unread (unread-char arg1 syn))
 	  (t
 	   (funcall (lisp-stream-misc syn) syn operation arg1 arg2)))
 	(stream-misc-dispatch syn operation arg1 arg2))))
@@ -877,10 +884,11 @@
 		      (sout #'two-way-sout)
 		      (misc #'two-way-misc))
 	    (:print-function %print-two-way-stream)
-	    (:constructor #-high-security-support
+	    (:constructor ;#-high-security-support
 			  make-two-way-stream
-			  #+high-security-support
-			  %make-two-way-stream (input-stream output-stream)))
+			  ;#+high-security-support
+			  ;%make-two-way-stream 
+			  (input-stream output-stream)))
   ;; We read from this stream...
   (input-stream (required-argument) :type stream :read-only t)
   ;; And write to this one
@@ -892,11 +900,15 @@
 	  (two-way-stream-input-stream s)
 	  (two-way-stream-output-stream s)))
 
-#-high-security-support
+;#-high-security-support
 (setf (documentation 'make-two-way-stream 'function)
   "Returns a bidirectional stream which gets its input from Input-Stream and
    sends its output to Output-Stream.")
-#+high-security-support
+
+(defun %make-two-way-stream (&rest streams)
+  (apply #'make-two-way-stream streams))
+
+#+nil ;#+high-security-support
 (defun make-two-way-stream (input-stream output-stream)
   "Returns a bidirectional stream which gets its input from Input-Stream and
    sends its output to Output-Stream."
@@ -949,10 +961,8 @@
        (if out-lisp-stream-p
 	   (funcall (lisp-stream-misc out) out operation arg1 arg2)
 	   (stream-misc-dispatch out operation arg1 arg2)))
-      ((:clear-input :unread)
-       (if in-lisp-stream-p
-	   (funcall (lisp-stream-misc in) in operation arg1 arg2)
-	   (stream-misc-dispatch in operation arg1 arg2)))
+      (:clear-input (clear-input in))
+      (:unread (unread-char arg1 in))
       (:element-type
        (let ((in-type (stream-element-type in))
 	     (out-type (stream-element-type out)))
@@ -975,29 +985,28 @@
 	    (:include lisp-stream
 		      (in #'concatenated-in)
 		      (bin #'concatenated-bin)
+		      (n-bin #'concatenated-n-bin)
 		      (misc #'concatenated-misc))
 	    (:print-function %print-concatenated-stream)
-	    (:constructor
-	     #-high-security-support make-concatenated-stream
-	     #+high-security-support %make-concatenated-stream 
-	         (&rest streams &aux (current streams))))
-  ;; The car of this is the stream we are reading from now.
-  current
-  ;; This is a list of all the streams.  We need to remember them so that
-  ;; we can close them.
-  (streams nil :type list :read-only t))
+	    (:constructor make-concatenated-stream (&rest streams)))
+  ;; This is a list of all the streams. The car of this is the stream
+  ;; we are reading from now.
+  (streams nil :type list))
 
 (defun %print-concatenated-stream (s stream d)
   (declare (ignore d))
   (format stream "#<Concatenated Stream, Streams = ~S>"
 	  (concatenated-stream-streams s)))
 
-#-high-security-support
+;#-high-security-support
 (setf (documentation 'make-concatenated-stream 'function)
   "Returns a stream which takes its input from each of the Streams in turn,
    going on to the next at EOF.")
 
-#+high-security-support
+(defun %make-concatenatedt-stream (&rest streams)
+  (apply #'make-concatenated-stream streams))
+
+#+nil ; #+high-security-support
 (defun make-concatenated-stream (&rest streams)
   "Returns a stream which takes its input from each of the Streams in turn,
    going on to the next at EOF."
@@ -1013,18 +1022,35 @@
 
 (macrolet ((in-fun (name fun)
 	     `(defun ,name (stream eof-errorp eof-value)
-		(do ((current (concatenated-stream-current stream) (cdr current)))
+		(do ((current (concatenated-stream-streams stream)
+			      (cdr current)))
 		    ((null current)
 		     (eof-or-lose stream eof-errorp eof-value))
 		  (let* ((stream (car current))
 			 (result (,fun stream nil nil)))
 		    (when result (return result)))
-		  (setf (concatenated-stream-current stream) current)))))
+		  (setf (concatenated-stream-streams stream) (cdr current))))))
   (in-fun concatenated-in read-char)
   (in-fun concatenated-bin read-byte))
 
+(defun concatenated-n-bin (stream buffer start numbytes eof-errorp)
+  (do ((current (concatenated-stream-streams stream) (cdr current))
+       (current-start start)
+       (remaining-bytes numbytes))
+      ((null current)
+       (if eof-errorp
+	(error 'end-of-file :stream stream)
+	(- numbytes remaining-bytes)))
+    (let* ((stream (car current))
+           (bytes-read (read-n-bytes stream buffer current-start
+	                             remaining-bytes nil)))
+      (incf current-start bytes-read)
+      (decf remaining-bytes bytes-read)
+      (when (zerop remaining-bytes) (return numbytes)))
+    (setf (concatenated-stream-streams stream) (cdr current))))
+
 (defun concatenated-misc (stream operation &optional arg1 arg2)
-  (let ((left (concatenated-stream-current stream)))
+  (let ((left (concatenated-stream-streams stream)))
     (when left
       (let* ((current (car left)))
 	(case operation
@@ -1036,9 +1062,9 @@
 			      (stream-misc-dispatch current :listen))))
 	       (cond ((eq stuff :eof)
 		      ;; Advance current, and try again.
-		      (pop (concatenated-stream-current stream))
+		      (pop (concatenated-stream-streams stream))
 		      (setf current
-			    (car (concatenated-stream-current stream)))
+			    (car (concatenated-stream-streams stream)))
 		      (unless current
 			;; No further streams.  EOF.
 			(return :eof)))
@@ -1050,6 +1076,8 @@
 		      (return nil))))))
 	  (:close
 	   (set-closed-flame stream))
+	  (:clear-input (clear-input current))
+	  (:unread (unread-char arg1 current))
 	  (t
 	   (if (lisp-stream-p current)
 	       (funcall (lisp-stream-misc current) current operation arg1 arg2)
@@ -1337,7 +1365,7 @@
       (let ((offset-current (+ start current)))
 	(declare (fixnum offset-current))
 	(if (= offset-current end)
-	    (let* ((new-length (* current 2))
+	    (let* ((new-length (if (zerop current) 1 (* current 2)))
 		   (new-workspace (make-string new-length)))
 	      (declare (simple-string new-workspace))
 	      (%primitive byte-blt workspace start new-workspace 0 current)
@@ -1407,7 +1435,7 @@
   (indentation 0))
 
 (setf (documentation 'make-indenting-stream 'function)
- "Returns an ouput stream which indents its output by some amount.")
+ "Returns an output stream which indents its output by some amount.")
 
 (defun %print-indenting-stream (s stream d)
   (declare (ignore s d))
@@ -1481,7 +1509,7 @@
 	   (stream-misc-dispatch sub-stream operation arg1 arg2))))))
 
 
-(proclaim '(maybe-inline read-char unread-char read-byte listen))
+(declaim (maybe-inline read-char unread-char read-byte listen))
 
 
 
@@ -1769,6 +1797,8 @@
 	   (type index start)
 	   (type sequence-end end)
 	   (values index))
+  (when (not (cl::lisp-stream-p stream))
+     (return-from read-sequence (stream-read-sequence seq stream start end)))
   (let ((end (or end (length seq))))
     (declare (type index end))
     (etypecase seq
@@ -1792,12 +1822,17 @@
 	   ((or (simple-array (unsigned-byte 8) (*))
 		(simple-array (signed-byte 8) (*))
 		simple-string)
-	    (let* ((numbytes (- end start))
-		   (bytes-read (system:read-n-bytes
-				stream data offset-start numbytes nil)))
-	      (if (< bytes-read numbytes)
-		  (+ start bytes-read)
-		  end)))
+	    (let ((required (- end start)))
+	      (loop
+	       (let ((bytes-read (system:read-n-bytes stream data offset-start
+						      required nil)))
+		 (cond ((= bytes-read required)
+			(return end))
+		       ((zerop bytes-read)
+			(return (- end required)))
+		       (t
+			(decf required bytes-read)
+			(incf offset-start bytes-read)))))))
 	   (t
 	    (let ((read-function
 		   (if (subtypep (stream-element-type stream) 'character)
@@ -1820,6 +1855,8 @@
 	   (type index start)
 	   (type sequence-end end)
 	   (values sequence))
+  (when (not (cl::lisp-stream-p stream))
+    (return-from write-sequence (stream-write-sequence seq stream start end)))
   (let ((end (or end (length seq))))
     (declare (type index start end))
     (etypecase seq
@@ -1845,3 +1882,16 @@
 	     ((>= i end) seq)
 	   (declare (type index i))
 	   (funcall write-function (aref seq i) stream)))))))
+
+
+
+;;; finish-standard-output-streams  --  Public
+;;;
+;;; Finish output on some of the standard output streams.
+;;;
+(defun finish-standard-output-streams ()
+  (dolist (stream '(*standard-output* *error-output* *trace-output*))
+    (when (boundp stream)
+      (let ((stream (symbol-value stream)))
+	(when (and (streamp stream) (open-stream-p stream))
+	  (finish-output stream))))))
