@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/code/type.lisp,v 1.32 2000/01/10 14:47:02 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/code/type.lisp,v 1.39 2001/11/22 13:39:15 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -55,7 +55,7 @@
    affects array types.")
 
 (cold-load-init (setq *use-implementation-types* t))
-(proclaim '(type boolean *use-implementation-types*))
+(declaim (type boolean *use-implementation-types*))
 
 ;;; DELEGATE-COMPLEX-{SUBTYPEP-ARG2,INTERSECTION}  --  Interface
 ;;;
@@ -198,6 +198,8 @@
 		      (:class-info (type-class-or-lose 'values)))
 	    (:print-function %print-type)))
 
+(declaim (freeze-type values-type))
+
 (define-type-method (values :unparse) (type)
   (cons 'values (unparse-args-types type)))
 
@@ -232,7 +234,7 @@
 	   (values nil nil))
 	  ((and rest1 rest2 (type/= rest1 rest2))
 	   (type= rest1 rest2))
-	  ((or rest1 rest2)
+	  ((if rest1 (not rest2) rest2)
 	   (values nil t))
 	  (t
 	   (multiple-value-bind (req-val req-win)
@@ -333,8 +335,8 @@
 ;;; structure, fill in the slots in the structure accordingly.  This is used
 ;;; for both FUNCTION and VALUES types.
 ;;;
-(proclaim '(function parse-args-types (list args-type) void))
 (defun parse-args-types (lambda-list result)
+  (declare (list lambda-list) (type args-type result))
   (multiple-value-bind (required optional restp rest keyp keys allowp aux)
 		       (parse-lambda-list lambda-list)
     (when aux
@@ -361,8 +363,8 @@
 ;;;    Return the lambda-list like type specification corresponding
 ;;; to a Args-Type.
 ;;;
-(proclaim '(function unparse-args-types (args-type) list))
 (defun unparse-args-types (type)
+  (declare (type args-type type) (values list))
   (collect ((result))
 
     (dolist (arg (args-type-required type))
@@ -413,16 +415,27 @@
 ;;; Single-Value-Type  --  Interface
 ;;;
 ;;;    Return the type of the first value indicated by Type.  This is used by
-;;; people who don't want to have to deal with values types.
+;;; people who don't want to have to deal with values types. If the first
+;;; values is an optional or rest argument then return the union with the null
+;;; type. If the first values is a keyword then give up and return the
+;;; universal type.
 ;;;
-(declaim (freeze-type values-type) (inline single-value-type))
 (defun single-value-type (type)
   (declare (type ctype type))
   (cond ((values-type-p type)
-	 (or (car (args-type-required type))
-	     (car (args-type-optional type))
-	     (args-type-rest type)
-	     *universal-type*))
+	 (let ((required (args-type-required type)))
+	   (if required
+	       (first required)
+	       (let ((otype (args-type-optional type)))
+		 (cond (otype
+			(type-union (first otype) *null-type*))
+		       ((or (args-type-keyp type) (args-type-allowp type))
+			*universal-type*)
+		       (t
+			(let ((rest (args-type-rest type)))
+			  (if rest
+			      (type-union rest *null-type*)
+			      *null-type*))))))))
 	((eq type *wild-type*)
 	 *universal-type*)
 	(t
@@ -511,13 +524,24 @@
 ;;; Coerce-To-Values  --  Internal
 ;;;
 ;;; If Type isn't a values type, then make it into one:
-;;;    <type>  ==>  (values type &rest t)
+;;;    <type>  ==>  (values type)
 ;;;
 (defun coerce-to-values (type)
   (declare (type ctype type))
   (if (values-type-p type)
       type
-      (make-values-type :required (list type) :rest *universal-type*)))
+      (make-values-type :required (list type))))
+
+
+;;; Make-canonical-values-type  --  Internal
+;;;
+;;; Make a single value type is possible, otherwise a values-type.
+;;;    (values type)  ==>  <type>
+;;;
+(defun make-canonical-values-type (required optional rest)
+  (if (and required (endp (rest required)) (not optional) (not rest))
+      (first required)
+      (make-values-type :required required :optional optional :rest rest)))
 
 
 ;;; Args-Type-Op  --  Internal
@@ -574,12 +598,12 @@
 					   :from-end t)))
 		  (if (find *empty-type* required :test #'type=)
 		      (values *empty-type* t)
-		      (values (make-values-type
-			       :required required
-			       :optional (if opt-last
-					     (subseq opt 0 (1+ opt-last))
-					     ())
-			       :rest (if (eq rest *empty-type*) nil rest))
+		      (values (make-canonical-values-type
+			       required
+			       (if opt-last
+				   (subseq opt 0 (1+ opt-last))
+				   ())
+			       (if (eq rest *empty-type*) nil rest))
 			      (and rest-exact res-exact)))))))))
       (funcall operation type1 type2)))
 
@@ -897,8 +921,8 @@
 ;;;    Take a list of type specifiers, compute the translation and define it as
 ;;; a builtin type.
 ;;;
-(proclaim '(function precompute-types (list) void)) 
 (defun precompute-types (specs)
+  (declare (list specs))
   (dolist (spec specs)
     (let ((res (specifier-type spec)))
       (unless (unknown-type-p res)
@@ -2172,6 +2196,77 @@
 	    (%make-alien-type-type alien-type)))
       *universal-type*))
 
+
+;;;; Cons types:
+
+;;; The Cons-Type is used to represent cons types.
+;;;
+(defstruct (cons-type (:include ctype
+				(:class-info (type-class-or-lose 'cons)))
+		      (:print-function %print-type))
+  ;;
+  ;; The car element type.
+  (car-type *wild-type* :type ctype)
+  ;;
+  ;; The cdr element type.
+  (cdr-type *wild-type* :type ctype))
+
+(define-type-class cons)
+
+(def-type-translator cons (&optional car-type cdr-type)
+  (make-cons-type :car-type (specifier-type car-type)
+		  :cdr-type (specifier-type cdr-type)))
+
+(define-type-method (cons :unparse) (type)
+  (let ((car-eltype (type-specifier (cons-type-car-type type)))
+	(cdr-eltype (type-specifier (cons-type-cdr-type type))))
+    (cond ((and (eq car-eltype '*) (eq cdr-eltype '*))
+	   'cons)
+	  (t
+	   `(cons ,car-eltype ,cdr-eltype)))))
+
+(define-type-method (cons :simple-=) (type1 type2)
+  (declare (type cons-type type1 type2))
+  (and (type= (cons-type-car-type type1) (cons-type-car-type type2))
+       (type= (cons-type-cdr-type type1) (cons-type-cdr-type type2))))
+
+(define-type-method (cons :simple-subtypep) (type1 type2)
+  (declare (type cons-type type1 type2))
+  (multiple-value-bind (val-car win-car)
+      (csubtypep (cons-type-car-type type1) (cons-type-car-type type2))
+    (multiple-value-bind (val-cdr win-cdr)
+	(csubtypep (cons-type-cdr-type type1) (cons-type-cdr-type type2))
+      (if (and val-car val-cdr)
+	  (values t (and win-car win-cdr))
+	  (values nil (or win-car win-cdr))))))
+
+;;; CONS :simple-union method  -- Internal
+;;;
+;;; Give up if a precise type in not possible, to avoid returning overly
+;;; general types.
+;;;
+(define-type-method (cons :simple-union) (type1 type2)
+  (declare (type cons-type type1 type2))
+  (let ((car-type1 (cons-type-car-type type1))
+	(car-type2 (cons-type-car-type type2))
+	(cdr-type1 (cons-type-cdr-type type1))
+	(cdr-type2 (cons-type-cdr-type type2)))
+    (cond ((type= car-type1 car-type2)
+	   (make-cons-type :car-type car-type1
+			   :cdr-type (type-union cdr-type1 cdr-type2)))
+	  ((type= cdr-type1 cdr-type2)
+	   (make-cons-type :car-type (type-union cdr-type1 cdr-type2)
+			   :cdr-type cdr-type1)))))
+
+(define-type-method (cons :simple-intersection) (type1 type2)
+  (declare (type cons-type type1 type2))
+  (multiple-value-bind (int-car win-car)
+      (type-intersection (cons-type-car-type type1) (cons-type-car-type type2))
+    (multiple-value-bind (int-cdr win-cdr)
+	(type-intersection (cons-type-cdr-type type1) (cons-type-cdr-type type2))
+      (values (make-cons-type :car-type int-car :cdr-type int-cdr)
+	      (and win-car win-cdr)))))
+
 
 ;;; TYPE-DIFFERENCE  --  Interface
 ;;;
@@ -2265,7 +2360,7 @@
 (defun ctypep (obj type)
   (declare (type ctype type))
   (etypecase type
-    ((or numeric-type named-type member-type array-type built-in-class)
+    ((or numeric-type named-type member-type array-type built-in-class cons-type)
      (values (%typep obj type) t))
     (class
      (if (if (csubtypep type (specifier-type 'funcallable-instance))
@@ -2340,7 +2435,6 @@
 ;;; than trying to come up with the one that the user might find most
 ;;; informative.
 ;;;
-(proclaim '(function ctype-of (t) ctype))
 (defun-cached (ctype-of
 	       :hash-function (lambda (x)
 				(the fixnum
@@ -2349,6 +2443,7 @@
 	       :hash-bits 9
 	       :init-form cold-load-init)
 	      ((x eq))
+  (declare (values ctype))
   (typecase x
     (function
      (if (funcallable-instance-p x)
@@ -2356,31 +2451,40 @@
 	 (extract-function-type x)))
     (symbol
      (make-member-type :members (list x)))
+    (complex
+     (let ((real (realpart x))
+	   (imag (imagpart x)))
+       (make-numeric-type :class (etypecase real
+				   (integer
+				    (etypecase imag
+				      (integer 'integer)
+				      (rational 'rational)))
+				   (rational 'rational)
+				   (float 'float))
+			  :format (if (floatp real)
+				      (float-format-name real)
+				      nil)
+			  :complexp :complex
+			  :low (min real imag)
+			  :high (max real imag))))
     (number
-     (let* ((num (if (complexp x) (realpart x) x))
-	    (res (make-numeric-type
-		  :class (etypecase num
-			   (integer 'integer)
-			   (rational 'rational)
-			   (float 'float))
-		  :format (if (floatp num)
-			      (float-format-name num)
-			      nil))))
-       (cond ((complexp x)
-	      (setf (numeric-type-complexp res) :complex)
-	      (let ((imag (imagpart x)))
-		(setf (numeric-type-low res) (min num imag))
-		(setf (numeric-type-high res) (max num imag))))
-	     (t
-	      (setf (numeric-type-low res) num)
-	      (setf (numeric-type-high res) num)))
-       res))
+     (make-numeric-type :class (etypecase x
+				 (integer 'integer)
+				 (rational 'rational)
+				 (float 'float))
+			:format (if (floatp x)
+				    (float-format-name x)
+				    nil)
+			:low x
+			:high x))
     (array
      (let ((etype (specifier-type (array-element-type x))))
        (make-array-type :dimensions (array-dimensions x)
 			:complexp (not (typep x 'simple-array))
 			:element-type etype
 			:specialized-element-type etype)))
+    (cons
+     (make-cons-type))
     (t
      (class-of x))))
 
