@@ -5,7 +5,7 @@
 ;;; the Public domain, and is provided 'as is'.
 ;;;
 (ext:file-comment
-  "$Header: /home/dtc/cvs/src/cmucl/p86/code/unix-syscall.lisp,v 1.11 1999/09/13 15:38:11 dtc Exp $")
+ "$Header: /home/dtc/cvs/src/cmucl/p86/code/unix-syscall.lisp,v 1.11 1999/09/13 15:38:11 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -23,27 +23,27 @@
 
 #+linux
 (def-alien-type nil
-  (struct kernel-stat
-    (st-dev unsigned-short)
-    (st-pad1 unsigned-short)
-    (st-ino long)
-    (st-mode unsigned-short)
-    (st-nlink unsigned-short)
-    (st-uid unsigned-short)
-    (st-gid unsigned-short)
-    (st-rdev unsigned-short)
-    (st-pad2 unsigned-short)
-    (st-size unsigned-long)
-    (st-blksize unsigned-long)
-    (st-blocks unsigned-long)
-    (st-atime unsigned-long)
-    (unused-1 unsigned-long)
-    (st-mtime unsigned-long)
-    (unused-2 unsigned-long)
-    (st-ctime unsigned-long)
-    (unused-3 unsigned-long)
-    (unused-4 unsigned-long)
-    (unused-5 unsigned-long)))
+    (struct kernel-stat
+            (st-dev unsigned-short)
+            (st-pad1 unsigned-short)
+            (st-ino long)
+            (st-mode unsigned-short)
+            (st-nlink unsigned-short)
+            (st-uid unsigned-short)
+            (st-gid unsigned-short)
+            (st-rdev unsigned-short)
+            (st-pad2 unsigned-short)
+            (st-size unsigned-long)
+            (st-blksize unsigned-long)
+            (st-blocks unsigned-long)
+            (st-atime unsigned-long)
+            (unused-1 unsigned-long)
+            (st-mtime unsigned-long)
+            (unused-2 unsigned-long)
+            (st-ctime unsigned-long)
+            (unused-3 unsigned-long)
+            (unused-4 unsigned-long)
+            (unused-5 unsigned-long)))
 
 ;;; STAT and friends.
 
@@ -66,6 +66,166 @@
 
 
 
+(defvar *kernel-errno-values*
+  (let ((hash (make-hash-table :test #'eq)))
+    (loop for (code string) in (list 
+                                (list 1	"Operation not permitted ")
+                                (list 2	"No such file or directory ")
+                                (list 3	"No such process ")
+                                (list 4	"Interrupted system call ")
+                                (list 5	"I/O error ")
+                                (list 6	"No such device or address ")
+                                (list 7	"Arg list too long ")
+                                (list 8	"Exec format error ")
+                                (list 9	"Bad file number ")
+                                (list 10	"No child processes ")
+                                (list 11	"Try again ")
+                                (list 12	"Out of memory ")
+                                (list 13	"Permission denied ")
+                                (list 14	"Bad address ")
+                                (list 15	"Block device required ")
+                                (list 16	"Device or resource busy ")
+                                (list 17	"File exists ")
+                                (list 18	"Cross-device link ")
+                                (list 19	"No such device ")
+                                (list 20	"Not a directory ")
+                                (list 21	"Is a directory ")
+                                (list 22	"Invalid argument ")
+                                (list 23	"File table overflow ")
+                                (list 24	"Too many open files ")
+                                (list 25	"Not a typewriter ")
+                                (list 26	"Text file busy ")
+                                (list 27	"File too large ")
+                                (list 28	"No space left on device ")
+                                (list 29	"Illegal seek ")
+                                (list 30	"Read-only file system ")
+                                (list 31	"Too many links ")
+                                (list 32	"Broken pipe ")
+                                (list 33	"Math argument out of domain of func ")
+                                (list 34	"Math result not representable ")
+                                (list 35	"Resource deadlock would occur ")
+                                (list 36	"File name too long ")
+                                (list 37	"No record locks available ")
+                                (list 38	"Function not implemented ")
+                                (list 39	"Directory not empty ")
+                                (list 40	"Too many symbolic links encountered ")
+                                (list 41 	"Operation would block")
+                                (list 42	"No message of desired type ")
+                                (list 43	"Identifier removed ")
+                                (list 44	"Channel number out of range ")
+                                (list 45	"Level 2 not synchronized ")
+                                (list 46	"Level 3 halted ")
+                                (list 47	"Level 3 reset ")
+                                (list 48	"Link number out of range ")
+                                (list 49	"Protocol driver not attached ")
+                                (list 50	"No CSI structure available ")
+                                (list 51	"Level 2 halted ")
+                                (list 52	"Invalid exchange ")
+                                (list 53	"Invalid request descriptor ")
+                                (list 54	"Exchange full ")
+                                (list 55	"No anode ")
+                                (list 56	"Invalid request code ")
+                                (list 57	"Invalid slot ")
+                                (list 59	"Bad font file format ")
+                                (list 60	"Device not a stream ")
+                                (list 61	"No data available ")
+                                (list 62	"Timer expired ")
+                                (list 63	"Out of streams resources ")
+                                (list 64	"Machine is not on the network ")
+                                (list 65	"Package not installed ")
+                                (list 66	"Object is remote ")
+                                (list 67	"Link has been severed ")
+                                (list 68	"Advertise error ")
+                                (list 69	"Srmount error ")
+                                (list 70	"Communication error on send ")
+                                (list 71	"Protocol error ")
+                                (list 72	"Multihop attempted ")
+                                (list 73	"RFS specific error ")
+                                (list 74	"Not a data message ")
+                                (list 75	"Value too large for defined data type ")
+                                (list 76	"Name not unique on network ")
+                                (list 77	"File descriptor in bad state ")
+                                (list 78	"Remote address changed ")
+                                (list 79	"Can not access a needed shared library ")
+                                (list 80	"Accessing a corrupted shared library ")
+                                (list 81	".lib section in a.out corrupted ")
+                                (list 82	"Attempting to link in too many shared libraries ")
+                                (list 83	"Cannot exec a shared library directly ")
+                                (list 84	"Illegal byte sequence ")
+                                (list 85	"Interrupted system call should be restarted ")
+                                (list 86	"Streams pipe error ")
+                                (list 87	"Too many users ")
+                                (list 88	"Socket operation on non-socket ")
+                                (list 89	"Destination address required ")
+                                (list 90	"Message too long ")
+                                (list 91	"Protocol wrong type for socket ")
+                                (list 92	"Protocol not available ")
+                                (list 93	"Protocol not supported ")
+                                (list 94	"Socket type not supported ")
+                                (list 95	"Operation not supported on transport endpoint ")
+                                (list 96	"Protocol family not supported ")
+                                (list 97	"Address family not supported by protocol ")
+                                (list 98	"Address already in use ")
+                                (list 99	"Cannot assign requested address ")
+                                (list 100	"Network is down ")
+                                (list 101	"Network is unreachable ")
+                                (list 102	"Network dropped connection because of reset ")
+                                (list 103	"Software caused connection abort ")
+                                (list 104	"Connection reset by peer ")
+                                (list 105	"No buffer space available ")
+                                (list 106	"Transport endpoint is already connected ")
+                                (list 107	"Transport endpoint is not connected ")
+                                (list 108	"Cannot send after transport endpoint shutdown ")
+                                (list 109	"Too many references: cannot splice ")
+                                (list 110	"Connection timed out ")
+                                (list 111	"Connection refused ")
+                                (list 112	"Host is down ")
+                                (list 113	"No route to host ")
+                                (list 114	"Operation already in progress ")
+                                (list 115	"Operation now in progress ")
+                                (list 116	"Stale NFS file handle ")
+                                (list 117	"Structure needs cleaning ")
+                                (list 118	"Not a XENIX named type file ")
+                                (list 119	"No XENIX semaphores available ")
+                                (list 120	"Is a named type file ")
+                                (list 121	"Remote I/O error ")
+                                (list 122	"Quota exceeded ")
+                                (list 123	"No medium found ")
+                                (list 124	"Wrong medium type ")
+                                (list 512    "ERESTARTSYS")
+                                (list 512    "ERESTARTNOINTR")
+                                (list 514	"restart if no handler.. ")
+                                (list 515	"No ioctl command ")
+                                (list 522	"Update synchronization mismatch ")
+                                (list 523	"Cookie is stale ")
+                                (list 524	"Operation is not supported ")
+                                (list 525	"Buffer or request is too small ")
+                                (list 526	"An untranslatable error occurred ")
+                                (list 527	"Type not supported by server ")
+                                (list 528	"Request initiated, but will not complete before timeout ")
+                                ) 
+          do
+          (setf (gethash code hash) string))
+    hash))
+
+(defun sanity-check-errno (errno)
+  "Checks if the errno is sane..."
+  (when (or (member errno (list 4 7 11 14))
+            (>= errno 512)
+            (eq :EOF 
+                (gethash errno *kernel-errno-values*
+                         :EOF)))
+    (error "Inpossible errno: ~S ~S" errno 
+           (gethash errno *kernel-errno-values* "Unknown error"))))
+
+(defmacro with-syscall-safe (&body code)
+  "Ensure that in the body the blocks of memory won't move round
+  and confuse the FFI"
+  `(system:without-gcing
+    (without-interrupts
+     (progn
+       ,@code))))
+
 ;;; Unix-access accepts a path and a mode.  It returns two values the
 ;;; first is T if the file is accessible and NIL otherwise.  The second
 ;;; only has meaning in the second case and is the unix errno value.
@@ -85,6 +245,7 @@
 	   (type (mod 8) mode))
   (multiple-value-bind (result errno)
       (%unix-access path mode)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -98,6 +259,7 @@
   (declare (type unix-pathname path))
   (multiple-value-bind (result errno)
       (%unix-chdir path)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -126,6 +288,7 @@
 	   (type unix-file-mode mode))
   (multiple-value-bind (result errno)
       (%unix-chmod path mode)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -143,6 +306,7 @@
 	   (type unix-file-mode mode))
   (multiple-value-bind (result errno)
       (%unix-fchmod fd mode)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -158,6 +322,7 @@
 	   (type (or unix-gid (integer -1 -1)) gid))
   (multiple-value-bind (result errno)
       (%unix-chown path uid gid)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -173,6 +338,7 @@
 	   (type (or unix-gid (integer -1 -1)) gid))
   (multiple-value-bind (result errno)
       (%unix-fchown fd uid gid)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -187,6 +353,7 @@
   (declare (type unix-fd fd))
   (multiple-value-bind (result errno)
       (%unix-close fd)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -203,7 +370,9 @@
   (declare (type unix-pathname name)
 	   (type unix-file-mode mode))
   (multiple-value-bind (result errno)
-      (%unix-open name (logior o_creat o_trunc o_wronly) mode)
+      (with-syscall-safe
+          (%unix-open name (logior o_creat o_trunc o_wronly) mode))
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values result 0))))
@@ -218,6 +387,7 @@
   (declare (type unix-fd fd))
   (multiple-value-bind (result errno)
       (%unix-dup fd)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values result 0))))
@@ -236,6 +406,7 @@
   (declare (type unix-fd fd1 fd2))
   (multiple-value-bind (result errno)
       (%unix-dup2 fd1 fd2)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -270,6 +441,7 @@
 	   (type (unsigned-byte 16) cmd arg))
   (multiple-value-bind (result errno)
       (%unix-fcntl fd cmd arg)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values result errno))))
@@ -282,6 +454,7 @@
   (declare (type unix-pathname name1 name2))
   (multiple-value-bind (result errno)
       (%unix-link name1 name2)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -347,6 +520,7 @@
 	   (type unix-file-mode mode))
   (multiple-value-bind (result errno)
       (%unix-mkdir name mode)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -387,7 +561,9 @@
   the unix error code."
   (with-alien ((fds (array int 2)))
     (multiple-value-bind (result errno)
-	(%unix-pipe (alien-sap (addr fds)))
+        (with-syscall-safe
+            (%unix-pipe (alien-sap (addr fds))))
+      (when (minusp result)  (sanity-check-errno errno))
       (if (minusp result)
 	  (values nil errno)
 	  (values (deref fds 0) (deref fds 1))))))
@@ -422,7 +598,9 @@
   (declare (type unix-pathname path))
   (with-alien ((buf (array char 1024)))
     (multiple-value-bind (result errno)
-	(%unix-readlink path (alien-sap (addr buf)) 1024)
+        (with-syscall-safe
+            (%unix-readlink path (alien-sap (addr buf)) 1024))
+      (when (minusp result)  (sanity-check-errno errno))
       (if (minusp result)
 	  (values nil errno)
 	  (let ((string (make-string result)))
@@ -440,6 +618,7 @@
   (declare (type unix-pathname name1 name2))
   (multiple-value-bind (result errno)
       (%unix-rename name1 name2)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -452,6 +631,7 @@
   (declare (type unix-pathname name))
   (multiple-value-bind (result errno)
       (%unix-rmdir name)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -477,13 +657,14 @@
 	 (setf (slot tv 'tv-sec) timeout-secs)
 	 (setf (slot tv 'tv-usec) ,timeout-usecs))
        (multiple-value-bind (nfds-ready errno)
-	   (%unix-select ,num-descriptors
-			 (if read-fds (alien-sap read-fds) (int-sap 0))
-			 (if write-fds (alien-sap write-fds) (int-sap 0))
-			 (if exception-fds
-			     (alien-sap exception-fds)
-			     (int-sap 0))
-			 (if timeout-secs (alien-sap (addr tv)) (int-sap 0)))
+           (with-syscall-safe
+               (%unix-select ,num-descriptors
+                             (if read-fds (alien-sap read-fds) (int-sap 0))
+                             (if write-fds (alien-sap write-fds) (int-sap 0))
+                             (if exception-fds
+                                 (alien-sap exception-fds)
+                                 (int-sap 0))
+                             (if timeout-secs (alien-sap (addr tv)) (int-sap 0))))
 	 (if (minusp nfds-ready)
 	     (values nil errno)
 	     (values nfds-ready errno))))))
@@ -512,8 +693,9 @@
 		      (int-sap 0)
 		      (alien-sap (addr ,alienvar)))))
       (multiple-value-bind (nfds-ready errno)
-	  (%unix-select nfds (frob rdfds rdf) (frob wrfds wrf) (frob xpfds xpf)
-			(if to-secs (alien-sap (addr tv)) (int-sap 0)))
+          (with-syscall-safe
+              (%unix-select nfds (frob rdfds rdf) (frob wrfds wrf) (frob xpfds xpf)
+                            (if to-secs (alien-sap (addr tv)) (int-sap 0))))
 	(if (minusp nfds-ready)
 	    (values nil errno)
 	    (values nfds-ready
@@ -530,6 +712,7 @@
    occured."
   (multiple-value-bind (result errno)
       (%unix-sync)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -543,6 +726,7 @@
   (declare (type unix-fd fd))
   (multiple-value-bind (result errno)
       (%unix-fsync fd)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -558,6 +742,7 @@
 	   (type (unsigned-byte 32) len))
   (multiple-value-bind (result errno)
       (%unix-truncate name len)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -569,6 +754,7 @@
 	   (type (unsigned-byte 32) len))
   (multiple-value-bind (result errno)
       (%unix-ftruncate fd len)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -580,6 +766,7 @@
   (declare (type unix-pathname name1 name2))
   (multiple-value-bind (result errno)
       (%unix-symlink name1 name2)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -593,6 +780,7 @@
   (declare (type unix-pathname name))
   (multiple-value-bind (result errno)
       (%unix-unlink name)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -633,6 +821,7 @@
 	   (type system-area-pointer arg))
   (multiple-value-bind (result errno)
       (%unix-ioctl fd cmd arg)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -647,6 +836,7 @@
   (declare (type (signed-byte 32) code))
   (multiple-value-bind (result errno)
       (%unix-exit code)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -661,20 +851,25 @@
   (when (string= name "")
     (setf name "."))
   (with-alien ((buf (struct #-linux stat #+linux kernel-stat)))
-    (multiple-value-bind (result errno)
-	(%unix-stat name (alien-sap (addr buf)))
+    (multiple-value-bind (result errno)        
+        (with-syscall-safe
+            (%unix-stat name (alien-sap (addr buf))))
+      (when (minusp result)  (sanity-check-errno errno))
       (if (minusp result)
 	  (values nil errno)
 	  #-linux (extract-stat-results buf)
 	  #+linux (extract-kernel-stat-results buf)))))
 
+
 (defun unix-lstat (name)
   "Unix-lstat is similar to unix-stat except the specified
    file must be a symbolic link."
   (declare (type unix-pathname name))
   (with-alien ((buf (struct #-linux stat #+linux kernel-stat)))
     (multiple-value-bind (result errno)
-	(%unix-lstat name (alien-sap (addr buf)))
+        (with-syscall-safe
+            (%unix-lstat name (alien-sap (addr buf))))
+      (when (minusp result)  (sanity-check-errno errno))
       (if (minusp result)
 	  (values nil errno)
 	  #-linux (extract-stat-results buf)
@@ -686,7 +881,9 @@
   (declare (type unix-fd fd))
   (with-alien ((buf (struct #-linux stat #+linux kernel-stat)))
     (multiple-value-bind (result errno)
-	(%unix-fstat fd (alien-sap (addr buf)))
+        (with-syscall-safe
+            (%unix-fstat fd (alien-sap (addr buf))))
+      (when (minusp result)  (sanity-check-errno errno))
       (if (minusp result)
 	  (values nil errno)
 	  #-linux (extract-stat-results buf)
@@ -702,7 +899,9 @@
 		   (unsigned-byte 31) (mod 1000000)))
   (with-alien ((usage (struct rusage)))
     (multiple-value-bind (result errno)
-	(%unix-getrusage who (alien-sap (addr usage)))
+        (with-syscall-safe
+            (%unix-getrusage who (alien-sap (addr usage))))
+      (when (minusp result)  (sanity-check-errno errno))
       (if (minusp result)
 	  (error "Getrusage failed: ~A" (get-unix-error-msg errno))
 	  (values t
@@ -720,7 +919,9 @@
   (declare (type (member #.rusage_self #.rusage_children) who))
   (with-alien ((usage (struct rusage)))
     (multiple-value-bind (result errno)
-	(%unix-getrusage who (alien-sap (addr usage)))
+        (with-syscall-safe
+            (%unix-getrusage who (alien-sap (addr usage))))
+      (when (minusp result)  (sanity-check-errno errno))
       (if (minusp result)
 	  (values nil errno)
 	  (values t
@@ -752,7 +953,8 @@
   (with-alien ((tv (struct timeval))
 	       (tz (struct timezone)))
     (multiple-value-bind (res errno)
-	(%unix-gettimeofday (alien-sap (addr tv)) (alien-sap (addr tz)))
+        (with-syscall-safe
+            (%unix-gettimeofday (alien-sap (addr tv)) (alien-sap (addr tz))))
       (if (minusp res)
 	  (error "Gettimeofday failed: ~A" (get-unix-error-msg errno))
 	  (values T (slot tv 'tv-sec) (slot tv 'tv-usec)
@@ -773,7 +975,9 @@
     (setf (slot (deref tvp 1) 'tv-sec) mtime-sec)
     (setf (slot (deref tvp 1) 'tv-usec) mtime-usec)
     (multiple-value-bind (result errno)
-	(%unix-utimes file (alien-sap (addr tvp)))
+        (with-syscall-safe
+            (%unix-utimes file (alien-sap (addr tvp))))
+      (when (minusp result)  (sanity-check-errno errno))
       (if (minusp result)
 	  (values nil errno)
 	  (values t 0)))))
@@ -786,14 +990,16 @@
   (declare (type unix-pathname file)
 	   (type (alien unsigned-long) atime-sec mtime-sec)
 	   (ignore mtime-usec atime-usec))
-    (with-alien ((buf (struct utimbuf)))
-      (setf (slot buf 'actime) atime-sec)
-      (setf (slot buf 'modtime) mtime-sec) 
-      (multiple-value-bind (result errno)
-	  (%unix-utime file (alien-sap (addr buf)))
-	(if (minusp result)
-	    (values nil errno)
-	    (values t 0)))))
+  (with-alien ((buf (struct utimbuf)))
+    (setf (slot buf 'actime) atime-sec)
+    (setf (slot buf 'modtime) mtime-sec) 
+    (multiple-value-bind (result errno)
+        (with-syscall-safe
+            (%unix-utime file (alien-sap (addr buf))))
+      (when (minusp result)  (sanity-check-errno errno))
+      (if (minusp result)
+          (values nil errno)
+          (values t 0)))))
 
 ;;; Unix-setreuid sets the real and effective user-id's of the current
 ;;; process to the arguments "ruid" and "euid", respectively.  Usage is
@@ -806,6 +1012,7 @@
    if the call fails."
   (multiple-value-bind (result errno)
       (%unix-setreuid ruid euid)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -821,6 +1028,7 @@
    returned if the call fails."
   (multiple-value-bind (result errno)
       (%unix-setregid rgid egid)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -861,6 +1069,7 @@
    pgrp.  NIL and an error number are returned upon failure."
   (multiple-value-bind (result errno)
       (%unix-setpgid pid pgrp)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -871,6 +1080,7 @@
    group leader. NIL and an error number are returned upon failure."
   (multiple-value-bind (result errno)
       (%unix-setpgid pid pgrp)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
@@ -895,6 +1105,7 @@
    doesn't work."
   (multiple-value-bind (result errno)
       (%unix-fork)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values result 0))))
@@ -907,21 +1118,22 @@
 	(envp nil)
 	(envp-bytes 0))
     (unwind-protect
-	(progn
-	  ;; Blast the stuff into the proper format
-	  (multiple-value-setq
-	      (argv argv-bytes)
-	    (string-list-to-c-strvec arg-list))
-	  (multiple-value-setq
-	      (envp envp-bytes)
-	    (string-list-to-c-strvec env-list))
-	  ;;
-	  ;; Now do the system call
-	  (multiple-value-bind (result errno)
-	    (%unix-execve program argv envp)
-	    (if (minusp result)
-		(values nil errno)
-		(values t 0))))
+         (progn
+           ;; Blast the stuff into the proper format
+           (multiple-value-setq
+               (argv argv-bytes)
+             (string-list-to-c-strvec arg-list))
+           (multiple-value-setq
+               (envp envp-bytes)
+             (string-list-to-c-strvec env-list))
+           ;;
+           ;; Now do the system call
+           (multiple-value-bind (result errno)
+               (%unix-execve program argv envp)
+             (when (minusp result)  (sanity-check-errno errno))
+             (if (minusp result)
+                 (values nil errno)
+                 (values t 0))))
       ;;
       ;; Deallocate memory
       (when argv
@@ -940,15 +1152,17 @@
 		 (:virtual ITIMER-VIRTUAL)
 		 (:profile ITIMER-PROF))))
     (with-alien ((itv (struct itimerval)))
-       (multiple-value-bind (result errno)
-	   (%unix-getitimer which (alien-sap (addr itv)))
-	 (if (minusp result)
-	     (values nil errno)
-	     (values T
-		     (slot (slot itv 'it-interval) 'tv-sec)
-		     (slot (slot itv 'it-interval) 'tv-usec)
-		     (slot (slot itv 'it-value) 'tv-sec)
-		     (slot (slot itv 'it-value) 'tv-usec)))))))
+      (multiple-value-bind (result errno)
+          (with-syscall-safe
+              (%unix-getitimer which (alien-sap (addr itv))))
+        (when (minusp result)  (sanity-check-errno errno))
+        (if (minusp result)
+            (values nil errno)
+            (values T
+                    (slot (slot itv 'it-interval) 'tv-sec)
+                    (slot (slot itv 'it-interval) 'tv-usec)
+                    (slot (slot itv 'it-value) 'tv-sec)
+                    (slot (slot itv 'it-value) 'tv-usec)))))))
 
 (defun unix-setitimer(which int-secs int-usec val-secs val-usec)
   "Unix-setitimer sets the INTERVAL and VALUE slots of one of
@@ -972,17 +1186,19 @@
 	    (slot (slot itvn 'it-interval) 'tv-usec) int-usec
 	    (slot (slot itvn 'it-value   ) 'tv-sec ) val-secs
 	    (slot (slot itvn 'it-value   ) 'tv-usec) val-usec)
-       (multiple-value-bind (result errno)
-	   (%unix-setitimer which
-			    (alien-sap (addr itvn))
-			    (alien-sap (addr itvo)))
-	 (if (minusp result)
-	     (values nil errno)
-	     (values T
-		     (slot (slot itvo 'it-interval) 'tv-sec)
-		     (slot (slot itvo 'it-interval) 'tv-usec)
-		     (slot (slot itvo 'it-value) 'tv-sec)
-		     (slot (slot itvo 'it-value) 'tv-usec)))))))
+      (multiple-value-bind (result errno)
+          (with-syscall-safe
+              (%unix-setitimer which
+                               (alien-sap (addr itvn))
+                               (alien-sap (addr itvo))))
+        (when (minusp result)  (sanity-check-errno errno))
+        (if (minusp result)
+            (values nil errno)
+            (values T
+                    (slot (slot itvo 'it-interval) 'tv-sec)
+                    (slot (slot itvo 'it-interval) 'tv-usec)
+                    (slot (slot itvo 'it-value) 'tv-sec)
+                    (slot (slot itvo 'it-value) 'tv-usec)))))))
 
 
 #+linux
@@ -991,6 +1207,7 @@
    occured."
   (multiple-value-bind (result errno)
       (%unix-sched-yield)
+    (when (minusp result)  (sanity-check-errno errno))
     (if (minusp result)
 	(values nil errno)
 	(values t 0))))
