@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/aliencomp.lisp,v 1.24 1997/02/22 19:19:54 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/aliencomp.lisp,v 1.26 2001/06/01 12:51:56 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -487,8 +487,10 @@
        '(lambda (sap type)
 	  (declare (ignore type))
 	  sap))
+      (ref
+       `(alien::alien-value-sap alien))
       (t
-       (give-up)))))
+       (give-up "Could not optimize away ALIEN-SAP")))))
 
 (defoptimizer (%sap-alien derive-type) ((sap type))
   (declare (ignore sap))
@@ -688,21 +690,15 @@
 	(let* ((arg (pop args))
 	       (sc (tn-sc tn))
 	       (scn (sc-number sc))
-	       (temp-tn
-		(unless (backend-featurep :x86)
-		  (make-representation-tn (tn-primitive-type tn) scn)))
+	       (temp-tn (make-representation-tn (tn-primitive-type tn) scn))
 	       (move-arg-vops (svref (sc-move-arg-vops sc) scn)))
 	  (assert arg)
 	  (assert (= (length move-arg-vops) 1) ()
 		  "No unique move-arg-vop for moves in SC ~S."
 		  (sc-name sc))
-	  (if (backend-featurep :x86)
-	      (emit-move-arg-template call block (first move-arg-vops)
-				      (continuation-tn call block arg) nsp tn)
-	    (progn
-	      (emit-move call block (continuation-tn call block arg) temp-tn)
-	      (emit-move-arg-template call block (first move-arg-vops)
-				      temp-tn nsp tn)))))
+	  (emit-move call block (continuation-tn call block arg) temp-tn)
+	  (emit-move-arg-template call block (first move-arg-vops)
+				  temp-tn nsp tn)))
       (assert (null args))
       (unless (listp result-tns)
 	(setf result-tns (list result-tns)))
