@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/byte-comp.lisp,v 1.27 1999/02/25 13:03:00 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/byte-comp.lisp,v 1.29 2000/09/26 16:39:30 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -1310,7 +1310,11 @@
 	 (t
 	  (etypecase leaf
 	    (constant
-	     (output-push-constant-leaf segment leaf))
+	     (cond ((legal-immediate-constant-p leaf)
+		    (output-push-constant-leaf segment leaf))
+		   (t
+		    (output-push-constant segment (leaf-name leaf))
+		    (output-do-inline-function segment 'symbol-value))))
 	    (clambda
 	     (let* ((refered-env (lambda-environment leaf))
 		    (closure (environment-closure refered-env)))
@@ -1349,17 +1353,24 @@
 	 (values (if info
 		     (byte-continuation-info-results info)
 		     0)))
-    (unless (eql values 0)
-      ;; Someone wants the value, so copy it.
-      (output-do-xop segment 'dup))
     (etypecase leaf
       (global-var
+       (unless (eql values 0)
+	 ;; Someone wants the value, so copy it.
+	 (output-do-xop segment 'dup))
        (ecase (global-var-kind leaf)
 	 ((:special :global)
 	  (output-push-constant segment (global-var-name leaf))
 	  (output-do-inline-function segment 'setf-symbol-value))))
       (lambda-var
-       (output-set-lambda-var segment leaf (node-environment set))))
+       (cond ((leaf-refs leaf)
+	      (unless (eql values 0)
+		;; Someone wants the value, so copy it.
+		(output-do-xop segment 'dup))
+	      (output-set-lambda-var segment leaf (node-environment set)))
+	     ;; If no-one wants the value then pop it else leave it for them.
+	     ((eql values 0)
+	      (output-byte-with-operand segment byte-pop-n 1)))))
     (unless (eql values 0)
       (checked-canonicalize-values segment cont 1)))
   (undefined-value))
