@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/checkgen.lisp,v 1.26 1998/03/01 21:55:39 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/checkgen.lisp,v 1.30 2000/07/09 13:59:58 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -80,6 +80,12 @@
 	    (+ 1
 	       (if (numeric-type-low type) 1 0)
 	       (if (numeric-type-high type) 1 0))))
+	(cons-type
+	 (+ (type-test-cost (specifier-type 'cons))
+	    (function-cost 'car)
+	    (type-test-cost (cons-type-car-type type))
+	    (function-cost 'cdr)
+	    (type-test-cost (cons-type-cdr-type type))))
 	(t
 	 (function-cost 'typep)))))
 
@@ -121,20 +127,58 @@
 	       *universal-type*)))))
 
 
-;;; NO-FUNCTION-VALUES-TYPES  --  Internal
+;;; NO-FUNCTION-TYPES  --  Internal
 ;;;
-;;;    Like VALUES-TYPES, only mash any complex function types to FUNCTION.
+;;;    Mash any complex function types to FUNCTION.
 ;;;
-(defun no-function-values-types (type)
-  (declare (type ctype type))
-  (multiple-value-bind (res count)
-		       (values-types type)
-    (values (mapcar #'(lambda (type)
-			(if (function-type-p type)
-			    (specifier-type 'function)
-			    type))
-		    res)
-	    count)))
+(defun no-function-types (types)
+  (declare (type list types))
+  (mapcar #'(lambda (type)
+	      (if (function-type-p type)
+		  (specifier-type 'function)
+		  type))
+	  types))
+
+
+;;; Values-types-asserted  --  Internal
+;;;
+;;;    Like values-types, but when an argument is proven to be delivered,
+;;; convert asserted optional and rest arguments to required arguments. This
+;;; makes it clear that these required arguments may all be type checked.
+;;;
+(defun values-types-asserted (atype ptype)
+  (declare (type ctype atype ptype))
+  (cond ((eq atype *wild-type*)
+	 (values nil :unknown))
+	((not (values-type-p atype))
+	 (values (list atype) 1))
+	((or (args-type-keyp atype)
+	     (args-type-allowp atype))
+	 (values nil :unknown))
+	(t
+	 (let* ((ptype (kernel::coerce-to-values ptype))
+		(preq (args-type-required ptype))
+		(popt (args-type-optional ptype))
+		(prest (args-type-rest ptype)))
+	   (collect ((types))
+	     (do ((args (args-type-required atype) (rest args)))
+		 ((endp args))
+	       (if (or (pop preq) (pop popt) prest)
+		   (types (single-value-type (first args)))
+		   (return-from values-types-asserted (values nil :unknown))))
+	     (do ((args (args-type-optional atype) (rest args)))
+		 ((endp args))
+	       (if (pop preq)
+		   (types (single-value-type (first args)))
+		   (return-from values-types-asserted (values nil :unknown))))
+	     (let ((arest (args-type-rest atype)))
+	       (when arest
+		 (do ((arg (pop preq) (pop preq)))
+		     ((null arg))
+		   (types (single-value-type arest)))
+		 (when (or popt prest)
+		   (return-from values-types-asserted (values nil :unknown)))))
+	     (values (types) (length (types))))))))
 
 
 ;;; Switch to disable check complementing, for evaluation.
@@ -162,7 +206,7 @@
   (declare (type continuation cont) (list types))
   (multiple-value-bind
       (ptypes count)
-      (no-function-values-types (continuation-proven-type cont))
+      (values-types (continuation-proven-type cont))
     (if (eq count :unknown)
 	(if (and (every #'type-check-template types) (not force-hairy))
 	    (values :simple types)
@@ -171,15 +215,17 @@
 				(list nil (maybe-weaken-check x cont) x))
 			    types)))
 	(let ((res (mapcar #'(lambda (p c)
-			       (let ((diff (type-difference p c))
-				     (weak (maybe-weaken-check c cont)))
-				 (if (and diff
-					  (< (type-test-cost diff)
-					     (type-test-cost weak))
-					  *complement-type-checks*)
-				     (list t diff c)
-				     (list nil weak c))))
-			   ptypes types)))
+			       (if (csubtypep p c)
+				   (list nil *universal-type* c)
+				   (let ((diff (type-difference p c))
+					 (weak (maybe-weaken-check c cont)))
+				     (if (and diff
+					      (< (type-test-cost diff)
+						 (type-test-cost weak))
+					      *complement-type-checks*)
+					 (list t diff c)
+					 (list nil weak c)))))
+			   (no-function-types ptypes) types)))
 	  (cond ((or force-hairy (find-if #'first res))
 		 (values :hairy res))
 		((every #'type-check-template types)
@@ -228,29 +274,37 @@
 ;;; derived type and the asserted type, then we check for the negation of this
 ;;; type instead.
 ;;;
+;;; When the proven type represents an unknown number of values, but Cont's
+;;; destination receives only a single value, a :hairy type check is
+;;; generated for the single-values-type of the asserted type.
+;;;
 (defun continuation-check-types (cont)
   (declare (type continuation cont))
-  (let ((type (continuation-asserted-type cont))
+  (let ((atype (continuation-asserted-type cont))
 	(dest (continuation-dest cont)))
-    (assert (not (eq type *wild-type*)))
+    (assert (not (eq atype *wild-type*)))
     (multiple-value-bind (types count)
-			 (no-function-values-types type)
+	(values-types-asserted atype (continuation-proven-type cont))
       (cond ((not (eq count :unknown))
-	     (if (or (exit-p dest)
-		     (and (return-p dest)
-			  (multiple-value-bind
-			      (ignore count)
-			      (values-types (return-result-type dest))
-			    (declare (ignore ignore))
-			    (eq count :unknown))))
-		 (maybe-negate-check cont types t)
-		 (maybe-negate-check cont types nil)))
+	     (let ((types (no-function-types types)))
+	       (if (or (exit-p dest)
+		       (and (return-p dest)
+			    (multiple-value-bind
+				  (ignore count)
+				(values-types (return-result-type dest))
+			      (declare (ignore ignore))
+			      (eq count :unknown))))
+		   (maybe-negate-check cont types t)
+		   (maybe-negate-check cont types nil))))
 	    ((and (mv-combination-p dest)
 		  (eq (basic-combination-kind dest) :local))
-	     (assert (values-type-p type))
-	     (maybe-negate-check cont (args-type-optional type) nil))
+	     (assert (values-type-p atype))
+	     (assert (null (args-type-required atype)))
+	     (maybe-negate-check cont (args-type-optional atype) nil))
+	    ((or (exit-p dest) (return-p dest) (mv-combination-p dest))
+	     (values :too-hairy nil))
 	    (t
-	     (values :too-hairy nil))))))
+	     (maybe-negate-check cont (list (single-value-type atype)) t))))))
 
 
 ;;; Probable-Type-Check-P  --  Internal
