@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/constraint.lisp,v 1.20 1999/01/25 12:25:47 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/constraint.lisp,v 1.23 2001/04/12 19:45:56 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -92,7 +92,7 @@
 ;;;    If Ref is to a Lambda-Var with Constraints (i.e. we can do flow analysis
 ;;; on it), then return the Lambda-Var, otherwise NIL.
 ;;;
-(proclaim '(inline ok-ref-lambda-var))
+(declaim (inline ok-ref-lambda-var))
 (defun ok-ref-lambda-var (ref)
   (declare (type ref ref))
   (let ((leaf (ref-leaf ref)))
@@ -106,7 +106,7 @@
 ;;;    If Cont's Use is a Ref, then return OK-REF-LAMBDA-VAR of the Use,
 ;;; otherwise NIL.
 ;;;
-(proclaim '(inline ok-cont-lambda-var))
+(declaim (inline ok-cont-lambda-var))
 (defun ok-cont-lambda-var (cont)
   (declare (type continuation cont))
   (let ((use (continuation-use cont)))
@@ -136,7 +136,7 @@
 ;;;    Add complementary constraints to the consequent and alternative blocks
 ;;; of If.  We do nothing if X is NIL.
 ;;;
-(proclaim '(inline add-complement-constraints))
+(declaim (inline add-complement-constraints))
 (defun add-complement-constraints (if fun x y not-p)
   (when x
     (add-test-constraint (if-consequent if) fun x y not-p)
@@ -243,7 +243,7 @@
 	   (when (continuation-type-check cont)
 	     (let ((var (ok-ref-lambda-var node)))
 	       (when var
-		 (let* ((atype (continuation-derived-type cont))
+		 (let* ((atype (continuation-type cont))
 			(con (find-constraint 'typep var atype nil)))
 		   (sset-adjoin con gen))))))
 	  (cset
@@ -409,7 +409,6 @@
 		      (let ((greater (if not-p (not greater) greater)))
 			(setq res
 			      (constrain-integer-type res y greater not-p)))))
-		   #+constrain-float-type
 		   ((and (float-type-p res) (float-type-p y))
 		    (let ((greater (eq kind '>)))
 		      (let ((greater (if not-p (not greater) greater)))
@@ -456,7 +455,8 @@
 		 (when (continuation-type-check cont)
 		   (sset-adjoin
 		    (find-constraint 'typep var
-				     (continuation-asserted-type cont)
+				     (single-value-type
+				      (continuation-asserted-type cont))
 				     nil)
 		    in)))))))
 	(cset
