@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/eval.lisp,v 1.32 1999/02/25 13:03:04 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/eval.lisp,v 1.35 2001/03/04 20:12:15 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -136,15 +136,15 @@
   "If an interpreted function goes uncalled for more than this many GCs, then
   it is eligible for flushing from the cache.")
 
-(proclaim '(type c::index
-		 *interpreted-function-cache-minimum-size*
-		 *interpreted-function-cache-threshold*))
+(declaim (type c::index
+	       *interpreted-function-cache-minimum-size*
+	       *interpreted-function-cache-threshold*))
 
 
 ;;; The list of INTERPRETED-FUNCTIONS that have translated definitions.
 ;;;
 (defvar *interpreted-function-cache* nil)
-(proclaim '(type list *interpreted-function-cache*))
+(declaim (type list *interpreted-function-cache*))
 
 
 ;;; MAKE-INTERPRETED-FUNCTION  --  Interface
@@ -625,35 +625,14 @@
   (setf *internal-apply-node-trace* on))
 
 
-;;;; INTERNAL-EVAL:
-
-(proclaim '(special lisp::*already-evaled-this*))
-
 ;;; INTERNAL-EVAL  --  Interface
 ;;;
 ;;;    Evaluate an arbitary form.  We convert the form, then call internal
-;;; apply on it.  If *ALREADY-EVALED-THIS* is true, then we bind it to NIL
-;;; around the apply to limit the inhibition to the lexical scope of the
-;;; EVAL-WHEN.
+;;; apply on it.
 ;;;
 (defun internal-eval (form &optional quietly)
   (let ((res (c:compile-for-eval form quietly)))
-    (if lisp::*already-evaled-this*
-	(let ((lisp::*already-evaled-this* nil))
-	  (internal-apply res nil '#()))
-	(internal-apply res nil '#()))))
-
-
-;;; MAKE-INDIRECT-VALUE-CELL -- Internal.
-;;;
-;;; Later this will probably be the same weird internal thing the compiler
-;;; makes to represent these things.
-;;;
-(defun make-indirect-value-cell (value)
-  (list value))
-;;;
-(defmacro indirect-value (value-cell)
-  `(car ,value-cell))
+    (internal-apply res nil '#())))
 
 
 ;;; VALUE -- Internal.
@@ -708,7 +687,7 @@
 	(cond ((c::leaf-refs var)
 	       (setf (eval-stack-local frame-ptr (c::lambda-var-info var))
 		     (if (c::lambda-var-indirect var)
-			 (make-indirect-value-cell (pop args))
+			 (c:make-value-cell (pop args))
 			 (pop args))))
 	      (ignore-unused (pop args)))))
     (internal-apply-loop (c::lambda-bind lambda) frame-ptr lambda args
@@ -917,24 +896,27 @@
 ;;; SET-LEAF-VALUE-LAMBDA-VAR -- Internal Interface.
 ;;;
 ;;; This does SET-LEAF-VALUE for a lambda-var leaf.  The debugger tools'
-;;; internals uses this also to set interpreted local variables.
+;;; internals uses this also to set interpreted local variables. If the var
+;;; is a lexical variable with no refs, then we don't actually set anything,
+;;; since the variable has been deleted.
 ;;;
 (defun set-leaf-value-lambda-var (node var frame-ptr closure value)
-  (let ((env (c::node-environment node)))
-    (cond ((not (eq (c::lambda-environment (c::lambda-var-home var))
-		    env))
-	   (setf (indirect-value
-		  (svref closure
-			 (position var (c::environment-closure env)
-				   :test #'eq)))
-		 value))
-	  ((c::lambda-var-indirect var)
-	   (setf (indirect-value
-		  (eval-stack-local frame-ptr (c::lambda-var-info var)))
-		 value))
-	  (t
-	   (setf (eval-stack-local frame-ptr (c::lambda-var-info var))
-		 value)))))
+  (when (c::leaf-refs var)
+    (let ((env (c::node-environment node)))
+      (cond ((not (eq (c::lambda-environment (c::lambda-var-home var))
+		      env))
+	     (c:value-cell-set
+	      (svref closure (position var (c::environment-closure env)
+				       :test #'eq))
+	      value))
+	    ((c::lambda-var-indirect var)
+	     (c:value-cell-set
+	      (eval-stack-local frame-ptr (c::lambda-var-info var))
+	      value))
+	    (t
+	     (setf (eval-stack-local frame-ptr (c::lambda-var-info var))
+		   value)))))
+  value)
 
 ;;; LEAF-VALUE -- Internal.
 ;;;
@@ -1028,7 +1010,7 @@
 		     (position leaf (c::environment-closure env)
 			       :test #'eq)))))
     (if (c::lambda-var-indirect leaf)
-	(indirect-value temp)
+	(c:value-cell-ref temp)
 	temp)))
 
 ;;; COMPUTE-CLOSURE -- Internal.
@@ -1172,7 +1154,7 @@
       (when (c::leaf-refs v)
 	(setf (eval-stack-local frame-ptr (c::lambda-var-info v))
 	      (if (c::lambda-var-indirect v)
-		  (make-indirect-value-cell (pop args))
+		  (c:make-value-cell (pop args))
 		  (pop args)))))))
 
 ;;; STORE-MV-LET-VARS -- Internal.
@@ -1191,7 +1173,7 @@
       (if (c::leaf-refs v)
 	  (setf (eval-stack-local frame-ptr (c::lambda-var-info v))
 		(if (c::lambda-var-indirect v)
-		    (make-indirect-value-cell (pop args))
+		    (c:make-value-cell (pop args))
 		    (pop args)))
 	  (pop args)))))
 
@@ -1223,7 +1205,7 @@
 	(when (c::leaf-refs v)
 	  (setf (eval-stack-local frame-ptr (c::lambda-var-info v))
 		(if (c::lambda-var-indirect v)
-		    (make-indirect-value-cell (car remaining-args))
+		    (c:make-value-cell (car remaining-args))
 		    (car remaining-args))))
 	(cdr remaining-args))
       args))
