@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/float-tran.lisp,v 1.80 1999/09/06 15:27:45 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/float-tran.lisp,v 1.84 2001/09/24 15:25:38 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -118,9 +118,9 @@
 #+new-random
 (deftransform random ((num &optional state)
 		      ((integer 1
-				#+(and propagate-float-type x86) #xffffffff
-				#+(and propagate-float-type (not x86)) #x7fffffff
-				#-propagate-float-type #.most-positive-fixnum)
+				#+x86 #xffffffff
+				#-x86 #x7fffffff
+				)
 		       &optional *))
   #+x86 "use inline (unsigned-byte 32) operations"
   #-x86 "use inline (signed-byte 32) operations"
@@ -282,9 +282,6 @@
 ;;; Optimizers for scale-float.  If the float has bounds, new bounds
 ;;; are computed for the result, if possible.
 
-#+propagate-float-type
-(progn
-
 (defun scale-float-derive-type-aux (f ex same-arg)
   (declare (ignore same-arg))
   (flet ((scale-bound (x n)
@@ -347,7 +344,6 @@
 	     (one-arg-derive-type num #',aux-name #',fun))))))
   (frob %single-float single-float)
   (frob %double-float double-float))
-) ; end progn  
 
 
 ;;;; Float contagion:
@@ -395,36 +391,6 @@
   (frob >)
   (frob =))
 
-
-;;;; Irrational derive-type methods:
-
-;;; Derive the result to be float for argument types in the appropriate domain.
-;;;
-#-propagate-fun-type
-(dolist (stuff '((asin (real -1.0 1.0))
-		 (acos (real -1.0 1.0))
-		 (acosh (real 1.0))
-		 (atanh (real -1.0 1.0))
-		 (sqrt (real 0.0))))
-  (destructuring-bind (name type) stuff
-    (let ((type (specifier-type type)))
-      (setf (function-info-derive-type (function-info-or-lose name))
-	    #'(lambda (call)
-		(declare (type combination call))
-		(when (csubtypep (continuation-type
-				  (first (combination-args call)))
-				 type)
-		  (specifier-type 'float)))))))
-
-#-propagate-fun-type
-(defoptimizer (log derive-type) ((x &optional y))
-  (when (and (csubtypep (continuation-type x)
-			(specifier-type '(real 0.0)))
-	     (or (null y)
-		 (csubtypep (continuation-type y)
-			    (specifier-type '(real 0.0)))))
-    (specifier-type 'float)))
-
 
 ;;;; Irrational transforms:
 
@@ -580,9 +546,6 @@
        (float pi x)
        (float 0 x)))
 
-#+(or propagate-float-type propagate-fun-type)
-(progn
-
 ;;; The number is of type REAL.
 (declaim (inline numeric-type-real-p))
 (defun numeric-type-real-p (type)
@@ -597,10 +560,6 @@
 	(list (coerce (car bound) type))
 	(coerce bound type))))
 
-)  ; end progn
-
-#+propagate-fun-type
-(progn
 
 ;;;; Optimizers for elementary functions
 ;;;;
@@ -695,9 +654,7 @@
   (etypecase arg
     (numeric-type
      (cond ((eq (numeric-type-complexp arg) :complex)
-	    (make-numeric-type :class (numeric-type-class arg)
-			       :format (numeric-type-format arg)
-			       :complexp :complex))
+	    (complex-float-type arg))
 	   ((numeric-type-real-p arg)
 	    ;; The argument is real, so let's find the intersection
 	    ;; between the argument and the domain of the function.
@@ -1120,7 +1077,6 @@
 (defoptimizer (phase derive-type) ((num))
   (one-arg-derive-type num #'phase-derive-type-aux #'phase))
 
-) ;end progn for propagate-fun-type
 
 (deftransform realpart ((x) ((complex rational)) *)
   '(kernel:%realpart x))
@@ -1225,8 +1181,9 @@
 
 
 ;;; Define some transforms for complex operations.  We do this in lieu
-;;; of complex operation VOPs.
+;;; of complex operation VOPs.  Some architectures have vops, though.
 ;;;
+#-complex-fp-vops
 (macrolet ((frob (type)
 	     `(progn
 	       ;; Negation
@@ -1265,11 +1222,11 @@
 			 (iy (imagpart y)))
 		    (if (> (abs ry) (abs iy))
 			(let* ((r (/ iy ry))
-			       (dn (* ry (+ 1 (* r r)))))
+			       (dn (+ ry (* r iy))))
 			  (complex (/ (+ rx (* ix r)) dn)
 				   (/ (- ix (* rx r)) dn)))
 			(let* ((r (/ ry iy))
-			       (dn (* iy (+ 1 (* r r)))))
+			       (dn (+ iy (* r ry))))
 			  (complex (/ (+ (* rx r) ix) dn)
 				   (/ (- (* ix r) rx) dn))))))
 	       ;; Multiply a complex by a real or vice versa
@@ -1280,6 +1237,19 @@
 	       ;; Divide a complex by a real
 	       (deftransform / ((w z) ((complex ,type) real) *)
 		 '(complex (/ (realpart w) z) (/ (imagpart w) z)))
+	       ;; Divide a real by a complex
+	       (deftransform / ((rx y) (real (complex ,type)) *)
+		 '(let* ((ry (realpart y))
+			 (iy (imagpart y)))
+		    (if (> (abs ry) (abs iy))
+			(let* ((r (/ iy ry))
+			       (dn (+ ry (* r iy))))
+			  (complex (/ rx dn)
+				   (/ (- (* rx r)) dn)))
+			(let* ((r (/ ry iy))
+			       (dn (+ iy (* r ry))))
+			  (complex (/ (* rx r) dn)
+				   (/ (- rx) dn))))))
 	       ;; Conjugate of complex number
 	       (deftransform conjugate ((z) ((complex ,type)) *)
 		 '(complex (realpart z) (- (imagpart z))))
@@ -1299,22 +1269,35 @@
   (frob single-float)
   (frob double-float))
 
+#+complex-fp-vops
+(macrolet
+    ((frob (type)
+       `(progn
+	 ;; Cis.
+	 (deftransform cis ((z) ((,type)) *)
+	   '(complex (cos z) (sin z)))
+	 ;; Comparison
+	 (deftransform = ((w z) ((complex ,type) real) *)
+	   '(and (= (realpart w) z) (zerop (imagpart w))))
+	 (deftransform = ((w z) (real (complex ,type)) *)
+	   '(and (= (realpart z) w) (zerop (imagpart z))))
+	 )))
+
+  (frob single-float)
+  (frob double-float))
+
 ;;; Here are simple optimizers for sin, cos, and tan.  They do not
 ;;; produce a minimal range for the result; the result is the widest
 ;;; possible answer.  This gets around the problem of doing range
 ;;; reduction correctly but still provides useful results when the
 ;;; inputs are union types.
 
-#+propagate-fun-type
-(progn
 (defun trig-derive-type-aux (arg domain fcn
 				 &optional def-lo def-hi (increasingp t))
   (etypecase arg
     (numeric-type
      (cond ((eq (numeric-type-complexp arg) :complex)
-	    (make-numeric-type :class (numeric-type-class arg)
-			       :format (numeric-type-format arg)
-			       :complexp :complex))
+	    (complex-float-type arg))
 	   ((numeric-type-real-p arg)
 	    (let* ((format (case (numeric-type-class arg)
 			     ((integer rational) 'single-float)
@@ -1387,8 +1370,6 @@
 (defoptimizer (cis derive-type) ((num))
   (one-arg-derive-type num
      #'(lambda (arg)
-	 (c::specifier-type
-	  `(complex ,(or (numeric-type-format arg) 'float))))
+	 (specifier-type `(complex ,(or (numeric-type-format arg) 'float))))
      #'cis))
 
-) ; end progn
