@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/fndb.lisp,v 1.85 1999/02/02 12:18:36 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/fndb.lisp,v 1.96 2001/10/16 19:19:54 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -119,10 +119,10 @@
 (defknown makunbound (symbol) symbol)
 (defknown fmakunbound ((or symbol cons)) (or symbol cons)
   (unsafe explicit-check))
-(defknown (get-setf-method get-setf-method-multiple-value)
-  ((or list symbol) &optional lexical-environment)
-  (values list list list form form)
-  (flushable))
+(defknown get-setf-expansion ((or list symbol) &optional lexical-environment)
+ (values list list list form form)
+ (flushable))
+
 (defknown apply (callable t &rest t) *) ; ### Last arg must be List...
 (defknown funcall (callable &rest t) *)
 
@@ -242,12 +242,6 @@
 (defknown lcm (&rest integer) unsigned-byte
   (movable foldable flushable explicit-check))
 
-#-propagate-fun-type
-(defknown exp (number) irrational
-  (movable foldable flushable explicit-check recursive)
-  :derive-type #'result-type-float-contagion)
-
-#+propagate-fun-type
 (defknown exp (number) irrational
   (movable foldable flushable explicit-check recursive))
 
@@ -266,25 +260,6 @@
 (defknown cis (real) (complex float)
   (movable foldable flushable explicit-check))
 
-#-propagate-fun-type
-(progn
-(defknown (sin cos) (number)
-  (or (float -1.0 1.0) (complex float))
-  (movable foldable flushable explicit-check recursive)
-  :derive-type #'result-type-float-contagion)
-
-(defknown atan
-  (number &optional real) irrational
-  (movable foldable flushable explicit-check recursive)
-  :derive-type #'result-type-float-contagion)
-
-(defknown (tan sinh cosh tanh asinh)
-  (number) irrational (movable foldable flushable explicit-check recursive)
-  :derive-type #'result-type-float-contagion)
-)	; end progn
-
-#+propagate-fun-type
-(progn
 (defknown (sin cos) (number)
   (or (float -1.0 1.0) (complex float))
   (movable foldable flushable explicit-check recursive))
@@ -295,7 +270,6 @@
 
 (defknown (tan sinh cosh tanh asinh)
   (number) irrational (movable foldable flushable explicit-check recursive))
-)	; end progn
 
 (defknown (asin acos acosh atanh)
   (number) irrational
@@ -837,7 +811,7 @@
 (defknown stream-element-type (stream) type-specifier (movable foldable flushable))
 (defknown (output-stream-p input-stream-p) (stream) boolean (movable foldable
 								     flushable))
-(defknown close (stream &key (:abort t)) stream ())
+(defknown close (stream &key (:abort t)) t ())
 
 
 ;;;; In the "Input/Output" chapter:
@@ -846,7 +820,8 @@
 ;;; code motion over I/O operations is particularly confusing and not very
 ;;; important for efficency.
 
-(defknown copy-readtable (&optional (or readtable null) readtable) readtable
+(defknown copy-readtable (&optional (or readtable null) (or readtable null))
+  readtable
   ())
 (defknown readtablep (t) boolean (movable foldable flushable))
 
@@ -856,7 +831,7 @@
 
 (defknown set-macro-character (character callable &optional t readtable) void
   (unsafe))
-(defknown get-macro-character (character &optional readtable)
+(defknown get-macro-character (character &optional (or readtable null))
   (values callable boolean) (flushable))
 
 (defknown make-dispatch-macro-character (character &optional t readtable)
@@ -865,7 +840,7 @@
   (character character callable &optional readtable) void
   (unsafe))
 (defknown get-dispatch-macro-character
-  (character character &optional readtable) callable
+  (character character &optional (or readtable null)) callable
   (flushable))
 
 ;;; May return any type due to eof-value...
@@ -969,9 +944,9 @@
 (defknown truename (pathnamelike) pathname ())
 
 (defknown parse-namestring
-  (pathnamelike &optional pathname-host pathnamelike
+  (pathnamelike &optional (or string pathname-host) pathnamelike
 		&key (:start index) (:end sequence-end) (:junk-allowed t))
-  (values (or pathname null) index)
+  (values (or pathname null) sequence-end)
   ())
 
 (defknown merge-pathnames
@@ -1021,7 +996,8 @@
 				   :rename-and-delete :overwrite :append
 				   :supersede nil))
 		(:if-does-not-exist (member :error :create nil))
-		(:external-format (member :default)))
+		(:external-format (member :default))
+		(:class (or symbol class)))
   (or stream null))
 
 (defknown rename-file (pathnamelike filename) (values pathname pathname pathname))
@@ -1039,11 +1015,12 @@
   ((or filename stream)
    &key (:verbose t) (:print t) (:if-does-not-exist (member :error :create nil))
    (:if-source-newer (member :load-source :load-object :query :compile))
-   (:contents (or null (member :source :binary))))
+   (:contents (or null (member :source :binary)))
+   (:external-format t))
   t)
 
 (defknown directory (pathnamelike &key (:check-for-subdirs t) (:all t)
-				  (:follow-links t))
+				  (:truenamep t) (:follow-links t))
   list (flushable))
 
 
@@ -1075,19 +1052,19 @@
 
 (defknown disassemble (callable &key (:stream stream) (:backend backend)
 				(:use-labels t))
-  void)
+  (values))
 
 (defknown documentation (t symbol)
   (or string null)
   (flushable))
 
 (defknown describe (t &optional (or stream (member t nil))) (values))
-(defknown inspect (t) (values))
+(defknown inspect (t) t)
 
-(defknown room (&optional (member t nil :default)) void)
+(defknown room (&optional (member t nil :default)) (values))
 (defknown ed (&optional (or symbol cons filename) &key (:init t) (:display t))
   t)
-(defknown dribble (&optional filename &key (:if-exists t)) t)
+(defknown dribble (&optional filename &key (:if-exists t)) (values))
 
 (defknown apropos (stringable &optional packagelike t) (values))
 (defknown apropos-list (stringable &optional packagelike t) list (flushable))
