@@ -4,7 +4,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/generic/new-genesis.lisp,v 1.39 1999/09/09 16:31:08 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/generic/new-genesis.lisp,v 1.42 2001/12/06 19:15:42 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -711,6 +711,8 @@
 
     (frob *free-interrupt-context-index* (make-fixnum-descriptor 0))
 
+    (frob *batch-mode* (cold-intern nil))
+
     (frob *initial-layouts* (list-all-layouts))
 
     (let ((res *nil-descriptor*))
@@ -860,7 +862,8 @@
     (write-indexed fdefn vm:fdefn-raw-addr-slot
 		   (ecase type
 		     (#.vm:function-header-type
-		      (if (c:backend-featurep :sparc)
+		      (if (or (c:backend-featurep :sparc)
+			      (c:backend-featurep :ppc))
 			  defn
 			  (make-random-descriptor
 			   (+ (logandc2 (descriptor-bits defn) vm:lowtag-mask)
@@ -1806,13 +1809,13 @@
   (let ((linux-p (and (eq (c:backend-fasl-file-implementation c:*backend*)
 			  #.c:x86-fasl-file-implementation)
 		      (c:backend-featurep :linux)))
-	(freebsd-p (and (eq (c:backend-fasl-file-implementation c:*backend*)
-			    #.c:x86-fasl-file-implementation)
-			(c:backend-featurep :freebsd))))
+	(bsd-p (and (eq (c:backend-fasl-file-implementation c:*backend*)
+			#.c:x86-fasl-file-implementation)
+		    (c:backend-featurep :bsd))))
     
     (cond
-     ((and freebsd-p (gethash (concatenate 'string "_" name)
-			      *cold-foreign-symbol-table* nil)))
+     ((and bsd-p (gethash (concatenate 'string "_" name)
+			  *cold-foreign-symbol-table* nil)))
      ((and linux-p (gethash (concatenate 'string "PVE_stub_" name)
 			    *cold-foreign-symbol-table* nil)))
      ;; Non-linux case
@@ -1837,6 +1840,7 @@
 ;; syscalls and Unix library things. Linux doesn't or maybe does
 ;; depending on things I don't know about yet? FreeBSD version 3
 ;; is ELF based and looks more like the other systems.
+;; For the moment we assume all non-elf BSDs are the same as non-elf FreeBSD.
 ;; Maybe these x86 hacks can be fixed when the non-elf's are obsoleted.
 
 (defun lookup-maybe-prefix-foreign-symbol (name)
@@ -1848,16 +1852,17 @@
 		    #.c:rt-afpa-fasl-file-implementation
 		    #.c:hppa-fasl-file-implementation
 		    #.c:alpha-fasl-file-implementation
-		    #.c:sgi-fasl-file-implementation)
+		    #.c:sgi-fasl-file-implementation
+		    #.c:ppc-fasl-file-implementation)
 		   "")
 		  (#.c:sparc-fasl-file-implementation
 		   (if (c:backend-featurep :svr4)
 		       ""
 		       "_"))
 		  (#.c:x86-fasl-file-implementation
-		   (if (and (c:backend-featurep :freebsd)
+		   (if (and (c:backend-featurep :bsd)
 			    (not (c:backend-featurep :elf)))
-		       "_" ; older FreeBSD
+		       "_" ; older FreeBSD, OpenBSD
 		       "")) ; Linux and FreeBSD V3+
 		  )
 		name)))
@@ -2075,7 +2080,20 @@
 	 (:addi
 	  (setf (sap-ref-16 sap 2)
 		(maybe-byte-swap-short
-		 (ldb (byte 16 0) value))))))))
+		 (ldb (byte 16 0) value))))))
+      (#.c:ppc-fasl-file-implementation
+       (ecase kind
+	 (:ba
+ 	  (setf (sap-ref-32 sap 0)
+ 		(dpb (ash value -2) (byte 24 2) (sap-ref-32 sap 0))))
+ 	 (:ha
+ 	  (let* ((h (ldb (byte 16 16) value))
+ 		 (l (ldb (byte 16 0) value)))
+ 	    (setf (sap-ref-16 sap 2)
+ 		  (if (logbitp 15 l) (ldb (byte 16 0) (1+ h)) h))))
+ 	 (:l
+  	  (setf (sap-ref-16 sap 2)
+  		(ldb (byte 16 0) value)))))))
   (undefined-value))
 
 (defun linkage-info-to-core ()
