@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/generic/vm-fndb.lisp,v 1.62 1999/02/25 13:03:13 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/generic/vm-fndb.lisp,v 1.63 2000/07/09 16:08:43 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -283,23 +283,23 @@
 
 (defknown copy-to-system-area
 	  ((simple-unboxed-array (*)) index system-area-pointer index index)
-  null
+  t
   ())
 
 (defknown copy-from-system-area
 	  (system-area-pointer index (simple-unboxed-array (*)) index index)
-  null
+  t
   ())
 
 (defknown system-area-copy
 	  (system-area-pointer index system-area-pointer index index)
-  null
+  t
   ())
 
 (defknown bit-bash-copy
 	  ((simple-unboxed-array (*)) index
 	   (simple-unboxed-array (*)) index index)
-  null
+  t
   ())
 
 
