@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/generic/vm-tran.lisp,v 1.39 2000/01/14 19:48:43 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/generic/vm-tran.lisp,v 1.43 2001/06/05 13:58:59 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -41,7 +41,7 @@
   `(%instance-set ,x 0 (the layout ,val)))
 
 
-;;;; Charater support.
+;;;; Character support.
 
 ;;; There are really only base-chars.
 ;;;
@@ -75,6 +75,31 @@
 					 (%array-data-vector array))
 			      index))))))
 
+(deftransform data-vector-ref ((array index) (array t) *
+			       :node node  :policy (> speed space))
+  (let ((array-type (continuation-type array)))
+    (unless (and (array-type-p array-type) (array-type-complexp array-type)
+		 (not (eq (array-type-specialized-element-type array-type)
+			  *wild-type*)))
+      (give-up))
+    (delay-transform node :optimize)
+    (let* ((dims (array-type-dimensions array-type))
+	   (el-type (array-type-element-type array-type))
+	   (total-size (if (or (atom dims) (member '* dims))
+			   '*
+			   (reduce #'* dims)))
+	   (vector-type `(simple-array ,(type-specifier el-type)
+				       (,total-size))))
+      (if (and (consp dims) (> (length dims) 1))
+	  `(multiple-value-bind (vector index)
+	       (%with-array-data array index nil)
+	     (data-vector-ref (truly-the ,vector-type vector) index))
+	  `(multiple-value-bind (vector index)
+	       (if (array-header-p array)
+		   (%with-array-data array index nil)
+		   (values array index))
+	     (data-vector-ref (truly-the ,vector-type vector) index))))))
+
 (deftransform data-vector-set ((array index new-value)
 			       (simple-array t t))
   (let ((array-type (continuation-type array)))
@@ -101,6 +126,35 @@
 			      index
 			      new-value))))))
 
+(deftransform data-vector-set ((array index new-value) (array t t) *
+			       :node node  :policy (> speed space))
+  (let ((array-type (continuation-type array)))
+    (unless (and (array-type-p array-type) (array-type-complexp array-type)
+		 (not (eq (array-type-specialized-element-type array-type)
+			  *wild-type*)))
+      (give-up))
+    (delay-transform node :optimize)
+    (let* ((dims (array-type-dimensions array-type))
+	   (el-type (array-type-element-type array-type))
+	   (total-size (if (or (atom dims) (member '* dims))
+			   '*
+			   (reduce #'* dims)))
+	   (vector-type `(simple-array ,(type-specifier el-type)
+				       (,total-size))))
+      (if (and (consp dims) (> (length dims) 1))
+	  `(multiple-value-bind (vector index)
+	       (%with-array-data array index nil)
+	     (data-vector-set (truly-the ,vector-type vector)
+			      index
+			      new-value))
+	  `(multiple-value-bind (vector index)
+	       (if (array-header-p array)
+		   (%with-array-data array index nil)
+		   (values array index))
+	     (data-vector-set (truly-the ,vector-type vector)
+			      index
+			      new-value))))))
+
 
 ;;; Transforms for getting at arrays of unsigned-byte n when n < 8.
 
@@ -140,9 +194,11 @@
 
 (deftransform subseq ((string start &optional (end nil))
 		      (simple-string t &optional t))
-  '(let* ((length (- (or end (length string))
-		     start))
-	  (result (make-string length)))
+  '(let* ((length (length string))
+	  (end (if end (min end length) length))
+	  (start (min start end))
+	  (size (- end start))
+	  (result (make-string size)))
      (declare (optimize (safety 0)))
      (bit-bash-copy string
 		    (the index
@@ -150,7 +206,7 @@
 			    vector-data-bit-offset))
 		    result
 		    vector-data-bit-offset
-		    (the index (* length vm:byte-bits)))
+		    (the index (* size vm:byte-bits)))
      result))
 
 
@@ -186,6 +242,11 @@
 			    vm:byte-bits)))
      string1))
 
+;; The original version of this deftransform seemed to cause the
+;; compiler to spend huge amounts of time deriving the type of the
+;; START variable.  The following version uses nested lets to prevent
+;; the compiler from doing this analysis.  This only hides the
+;; symptom.
 
 (deftransform concatenate ((rtype &rest sequences)
 			   (t &rest simple-string)
@@ -201,19 +262,32 @@
 	(args n-seq)
 	(lets `(,n-length (the index (* (length ,n-seq) vm:byte-bits))))
 	(all-lengths n-length)
-	(forms `(bit-bash-copy ,n-seq vector-data-bit-offset
-			       res start
-			       ,n-length))
-	(forms `(setq start (+ start ,n-length)))))
-    `(lambda (rtype ,@(args))
-       (declare (ignore rtype))
-       (let* (,@(lets)
-	      (res (make-string (truncate (the index (+ ,@(all-lengths)))
-					  vm:byte-bits)))
-	      (start vector-data-bit-offset))
-	 (declare (type index start ,@(all-lengths)))
-	 ,@(forms)
-	 res))))
+	(forms `((bit-bash-copy ,n-seq vector-data-bit-offset
+		  res start
+		  ,n-length)
+		 (start (+ start ,n-length))))))
+    (flet ((nestify (lists)
+	     (let* ((lists (reverse lists))
+		    (result `(,(caar lists))))
+	       (dolist (item (rest lists))
+		 (destructuring-bind (bit-bash init)
+		     item
+		   (setf result `(,bit-bash
+				  (let (,init)
+				    ,@result)))))
+	       result)))
+      (let ((result 
+	     `(lambda (rtype ,@(args))
+	       (declare (ignore rtype))
+	       (let* (,@(lets)
+			(res (make-string (truncate (the index (+ ,@(all-lengths)))
+						    vm:byte-bits))))
+		 (declare (type index ,@(all-lengths)))
+		 (let ((start vector-data-bit-offset))
+		   ,@(nestify (forms)))
+		 res))))
+	result))))
+
 
 
 ;;;; Bit vector hackery:
