@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/generic/vm-type.lisp,v 1.38 2000/01/10 14:47:45 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/generic/vm-type.lisp,v 1.41 2000/07/07 09:33:11 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -119,8 +119,8 @@
 ;;;
 ;;;    Return the symbol that describes the format of Float.
 ;;;
-(proclaim '(function float-format-name (float) symbol))
 (defun float-format-name (x)
+  (declare (float x) (values symbol))
   (etypecase x
     (single-float 'single-float)
     (double-float 'double-float)
@@ -132,8 +132,8 @@
 ;;; array will actually be implemented.  We set the Specialized-Element-Type to
 ;;; correspond to the actual specialization used in this implementation.
 ;;;
-(proclaim '(function specialize-array-type (array-type) array-type))
 (defun specialize-array-type (type)
+  (declare (type array-type type) (values array-type))
   (let ((eltype (array-type-element-type type)))
 
     (setf (array-type-specialized-element-type type)
@@ -172,11 +172,6 @@
 (defun hairy-type-check-template (type)
   (declare (type ctype type))
   (typecase type
-    (named-type
-     (case (named-type-name type)
-       (cons 'c:check-cons)
-       (symbol 'c:check-symbol)
-       (t nil)))
     (numeric-type
      (cond ((type= type (specifier-type 'fixnum))
 	    'c:check-fixnum)
@@ -189,6 +184,13 @@
      (if (type= type (specifier-type 'bignum))
 	 'c:check-bignum
 	 nil))
+    (cons-type
+     (if (type= type (specifier-type 'cons))
+	  'c:check-cons
+	   nil))
+    (built-in-class
+     (case (class-name type)
+       (symbol 'c:check-symbol)))
     (function-type
      'c:check-function)
     (t
