@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/globaldb.lisp,v 1.36 1998/03/01 21:55:43 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/globaldb.lisp,v 1.41 2001/03/13 16:52:16 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -39,8 +39,8 @@
 (in-package "C")
 
 ;;; The defvar for this appears later.
-(proclaim '(special *universal-type*))
-(proclaim '(type list type-specifier-symbols))
+(declaim (special *universal-type*))
+(declaim (type list type-specifier-symbols))
 
 
 ;;; PRIMIFY  --  Internal
@@ -113,7 +113,7 @@
 ;;; GLOBALDB-INIT...
 ;;;
 (defvar *info-classes* (make-hash-table :test #'equal))
-(proclaim '(hash-table *info-classes*))
+(declaim (hash-table *info-classes*))
 
 
 ;;; FIND-TYPE-INFO  --  Internal
@@ -132,13 +132,13 @@
 ;;;
 ;;;    Return the info structure for an info class or type, or die trying.
 ;;;
-(proclaim '(function class-info-or-lose (string) class-info))
 (defun class-info-or-lose (class)
+  (declare (string class) (values class-info))
   (or (gethash class *info-classes*)
       (error "~S is not a defined info class." class)))
 ;;;
-(proclaim '(function type-info-or-lose (string string) type-info))
 (defun type-info-or-lose (class type)
+  (declare (string class type) (values type-info))
   (or (find-type-info type (class-info-or-lose class))
       (error "~S is not a defined info type." type)))
 
@@ -163,8 +163,8 @@
 ;;;
 ;;;    If there is no info for the class, then create it, otherwise do nothing.
 ;;;
-(proclaim '(function %define-info-class (string) void))
 (defun %define-info-class (class)
+  (declare (string class))
   (unless (gethash class *info-classes*)
     (setf (gethash class *info-classes*) (make-class-info class))))
 
@@ -252,7 +252,7 @@
 ;;; initialized in GLOBALDB-INIT.
 
 (defvar *info-environment*)
-(proclaim '(type list *info-environment*))
+(declaim (type list *info-environment*))
 
 
 (defun %print-info-environment (s stream d)
@@ -285,7 +285,7 @@
 ;;; does, but we get there more expeditiously.  With the two-list, we LOGXOR a
 ;;; random constant with the hash of the second symbol.
 ;;;
-(proclaim '(inline info-hash))
+(declaim (inline info-hash))
 (defun info-hash (x)
   (cond
    ((symbolp x)
@@ -323,11 +323,11 @@
   (let* ((class (symbol-name class))
 	 (type (symbol-name type))
 	 (info (type-info-or-lose class type)))
-    `(truly-the ,(type-info-type info)
+    `(truly-the (values ,(type-info-type info) t)
 		(get-info-value ,name ,(type-info-number info)
 				,@(when env-list `(,env-list))))))
 ;;;
-(define-setf-method info (class type name &optional env-list)
+(define-setf-expander info (class type name &optional env-list)
   "Set the global information for Name."
   (let* ((n-name (gensym))
 	 (n-env-list (if env-list (gensym)))
@@ -504,7 +504,7 @@
 ;;;
 ;;;    If the info cache is invalid, then clear it.
 ;;;
-(proclaim '(inline clear-invalid-info-cache))
+(declaim (inline clear-invalid-info-cache))
 (defun clear-invalid-info-cache ()
   (unless (eq *info-environment* *cached-info-environment*)
     (without-interrupts
@@ -573,7 +573,7 @@
 ;;;    Return the value of the type corresponding to Number for the currently
 ;;; cached name in Env.
 ;;;
-(proclaim '(inline compact-info-cache-hit))
+(declaim (inline compact-info-cache-hit))
 (defun compact-info-cache-hit (env number)
   (declare (type compact-info-env env) (type type-number number))
   (let ((entries-info (compact-info-env-entries-info env))
@@ -753,7 +753,7 @@
 ;;;
 ;;;    Just like COMPACT-INFO-CACHE-HIT, only do it on a volatile environment.
 ;;;
-(proclaim '(inline volatile-info-cache-hit))
+(declaim (inline volatile-info-cache-hit))
 (defun volatile-info-cache-hit (env number)
   (declare (type volatile-info-env env) (type type-number number))
   (dolist (type (volatile-info-env-cache-types env) (values nil nil))
@@ -804,7 +804,7 @@
 ;;; This is always the first environment in the list, and must be a
 ;;; VOLATILE-INFO-ENV.
 ;;;
-(proclaim '(inline get-write-info-env))
+(declaim (inline get-write-info-env))
 (defun get-write-info-env (&optional (env-list *info-environment*))
   (let ((env (car env-list)))
     (unless env
@@ -1070,7 +1070,8 @@
 (define-info-class variable)
 
 ;;; The kind of variable-like thing described.
-(define-info-type variable kind (member :special :constant :global :alien)
+(define-info-type variable kind (member :special :constant :global :alien
+					:macro)
   (if (or (eq (symbol-package name) (symbol-package :end))
 	  (member name '(t nil)))
       :constant
@@ -1091,6 +1092,8 @@
 
 (define-info-type variable alien-info (or heap-alien-info null) nil)
 
+(define-info-type variable macro-expansion t nil)
+
 (define-info-type variable documentation (or string null) nil)
 
 (define-info-class type)
