@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/hppa/type-vops.lisp,v 1.6 1998/07/24 17:22:34 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/hppa/type-vops.lisp,v 1.7 2000/01/10 14:45:53 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;; 
@@ -230,7 +230,7 @@
 (def-type-vops %instancep check-instance instance object-not-instance-error
   instance-pointer-type)
 
-(def-type-vops bignump check-bigunm bignum
+(def-type-vops bignump check-bignum bignum
   object-not-bignum-error bignum-type)
 
 (def-type-vops ratiop check-ratio ratio
