@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/ir1opt.lisp,v 1.67 1999/02/25 13:03:05 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/ir1opt.lisp,v 1.74 2001/03/04 20:12:17 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -26,8 +26,8 @@
 ;;;
 ;;;    Return true if the sole use of Cont is a reference to a constant leaf.
 ;;;
-(proclaim '(function constant-continuation-p (continuation) boolean))
 (defun constant-continuation-p (cont)
+  (declare (type continuation cont) (values boolean))
   (let ((use (continuation-use cont)))
     (and (ref-p use)
 	 (constant-p (ref-leaf use)))))
@@ -38,8 +38,8 @@
 ;;;    Return the constant value for a continuation whose only use is a
 ;;; constant node.
 ;;;
-(proclaim '(function continuation-value (continuation) t))
 (defun continuation-value (cont)
+  (declare (type continuation cont))
   (assert (constant-continuation-p cont))
   (constant-value (ref-leaf (continuation-use cont))))
 
@@ -88,7 +88,7 @@
 ;;; slot is true, just return that value, otherwise recompute and stash the
 ;;; value there.
 ;;;
-(proclaim '(inline continuation-derived-type))
+(declaim (inline continuation-derived-type))
 (defun continuation-derived-type (cont)
   (declare (type continuation cont))
   (or (continuation-%derived-type cont)
@@ -116,7 +116,7 @@
 ;;;    Call CONTINUATION-DERIVED-TYPE to make sure the slot is up to date, then
 ;;; return it.
 ;;;
-(proclaim '(inline continuation-type-check))
+(declaim (inline continuation-type-check))
 (defun continuation-type-check (cont)
   (declare (type continuation cont))
   (continuation-derived-type cont)
@@ -128,8 +128,8 @@
 ;;;    Return the derived type for Cont's first value.  This is guaranteed not
 ;;; to be a Values or Function type.
 ;;;
-(proclaim '(function continuation-type (continuation) ctype))
 (defun continuation-type (cont)
+  (declare (type continuation cont) (values ctype))
   (single-value-type (continuation-derived-type cont)))
 
 
@@ -200,7 +200,8 @@
 	  (reoptimize-continuation (node-cont node))))))
   (undefined-value))
 
-(declaim (start-block assert-continuation-type assert-call-type))
+(declaim (start-block assert-continuation-type
+		      assert-continuation-optional-type assert-call-type))
 
 ;;; Assert-Continuation-Type  --  Interface
 ;;;
@@ -224,6 +225,18 @@
   (undefined-value))
 
 
+;;; Assert-continuation-optional-type  --  Interface
+;;;
+;;;    Similar to Assert-Continuation-Type, but asserts that the type is
+;;; for an optional argument and that other arguments may be received.
+;;;
+(defun assert-continuation-optional-type (cont type)
+  (declare (type continuation cont) (type ctype type))
+  (let ((opt-type (make-values-type :optional (list type)
+				    :rest *universal-type*)))
+    (assert-continuation-type cont opt-type)))
+
+
 ;;; Assert-Call-Type  --  Interface
 ;;;
 ;;;    Assert that Call is to a function of the specified Type.  It is assumed
@@ -236,23 +249,23 @@
     (dolist (req (function-type-required type))
       (when (null args) (return-from assert-call-type))
       (let ((arg (pop args)))
-	(assert-continuation-type arg req)))
+	(assert-continuation-optional-type arg req)))
     (dolist (opt (function-type-optional type))
       (when (null args) (return-from assert-call-type))
       (let ((arg (pop args)))
-	(assert-continuation-type arg opt)))
+	(assert-continuation-optional-type arg opt)))
 
     (let ((rest (function-type-rest type)))
       (when rest
 	(dolist (arg args)
-	  (assert-continuation-type arg rest))))
+	  (assert-continuation-optional-type arg rest))))
 
     (dolist (key (function-type-keywords type))
       (let ((name (key-info-name key)))
 	(do ((arg args (cddr arg)))
 	    ((null arg))
 	  (when (eq (continuation-value (first arg)) name)
-	    (assert-continuation-type
+	    (assert-continuation-optional-type
 	     (second arg) (key-info-type key)))))))
   (undefined-value))
 
@@ -538,9 +551,8 @@
 		   (return-from find-result-type (undefined-value)))))
 	      (t
 	       (use-union (node-derived-type use)))))
-      (let ((int (values-type-intersection
-		  (continuation-asserted-type result)
-		  (use-union))))
+      (let ((int (values-type-intersection (continuation-asserted-type result)
+					   (use-union))))
 	(setf (return-result-type node) int))))
   (undefined-value))
 
@@ -725,8 +737,8 @@
 ;;;
 ;;;    Do IR1 optimizations on a Combination node.
 ;;;
-(proclaim '(function ir1-optimize-combination (combination) void))
 (defun ir1-optimize-combination (node)
+  (declare (type combination node))
   (when (continuation-reoptimize (basic-combination-fun node))
     (propagate-function-change node))
   (let ((args (basic-combination-args node))
@@ -1038,7 +1050,10 @@
 		      (record-optimization-failure node transform args))
 		    (setf (gethash node table)
 			  (remove transform (gethash node table) :key #'car)))
-		t))))
+		t)
+	       (:delayed
+		(remhash node table)
+		nil))))
 	  ((and flame
 		(valid-function-use node type
 				    :argument-test #'types-intersect
@@ -1050,16 +1065,16 @@
 
 (declaim (end-block))
 
-;;; GIVE-UP, ABORT-TRANSFORM  --  Interface
+;;; give-up, abort-transform  --  Interface
 ;;;
 ;;;    Just throw the severity and args...
 ;;;
-(proclaim '(function give-up (&rest t) nil))
 (defun give-up (&rest args)
   "This function is used to throw out of an IR1 transform, aborting this
   attempt to transform the call, but admitting the possibility that this or
   some other transform will later suceed.  If arguments are supplied, they are
   format arguments for an efficiency note."
+  (values nil)
   (throw 'give-up (values :failure args)))
 ;;;
 (defun abort-transform (&rest args)
@@ -1068,6 +1083,50 @@
   attempted."
   (throw 'give-up (values :aborted args)))
 
+(defvar *delayed-transforms*)
+
+;;; delay-transform  --  Interface
+;;;
+(defun delay-transform (node &rest reasons)
+  "This function is used to throw out of an IR1 transform, and delay the
+  transform on the node until later. The reasons specifies when the transform
+  will be later retried. The :optimize reason causes the transform to be
+  delayed until after the current IR1 optimization pass. The :constraint
+  reason causes the transform to be delayed until after constraint
+  propagation."
+  (let ((assoc (assoc node *delayed-transforms*)))
+    (cond ((not assoc)
+	   (setf *delayed-transforms*
+		 (acons node reasons *delayed-transforms*))
+	   (throw 'give-up :delayed))
+	  ((cdr assoc)
+	   (dolist (reason reasons)
+	     (pushnew reason (cdr assoc)))
+	   (throw 'give-up :delayed)))))
+
+;;; retry-delayed-transforms  --  Interface.
+;;;
+;;; Clear any delayed transform with no reasons - these should have been tried
+;;; in the last pass. Then remove the reason from the delayed transform
+;;; reasons, and if any become empty then set reoptimize flags for the
+;;; node. Returns true if any transforms are to be retried.
+;;;
+(defun retry-delayed-transforms (reason)
+  (setf *delayed-transforms* (remove-if-not #'cdr *delayed-transforms*))
+  (let ((reoptimize nil))
+    (dolist (assoc *delayed-transforms*)
+      (let ((reasons (remove reason (cdr assoc))))
+	(setf (cdr assoc) reasons)
+	(unless reasons
+	  (let ((node (car assoc)))
+	    (unless (node-deleted node)
+	      (setf reoptimize t)
+	      (setf (node-reoptimize node) t)
+	      (let ((block (node-block node)))
+		(setf (block-reoptimize block) t)
+		(setf (component-reoptimize (block-component block)) t)))))))
+    reoptimize))
+
 
 ;;; Transform-Call  --  Internal
 ;;;
@@ -1236,6 +1295,8 @@
 ;;; -- either continuation has a funky TYPE-CHECK annotation.
 ;;; -- the continuations have incompatible assertions, so the new asserted type
 ;;;    would be NIL.
+;;; -- CONT's assertion is incompatbile with the proven type of ARG's, such as
+;;;    when ARG returns multiple values and CONT has a single value assertion.
 ;;; -- the var's DEST has a different policy than the ARG's (think safety).
 ;;;
 ;;;    We change the Ref to be a reference to NIL with unused value, and let it
@@ -1256,8 +1317,10 @@
 	       (member (continuation-type-check arg) '(t nil))
 	       (member (continuation-type-check cont) '(t nil))
 	       (not (eq (values-type-intersection
-			 cont-atype
-			 (continuation-asserted-type arg))
+			 cont-atype (continuation-asserted-type arg))
+			*empty-type*))
+	       (not (eq (values-type-intersection
+			 cont-atype (continuation-proven-type arg))
 			*empty-type*))
 	       (eq (lexenv-cookie (node-lexenv dest))
 		   (lexenv-cookie (node-lexenv (continuation-dest arg)))))
@@ -1347,7 +1410,7 @@
 	     (substitute-single-use-continuation arg var)))
        (t
 	(propagate-to-refs var (continuation-type arg))))))
-  
+
   (when (every #'null (combination-args call))
     (delete-let fun))
 
@@ -1460,6 +1523,42 @@
   (undefined-value))
 
   
+;;; Values-types-defaulted  --  Internal
+;;;
+;;;    Like values-types, but returns the types of the given number of
+;;; arguments. If optional of rest values must be used then the union
+;;; with the null type is computed in case of defaulting, and if no
+;;; values are available then they are defaulted to the null type.
+;;;
+(defun values-types-defaulted (type count)
+  (declare (type ctype type) (type index count))
+  (cond ((eq type *wild-type*)
+	 (let ((types nil))
+	   (dotimes (i count types)
+	     (push *universal-type* types))))
+	((not (values-type-p type))
+	 (let ((types nil))
+	   (dotimes (i (1- count))
+	     (push *null-type* types))
+	   (push type types)))
+	(t
+	 (let ((required (args-type-required type))
+	       (optional (args-type-optional type))
+	       (keyp-allowp (or (args-type-keyp type) (args-type-allowp type)))
+	       (rest (args-type-rest type)))
+	   (collect ((types))
+	     (dotimes (i count)
+	       (types (cond (required (single-value-type (pop required)))
+			    (optional (values-type-union
+				       (single-value-type (pop optional))
+				       *null-type*))
+			    (keyp-allowp *universal-type*)
+			    (rest (values-type-union (single-value-type rest)
+						     *null-type*))
+			    (t *null-type*))))
+	     (types))))))
+
+
 ;;; IR1-OPTIMIZE-MV-BIND  --  Internal
 ;;;
 ;;;    Propagate derived type info from the values continuation to the vars.
@@ -1468,17 +1567,13 @@
   (declare (type mv-combination node))
   (let ((arg (first (basic-combination-args node)))
 	(vars (lambda-vars (combination-lambda node))))
-    (multiple-value-bind (types nvals)
-			 (values-types (continuation-derived-type arg))
-      (unless (eq nvals :unknown)
-	(mapc #'(lambda (var type)
-		  (if (basic-var-sets var)
-		      (propagate-from-sets var type)
-		      (propagate-to-refs var type)))
-		vars
-		(append types
-			(make-list (max (- (length vars) nvals) 0)
-				   :initial-element *null-type*)))))
+    (let ((types (values-types-defaulted (continuation-derived-type arg)
+					 (length vars))))
+      (mapc #'(lambda (var type)
+		(if (basic-var-sets var)
+		    (propagate-from-sets var type)
+		    (propagate-to-refs var type)))
+	    vars types))
 
     (setf (continuation-reoptimize arg) nil))
   (undefined-value))
@@ -1659,17 +1754,48 @@
 ;;;
 ;;;    If VALUES appears in a non-MV context, then effectively convert it to a
 ;;; PROG1.  This allows the computation of the additional values to become dead
-;;; code.
+;;; code.  Some attempt is made to correct the node derived type, setting it to
+;;; the received single-value-type. The node continuation asserted type must
+;;; also be adjusted, taking care when the continuation has multiple uses.
 ;;;
 (deftransform values ((&rest vals) * * :node node)
-  (when (typep (continuation-dest (node-cont node))
-	       '(or creturn exit mv-combination))
-    (give-up))
-  (setf (node-derived-type node) *wild-type*)
-  (if vals
-      (let ((dummies (loop repeat (1- (length vals))
-		       collect (gensym))))
-	`(lambda (val ,@dummies)
-	   (declare (ignore ,@dummies))
-	   val))
-      'nil))
+  (let ((cont (node-cont node)))
+    (when (typep (continuation-dest cont) '(or creturn exit mv-combination))
+      (give-up))
+    (flet ((first-value-type (type)
+	     (declare (type ctype type))
+	     (cond ((values-type-p type)
+		    (let ((required (args-type-required type)))
+		      (if required
+			  (first required)
+			  (let ((otype (args-type-optional type)))
+			    (cond (otype (first otype))
+				  ((or (args-type-keyp type)
+				       (args-type-allowp type))
+				   *universal-type*)
+				  ((args-type-rest type))
+				  (t *null-type*))))))
+		   ((eq type *wild-type*)
+		    *universal-type*)
+		   (t
+		    type))))
+      (cond ((= (length (find-uses cont)) 1)
+	     (setf (node-derived-type node)
+		   (single-value-type (node-derived-type node)))
+	     (setf (continuation-asserted-type cont)
+		   (first-value-type (continuation-asserted-type cont))))
+	    (t
+	     (setf (node-derived-type node)
+		   (single-value-type (node-derived-type node)))
+	     (setf (continuation-asserted-type cont)
+		   (values-type-union (continuation-asserted-type cont)
+				      (first-value-type
+				       (continuation-asserted-type cont)))))))
+    (reoptimize-continuation cont)
+    (if vals
+	(let ((dummies (loop repeat (1- (length vals))
+			     collect (gensym))))
+	  `(lambda (val ,@dummies)
+	     (declare (ignore ,@dummies))
+	     val))
+	'nil)))
