@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/ir1util.lisp,v 1.78 1998/02/24 19:19:00 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/ir1util.lisp,v 1.83 2001/06/17 19:12:36 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -76,8 +76,8 @@
 ;;;
 ;;;    Return a list of all the nodes which use Cont.
 ;;;
-(proclaim '(function find-uses (continuation) list))
 (defun find-uses (cont)
+  (declare (type continuation cont) (values list))
   (ecase (continuation-kind cont)
     ((:block-start :deleted-block-start)
      (block-start-uses (continuation-block cont)))
@@ -85,7 +85,7 @@
     (:unused nil)
     (:deleted nil)))
 
-      
+
 ;;; Delete-Continuation-Use  --  Interface
 ;;;
 ;;;    Update continuation use information so that Node is no longer a use of
@@ -96,8 +96,8 @@
 ;;; REOPTIMIZE-CONTINUATION to inform IR1 optimization that something has
 ;;; changed.
 ;;;
-(proclaim '(function delete-continuation-use (node) void))
 (defun delete-continuation-use (node)
+  (declare (type node node))
   (let* ((cont (node-cont node))
 	 (block (continuation-block cont)))
     (ecase (continuation-kind cont)
@@ -124,8 +124,8 @@
 ;;; REOPTIMIZE-CONTINUATION to inform IR1 optimization that something has
 ;;; changed.
 ;;;
-(proclaim '(function add-continuation-use (node continuation) void))
 (defun add-continuation-use (node cont)
+  (declare (type node node) (type continuation cont))
   (assert (not (node-cont node)))
   (let ((block (continuation-block cont)))
     (ecase (continuation-kind cont)
@@ -201,11 +201,14 @@
   (unless (and (eq (continuation-kind new) :unused)
 	       (eq (continuation-kind old) :inside-block))
     (ensure-block-start new))
-  
+
   (do-uses (node old)
     (delete-continuation-use node)
     (add-continuation-use node new))
 
+  (dolist (cont-ref (continuation-refs old))
+    (setf (cont-ref-cont cont-ref) new))
+
   (reoptimize-continuation new)
   (undefined-value))
 
@@ -536,9 +539,9 @@
 ;;;    Unlink a block from the next/prev chain.  We also null out the
 ;;; Component.
 ;;;
-(proclaim '(function remove-from-dfo (cblock) void))
 (declaim (inline remove-from-dfo))
 (defun remove-from-dfo (block)
+  (declare (type cblock block))
   (let ((next (block-next block))
 	(prev (block-prev block)))
     (setf (block-component block) nil)
@@ -569,8 +572,8 @@
 ;;;    Set the Flag for all the blocks in Component to NIL, except for the head
 ;;; and tail which are set to T.
 ;;;
-(proclaim '(function clear-flags (component) void))
 (defun clear-flags (component)
+  (declare (type component component))
   (let ((head (component-head component))
 	(tail (component-tail component)))
     (setf (block-flag head) t)
@@ -584,8 +587,8 @@
 ;;;    Make a component with no blocks in it.  The Block-Flag is initially true
 ;;; in the head and tail blocks.
 ;;;
-(proclaim '(function make-empty-component () component))
 (defun make-empty-component ()
+  (declare (values component))
   (let* ((head (make-block-key :start nil :component nil))
 	 (tail (make-block-key :start nil :component nil))
 	 (res (make-component :head head  :tail tail)))
@@ -1067,10 +1070,8 @@
     (unless (or (leaf-ever-used var)
 		(lambda-var-ignorep var))
       (let ((*compiler-error-context* (lambda-bind fun)))
-	(unless (policy *compiler-error-context* (= brevity 3))
-	  (compiler-warning "Variable ~S defined but never used."
-			    (leaf-name var)))
-	(setf (leaf-ever-used var) t))))
+	(compiler-note "Variable ~S defined but never used." (leaf-name var)))
+      (setf (leaf-ever-used var) t)))
   (undefined-value))
 
 
@@ -1393,8 +1394,8 @@
 ;;;    If Functional is a Lambda, just return it; if it is an
 ;;; optional-dispatch, return the main-entry.
 ;;;
-(proclaim '(function main-entry (functional) clambda))
 (defun main-entry (functional)
+  (declare (type functional functional) (values clambda))
   (etypecase functional
     (clambda functional)
     (optional-dispatch
@@ -1407,8 +1408,8 @@
 ;;; null default and no supplied-p.  There must be a rest arg with no
 ;;; references.
 ;;;
-(proclaim '(function looks-like-an-mv-bind (functional) boolean))
 (defun looks-like-an-mv-bind (functional)
+  (declare (type functional functional) (values boolean))
   (and (optional-dispatch-p functional)
        (do ((arg (optional-dispatch-arglist functional) (cdr arg)))
 	   ((null arg) nil)
@@ -1514,14 +1515,14 @@
 
 ;;;; Compiler error context determination:
 
-(proclaim '(special *current-path*))
+(declaim (special *current-path*))
 
 
 ;;; We bind print level and length when printing out messages so that we don't
 ;;; dump huge amounts of garbage.
 ;;;
-(proclaim '(type (or unsigned-byte null) *error-print-level*
-		 *error-print-length* *error-print-lines*))
+(declaim (type (or unsigned-byte null) *error-print-level*
+	       *error-print-length* *error-print-lines*))
 
 (defvar *error-print-level* 3
   "The value for *Print-Level* when printing compiler error messages.")
@@ -1534,7 +1535,7 @@
   "The maximum number of enclosing non-original source forms (i.e. from
   macroexpansion) that we print in full.  For additional enclosing forms, we
   print only the CAR.")
-(proclaim '(type unsigned-byte *enclosing-source-cutoff*))
+(declaim (type unsigned-byte *enclosing-source-cutoff*))
 
 
 ;;; We separate the determination of compiler error contexts from the actual
@@ -1579,7 +1580,7 @@
 ;;; If true, this is the node which is used as context in compiler warning
 ;;; messages.
 ;;;
-(proclaim '(type (or null compiler-error-context node)
+(declaim (type (or null compiler-error-context node)
 		 *compiler-error-context*))
 (defvar *compiler-error-context* nil)
 
@@ -1744,14 +1745,14 @@
 
 ;;; A function that is called to unwind out of Compiler-Error.
 ;;;
-(proclaim '(type (function () nil) *compiler-error-bailout*))
+(declaim (type (function () nil) *compiler-error-bailout*))
 (defvar *compiler-error-bailout*
   #'(lambda () (error "Compiler-Error with no bailout.")))
 
 ;;; The stream that compiler error output is directed to.
 ;;;
 (defvar *compiler-error-output* (make-synonym-stream '*error-output*))
-(proclaim '(type stream *compiler-error-output*))
+(declaim (type stream *compiler-error-output*))
 
 ;;; We save the context information that we printed out most recently so that
 ;;; we don't print it out redundantly.
@@ -1759,19 +1760,19 @@
 ;;; The last COMPILER-ERROR-CONTEXT that we printed.
 ;;;
 (defvar *last-error-context* nil)
-(proclaim '(type (or compiler-error-context null) *last-error-context*))
+(declaim (type (or compiler-error-context null) *last-error-context*))
 
 ;;; The format string and args for the last error we printed.
 ;;;
 (defvar *last-format-string* nil)
 (defvar *last-format-args* nil)
-(proclaim '(type (or string null) *last-format-string*))
-(proclaim '(type list *last-format-args*))
+(declaim (type (or string null) *last-format-string*))
+(declaim (type list *last-format-args*))
 
 ;;; The number of times that the last error message has been emitted, so that
 ;;; we can compress duplicate error messages.
 (defvar *last-message-count* 0)
-(proclaim '(type index *last-message-count*))
+(declaim (type index *last-message-count*))
 
 (defvar *compiler-notification-function* nil
   "This is the function called by the compiler to specially note a warning,
@@ -1915,8 +1916,8 @@
 ;;; needs to be reprinted, and we also Force-Output so that the message gets
 ;;; seen right away.
 ;;;
-(proclaim '(function compiler-mumble (string &rest t) void))
 (defun compiler-mumble (format-string &rest format-args)
+  (declare (string format-string))
   (note-message-repeats)
   (setq *last-error-context* nil)
   (apply #'format *compiler-error-output* format-string format-args)
@@ -1929,8 +1930,8 @@
 ;;; source path for the bind node for an arbitrary entry point to find the
 ;;; source context, then return that as a string.
 ;;;
-(proclaim  '(function find-component-name (component) simple-string))
 (defun find-component-name (component)
+  (declare (type component component) (values simple-string))
   (let ((ep (first (block-succ (component-head component)))))
     (assert ep () "No entry points?")
     (multiple-value-bind
@@ -1947,8 +1948,8 @@
 
 ;;; Keep track of how many times each kind of warning happens.
 ;;;
-(proclaim '(type index *compiler-error-count* *compiler-warning-count*
-		 *compiler-note-count*))
+(declaim (type index *compiler-error-count* *compiler-warning-count*
+	       *compiler-note-count*))
 (defvar *compiler-error-count* 0)
 (defvar *compiler-warning-count* 0)
 (defvar *compiler-note-count* 0)
@@ -1957,7 +1958,7 @@
 ;;; between internal and user errors.  Non-compiler errors put us in the
 ;;; debugger.
 ;;;
-(define-condition compiler-error (simple-error) ())
+(define-condition compiler-error (simple-program-error) ())
 
 ;;; COMPILER-{ERROR,WARNING,STYLE-WARNING}-HANDLER  --  Interface
 ;;;
@@ -1992,10 +1993,8 @@
 ;;; Compiler-Error-Message returns like Compiler-Warning, but signals a
 ;;; Compiler-Error.
 ;;;
-(proclaim '(ftype (function (string &rest t) void)
-		  compiler-error compiler-warning compiler-note))
-;;;
 (defun compiler-error (format-string &rest format-args)
+  (declare (string format-string))
   (cerror "replace form with call to ERROR."
 	  'compiler-error :format-control format-string
 	  :format-arguments format-args)
@@ -2007,9 +2006,11 @@
 	  :format-arguments format-args))
 ;;;
 (defun compiler-warning (format-string &rest format-args)
+  (declare (string format-string))
   (apply #'warn format-string format-args))
 ;;;
 (defun compiler-note (format-string &rest format-args)
+  (declare (string format-string))
   (unless (if *compiler-error-context*
 	      (policy *compiler-error-context* (= brevity 3))
 	      (policy nil (= brevity 3)))
@@ -2064,9 +2065,9 @@
 ;;; value to indicate this.  Node is used as the error context for any error
 ;;; message, and Context is a string that is spliced into the warning.
 ;;;
-(proclaim '(function careful-call ((or symbol function) list node string)
-		     (values list boolean)))
 (defun careful-call (function args node context)
+  (declare (type (or symbol function) function) (list args) (type node node)
+	   (string context) (values list boolean))
   (values
    (multiple-value-list
     (handler-case (apply function args)
@@ -2087,8 +2088,8 @@
 ;;; or NIL otherwise.  The legality and constantness of the keywords should
 ;;; already have been checked. 
 ;;;
-(proclaim '(function find-keyword-continuation (list keyword) (or continuation null)))
 (defun find-keyword-continuation (args key)
+  (declare (list args) (type keyword key))
   (do ((arg args (cddr arg)))
       ((null arg) nil)
     (when (eq (continuation-value (first arg)) key)
@@ -2101,8 +2102,8 @@
 ;;; alternating continuations in Args are constant and that there is an even
 ;;; number of args.
 ;;;
-(proclaim '(function check-keywords-constant (list) boolean))
 (defun check-keywords-constant (args)
+  (declare (list args) (values boolean))
   (do ((arg args (cddr arg)))
       ((null arg) t)
     (unless (and (rest arg)
@@ -2116,8 +2117,8 @@
 ;;; the list of continuations Args is a well-formed keyword arglist and that
 ;;; only keywords present in the list Keys are supplied.
 ;;;
-(proclaim '(function check-transform-keys (list list) boolean))
 (defun check-transform-keys (args keys)
+  (declare (list args keys) (values boolean))
   (and (check-keywords-constant args)
        (do ((arg args (cddr arg)))
 	   ((null arg) t)
@@ -2129,8 +2130,8 @@
 ;;;
 ;;;    Called by the expansion of the EVENT macro.
 ;;;
-(proclaim '(function %event (event-info (or node null))))
 (defun %event (info node)
+  (declare (type event-info info) (type (or node null) node))
   (incf (event-info-count info))
   (when (and (>= (event-info-level info) *event-note-threshold*)
 	     (if node
