@@ -54,6 +54,12 @@
 (load "vm:pred")
 (load "vm:type-vops")
 
+(when (target-featurep :direct-syscall)
+  (cond ((target-featurep :freebsd)
+         (load "vm:syscall-freebsd"))
+        ((target-featurep :linux)
+         (load "vm:syscall-linux"))))
+
 (load "assem:assem-rtns")
 
 (load "assem:array")
