@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/locall.lisp,v 1.48 1999/12/03 16:28:28 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/locall.lisp,v 1.50 2000/09/07 12:16:23 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -46,7 +46,7 @@
     (let ((arg (car args))
 	  (var (car vars)))
       (cond ((leaf-refs var)
-	     (assert-continuation-type arg (leaf-type var)))
+	     (assert-continuation-optional-type arg (leaf-type var)))
 	    (t
 	     (flush-dest arg)
 	     (setf (car args) nil)))))
@@ -733,8 +733,7 @@
       (setf (lambda-lets fun) ()))
 
     (setf (lambda-calls home)
-	  (nunion (lambda-calls fun)
-		  (delete fun (lambda-calls home))))
+	  (delete fun (nunion (lambda-calls fun) (lambda-calls home))))
     (setf (lambda-calls fun) ())
 
     (setf (lambda-entries home)
