@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/macros.lisp,v 1.43 1999/11/25 15:57:39 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/macros.lisp,v 1.46 2001/03/13 16:52:17 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -23,7 +23,7 @@
 	  def-boolean-attribute attributes-union attributes-intersection
 	  attributes=))
 
-(proclaim '(special *wild-type* *universal-type* *compiler-error-context*))
+(declaim (special *wild-type* *universal-type* *compiler-error-context*))
 
 ;;;; Deftypes:
 
@@ -46,7 +46,7 @@
 
 ;;;; The Policy macro:
 
-(proclaim '(special *lexical-environment*))
+(declaim (special *lexical-environment*))
 
 (eval-when (compile load eval)
 (defconstant policy-parameter-slots
@@ -109,8 +109,9 @@
 ;;;    Concatenate together the names of some strings and symbols, producing
 ;;; a symbol in the current package.
 ;;;
-(proclaim '(function symbolicate (&rest (or string symbol)) symbol))
+(declaim (function symbolicate (&rest (or string symbol)) symbol))
 (defun symbolicate (&rest things)
+  (declare (values symbol))
   (values (intern (reduce #'(lambda (x y)
 			      (concatenate 'string (string x) (string y)))
 			  things))))
@@ -131,7 +132,7 @@
 ;;; Passed to parse-defmacro when we want compiler errors instead of real
 ;;; errors.
 ;;;
-(proclaim '(inline convert-condition-into-compiler-error))
+(declaim (inline convert-condition-into-compiler-error))
 (defun convert-condition-into-compiler-error (datum &rest stuff)
   (if (stringp datum)
       (apply #'compiler-error datum stuff)
@@ -166,7 +167,7 @@
 			      :environment n-env
 			      :error-fun 'convert-condition-into-compiler-error)
       `(progn
-	 (proclaim '(function ,fn-name (continuation continuation t) void))
+	 (declaim (function ,fn-name (continuation continuation t) void))
 	 (defun ,fn-name (,start-var ,cont-var ,n-form)
 	   (let ((,n-env *lexical-environment*))
 	     ,@decls
@@ -267,8 +268,8 @@
 ;;; bound.  We make the variables IGNORABLE so that we don't have to manually
 ;;; declare them Ignore if their only purpose is to make the syntax work.
 ;;;
-(proclaim '(function parse-deftransform (list list symbol t) list))
 (defun parse-deftransform (lambda-list body args error-form)
+  (declare (list lambda-list body) (symbol args))
   (multiple-value-bind (req opt restp rest keyp keys allowp)
 		       (parse-lambda-list lambda-list)
     (let* ((min-args (length req))
@@ -322,11 +323,7 @@
 			       `((check-transform-keys ,n-keys ',(keywords))))))
 		,error-form)
 	      (let ,(binds)
-		;;; ### Bootstrap hack...
-		#+new-compiler
 		(declare (ignorable ,@(vars)))
-		#-new-compiler
-		(progn ,@(vars))
 		,@body))
 	   (vars)))))))
 
@@ -768,10 +765,8 @@
 ;;;    These functions are called by the expansion of the Defprinter
 ;;; macro to do the actual printing.
 ;;;
-(proclaim '(ftype (function (symbol t stream &optional t) void)
-		  defprinter-prin1 defprinter-princ))
 (defun defprinter-prin1 (name value stream &optional indent)
-  (declare (ignore indent))
+  (declare (symbol name) (stream stream) (ignore indent))
   (write-string "  " stream)
   (when *print-pretty*
     (pprint-newline :linear stream))
@@ -780,7 +775,7 @@
   (prin1 value stream))
 ;;;
 (defun defprinter-princ (name value stream &optional indent)
-  (declare (ignore indent))
+  (declare (symbol name) (stream stream) (ignore indent))
   (write-string "  " stream)
   (when *print-pretty*
     (pprint-newline :linear stream))
@@ -927,8 +922,8 @@
 	   `(logtest ,(compute-attribute-mask attribute-names ,const-name)
 		     (the attributes ,attributes)))
 
-	 (define-setf-method ,test-name (place &rest attributes
-					       &environment env)
+	 (define-setf-expander ,test-name (place &rest attributes
+						 &environment env)
 	   
 	   "Automagically generated boolean attribute setter.  See
 	    Def-Boolean-Attribute."
@@ -970,9 +965,9 @@
   `(the attributes
 	(logand ,@(mapcar #'(lambda (x) `(the attributes ,x)) attributes))))
 ;;;
-(proclaim '(inline attributes=))
-(proclaim '(function attributes= (attributes attributes) boolean))
+(declaim (inline attributes=))
 (defun attributes= (attr1 attr2)
+  (declare (type attributes attr1 attr2))
   "Returns true if the attributes present in Attr1 are indentical to those in
   Attr2."
   (eql attr1 attr2))
@@ -1012,8 +1007,8 @@
 ;;;
 ;;;    Return the event info for Name or die trying.
 ;;;
-(proclaim '(function event-info-or-lose (t) event-info))
 (defun event-info-or-lose (name)
+  (declare (values event-info))
   (let ((res (gethash name *event-info*)))
     (unless res
       (error "~S is not the name of an event." name))
@@ -1024,34 +1019,36 @@
 
 ;;; Event-Count, Event-Action, Event-Level  --  Interface
 ;;;
-(proclaim '(function event-count (symbol) fixnum))
 (defun event-count (name)
   "Return the number of times that Event has happened."
+  (declare (symbol name) (values fixnum))
   (event-info-count (event-info-or-lose name)))
 ;;;
-(proclaim '(function event-action (symbol) (or function null)))
 (defun event-action (name)
   "Return the function that is called when Event happens.  If this is null,
   there is no action.  The function is passed the node to which the event
   happened, or NIL if there is no relevant node.  This may be set with SETF."
+  (declare (symbol name) (values (or function null)))
   (event-info-action (event-info-or-lose name)))
 ;;;
-(proclaim '(function %set-event-action (symbol (or function null)) (or function null)))
 (defun %set-event-action (name new-value)
+  (declare (symbol name) (type (or function null) new-value)
+	   (values (or function null)))
   (setf (event-info-action (event-info-or-lose name))
 	new-value))
 ;;;
 (defsetf event-action %set-event-action)
 ;;;
-(proclaim '(function event-level (symbol) unsigned-byte))
 (defun event-level (name)
   "Return the non-negative integer which represents the level of significance
   of the event Name.  This is used to determine whether to print a message when
   the event happens.  This may be set with SETF."
+  (declare (symbol name) (values unsigned-byte))
   (event-info-level (event-info-or-lose name)))
 ;;;
-(proclaim '(function %set-event-level (symbol unsigned-byte) unsigned-byte))
 (defun %set-event-level (name new-value)
+  (declare (symbol name) (type unsigned-byte new-value)
+	   (values unsigned-byte))
   (setf (event-info-level (event-info-or-lose name))
 	new-value))
 ;;;
@@ -1077,7 +1074,7 @@
        (setf (gethash ',name *event-info*) ,var-name)
        ',name)))
 
-(proclaim '(type unsigned-byte *event-note-threshold*))
+(declaim (type unsigned-byte *event-note-threshold*))
 (defvar *event-note-threshold* 1
   "This variable is a non-negative integer specifying the lowest level of
   event that will print a Note when it occurs.")
@@ -1096,8 +1093,8 @@
 
 ;;; Event-Statistics, Clear-Statistics  --  Interface
 ;;;
-(proclaim '(function event-statistics (&optional unsigned-byte stream) void))
 (defun event-statistics (&optional (min-count 1) (stream *standard-output*))
+  (declare (type unsigned-byte min-count) (stream stream) (values))
   "Print a listing of events and their counts, sorted by the count.  Events
   that happened fewer than Min-Count times will not be printed.  Stream is the
   stream to write to."
@@ -1112,8 +1109,8 @@
 	      (event-info-description event)))
     (values)))
 ;;;
-(proclaim '(function clear-statistics () void))
 (defun clear-statistics ()
+  (declare (values))
   (maphash #'(lambda (k v)
 	       (declare (ignore k))
 	       (setf (event-info-count v) 0))
@@ -1123,7 +1120,7 @@
 
 ;;;; Generic list (?) functions:
 
-(proclaim '(inline find-in position-in map-in))
+(declaim (inline find-in position-in map-in))
 
 ;;; Find-In  --  Interface
 ;;;
