@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/main.lisp,v 1.114 1999/11/25 16:03:43 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/main.lisp,v 1.123 2001/05/17 22:08:48 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -25,20 +25,20 @@
 			    compile-file-pathname))
 (in-package "C")
 
-(proclaim '(special *constants* *free-variables* *compile-component*
-		    *code-vector* *next-location* *result-fixups*
-		    *free-functions* *source-paths*
-		    *continuation-number* *continuation-numbers*
-		    *number-continuations* *tn-id* *tn-ids* *id-tns*
-		    *label-ids* *label-id* *id-labels*
-		    *undefined-warnings* *compiler-error-count*
-		    *compiler-warning-count* *compiler-note-count*
-		    *compiler-error-output* *compiler-error-bailout*
-		    *compiler-trace-output*
-		    *last-source-context* *last-original-source*
-		    *last-source-form* *last-format-string* *last-format-args*
-		    *last-message-count* *lexical-environment*
-		    *coalesce-constants*))
+(declaim (special *constants* *free-variables* *compile-component*
+		  *code-vector* *next-location* *result-fixups*
+		  *free-functions* *source-paths*
+		  *continuation-number* *continuation-numbers*
+		  *number-continuations* *tn-id* *tn-ids* *id-tns*
+		  *label-ids* *label-id* *id-labels*
+		  *undefined-warnings* *compiler-error-count*
+		  *compiler-warning-count* *compiler-note-count*
+		  *compiler-error-output* *compiler-error-bailout*
+		  *compiler-trace-output*
+		  *last-source-context* *last-original-source*
+		  *last-source-form* *last-format-string* *last-format-args*
+		  *last-message-count* *lexical-environment*
+		  *coalesce-constants*))
 
 ;;; Exported:
 (defvar *block-compile-default* :specified
@@ -146,7 +146,7 @@
 
 ;;;; Component compilation:
 
-(defparameter max-optimize-iterations 3
+(defparameter max-optimize-iterations 6
   "The upper limit on the number of times that we will consecutively do IR1
   optimization that doesn't introduce any new code.  A finite limit is
   necessary, since type inference may take arbitrarily long to converge.")
@@ -158,7 +158,7 @@
 ;;;
 ;;;    Repeatedly optimize Component until no further optimizations can be
 ;;; found or we hit our iteration limit.  When we hit the limit, we clear the
-;;; component and block REOPTIMIZE flags to discourage following the next
+;;; component and block REOPTIMIZE flags to discourage the following
 ;;; optimization attempt from pounding on the same code.
 ;;;
 (defun ir1-optimize-until-done (component)
@@ -169,29 +169,37 @@
 	(cleared-reanalyze nil))
     (loop
       (when (component-reanalyze component)
-	(setq count 0)
-	(setq cleared-reanalyze t)
+	(setf count 0)
+	(setf cleared-reanalyze t)
 	(setf (component-reanalyze component) nil))
       (setf (component-reoptimize component) nil)
       (ir1-optimize component)
-      (unless (component-reoptimize component)
-	(maybe-mumble " ")
-	(return))
-      (incf count)
-      (when (= count max-optimize-iterations)
-	(event ir1-optimize-maxed-out)
-	(maybe-mumble "* ")
-	(setf (component-reoptimize component) nil)
-	(do-blocks (block component)
-	  (setf (block-reoptimize block) nil))
-	(return))
+      (cond ((component-reoptimize component)
+	     (incf count)
+	     (when (= count max-optimize-iterations)
+	       (maybe-mumble "*")
+	       (cond ((retry-delayed-transforms :optimize)
+		      (maybe-mumble "+")
+		      (setf count 0))
+		     (t
+		      (event ir1-optimize-maxed-out)
+		      (setf (component-reoptimize component) nil)
+		      (do-blocks (block component)
+			(setf (block-reoptimize block) nil))
+		      (return)))))
+	    ((retry-delayed-transforms :optimize)
+	     (setf count 0)
+	     (maybe-mumble "+"))
+	    (t
+	     (return)))
       (maybe-mumble "."))
     (when cleared-reanalyze
-      (setf (component-reanalyze component) t)))
+      (setf (component-reanalyze component) t))
+    (maybe-mumble " "))
   (undefined-value))
 
 (defparameter *constraint-propagate* t)
-(defparameter *reoptimize-after-type-check-max* 5)
+(defparameter *reoptimize-after-type-check-max* 10)
 
 (defevent reoptimize-maxed-out
   "*REOPTIMIZE-AFTER-TYPE-CHECK-MAX* exceeded.")
@@ -221,38 +229,40 @@
 (defun ir1-phases (component)
   (declare (type component component))
   (let ((*constraint-number* 0)
-	(loop-count 1))
-    (declare (special *constraint-number*))
+	(loop-count 1)
+	(*delayed-transforms* nil))
+    (declare (special *constraint-number* *delayed-transforms*))
     (loop
-      (ir1-optimize-until-done component)
-      (when (or (component-new-functions component)
-		(component-reanalyze-functions component))
-	(maybe-mumble "Locall ")
-	(local-call-analyze component))
-      (dfo-as-needed component)
-      (when *constraint-propagate*
-	(maybe-mumble "Constraint ")
-	(constraint-propagate component))
-      (maybe-mumble "Type ")
-      ;; Delay the generation of type checks until the type
-      ;; constraints have had time to propagate, else the compiler can
-      ;; confuse itself.
-      (unless (and (or (component-reoptimize component)
-		       (component-reanalyze component)
-		       (component-new-functions component)
-		       (component-reanalyze-functions component))
-		   (< loop-count (- *reoptimize-after-type-check-max* 2)))
-	      (generate-type-checks component)
-	      (unless (or (component-reoptimize component)
-			  (component-reanalyze component)
-			  (component-new-functions component)
-			  (component-reanalyze-functions component))
-		      (return)))
-      (when (>= loop-count *reoptimize-after-type-check-max*)
-	(maybe-mumble "[Reoptimize Limit]")
-	(event reoptimize-maxed-out)
-	(return))
-      (incf loop-count)))
+     (ir1-optimize-until-done component)
+     (when (or (component-new-functions component)
+	       (component-reanalyze-functions component))
+       (maybe-mumble "Locall ")
+       (local-call-analyze component))
+     (dfo-as-needed component)
+     (when *constraint-propagate*
+       (maybe-mumble "Constraint ")
+       (constraint-propagate component))
+     (when (retry-delayed-transforms :constraint)
+       (maybe-mumble "Rtran "))
+     ;; Delay the generation of type checks until the type constraints have
+     ;; had time to propagate, else the compiler can confuse itself.
+     (unless (and (or (component-reoptimize component)
+		      (component-reanalyze component)
+		      (component-new-functions component)
+		      (component-reanalyze-functions component))
+		  (< loop-count (- *reoptimize-after-type-check-max* 4)))
+       (maybe-mumble "Type ")
+       (generate-type-checks component)
+       (unless (or (component-reoptimize component)
+		   (component-reanalyze component)
+		   (component-new-functions component)
+		   (component-reanalyze-functions component))
+	 (return)))
+     (when (>= loop-count *reoptimize-after-type-check-max*)
+       (maybe-mumble "[Reoptimize Limit]")
+       (event reoptimize-maxed-out)
+       (return))
+     (incf loop-count)))
 
   (ir1-finalize component)
   (undefined-value))
@@ -1055,7 +1065,7 @@
       (convert-and-maybe-compile form path)))
 
 
-(proclaim '(special *compiler-error-bailout*))
+(declaim (special *compiler-error-bailout*))
 
 ;;; PROCESS-FORM  --  Internal
 ;;;
@@ -1081,7 +1091,7 @@
 	  (case (car form)
 	    ((make-package shadow shadowing-import export
 	      unexport use-package unuse-package import
-	      old-in-package %in-package)
+	      old-in-package %in-package %defpackage)
 	     (process-cold-load-form form path t))
 	    ((error cerror break signal)
 	     (process-cold-load-form form path nil))
@@ -1094,7 +1104,8 @@
 	     (do-eval-when-stuff
 	      (cadr form) (cddr form)
 	      #'(lambda (forms)
-		  (process-progn forms path))))
+		  (process-progn forms path))
+	      t))
 	    ((macrolet)
 	     (unless (>= (length form) 2)
 	       (compiler-error "MACROLET form is too short: ~S." form))
@@ -1541,7 +1552,7 @@
 	(error "Can't compile with no source files."))
       (mapcar #'(lambda (x)
 		  (let ((x (pathname x)))
-		    (cond ((logical-pathname-p x)
+		    (cond ((typep x 'logical-pathname)
 			   (try-with-type x "LISP" t))
 			  ((probe-file x) x)
 			  ((try-with-type x "lisp"  nil))
@@ -1692,7 +1703,7 @@
       which is initially :MAYBE."
   (let* ((fasl-file nil)
 	 (error-file-stream nil)
-	 (output-file-name nil)
+	 (output-file-pathname nil)
 	 (*compiler-error-output* *compiler-error-output*)
 	 (*compiler-trace-output* nil)
 	 (compile-won nil)
@@ -1704,19 +1715,18 @@
 	(progn
 	  (flet ((frob (file type)
 		   (if (eq file t)
-		       (make-pathname :type type
-				      :defaults
-				      (if (logical-pathname-p default)
-					  (translate-logical-pathname default)
-					  default))
+		       (make-pathname
+			:type type
+			:defaults (translate-logical-pathname default))
 		       (pathname file))))
 	    
 	    (when output-file
-	      (setq output-file-name
-		    (compile-file-pathname (first source)
-					   :output-file output-file
-					   :byte-compile *byte-compile*))
-	      (setq fasl-file (open-fasl-file output-file-name
+	      (setq output-file-pathname
+		    (translate-logical-pathname
+		     (compile-file-pathname (first source)
+					    :output-file output-file
+					    :byte-compile *byte-compile*)))
+	      (setq fasl-file (open-fasl-file output-file-pathname
 					      (namestring (first source))
 					      (eq *byte-compile* t))))
 	    
@@ -1751,9 +1761,10 @@
 
       (when fasl-file
 	(close-fasl-file fasl-file (not compile-won))
-	(setq output-file-name (pathname (fasl-file-stream fasl-file)))
+	(setq output-file-pathname (pathname (fasl-file-stream fasl-file)))
 	(when (and compile-won *compile-verbose*)
-	  (compiler-mumble "~2&~A written.~%" (namestring output-file-name))))
+	  (compiler-mumble "~2&~A written.~%"
+			   (namestring output-file-pathname))))
 
       (when *compile-verbose*
 	(finish-error-output source-info compile-won))
@@ -1774,11 +1785,11 @@
     (when load
       (unless output-file
 	(error "Can't :LOAD with no output file."))
-      (load output-file-name :verbose *compile-verbose*))
+      (load output-file-pathname :verbose *compile-verbose*))
 
     (values (if output-file
 		;; Hack around filesystem race condition...
-		(or (probe-file output-file-name) output-file-name)
+		(or (probe-file output-file-pathname) output-file-pathname)
 		nil)
 	    (not (null error-severity))
 	    (if (member error-severity '(:warning :error)) t nil))))
@@ -1914,7 +1925,7 @@
     (cond ((not (eq output-file t))
 	   (when output-file
 	     (translate-logical-pathname (pathname output-file))))
-	  ((and (logical-pathname-p pathname) (not (eq byte-compile t)))
+	  ((and (typep pathname 'logical-pathname) (not (eq byte-compile t)))
 	   (make-pathname :type "FASL" :defaults pathname
 			  :case :common))
 	  (t
