@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/node.lisp,v 1.35 1999/02/25 13:03:11 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/node.lisp,v 1.37 2001/03/04 20:12:24 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -19,11 +19,11 @@
 (export '(component component-info block-number))
 
 ;;; Defvars for these variables appear later.
-(proclaim '(special *current-path* *lexical-environment* *current-component*
-		    *default-cookie* *default-interface-cookie*))
+(declaim (special *current-path* *lexical-environment* *current-component*
+		  *default-cookie* *default-interface-cookie*))
   
 
-(proclaim '(inline internal-make-lexenv))
+(declaim (inline internal-make-lexenv))
 
 ;;; The LEXENV represents the lexical environment used for IR1 conversion.
 ;;;
@@ -85,6 +85,21 @@
   (options nil :type list))
 
 
+;;; A Cont-Ref represents a reference to a continuation.
+;;;
+;;; Note this is somewhat of a hack to handle continuation substitution and
+;;; correct references to continuations within the lexenv.
+;;;
+(defstruct (cont-ref
+	    (:print-function %print-cont-ref))
+  ;;
+  ;; The continuation referenced.
+  (cont (required-argument))) ; :type continuation))
+
+(defprinter cont-ref
+  cont)
+
+
 ;;; The front-end data structure (IR1) is composed of nodes and continuations.
 ;;; The general idea is that continuations contain top-down information and
 ;;; nodes contain bottom-up, derived information.  A continuation represents a
@@ -167,6 +182,10 @@
   ;; indicate whether NIL means no uses or more than one use.
   (use nil :type (or node null))
   ;;
+  ;; Other references to this continuation that must be substituted when this
+  ;; continuation is substituted. This is a list of cont-ref structures.
+  (refs nil :type list)
+  ;;
   ;; Basic block this continuation is in.  This is null only in :Deleted and
   ;; :Unused continuations.  Note that blocks that are unreachable but still in
   ;; the DFO may receive deleted continuations, so it isn't o.k. to assume that
