@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/sparc/arith.lisp,v 1.19 2000/01/17 18:08:19 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/sparc/arith.lisp,v 1.27 2001/11/05 21:38:15 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -14,7 +14,8 @@
 ;;; Written by Rob MacLachlan
 ;;;
 ;;; Converted by William Lott.
-;;; 
+;;;
+;;; Enhancements/debugging by Raymond Toy 1999, 2000
 
 (in-package "SPARC")
 
@@ -167,9 +168,27 @@
 (define-binop logxor 2 xor)
 (define-binop logeqv 2 xnor)
 
+;;; Special logand cases: (logand signed unsigned) => unsigned
+
+(define-vop (fast-logand/signed-unsigned=>unsigned
+	     fast-logand/unsigned=>unsigned)
+    (:args (x :target r :scs (signed-reg))
+	   (y :scs (unsigned-reg unsigned-stack)))
+  (:arg-types signed-num unsigned-num))
+
+(define-vop (fast-logand/unsigned-signed=>unsigned
+	     fast-logand/unsigned=>unsigned)
+    (:args (x :target r :scs (unsigned-reg))
+	   (y :scs (signed-reg signed-stack)))
+  (:arg-types unsigned-num signed-num))
+    
 ;;; Special case fixnum + and - that trap on overflow.  Useful when we
 ;;; don't know that the output type is a fixnum.
 
+;;; I (toy@rtp.ericsson.se) took these out.  They don't seem to be
+;;; used anywhere at all.
+#+nil
+(progn
 (define-vop (+/fixnum fast-+/fixnum=>fixnum)
   (:policy :safe)
   (:results (r :scs (any-reg descriptor-reg)))
@@ -202,84 +221,372 @@
   (:generator 3
     (inst tsubcctv r x (fixnum y))))
 
+)
+
+;;; Truncate
+
+;; This doesn't work for some reason.
+#+nil
+(define-vop (fast-v8-truncate/fixnum=>fixnum fast-safe-arith-op)
+  (:translate truncate)
+  (:args (x :scs (any-reg))
+	 (y :scs (any-reg)))
+  (:arg-types tagged-num tagged-num)
+  (:results (quo :scs (any-reg))
+	    (rem :scs (any-reg)))
+  (:result-types tagged-num tagged-num)
+  (:note "inline fixnum arithmetic")
+  (:temporary (:scs (any-reg) :target quo) q)
+  (:temporary (:scs (any-reg)) r)
+  (:temporary (:scs (signed-reg)) y-int)
+  (:vop-var vop)
+  (:save-p :compute-only)
+  (:guard (or (backend-featurep :sparc-v8)
+	      (and (backend-featurep :sparc-v9)
+		   (not (backend-featurep :sparc-64)))))
+  (:generator 12
+    (let ((zero (generate-error-code vop division-by-zero-error x y)))
+      (inst cmp y zero-tn)
+      (inst b :eq zero)
+      ;; Extend the sign of X into the Y register
+        (inst sra r x 31)
+      (inst wry r)
+      ;; Remove tag bits so Q and R will be tagged correctly.
+      (inst sra y-int y fixnum-tag-bits)
+      (inst nop)
+      (inst nop)
+
+      (inst sdiv q x y-int)		; Q is tagged.
+      ;; We have the quotient so we need to compute the remainder
+      (inst smul r q y-int)		; R is tagged
+      (inst sub rem x r)
+      (unless (location= quo q)
+	(move quo q)))))
+
+(define-vop (fast-v8-truncate/signed=>signed fast-safe-arith-op)
+  (:translate truncate)
+  (:args (x :scs (signed-reg))
+	 (y :scs (signed-reg)))
+  (:arg-types signed-num signed-num)
+  (:results (quo :scs (signed-reg))
+	    (rem :scs (signed-reg)))
+  (:result-types signed-num signed-num)
+  (:note "inline (signed-byte 32) arithmetic")
+  (:temporary (:scs (signed-reg) :target quo) q)
+  (:temporary (:scs (signed-reg)) r)
+  (:vop-var vop)
+  (:save-p :compute-only)
+  (:guard (or (backend-featurep :sparc-v8)
+	      (and (backend-featurep :sparc-v9)
+		   (not (backend-featurep :sparc-64)))))
+  (:generator 12
+    (let ((zero (generate-error-code vop division-by-zero-error x y)))
+      (inst cmp y zero-tn)
+      (inst b :eq zero #+sparc-v9 :pn)
+      ;; Extend the sign of X into the Y register
+        (inst sra r x 31)
+      (inst wry r)
+      (inst nop)
+      (inst nop)
+      (inst nop)
+
+      (inst sdiv q x y)
+      ;; We have the quotient so we need to compue the remainder
+      (inst smul r q y)		; rem
+      (inst sub rem x r)
+      (unless (location= quo q)
+	(move quo q)))))
+
+(define-vop (fast-v8-truncate/unsigned=>unsigned fast-safe-arith-op)
+  (:translate truncate)
+  (:args (x :scs (unsigned-reg))
+	 (y :scs (unsigned-reg)))
+  (:arg-types unsigned-num unsigned-num)
+  (:results (quo :scs (unsigned-reg))
+	    (rem :scs (unsigned-reg)))
+  (:result-types unsigned-num unsigned-num)
+  (:note "inline (unsigned-byte 32) arithmetic")
+  (:temporary (:scs (unsigned-reg) :target quo) q)
+  (:temporary (:scs (unsigned-reg)) r)
+  (:vop-var vop)
+  (:save-p :compute-only)
+  (:guard (or (backend-featurep :sparc-v8)
+	      (and (backend-featurep :sparc-v9)
+		   (not (backend-featurep :sparc-64)))))
+  (:generator 8
+    (let ((zero (generate-error-code vop division-by-zero-error x y)))
+      (inst cmp y zero-tn)
+      (inst b :eq zero #+sparc-v9 :pn)
+        (inst wry zero-tn)		; Clear out high part
+      (inst nop)
+      (inst nop)
+      (inst nop)
+      
+      (inst udiv q x y)
+      ;; Compute remainder
+      (inst umul r q y)
+      (inst sub rem x r)
+      (unless (location= quo q)
+	(inst move quo q)))))
+
+(define-vop (fast-v9-truncate/signed=>signed fast-safe-arith-op)
+  (:translate truncate)
+  (:args (x :scs (signed-reg))
+	 (y :scs (signed-reg)))
+  (:arg-types signed-num signed-num)
+  (:results (quo :scs (signed-reg))
+	    (rem :scs (signed-reg)))
+  (:result-types signed-num signed-num)
+  (:note "inline (signed-byte 32) arithmetic")
+  (:temporary (:scs (signed-reg) :target quo) q)
+  (:temporary (:scs (signed-reg)) r)
+  (:vop-var vop)
+  (:save-p :compute-only)
+  (:guard (backend-featurep :sparc-64))
+  (:generator 8
+    (let ((zero (generate-error-code vop division-by-zero-error x y)))
+      (inst cmp y zero-tn)
+      (inst b :eq zero :pn)
+      ;; Sign extend the numbers, just in case.
+        (inst sra x 0)
+      (inst sra y 0)
+      (inst sdivx q x y)
+      ;; Compute remainder
+      (inst mulx r q y)
+      (inst sub rem x r)
+      (unless (location= quo q)
+	(inst move quo q)))))
+
+(define-vop (fast-v9-truncate/unsigned=>unsigned fast-safe-arith-op)
+  (:translate truncate)
+  (:args (x :scs (unsigned-reg))
+	 (y :scs (unsigned-reg)))
+  (:arg-types unsigned-num unsigned-num)
+  (:results (quo :scs (unsigned-reg))
+	    (rem :scs (unsigned-reg)))
+  (:result-types unsigned-num unsigned-num)
+  (:note "inline (unsigned-byte 32) arithmetic")
+  (:temporary (:scs (unsigned-reg) :target quo) q)
+  (:temporary (:scs (unsigned-reg)) r)
+  (:vop-var vop)
+  (:save-p :compute-only)
+  (:guard (backend-featurep :sparc-64))
+  (:generator 8
+    (let ((zero (generate-error-code vop division-by-zero-error x y)))
+      (inst cmp y zero-tn)
+      (inst b :eq zero :pn)
+      ;; Zap the higher 32 bits, just in case
+        (inst srl x 0)
+      (inst srl y 0)
+      (inst udivx q x y)
+      ;; Compute remainder
+      (inst mulx r q y)
+      (inst sub rem x r)
+      (unless (location= quo q)
+	(inst move quo q)))))
+
 ;;; Shifting
 
-(define-vop (fast-ash/unsigned=>unsigned)
-  (:note "inline ASH")
-  (:args (number :scs (unsigned-reg) :to :save)
-	 (amount :scs (signed-reg immediate)))
-  (:arg-types unsigned-num signed-num)
-  (:results (result :scs (unsigned-reg)))
-  (:result-types unsigned-num)
-  (:translate ash)
-  (:policy :fast-safe)
-  (:temporary (:sc non-descriptor-reg) ndesc)
-  (:generator 3
-    (sc-case amount
-      (signed-reg
-       (let ((positive (gen-label))
-	     (done (gen-label)))
-	 (inst cmp amount)
-	 (inst b :ge positive)
-	 (inst neg ndesc amount)
-	 (inst cmp ndesc 31)
-	 (inst b :le done)
-	 (inst srl result number ndesc)
-	 (inst b done)
-	 (inst srl result number 31)
-
-	 (emit-label positive)
-	 ;; The result-type assures us that this shift will not overflow.
-	 (inst sll result number amount)
-
-	 (emit-label done)))
-
-      (immediate
-       (let ((amount (tn-value amount)))
-	 (if (minusp amount)
-	     (let ((amount (min 31 (- amount))))
-	       (inst srl result number amount))
-	     (inst sll result number amount)))))))
-
 (define-vop (fast-ash/signed=>signed)
-  (:note "inline ASH")
+  (:note "inline (signed-byte 32) ASH")
   (:args (number :scs (signed-reg) :to :save)
-	 (amount :scs (signed-reg immediate)))
+	 (amount :scs (signed-reg immediate) :to :save))
   (:arg-types signed-num signed-num)
   (:results (result :scs (signed-reg)))
   (:result-types signed-num)
   (:translate ash)
   (:policy :fast-safe)
   (:temporary (:sc non-descriptor-reg) ndesc)
-  (:generator 3
+  (:generator 5
     (sc-case amount
       (signed-reg
-       (let ((positive (gen-label))
-	     (done (gen-label)))
-	 (inst cmp amount)
-	 (inst b :ge positive)
-	 (inst neg ndesc amount)
-	 (inst cmp ndesc 31)
-	 (inst b :le done)
-	 (inst sra result number ndesc)
-	 (inst b done)
-	 (inst sra result number 31)
+       (cond ((backend-featurep :sparc-v9)
+	      (let ((done (gen-label)))
+		(inst cmp amount)
+		(inst b :ge done)
+		;; The result-type assures us that this shift will not
+		;; overflow.
+		(inst sll result number amount)
+		(inst neg ndesc amount)
+		;; ndesc = max(-amount, 31)
+		(inst cmp ndesc 31)
+		(inst cmove :ge ndesc 31)
+		(inst sra result number ndesc)
+		(emit-label done)))
+	     (t
+	      (let ((done (gen-label)))
+		(inst cmp amount)
+		(inst b :ge done)
+		;; The result-type assures us that this shift will not overflow.
+		(inst sll result number amount)
 
-	 (emit-label positive)
-	 ;; The result-type assures us that this shift will not overflow.
-	 (inst sll result number amount)
+		;; Handle right shifts here.
+		(inst neg ndesc amount)
+		(inst cmp ndesc 31)
+		(inst b :le done)
+		(inst sra result number ndesc)
+		;; Right shift of greater than 31 bits is the same as a shift
+		;; of 31 bits for signed 32-bit numbers.
+		(inst sra result number 31)
 
-	 (emit-label done)))
+		(emit-label done)))))
+
+	(immediate
+	 (let ((amount (tn-value amount)))
+	   (cond ((< amount -31)
+		  (inst li result -1))
+		 ((< amount 0)
+		  (inst sra result number (- amount)))
+		 ((> amount 0)
+		  (inst sll result number amount))
+		 (t
+		  ;; amount = 0.  Shouldn't happen because of a
+		  ;; deftransform, but it's easy.
+		  (move result number))))))))
+
+(define-vop (fast-ash/unsigned=>unsigned)
+  (:note "inline (unsigned-byte 32) ASH")
+  (:args (number :scs (unsigned-reg) :to :save)
+	 (amount :scs (signed-reg immediate) :to :save))
+  (:arg-types unsigned-num signed-num)
+  (:results (result :scs (unsigned-reg)))
+  (:result-types unsigned-num)
+  (:translate ash)
+  (:policy :fast-safe)
+  (:temporary (:sc non-descriptor-reg) ndesc)
+  (:generator 5
+    (sc-case amount
+      (signed-reg
+       (cond ((backend-featurep :sparc-v9)
+	      (let ((done (gen-label)))
+		(inst cmp amount)
+		(inst b :ge done)
+		;; The result-type assures us that this shift will not
+		;; overflow.
+		(inst sll result number amount)
+		(inst neg ndesc amount)
+		;; A right shift of 32 or more results in zero.
+		(inst cmp ndesc 32)
+		(inst srl result number ndesc)
+		(inst cmove :ge result zero-tn)
+		(emit-label done)))
+	     (t
+	      (let ((done (gen-label)))
+		(inst cmp amount)
+		(inst b :ge done)
+		;; The result-type assures us that this shift will not
+		;; overflow.
+		(inst sll result number amount)
+		(inst neg ndesc amount)
+		(inst cmp ndesc 32)
+		(inst b :lt done)
+		(inst srl result number ndesc)
+		;; Right shift of 32 or more is the same as zero for unsigned
+		;; 32-bit numbers.
+		(move result zero-tn)
+
+		(emit-label done)))))
 
       (immediate
        (let ((amount (tn-value amount)))
-	 (if (minusp amount)
-	     (let ((amount (min 31 (- amount))))
-	       (inst sra result number amount))
-	     (inst sll result number amount)))))))
+	 (cond ((< amount -31)
+		(move result zero-tn))
+	       ((< amount 0)
+		(inst srl result number (- amount)))
+	       ((> amount 0)
+		(inst sll result number amount))
+	       (t
+		;; amount = 0.  Shouldn't happen because of a
+		;; deftransform, but it's easy.
+		(move result number))))))))
 
+;; Some special cases where we know we want a left shift.  Just do the
+;; shift, instead of checking for the sign of the shift.
+(macrolet
+    ((frob (name sc-type type result-type cost)
+       `(define-vop (,name)
+	 (:note "inline ASH")
+	 (:translate ash)
+	 (:args (number :scs (,sc-type))
+	        (amount :scs (signed-reg unsigned-reg immediate)))
+	 (:arg-types ,type positive-fixnum)
+	 (:results (result :scs (,result-type)))
+	 (:result-types ,type)
+	 (:policy :fast-safe)
+	 (:generator ,cost
+	  ;; The result-type assures us that this shift will not
+	  ;; overflow. And for fixnum's, the zero bits that get
+	  ;; shifted in are just fine for the fixnum tag.
+	  (sc-case amount
+	   ((signed-reg unsigned-reg)
+	    (inst sll result number amount))
+	   (immediate
+	    (let ((amount (tn-value amount)))
+	      (assert (>= amount 0))
+	      (inst sll result number amount))))))))
+  (frob fast-ash-left/signed=>signed signed-reg signed-num signed-reg 3)
+  (frob fast-ash-left/fixnum=>fixnum any-reg tagged-num any-reg 2)
+  (frob fast-ash-left/unsigned=>unsigned unsigned-reg unsigned-num unsigned-reg 3))
+
+(defknown ash-right-signed ((signed-byte #.vm:word-bits)
+			    (and fixnum unsigned-byte))
+  (signed-byte #.vm:word-bits)
+  (movable foldable flushable))
+
+(defknown ash-right-unsigned ((unsigned-byte #.vm:word-bits)
+			      (and fixnum unsigned-byte))
+  (unsigned-byte #.vm:word-bits)
+  (movable foldable flushable))
+
+;; Some special cases where we want a right shift.  Just do the shift.
+;; (Needs appropriate deftransforms to call these, though.)
+
+(macrolet
+    ((frob (trans name sc-type type shift-inst cost)
+       `(define-vop (,name)
+	 (:note "inline right ASH")
+	 (:translate ,trans)
+	 (:args (number :scs (,sc-type))
+	        (amount :scs (signed-reg unsigned-reg immediate)))
+	 (:arg-types ,type positive-fixnum)
+	 (:results (result :scs (,sc-type)))
+	 (:result-types ,type)
+	 (:policy :fast-safe)
+	 (:generator ,cost
+	    (sc-case amount
+	     ((signed-reg unsigned-reg)
+		(inst ,shift-inst result number amount))
+	     (immediate
+	      (let ((amt (tn-value amount)))
+		(inst ,shift-inst result number amt))))))))
+  (frob ash-right-signed fast-ash-right/signed=>signed
+	signed-reg signed-num sra 3)
+  (frob ash-right-unsigned fast-ash-right/unsigned=>unsigned
+	unsigned-reg unsigned-num srl 3))
+
+(define-vop (fast-ash-right/fixnum=>fixnum)
+    (:note "inline right ASH")
+  (:translate ash-right-signed)
+  (:args (number :scs (any-reg))
+	 (amount :scs (signed-reg unsigned-reg immediate)))
+  (:arg-types tagged-num positive-fixnum)
+  (:results (result :scs (any-reg)))
+  (:result-types tagged-num)
+  (:temporary (:sc non-descriptor-reg :target result) temp)
+  (:policy :fast-safe)
+  (:generator 2
+    ;; Shift the fixnum right by the desired amount.  Then zap out the
+    ;; 2 LSBs to make it a fixnum again.  (Those bits are junk.)
+    (sc-case amount
+      ((signed-reg unsigned-reg)
+       (inst sra temp number amount))
+      (immediate
+       (inst sra temp number (tn-value amount))))
+    (inst andn result temp fixnum-tag-mask)))
+    
 
 
+
 (define-vop (signed-byte-32-len)
   (:translate integer-length)
   (:note "inline (signed-byte 32) integer-length")
@@ -337,20 +644,30 @@
 (define-vop (fast-v8-*/fixnum=>fixnum fast-fixnum-binop)
   (:temporary (:scs (non-descriptor-reg)) temp)
   (:translate *)
-  (:guard (backend-featurep :sparc-v8))
-  (:generator 4
-    (inst sra temp y 2)
+  (:guard (or (backend-featurep :sparc-v8)
+	      (and (backend-featurep :sparc-v9)
+		   (not (backend-featurep :sparc-64)))))
+  (:generator 2
+    ;; The cost here should be less than the cost for
+    ;; */signed=>signed.  Why?  A fixnum product using signed=>signed
+    ;; has to convert both args to signed-nums.  But using this, we
+    ;; don't have to and that saves an instruction.
+    (inst sra temp y fixnum-tag-bits)
     (inst smul r x temp)))
 
 (define-vop (fast-v8-*/signed=>signed fast-signed-binop)
   (:translate *)
-  (:guard (backend-featurep :sparc-v8))
+  (:guard (or (backend-featurep :sparc-v8)
+	      (and (backend-featurep :sparc-v9)
+		   (not (backend-featurep :sparc-64)))))
   (:generator 3
     (inst smul r x y)))
 
 (define-vop (fast-v8-*/unsigned=>unsigned fast-unsigned-binop)
   (:translate *)
-  (:guard (backend-featurep :sparc-v8))
+  (:guard (or (backend-featurep :sparc-v8)
+	      (and (backend-featurep :sparc-v9)
+		   (not (backend-featurep :sparc-64)))))
   (:generator 3
     (inst umul r x y)))
 
@@ -361,7 +678,7 @@
   (:translate *)
   (:guard (backend-featurep :sparc-64))
   (:generator 4
-    (inst sra temp y 2)
+    (inst sra temp y fixnum-tag-bits)
     (inst mulx r x temp)))
 
 (define-vop (fast-v9-*/signed=>signed fast-signed-binop)
@@ -635,6 +952,7 @@
   (:args (digit :scs (unsigned-reg)))
   (:arg-types unsigned-num)
   (:results (result :scs (descriptor-reg)))
+  (:guard (not (backend-featurep :sparc-v9)))
   (:generator 3
     (let ((done (gen-label)))
       (inst cmp digit)
@@ -643,6 +961,34 @@
       (load-symbol result t)
       (emit-label done))))
 
+(define-vop (v9-digit-0-or-plus-cmove)
+  (:translate bignum::%digit-0-or-plusp)
+  (:policy :fast-safe)
+  (:args (digit :scs (unsigned-reg)))
+  (:arg-types unsigned-num)
+  (:results (result :scs (descriptor-reg)))
+  (:guard (backend-featurep :sparc-v9))
+  (:generator 3
+    (inst cmp digit)
+    (load-symbol result t)
+    (inst cmove :lt result null-tn)))
+
+;; This doesn't work?
+#+nil
+(define-vop (v9-digit-0-or-plus-movr)
+  (:translate bignum::%digit-0-or-plusp)
+  (:policy :fast-safe)
+  (:args (digit :scs (unsigned-reg)))
+  (:arg-types unsigned-num)
+  (:results (result :scs (descriptor-reg)))
+  (:temporary (:scs (descriptor-reg)) temp)
+  (:guard (backend-featurep :sparc-v9))
+  (:generator 2
+    (load-symbol temp t)
+    (inst movr result null-tn digit :lz)
+    (inst movr result temp digit :gez)))
+
+
 (define-vop (add-w/carry)
   (:translate bignum::%add-with-carry)
   (:policy :fast-safe)
@@ -688,6 +1034,11 @@
   (cond ((backend-featurep :sparc-64)
 	 ;; Take advantage of V9's 64-bit multiplier.
 	 ;;
+	 ;; Make sure the multiplier and multiplicand are really
+	 ;; unsigned 64-bit numbers.
+	 (inst srl multiplier 0)
+	 (inst srl multiplicand 0)
+	 
 	 ;; Multiply the two numbers and put the result in
 	 ;; result-high.  Copy the low 32-bits to result-low.  Then
 	 ;; shift result-high so the high 32-bits end up in the low
@@ -695,7 +1046,8 @@
 	 (inst mulx result-high multiplier multiplicand)
 	 (inst move result-low result-high)
 	 (inst srax result-high 32))
-	((backend-featurep :sparc-v8)
+	((or (backend-featurep :sparc-v8)
+	     (backend-featurep :sparc-v9))
 	 ;; V8 has a multiply instruction.  This should also work for
 	 ;; the V9, but umul and the Y register is deprecated on the
 	 ;; V9.
@@ -782,9 +1134,9 @@
   (:results (digit :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 1
-    (inst sra digit fixnum 2)))
+    (inst sra digit fixnum fixnum-tag-bits)))
 
-(define-vop (bignum-floor-v8)
+(define-vop (bignum-floor)
   (:translate bignum::%floor)
   (:policy :fast-safe)
   (:args (div-high :scs (unsigned-reg) :target rem)
@@ -794,7 +1146,8 @@
   (:results (quo :scs (unsigned-reg) :from (:argument 1))
 	    (rem :scs (unsigned-reg) :from (:argument 0)))
   (:result-types unsigned-num unsigned-num)
-  (:guard (not (backend-featurep :sparc-64)))
+  (:guard (not (or (backend-featurep :sparc-v8)
+		   (backend-featurep :sparc-v9))))
   (:generator 300
     (move rem div-high)
     (move quo div-low)
@@ -809,6 +1162,36 @@
 	  (inst addx rem rem))))
     (inst not quo)))
 
+(define-vop (bignum-floor-v8)
+  (:translate bignum::%floor)
+  (:policy :fast-safe)
+  (:args (div-high :scs (unsigned-reg) :target rem)
+	 (div-low :scs (unsigned-reg) :target quo)
+	 (divisor :scs (unsigned-reg)))
+  (:arg-types unsigned-num unsigned-num unsigned-num)
+  (:results (quo :scs (unsigned-reg) :from (:argument 1))
+	    (rem :scs (unsigned-reg) :from (:argument 0)))
+  (:result-types unsigned-num unsigned-num)
+  (:temporary (:scs (unsigned-reg) :target quo) q)
+  ;; This vop is for a v8 or v9, provided we're also not using
+  ;; sparc-64, for which there a special sparc-64 vop.
+  (:guard (or (backend-featurep :sparc-v8)
+	      (and (backend-featurep :sparc-v9)
+		   (not (backend-featurep :sparc-64)))))
+  (:generator 15
+    (inst wry div-high)
+    (inst nop)
+    (inst nop)
+    (inst nop)
+    ;; Compute the quotient [Y, div-low] / divisor
+    (inst udiv q div-low divisor)
+    ;; Compute the remainder.  The high part of the result is in the Y
+    ;; register.
+    (inst umul rem q divisor)
+    (inst sub rem div-low rem)
+    (unless (location= quo q)
+      (move quo q))))
+
 (define-vop (bignum-floor-v9)
   (:translate bignum::%floor)
   (:policy :fast-safe)
@@ -841,7 +1224,7 @@
   (:generator 1
     (sc-case res
       (any-reg
-       (inst sll res digit 2))
+       (inst sll res digit fixnum-tag-bits))
       (signed-reg
        (move res digit)))))
 
@@ -890,3 +1273,86 @@
 (define-static-function two-arg-and (x y) :translate logand)
 (define-static-function two-arg-ior (x y) :translate logior)
 (define-static-function two-arg-xor (x y) :translate logxor)
+
+
+(defknown ext::is-plusp ((signed-byte 32))
+  (member nil t)
+  (movable foldable flushable))
+
+(define-vop (is-plusp)
+  (:translate ext::is-plusp)
+  (:policy :fast-safe)
+  (:args (num :scs (signed-reg)))
+  (:results (result :scs (descriptor-reg)))
+  (:arg-types signed-num)
+  (:temporary (:scs (descriptor-reg)) temp)
+  (:generator 2
+    (load-symbol temp t)
+    (inst movr result null-tn num :lz)
+    (inst movr result temp num :gez)))
+
+(defknown ext::f-move (double-float double-float (signed-byte 32))
+  double-float
+  (movable foldable flushable))
+
+(define-vop (fmovr-test)
+  (:translate ext::f-move)
+  (:policy :fast-safe)
+  (:args (x :scs (double-reg))
+	 (y :scs (double-reg))
+	 (num :scs (signed-reg)))
+  (:results (result :scs (double-reg)))
+  (:arg-types double-float double-float signed-num)
+  (:result-types double-float)
+  (:generator 2
+    (inst fmovrd result x num :lez)
+    (inst fmovrd result y num :gz)
+    (inst fcmpd x y :fcc1)
+    (inst cfmovd :le result x :fcc1)
+    (inst cfmovd :g result y :Fcc1)))
+
+;; Need these so constant folding works with the deftransform.
+
+(defun ash-right-signed (num shift)
+  (declare (type (signed-byte #.vm:word-bits) num)
+	   (type (integer 0 #.(1- vm:word-bits)) shift))
+  (ash num (- shift)))
+
+(defun ash-right-unsigned (num shift)
+  (declare (type (unsigned-byte #.vm:word-bits) num)
+	   (type (integer 0 #.(1- vm:word-bits)) shift))
+  (ash num (- shift)))
+
+;; If we can prove that we have a right shift, just do the right shift
+;; instead of calling the inline ASH which has to check for the
+;; direction of the shift at run-time.
+(in-package "C")
+
+(deftransform ash ((num shift) (integer integer))
+  (let ((num-type (continuation-type num))
+	(shift-type (continuation-type shift)))
+    ;; Can only handle right shifts
+    (unless (csubtypep shift-type (specifier-type '(integer * 0)))
+      (give-up))
+
+    ;; If we can prove the shift is so large that all bits are shifted
+    ;; out, return the appropriate constant.  If the shift is small
+    ;; enough, call the VOP.  Otherwise, check for the shift size and
+    ;; do the appropriate thing.  (Hmm, could we just leave the IF
+    ;; s-expr and depend on other parts of the compiler to delete the
+    ;; unreachable parts, if any?)
+    (cond ((csubtypep num-type (specifier-type '(signed-byte #.vm:word-bits)))
+	   ;; A right shift by 31 is the same as a right shift by
+	   ;; larger amount.  We get just the sign.
+	   (if (csubtypep shift-type (specifier-type '(integer #.(- 1 vm:word-bits) 0)))
+	       `(sparc::ash-right-signed num (- shift))
+	       `(sparc::ash-right-signed num (min (- shift) #.(1- vm:word-bits)))))
+	  ((csubtypep num-type (specifier-type '(unsigned-byte #.vm:word-bits)))
+	   (if (csubtypep shift-type (specifier-type '(integer #.(- 1 vm:word-bits) 0)))
+	       `(sparc::ash-right-unsigned num (- shift))
+	       `(if (<= shift #.(- vm:word-bits))
+		 0
+		 (sparc::ash-right-unsigned num (- shift)))))
+	  (t
+	   (give-up)))))
+
