@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/typetran.lisp,v 1.35 1998/07/24 17:22:28 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/typetran.lisp,v 1.39 2000/08/09 13:00:17 dtc Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -333,6 +333,29 @@
 			    types)))))))
 
 
+;;; Source-Transform-Cons-Typep  --  Internal
+;;;
+;;; If necessary recurse to check the cons type.
+;;;
+(defun source-transform-cons-typep (object type)
+  (let* ((car-type (cons-type-car-type type))
+	 (cdr-type (cons-type-cdr-type type)))
+    (let ((car-test-p (not (or (type= car-type *wild-type*)
+			       (type= car-type (specifier-type t)))))
+	  (cdr-test-p (not (or (type= cdr-type *wild-type*)
+			       (type= cdr-type (specifier-type t))))))
+      (if (and (not car-test-p) (not cdr-test-p))
+	  `(consp ,object)
+	  (once-only ((n-obj object))
+	    `(and (consp ,n-obj)
+		  ,@(if car-test-p
+			`((typep (car ,n-obj)
+				 ',(type-specifier car-type))))
+		  ,@(if cdr-test-p
+			`((typep (cdr ,n-obj)
+				 ',(type-specifier cdr-type))))))))))
+
+
 ;;; FIND-SUPERTYPE-PREDICATE  --  Internal
 ;;;
 ;;;    Return the predicate and type from the most specific entry in
@@ -405,7 +428,7 @@
 ;;; for the object is invalid and signal an error if so.  Otherwise, look up
 ;;; the indirect class-cell and call CLASS-CELL-TYPEP at runtime.
 ;;;
-(deftransform %instance-typep ((object spec))
+(deftransform %instance-typep ((object spec) (* *) * :node node :when :both)
   (assert (constant-continuation-p spec))
   (let* ((spec (continuation-value spec))
 	 (class (specifier-type spec))
@@ -425,6 +448,8 @@
 			class:~%  ~S"
 		       class))
       (t
+       ;; Delay the type transform to give type propagation a chance.
+       (delay-transform node :constraint)
        ;; Otherwise transform the type test.
        (multiple-value-bind
 	     (pred get-layout)
@@ -447,7 +472,7 @@
 				  (%layout-invalid-error object ',layout))))
 		      (eq ,n-layout ',layout)))))
 	   ((and (typep class 'basic-structure-class) layout)
-	    ;; Structure type tests; hierarchal layout depths.
+	    ;; Structure type tests; hierarchical layout depths.
 	    (let ((idepth (layout-inheritance-depth layout))
 		  (n-layout (gensym)))
 	      `(and (,pred object)
@@ -462,6 +487,23 @@
 				 (eq (svref (layout-inherits ,n-layout) 
 					    ,idepth)
 				     ',layout))))))))
+	   ((and layout (>= (layout-inheritance-depth layout) 0))
+	    ;; Hierarchical layout depths.
+	    (let ((idepth (layout-inheritance-depth layout))
+		  (n-layout (gensym))
+		  (n-inherits (gensym)))
+	      `(and (,pred object)
+		    (let ((,n-layout (,get-layout object)))
+		      ,@(when (policy nil (>= safety speed))
+			      `((when (layout-invalid ,n-layout)
+				  (%layout-invalid-error object ',layout))))
+		      (if (eq ,n-layout ',layout)
+			  t
+			  (let ((,n-inherits (layout-inherits ,n-layout)))
+			    (declare (optimize (safety 0)))
+			    (and (> (length ,n-inherits) ,idepth)
+				 (eq (svref ,n-inherits ,idepth)
+				     ',layout))))))))
 	   (t
 	    `(and (,pred object)
 		  (class-cell-typep (,get-layout object)
@@ -512,6 +554,8 @@
 		    `(%instance-typep ,object ,spec))
 		   (array-type
 		    (source-transform-array-typep object type))
+		   (cons-type
+		    (source-transform-cons-typep object type))
 		   (t nil)))
 	    `(%typep ,object ,spec)))
       (values nil t)))
