@@ -7,7 +7,7 @@
 ;;; Scott Fahlman or slisp-group@cs.cmu.edu.
 ;;;
 (ext:file-comment
- "$Header: /home/CVS-cmucl/src/compiler/x86/arith.lisp,v 1.12 2000/01/16 20:12:23 dtc Exp $")
+ "$Header: /home/CVS-cmucl/src/compiler/x86/arith.lisp,v 1.14 2001/11/03 22:27:01 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -16,7 +16,7 @@
 ;;; Written by William Lott.
 ;;;
 ;;; Debugged by Paul F. Werkowski Spring/Summer 1995.
-;;; Enhancements/debugging by Douglas T. Crosher 1996,1997.
+;;; Enhancements/debugging by Douglas T. Crosher 1996,1997,2000.
 ;;; 
 
 (in-package :x86)
@@ -330,6 +330,34 @@
 	       (inst inc r)
 	     (inst add r y))))))
 
+
+;;;; Special logand cases: (logand signed unsigned) => unsigned
+
+(define-vop (fast-logand/signed-unsigned=>unsigned
+	     fast-logand/unsigned=>unsigned)
+  (:args (x :target r :scs (signed-reg)
+	    :load-if (not (and (sc-is x signed-stack)
+			       (sc-is y unsigned-reg)
+			       (sc-is r unsigned-stack)
+			       (location= x r))))
+	 (y :scs (unsigned-reg unsigned-stack)))
+  (:arg-types signed-num unsigned-num))
+
+(define-vop (fast-logand-c/signed-unsigned=>unsigned
+	     fast-logand-c/unsigned=>unsigned)
+  (:args (x :target r :scs (signed-reg signed-stack)))
+  (:arg-types signed-num (:constant (unsigned-byte 32))))
+
+(define-vop (fast-logand/unsigned-signed=>unsigned
+	     fast-logand/unsigned=>unsigned)
+  (:args (x :target r :scs (unsigned-reg)
+	    :load-if (not (and (sc-is x unsigned-stack)
+			       (sc-is y signed-reg)
+			       (sc-is r unsigned-stack)
+			       (location= x r))))
+	 (y :scs (signed-reg signed-stack)))
+  (:arg-types unsigned-num signed-num))
+
 
 ;;;; Multiplication and division.
 
@@ -624,7 +652,7 @@
     (inst shl result :cl)))
 
 (define-vop (fast-ash-c/unsigned=>unsigned)
-  (:translate ash)
+	    (:translate ash)
   (:policy :fast-safe)
   (:args (number :scs (unsigned-reg) :target result
 		 :load-if (not (and (sc-is number unsigned-stack)
@@ -651,8 +679,10 @@
 		  ;; We don't have to worry about overflow because of the
 		  ;; result type restriction.
 		  (inst shl result amount))
+		 ((< amount -31)
+		  (inst mov result 0))
 		 (t
-		  (inst shr result (min 31 (- amount)))))))))
+		  (inst shr result (- amount))))))))
 
 (define-vop (fast-ash-c/signed=>signed)
   (:translate ash)
@@ -733,7 +763,7 @@
     (inst shl result :cl)))
 
 (define-vop (fast-ash/unsigned=>unsigned)
-  (:translate ash)
+	    (:translate ash)
   (:policy :fast-safe)
   (:args (number :scs (unsigned-reg) :target result)
 	 (amount :scs (signed-reg) :target ecx))
@@ -746,19 +776,20 @@
     (move result number)
     (move ecx amount)
     (inst or ecx ecx)
-    (inst jmp :ns positive)
+    (inst jmp :ns POSITIVE)
     (inst neg ecx)
     (inst cmp ecx 31)
-    (inst jmp :be okay)
-    (inst mov ecx 31)
+    (inst jmp :be OKAY)
+    (inst xor result result)
+    (inst jmp DONE)
     OKAY
     (inst shr result :cl)
-    (inst jmp done)
-      
+    (inst jmp DONE)
+    
     POSITIVE
     ;; The result-type assures us that this shift will not overflow.
     (inst shl result :cl)
-      
+    
     DONE))
 
 (define-vop (fast-ash/signed=>signed)
