@@ -7,7 +7,7 @@
 ;;; Scott Fahlman or slisp-group@cs.cmu.edu.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/compiler/x86/insts.lisp,v 1.19 1999/11/11 16:12:30 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/compiler/x86/insts.lisp,v 1.23 2001/06/26 03:14:43 toy Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -423,10 +423,35 @@
 	  (unless (or firstp (minusp offset))
 	    (write-char #\+ stream))
 	  (if firstp
-	      (disassem:princ16 offset stream)
+	      (progn
+		(disassem:princ16 offset stream)
+		(or (minusp offset)
+		    (nth-value 1
+			       (disassem::note-code-constant-absolute offset
+								      dstate))
+		    (disassem:maybe-note-assembler-routine offset
+							   nil
+							   dstate)
+		    (let ((offs (- offset disassem::nil-addr)))
+		      (when (typep offs 'offset)
+			(or (disassem::maybe-note-nil-indexed-symbol-slot-ref offs
+									      dstate)
+			    (disassem::maybe-note-static-function offs dstate))))))
 	      (princ offset stream))))))
   (write-char #\] stream))
 
+(defun print-imm-data (value stream dstate)
+  (let ((offset (- value disassem::nil-addr)))
+    (if (zerop offset)
+	(format stream "#x~X" value)
+	(format stream "~A" value))
+    (when (typep offset 'offset)
+      (or (disassem::maybe-note-nil-indexed-object offset dstate)
+	  (disassem::maybe-note-assembler-routine value stream dstate)
+	  (nth-value 1
+		     (disassem::note-code-constant-absolute offset
+							    dstate))))))
+
 (defun print-reg/mem (value stream dstate)
   (declare (type (or list reg) value)
 	   (type stream stream)
@@ -578,6 +603,7 @@
 		 (disassem:read-suffix
 		  (width-bits (disassem:dstate-get-prop dstate 'width))
 		  dstate))
+  :printer #'print-imm-data
   )
 
 (disassem:define-argument-type signed-imm-data
@@ -1696,7 +1722,10 @@
 
 (disassem:define-argument-type displacement
   :sign-extend t
-  :use-label #'offset-next)
+  :use-label #'offset-next
+  :printer #'(lambda (value stream dstate)
+	       (disassem:maybe-note-assembler-routine value nil dstate)
+	       (print-label value stream dstate)))
 
 (disassem:define-instruction-format (short-cond-jump 16)
   (op    :field (byte 4 4))
@@ -1850,6 +1879,30 @@
    (emit-byte segment #b11100000)
    (emit-byte-displacement-backpatch segment target)))
 
+
+;;; Conditional move.
+
+(disassem:define-instruction-format (cond-move 24
+				     :default-printer
+				        '('cmov cond :tab reg ", " reg/mem))
+  (prefix  :field (byte 8 0)	:value #b00001111)
+  (op      :field (byte 4 12)	:value #b0100)
+  (cond    :field (byte 4 8)	:type 'condition-code)
+  (reg/mem :fields (list (byte 2 22) (byte 3 16))
+	   			:type 'reg/mem)
+  (reg     :field (byte 3 19)	:type 'reg))
+
+(define-instruction cmov (segment cond dst src)
+  (:printer cond-move ())
+  (:emitter
+   (assert (register-p dst))
+   (let ((size (matching-operand-size dst src)))
+     (assert (or (eq size :word) (eq size :dword)))
+     (maybe-emit-operand-size-prefix segment size))
+   (emit-byte segment #b00001111)
+   (emit-byte segment (dpb (conditional-opcode cond) (byte 4 0) #b01000000))
+   (emit-ea segment src (reg-tn-encoding dst))))
+
 
 ;;;; Conditional byte set.
 
@@ -2670,6 +2723,23 @@
    (emit-byte segment #b11011001)
    (emit-byte segment #b11100100)))
 
+;;; Compare and move ST(i) to ST0.
+;;;
+;;; Intel syntal: FCMOVcc ST, ST(i)
+;;;
+(define-instruction fcmov (segment cond src)
+  #+nil (:printer floating-point ((op '(#b01? #b???))))
+  (:emitter
+   (assert (fp-reg-tn-p src))
+   (emit-byte segment (ecase cond
+			((:b :e :be :u) #b11011010)
+			((:nb :ne :nbe :nu) #b11011011)))
+   (emit-fp-op segment src (ecase cond
+			     ((:b :nb) #b000)
+			     ((:e :ne) #b000)
+			     ((:be :nbe) #b000)
+			     ((:u nu) #b000)))))
+
 ;;;
 ;;; 80387 Specials
 ;;;
