@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/hemlock/netnews.lisp,v 1.14 1994/10/31 04:50:12 ram Exp $")
+  "$Header: /home/CVS-cmucl/src/hemlock/netnews.lisp,v 1.15 2001/03/13 15:49:56 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -819,7 +819,7 @@
     (setf (nn-info-last-batch-p nn-info) last-batch-p)
     (setf (nn-info-from-end-p nn-info) nil)))
 
-(proclaim '(special *nn-last-command-issued*))
+(declaim (special *nn-last-command-issued*))
 
 (defun nn-send-many-head-requests (stream first last out-of-order-p)
   (do ((i first (1+ i)))
