@@ -1,4 +1,4 @@
-/* $Header: /home/CVS-cmucl/src/lisp/Linux-os.h,v 1.11 1999/09/22 14:42:08 dtc Exp $
+/* $Header: /home/CVS-cmucl/src/lisp/Linux-os.h,v 1.13 2000/10/21 12:40:56 dtc Exp $
 
  This code was written as part of the CMU Common Lisp project at
  Carnegie Mellon University, and has been placed in the public domain.
@@ -58,6 +58,11 @@ typedef struct sigcontext_struct sigcontext;
 
 #define POSIX_SIGS
 
+/* Don't want the SIGINFO flag on linux as it causes the creation
+   of real-time interrupt frames.
+*/
+#define USE_SA_SIGINFO 0
+
 /* Alpha uses OSF/1 signals which are the defaults in os.h,
    so there is no need to define the following for Alpha 
    Linux 
@@ -72,7 +77,7 @@ typedef struct sigcontext_struct sigcontext;
 #define GET_CONTEXT int code=0; struct sigcontext_struct *context=&contextstruct;
 #endif
 
-#define setfpucw(cw)	asm("fldcw %0" : : "m" (cw));
+#define setfpucw(cw)	asm("fldcw %0" : : "m" (cw))
 
 #define sigvec          sigaction
 #define sv_mask         sa_mask
