@@ -7,7 +7,7 @@
  *
  * Douglas Crosher, 1996, 1997.
  *
- * $Header: /home/CVS-cmucl/src/lisp/gencgc.h,v 1.3 2000/01/09 19:36:08 dtc Exp $
+ * $Header: /home/CVS-cmucl/src/lisp/gencgc.h,v 1.6 2000/10/27 19:25:55 dtc Exp $
  *
  */
 
@@ -137,9 +137,8 @@ struct page {
 #define PAGE_SIZE 4096
 
 
-/* The number of pages needed for the dynamic space - rounding up. */
-#define NUM_PAGES ((DYNAMIC_SPACE_SIZE + (PAGE_SIZE - 1)) / PAGE_SIZE)
-extern struct page page_table[NUM_PAGES];
+extern unsigned dynamic_space_pages;
+extern struct page *page_table;
 
 
 /*
@@ -164,8 +163,11 @@ extern struct alloc_region  unboxed_region;
 void  gencgc_pickup_dynamic(void);
 
 void sniff_code_object(struct code *code, unsigned displacement);
+lispobj *search_dynamic_space(lispobj *pointer);
+void update_x86_dynamic_space_free_pointer(void);
 
-int  update_x86_dynamic_space_free_pointer(void);
-void  gc_alloc_update_page_tables(int unboxed,
-				  struct alloc_region *alloc_region);
+lispobj * component_ptr_from_pc(lispobj *pc);
+
+void gc_alloc_update_page_tables(int unboxed,
+				 struct alloc_region *alloc_region);
 #endif _GENCGC_H_
