@@ -1,4 +1,4 @@
-/* $Header: /home/CVS-cmucl/src/lisp/interrupt.c,v 1.21 1999/11/11 16:14:16 dtc Exp $ */
+/* $Header: /home/CVS-cmucl/src/lisp/interrupt.c,v 1.26 2001/12/06 19:15:45 pmai Exp $ */
 
 /* Interrupt handing magic. */
 
@@ -34,7 +34,7 @@ void (*interrupt_low_level_handlers[NSIG]) (HANDLER_ARGS) = {0};
 
 static int pending_signal = 0;
 
-#ifdef SOLARIS
+#if defined(SOLARIS) || defined(__OpenBSD__)
 static siginfo_t *pending_code;
 #define PASSCODE(code) ((code))
 #define DEREFCODE(code) ((code))
@@ -233,7 +233,7 @@ interrupt_handle_pending(struct sigcontext *context)
 
     if (pending_signal) {
 	int signal;
-#ifdef SOLARIS
+#if defined(SOLARIS) || defined(__OpenBSD__)
 	siginfo_t *code;
 #else
 	int code;
@@ -271,7 +271,8 @@ interrupt_handle_now(HANDLER_ARGS)
      * Restore the FPU control word, setting the rounding mode to nearest.
      */
 
-    setfpucw(contextstruct.fpstate->cw & ~0xc00);
+    if (contextstruct.fpstate)
+      setfpucw(contextstruct.fpstate->cw & ~0xc00);
 #endif
 
     handler = interrupt_handlers[signal];
@@ -359,14 +360,6 @@ maybe_now_maybe_later(HANDLER_ARGS)
 
     SAVE_CONTEXT(); /**/
 
-#if defined(__linux__) && defined(i386)
-    /*
-     * Restore the FPU control word, setting the rounding mode to nearest.
-     */
-
-    setfpucw(contextstruct.fpstate->cw & ~0xc00);
-#endif
-
     if (SymbolValue(INTERRUPTS_ENABLED) == NIL) {
         pending_signal = signal;
         pending_code = DEREFCODE(code);
@@ -419,12 +412,22 @@ maybe_now_maybe_later(HANDLER_ARGS)
 #endif /* POSIX_SIGS */
 
 	arch_set_pseudo_atomic_interrupted(context);
-    } else
+    } else {
+#if defined(__linux__) && defined(i386)
+      /*
+       * Restore the FPU control word, setting the rounding mode to nearest.
+       */
+
+      if (contextstruct.fpstate)
+	setfpucw(contextstruct.fpstate->cw & ~0xc00);
+#endif
+
 #if ( defined( __linux__ ) && defined( i386 ) )
         interrupt_handle_now(signal, contextstruct);
 #else
         interrupt_handle_now(signal, code, context);
 #endif
+    }
 }
 
 /****************************************************************\
@@ -440,7 +443,7 @@ static boolean gc_trigger_hit(HANDLER_ARGS)
 	lispobj *badaddr=(lispobj *)arch_get_bad_addr(signal, code, context);
 
 	return (badaddr >= current_auto_gc_trigger &&
-		badaddr < current_dynamic_space + DYNAMIC_SPACE_SIZE);
+		badaddr < current_dynamic_space + dynamic_space_size);
     }
 }
 #endif
@@ -508,46 +511,16 @@ void interrupt_install_low_level_handler
     sa.sa_sigaction = handler;
     sigemptyset(&sa.sa_mask);
     FILLBLOCKSET(&sa.sa_mask);
-#if defined(__linux__)
-    /*
-     * Don't want the SIGINFO flag on linux as it causes the creation
-     * of real-time interrupt frames.
-     */
-    sa.sa_flags = SA_RESTART;
-#else
-    sa.sa_flags = SA_RESTART | SA_SIGINFO;
-#endif
+    sa.sa_flags = SA_RESTART | USE_SA_SIGINFO;
 
     sigaction(signal, &sa, NULL);
-#else
-#if defined __FreeBSD__ && 0	/* hack in progress */
-/* sort of like POSIX -- maybe fixed in new release */
-#define FILLBLOCKSET(s) (sigaddset(s, SIGHUP), sigaddset(s, SIGINT), \
-		   sigaddset(s, SIGQUIT), sigaddset(s, SIGPIPE), \
-		   sigaddset(s, SIGALRM), sigaddset(s, SIGURG), \
-		   sigaddset(s, SIGTSTP), sigaddset(s, SIGCHLD), \
-		   sigaddset(s, SIGIO), sigaddset(s, SIGXCPU), \
-                   sigaddset(s, SIGXFSZ), sigaddset(s, SIGVTALRM), \
-		   sigaddset(s, SIGPROF), sigaddset(s, SIGWINCH), \
-		   sigaddset(s, SIGUSR1), sigaddset(s,S IGUSR2))
-
-    struct sigaction sa;
-    sa.sa_handler = handler;
-    sigemptyset(&sa.sa_mask);
-    FILLBLOCKSET(&sa.sa_mask);
-    sa.sa_flags = SA_RESTART | SA_SIGINFO;
-    sa.sa_flags |= SA_ONSTACK;
 #else
     struct sigvec sv;
 
     sv.sv_handler = handler;
     sv.sv_mask = BLOCKABLE;
     sv.sv_flags = 0;
-#if defined USE_SIG_STACK
-    sv.sv_flags |= SV_ONSTACK;	/* use separate stack */
-#endif
     sigvec(signal, &sv, NULL);
-#endif
 #endif    
     interrupt_low_level_handlers[signal] =
       (handler == (void (*)(HANDLER_ARGS)) SIG_DFL) ? 0 : handler;
@@ -579,15 +552,7 @@ unsigned long install_handler(int signal,
 
 	sigemptyset(&sa.sa_mask);
 	FILLBLOCKSET(&sa.sa_mask);
-#if defined(__linux__)
-	/*
-	 * Don't want the SIGINFO flag on linux as it causes the
-	 * creation of real-time interrupt frames.
-	 */
-	sa.sa_flags = SA_RESTART;
-#else
-	sa.sa_flags = SA_SIGINFO | SA_RESTART;
-#endif
+	sa.sa_flags = USE_SA_SIGINFO | SA_RESTART;
 
 	sigaction(signal, &sa, NULL);
     }
@@ -618,9 +583,6 @@ unsigned long install_handler(int signal,
 
 	sv.sv_mask = BLOCKABLE;
 	sv.sv_flags = 0;
-#if defined USE_SIG_STACK
-	sv.sv_flags = SV_ONSTACK;
-#endif
 	sigvec(signal, &sv, NULL);
     }
 
