@@ -1,7 +1,7 @@
 /*
  * main() entry point for a stand alone lisp image.
  *
- * $Header: /home/CVS-cmucl/src/lisp/lisp.c,v 1.16 1999/09/09 16:22:44 pw Exp $
+ * $Header: /home/CVS-cmucl/src/lisp/lisp.c,v 1.24 2001/11/29 01:46:59 pmai Exp $
  *
  */
 
@@ -25,6 +25,7 @@
 #include "gc.h"
 #include "monitor.h"
 #include "validate.h"
+#include "interr.h"
 #if defined GENCGC
 #include "gencgc.h"
 #endif
@@ -102,29 +103,63 @@ int main(int argc, char *argv[], char *envp[])
     set_lossage_handler(ldb_monitor);
 
     monitor = FALSE;
+#ifdef DEFAULT_DYNAMIC_SPACE_SIZE
+    dynamic_space_size = DEFAULT_DYNAMIC_SPACE_SIZE;
+#else
+    dynamic_space_size = DYNAMIC_SPACE_SIZE;
+#endif
 
     lazy_memory_allocation = 0;
     argptr = argv;
-    while ((arg = *++argptr) != NULL) {
-        if (strcmp(arg, "-core") == 0) {
-            if (core != NULL) {
+    while ((arg = *++argptr) != NULL)
+      {
+        if (strcmp(arg, "-core") == 0)
+	  {
+            if (core != NULL)
+	      {
                 fprintf(stderr, "can only specify one core file.\n");
                 exit(1);
-            }
+	      }
             core = *++argptr;
-            if (core == NULL) {
-                fprintf(stderr, "-core must be followed by the name of the core file to use.\n");
+            if (core == NULL)
+	      {
+		fprintf(stderr, "-core must be followed by the name of the core file to use.\n");
                 exit(1);
             }
         }
 	else if (strcmp(arg, "-lazy") == 0) {
+          fprintf(stderr,"Lazy memory allocation doesn't work for now. Disabling...\n");
+	  lazy_memory_allocation = 0;
+#if 0           
 	  fprintf(stderr,"Using lazy memory allocation...\n");      
 	  lazy_memory_allocation = 1;
+#endif
 	}
-	else if (strcmp(arg, "-monitor") == 0) {
+	else if (strcmp(arg, "-nolazy") == 0) {
+	  fprintf(stderr,"Disabeling lazy memory allocation...\n");      
+	  lazy_memory_allocation = 0;
+	}
+        else if (strcmp(arg, "-dynamic-space-size") == 0)
+	  {
+            char *str = *++argptr;
+            if (str == NULL)
+	      {
+                fprintf(stderr, "-dynamic-space-size must be followed by the size to use in MBytes.\n");
+                exit(1);
+	      }
+	    dynamic_space_size = atoi(str) * 1024 * 1024;
+	    if (dynamic_space_size > DYNAMIC_SPACE_SIZE)
+	      {
+                fprintf(stderr, "-dynamic-space-size must be no greater than %d MBytes.\n",
+			DYNAMIC_SPACE_SIZE / (1024 * 1024));
+                exit(1);
+	      }
+	  }
+	else if (strcmp(arg, "-monitor") == 0)
+	  {
 	    monitor = TRUE;
-	}
-    }
+	  }
+      }
 
     default_core = arch_init();
     if (default_core == NULL)
@@ -160,11 +195,7 @@ int main(int argc, char *argv[], char *envp[])
 #ifdef MACH
 	    strcpy(buf, "/usr/misc/.cmucl/lib/");
 #else
-#ifdef __linux__
 	    strcpy(buf, "/usr/lib/cmucl/");
-#else
-	    strcpy(buf, "/usr/local/lib/cmucl/lib/");
-#endif
 #endif
 	    strcat(buf, default_core);
 	    core = buf;
@@ -172,8 +203,8 @@ int main(int argc, char *argv[], char *envp[])
     }
 
     os_init();
-    gc_init();
     validate();
+    gc_init();
 
     /* This is the first use of malloc() and must come after the
      * static memory layout is mmapped to avoid conflicts with possible
@@ -222,8 +253,22 @@ int main(int argc, char *argv[], char *envp[])
     SetSymbolValue(LISP_COMMAND_LINE_LIST, alloc_str_list(argv));
     SetSymbolValue(LISP_ENVIRONMENT_LIST, alloc_str_list(envp));
 
-    /* Pick off sigint until the lisp system gets far enough along to */
-    /* install it's own. */
+    /*
+     * Parse the command line again, picking up values that override
+     * those loaded from the core.
+     */
+
+    argptr = argv;
+    while ((arg = *++argptr) != NULL)
+      {
+	if (strcmp(arg, "-batch") == 0)
+	  SetSymbolValue(BATCH_MODE, T);
+      }
+
+    /*
+     * Pick off sigint until the lisp system gets far enough along to
+     * install it's own.
+     */
     sigint_init();
 
     if (monitor)
