@@ -1,6 +1,6 @@
 /*
 
- $Header: /home/CVS-cmucl/src/lisp/socket.c,v 1.3 1994/10/27 17:13:54 ram Exp $
+ $Header: /home/CVS-cmucl/src/lisp/socket.c,v 1.4 2000/10/26 19:50:19 dtc Exp $
 
  This code was written as part of the CMU Common Lisp project at
  Carnegie Mellon University, and has been placed in the public domain.
@@ -18,6 +18,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <X11/Xos.h>
 #include <X11/Xproto.h>
 #include <errno.h>
@@ -25,17 +26,13 @@
 #include <sys/ioctl.h>
 #include <netdb.h> 
 #include <sys/socket.h>
+#include <arpa/inet.h>
 #ifndef hpux
 #include <netinet/tcp.h>
 #endif
 
 #include "os.h"
 
-#ifndef __linux__
-extern int errno;		/* Certain (broken) OS's don't have this */
-				/* decl in errno.h */
-#endif
-
 #ifdef UNIXCONN
 #include <sys/un.h>
 #ifndef X_UNIX_PATH
@@ -47,12 +44,6 @@ extern int errno;		/* Certain (broken) OS's don't have this */
 #endif /* hpux */
 #endif /* X_UNIX_PATH */
 #endif /* UNIXCONN */
-#ifdef __linux__
-#include <string.h>
-#endif
-#ifndef bcopy
-void bcopy();
-#endif
 
 /* 
  * Attempts to connect to server, given host and display. Returns file 
@@ -68,8 +59,6 @@ int connect_to_server (char *host, int display)
 #ifdef UNIXCONN
   struct sockaddr_un unaddr;	/* UNIX socket address. */
 #endif
-  extern char *getenv();
-  extern struct hostent *gethostbyname();
   int fd;				/* Network socket */
   {
 #ifdef UNIXCONN
