@@ -5,7 +5,7 @@
 ;;; Carnegie Mellon University, and has been placed in the public domain.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/motif/lisp/transport.lisp,v 1.3 1998/12/19 18:44:25 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/motif/lisp/transport.lisp,v 1.4 2000/02/15 11:59:24 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -152,6 +152,7 @@
   (dolist (packet (message-packet-list message))
     (destroy-packet packet)))
 
+
 (defun message-add-packet (message)
   (let ((packet (create-packet)))
     (push packet (message-packet-list message))
@@ -275,23 +276,40 @@
   (warn "Cannot yet handle killing deferred messages."))
 
 (defun defer-packet (packet)
-  (declare (ignore packet))
-  (warn "Cannot yet handle deferring packets."))
+  (let* ((serial (packet-serial packet))
+	 (found (assoc serial *pending-msgs*))
+	 (message (or (cdr found) (make-message serial))))
+    (push packet (message-packet-list message))
+    (incf (message-packet-count message))
+    (cond ((= (message-packet-count message)(packet-sequence-length packet))
+	   (setq *pending-msgs* (delete found *pending-msgs*))
+	   (setf (message-packet-list message)
+		 ;; this can be nreverse if messages really arrive in order
+		 (sort (message-packet-list message) #'<
+		       :key (lambda(pkt)(packet-sequence-number pkt))))
+	   (setf (message-fill-packet message)
+		 (first (message-packet-list message)))
+	   message)
+	  (t (unless found
+	       (setq *pending-msgs* (acons serial message *pending-msgs*)))
+	     nil))))
 
 (defun receive-message (socket)
-  (let* ((first (receive-packet socket))
-	 (count (packet-sequence-length first)))
-    (cond
-     ((zerop count) (kill-deferred-message first))
-     ((= count 1)
-      (let ((message (make-message (packet-serial first))))
-	(setf (message-packet-count message) 1)
-	(push first (message-packet-list message))
-	(setf (message-fill-packet message) first)
-	message))
-     (t
-      (defer-packet first)))))
-
+  (loop
+    (let* ((first (receive-packet socket))
+	   (count (packet-sequence-length first)))
+      (cond
+       ((zerop count) (kill-deferred-message first))
+       ((= count 1)
+	(let ((message (make-message (packet-serial first))))
+	  (setf (message-packet-count message) 1)
+	  (push first (message-packet-list message))
+	  (setf (message-fill-packet message) first)
+	  (return message)))
+       (t
+	(let ((message (defer-packet first)))
+	  (when message
+	    (return message))))))))
 
 
 ;;;; Functions for handling requests
