@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
- "$Header: /home/CVS-cmucl/src/pcl/boot.lisp,v 1.20 1999/05/30 23:13:51 pw Exp $")
+ "$Header: /home/CVS-cmucl/src/pcl/boot.lisp,v 1.26 2001/05/10 21:28:41 pmai Exp $")
 
 (in-package :pcl)
 
@@ -74,13 +74,12 @@ work during bootstrapping.
 
 |#
 
-(proclaim '(notinline make-a-method
-		      add-named-method		      
-		      ensure-generic-function-using-class
-
-		      add-method
-		      remove-method
-		      ))
+(declaim (notinline make-a-method
+		    add-named-method		      
+		    ensure-generic-function-using-class
+		    add-method
+		    remove-method
+		    ))
 
 (defvar *early-functions*
 	'((make-a-method early-make-a-method
@@ -163,6 +162,12 @@ work during bootstrapping.
 ;;;
 ;;;
 ;;;
+(defun tell-compiler-about-gf (function-specifier lambda-list)
+  ;; Supress any undefined function warnings from compiler.
+  ;; I had originally lifted some code from c::%%defun but this
+  ;; seems to do the job just as well.
+  (proclaim-defgeneric function-specifier lambda-list))
+
 (defmacro defgeneric (function-specifier lambda-list &body options)
   (expand-defgeneric function-specifier lambda-list options))
 
@@ -217,6 +222,9 @@ work during bootstrapping.
 
 	(let ((declarations (initarg :declarations)))
 	  (when declarations (initarg :declarations `',declarations))))
+
+      (tell-compiler-about-gf function-specifier lambda-list)
+
     `(progn 
        (proclaim-defgeneric ',function-specifier ',lambda-list)
        ,(make-top-level-form
@@ -306,6 +314,7 @@ work during bootstrapping.
 	(let ((initargs-form (make-method-initargs-form 
 			      proto-gf proto-method
 			      method-function-lambda initargs env)))
+	  (tell-compiler-about-gf name lambda-list)
 	  `(progn
 	     (proclaim-defgeneric ',name ',lambda-list)
 	     ,@(when *make-instance-function-keys*
@@ -889,7 +898,7 @@ work during bootstrapping.
 				   (if (consp (car var))
 				       (values (caar var) (cadar var))
 				       (values (make-keyword (car var)) (car var)))
-				 `((,key (get-key-arg1 ,keyword ,args-tail))
+				 `((,key (get-key-arg1 ',keyword ,args-tail))
 				   (,variable (if (consp ,key)
 						  (car ,key)
 						  ,(cadr var))))))
@@ -898,7 +907,7 @@ work during bootstrapping.
 				   (if (consp (car var))
 				       (values (caar var) (cadar var))
 				       (values (make-keyword (car var)) (car var)))
-				 `((,key (get-key-arg1 ,keyword ,args-tail))
+				 `((,key (get-key-arg1 ',keyword ,args-tail))
 				   (,(caddr var) ,key)
 				   (,variable (if (consp ,key)
 						  (car ,key)
@@ -1078,7 +1087,6 @@ work during bootstrapping.
   (let ((method-spec (or (getf initargs ':method-spec)
 			 (make-method-spec name quals specls))))
     (setf (getf initargs ':method-spec) method-spec)
-    (record-definition 'method method-spec)
     (load-defmethod-internal class name quals specls ll initargs pv-table-symbol)))
 
 (defun load-defmethod-internal
@@ -1607,7 +1615,10 @@ work during bootstrapping.
 	       (find-method-combination (class-prototype ,gf-class)
 					(car combin)
 					(cdr combin)))))
-     ))
+     (let ((method-class (getf ,all-keys :method-class '.shes-not-there.)))
+       (unless (eq method-class '.shes-not-there.)
+	 (setf (getf ,all-keys :method-class)
+	       (find-class method-class t ,env))))))
      
 (defun real-ensure-gf-using-class--generic-function
        (existing
@@ -1911,7 +1922,7 @@ work during bootstrapping.
 ;;; implemented.
 ;;; 
 (defun parse-defmethod (cdr-of-form)
-  ;;(declare (values name qualifiers specialized-lambda-list body))
+  (declare (list cdr-of-form))
   (let ((name (pop cdr-of-form))
 	(qualifiers ())
 	(spec-ll ()))
@@ -1922,6 +1933,7 @@ work during bootstrapping.
     (values name qualifiers spec-ll cdr-of-form)))
 
 (defun parse-specializers (specializers)
+  (declare (list specializers))
   (flet ((parse (spec)
 	   (let ((result (specializer-from-type spec)))
 	     (if (specializerp result)
