@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/pcl/braid.lisp,v 1.15 1999/05/30 23:13:52 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/pcl/braid.lisp,v 1.19 2001/07/08 16:24:39 pw Exp $")
 ;;;
 ;;; Bootstrapping the meta-braid.
 ;;;
@@ -118,11 +118,6 @@
 	      classes)))		        
 
 (defun bootstrap-meta-braid ()
-  (let* ((name 'class)
-	 (predicate-name (make-type-predicate-name name)))
-    (setf (gdefinition predicate-name)
-	  #'(lambda (x) (declare (ignore x)) t))
-    (do-satisfies-deftype name predicate-name))  
   (let* ((*create-classes-from-internal-structure-definitions-p* nil)
 	 std-class-wrapper std-class
 	 standard-class-wrapper standard-class
@@ -512,17 +507,9 @@
 			:metaclass 'structure-class
 			:name symbol
 			:direct-superclasses
-			(cond ;; Handle the CMUCL structure based conditions.
-			      ((lisp:subtypep symbol 'condition)
-			       (mapcar #'lisp:class-name
-				       (kernel:class-direct-superclasses
-					(lisp:find-class symbol))))
-			      ;; Hack to add the stream class as a
-			      ;; mixin to the lisp-stream class.
-			      ((eq symbol 'sys:lisp-stream)
-			       '(structure-object stream))
-			      ((structure-type-included-type-name symbol)
-			       (list (structure-type-included-type-name symbol))))
+			(mapcar #'lisp:class-name
+				(kernel:class-direct-superclasses
+				 (lisp:find-class symbol)))
 			:direct-slots
 			(mapcar #'slot-initargs-from-structure-slotd
 				(structure-type-slot-description-list symbol)))))
@@ -576,8 +563,9 @@
   (let ((lclass (kernel:layout-class layout)))
     (unless (eq (kernel:class-layout lclass) layout)
       (setf (kernel:layout-inherits layout)
-	    (map 'vector #'class-wrapper
-		 (reverse (rest (class-precedence-list class)))))
+	    (kernel:order-layout-inherits
+	     (map 'simple-vector #'class-wrapper
+		  (reverse (rest (class-precedence-list class))))))
       (kernel:register-layout layout :invalidate nil)
 
       ;; Subclasses of formerly forward-referenced-class may be unknown
@@ -616,9 +604,26 @@
   (setq *boot-state* 'braid)
   )
 
+
+(define-condition no-applicable-method (type-error)
+  ((function :reader no-applicable-method-function :initarg :function)
+   (arguments :reader no-applicable-method-arguments :initarg :arguments))
+  (:report (lambda (condition stream)
+             (format stream "No matching method for the generic-function ~S,~@
+when called with arguments ~S."
+                     (no-applicable-method-function condition)
+                     (no-applicable-method-arguments condition)))))
+
 (defmethod no-applicable-method (generic-function &rest args)
   (cerror "Retry call to ~S"
-	  "No matching method for the generic-function ~S,~@
-          when called with arguments ~S."
-	  generic-function args)
+          'no-applicable-method
+          :function generic-function
+          :arguments args)
   (apply generic-function args))
+
+;; (defmethod no-applicable-method (generic-function &rest args)
+;;   (cerror "Retry call to ~S"
+;; 	  "No matching method for the generic-function ~S,~@
+;;           when called with arguments ~S."
+;; 	  generic-function args)
+;;   (apply generic-function args))
