@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/pcl/cache.lisp,v 1.13 1999/05/30 23:13:53 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/pcl/cache.lisp,v 1.14 2000/07/06 08:15:02 dtc Exp $")
 ;;;
 ;;; The basics of the PCL wrapper cache mechanism.
 ;;;
@@ -604,7 +604,7 @@
 	(values (logxor (the fixnum (1- cache-size)) (the fixnum (1- line-size)))
 		cache-size
 		line-size
-		(the fixnum (floor cache-size line-size))))
+		(the (values fixnum t) (floor cache-size line-size))))
       (let* ((line-size (power-of-two-ceiling (if valuep (1+ nkeys) nkeys)))
 	     (cache-size (if (typep nlines-or-cache-vector 'fixnum)
 			     (the fixnum
@@ -617,7 +617,7 @@
 	(values (logxor (the fixnum (1- cache-size)) (the fixnum (1- line-size)))
 		(the fixnum (1+ cache-size))
 		line-size
-		(the fixnum (floor cache-size line-size))))))
+		(the (values fixnum t) (floor cache-size line-size))))))
 
 
 
