@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/pcl/defclass.lisp,v 1.16 1999/05/30 23:13:55 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/pcl/defclass.lisp,v 1.19 2001/03/13 15:49:47 pw Exp $")
 ;;;
 
 (in-package :pcl)
@@ -97,12 +97,12 @@
     ;; TBD - ANSI compliant class and slot option checking.
     (dolist (option options)
       (if (not (listp option))
-          (error 'simple-program-error
+          (error 'kernel:simple-program-error
 		 :format-control "~S is not a legal defclass option."
 		 :format-arguments (list option))
           (when (eq (car option) ':metaclass)
             (unless (legal-class-name-p (cadr option))
-              (error 'simple-program-error
+              (error 'kernel:simple-program-error
 		     :format-control 
 		     "The value of the :metaclass option (~S) is not a~%~
 		      legal class name."
@@ -160,7 +160,10 @@
 	      (progn
 		(when (and (eq *boot-state* 'complete)
 			   (not (member 'compile *defclass-times*)))
-		  (inform-type-system-about-std-class name))
+		  (when (eq metaclass 'standard-class)
+		    ;; a funcallable-standard-class metaclass here causes
+		    ;; a compiler crash in construct.lisp
+		    (inform-type-system-about-std-class name)))
 		defclass-form)))))))
 
 (defun make-initfunction (initform)
@@ -393,7 +396,7 @@
 (defun early-class-direct-subclasses (class)
   (bootstrap-get-slot 'class class 'direct-subclasses))
 
-(proclaim '(notinline load-defclass))
+(declaim (notinline load-defclass))
 (defun load-defclass (name metaclass supers canonical-slots canonical-options)
   (setq supers  (copy-tree supers)
 	canonical-slots   (copy-tree canonical-slots)
