@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/pcl/defcombin.lisp,v 1.10 1999/05/30 23:13:55 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/pcl/defcombin.lisp,v 1.12 2001/09/23 19:02:12 pmai Exp $")
 ;;;
 
 (in-package :pcl)
@@ -42,6 +42,29 @@
       (expand-long-defcombin form)
       (expand-short-defcombin form)))
 
+
+;;;
+;;; Implementation of INVALID-METHOD-ERROR and METHOD-COMBINATION-ERROR
+;;;
+;;; See combin.lisp for rest of the implementation.  This method is
+;;; defined here because compute-effective-method is still a function
+;;; in combin.lisp.
+;;;
+(defmethod compute-effective-method :around
+    ((generic-function generic-function)
+     (method-combination method-combination)
+     applicable-methods)
+  (declare (ignore applicable-methods))
+  (flet ((real-invalid-method-error (method format-string &rest args)
+	   (declare (ignore method))
+	   (apply #'error format-string args))
+	 (real-method-combination-error (format-string &rest args)
+	   (apply #'error format-string args)))
+    (let ((*invalid-method-error* #'real-invalid-method-error)
+	  (*method-combination-error* #'real-method-combination-error))
+      (call-next-method))))
+
+
 
 ;;;
 ;;; STANDARD method combination
@@ -148,6 +171,7 @@
   (let ((type (method-combination-type combin))
 	(operator (short-combination-operator combin))
 	(ioa (short-combination-identity-with-one-argument combin))
+	(order (car (method-combination-options combin)))
 	(around ())
 	(primary ()))
     (dolist (m applicable-methods)
@@ -171,8 +195,9 @@
 		 (push m primary))
 		(t
 		 (lose m "has an illegal qualifier"))))))
-    (setq around (nreverse around)
-	  primary (nreverse primary))
+    (setq around (nreverse around))
+    (unless (eq order :most-specific-last)
+      (setq primary (nreverse primary)))
     (let ((main-method
 	    (if (and (null (cdr primary))
 		     (not (null ioa)))
