@@ -1,4 +1,4 @@
-;;;-*-Mode:LISP; Package:(PCL LISP 1000); Base:10; Syntax:Common-lisp -*-
+;;;-*-Mode:LISP; Package:PCL  -*-
 ;;;
 ;;; *************************************************************************
 ;;; Copyright (c) 1985, 1986, 1987, 1988, 1989, 1990 Xerox Corporation.
@@ -70,55 +70,55 @@
 ;;;
 ;;; If symbol names a function which is traced or advised, redefine
 ;;; the `real' definition without affecting the advise.
-;;;
+;;
 (defun (setf gdefinition) (new-definition name)
   (c::%%defun name new-definition nil)
   (c::note-name-defined name :function)
   new-definition)
 
-(proclaim '(special *the-class-t* 
-	            *the-class-vector* *the-class-symbol*
-                    *the-class-string* *the-class-sequence*
-                    *the-class-rational* *the-class-ratio*
-                    *the-class-number* *the-class-null* *the-class-list*
-                    *the-class-integer* *the-class-float* *the-class-cons*
-                    *the-class-complex* *the-class-character*
-                    *the-class-bit-vector* *the-class-array*
-		    *the-class-stream*
+(declaim (special *the-class-t* 
+		  *the-class-vector* *the-class-symbol*
+		  *the-class-string* *the-class-sequence*
+		  *the-class-rational* *the-class-ratio*
+		  *the-class-number* *the-class-null* *the-class-list*
+		  *the-class-integer* *the-class-float* *the-class-cons*
+		  *the-class-complex* *the-class-character*
+		  *the-class-bit-vector* *the-class-array*
+		  *the-class-stream*
 
-                    *the-class-slot-object*
-                    *the-class-structure-object*
-                    *the-class-std-object*
-                    *the-class-standard-object*
-                    *the-class-funcallable-standard-object*
-                    *the-class-class*
-                    *the-class-generic-function*
-                    *the-class-built-in-class*
-                    *the-class-slot-class*
-                    *the-class-structure-class*
-                    *the-class-std-class*
-                    *the-class-standard-class*
-                    *the-class-funcallable-standard-class*
-                    *the-class-method*
-                    *the-class-standard-method*
-	            *the-class-standard-reader-method*
-	            *the-class-standard-writer-method*
-	            *the-class-standard-boundp-method*
-                    *the-class-standard-generic-function*
-                    *the-class-standard-effective-slot-definition*
+		  *the-class-slot-object*
+		  *the-class-structure-object*
+		  *the-class-std-object*
+		  *the-class-standard-object*
+		  *the-class-funcallable-standard-object*
+		  *the-class-class*
+		  *the-class-generic-function*
+		  *the-class-built-in-class*
+		  *the-class-slot-class*
+		  *the-class-structure-class*
+		  *the-class-std-class*
+		  *the-class-standard-class*
+		  *the-class-funcallable-standard-class*
+		  *the-class-method*
+		  *the-class-standard-method*
+		  *the-class-standard-reader-method*
+		  *the-class-standard-writer-method*
+		  *the-class-standard-boundp-method*
+		  *the-class-standard-generic-function*
+		  *the-class-standard-effective-slot-definition*
+		  
+		  *the-eslotd-standard-class-slots*
+		  *the-eslotd-funcallable-standard-class-slots*))
 
-                    *the-eslotd-standard-class-slots*
-                    *the-eslotd-funcallable-standard-class-slots*))
-
-(proclaim '(special *the-wrapper-of-t*
-                    *the-wrapper-of-vector* *the-wrapper-of-symbol*
-                    *the-wrapper-of-string* *the-wrapper-of-sequence*
-                    *the-wrapper-of-rational* *the-wrapper-of-ratio*
-                    *the-wrapper-of-number* *the-wrapper-of-null*
-                    *the-wrapper-of-list* *the-wrapper-of-integer*
-                    *the-wrapper-of-float* *the-wrapper-of-cons*
-                    *the-wrapper-of-complex* *the-wrapper-of-character*
-                    *the-wrapper-of-bit-vector* *the-wrapper-of-array*))
+(declaim (special *the-wrapper-of-t*
+		  *the-wrapper-of-vector* *the-wrapper-of-symbol*
+		  *the-wrapper-of-string* *the-wrapper-of-sequence*
+		  *the-wrapper-of-rational* *the-wrapper-of-ratio*
+		  *the-wrapper-of-number* *the-wrapper-of-null*
+		  *the-wrapper-of-list* *the-wrapper-of-integer*
+		  *the-wrapper-of-float* *the-wrapper-of-cons*
+		  *the-wrapper-of-complex* *the-wrapper-of-character*
+		  *the-wrapper-of-bit-vector* *the-wrapper-of-array*))
 
 ;;;; Type specifier hackery:
 
@@ -181,22 +181,21 @@
   (specializer-type (class-eq-specializer class)))
 
 (defun inform-type-system-about-std-class (name)
-  (let ((predicate-name (make-type-predicate-name name)))
-    (setf (gdefinition predicate-name) (make-type-predicate name))
-    (do-satisfies-deftype name predicate-name)))
-
-(defun make-type-predicate (name)
-  (let ((cell (find-class-cell name)))
-    #'(lambda (x)
-	(funcall (the function (find-class-cell-predicate cell)) x))))
-
-
-;This stuff isn't right.  Good thing it isn't used.
-;The satisfies predicate has to be a symbol.  There is no way to
-;construct such a symbol from a class object if class names change.
-(defun class-predicate (class)
-  (when (symbolp class) (setq class (find-class class)))
-  #'(lambda (object) (memq class (class-precedence-list (class-of object)))))
+  ;; This should only be called if metaclass is standard-class.
+  ;; Compiler problems have been seen if the metaclass is 
+  ;; funcallable-standard-class and this is called from the defclass macro
+  ;; expander. However, bootstrap-meta-braid calls this for funcallable-
+  ;; standard-class metaclasses but *boot-state* is not 'complete then.
+  ;;
+  ;; The only effect of this code is to ensure a lisp:standard-class class
+  ;; exists so as to avoid undefined-function compiler warnings. The
+  ;; skeleton class will be replaced at load-time with the correct object.
+  ;; Earlier revisions (<= 1.17) of this function were essentially NOOPs.
+  (declare (ignorable name))
+  (when (and (eq *boot-state* 'complete)
+	     (null (lisp:find-class name nil)))
+    (setf (lisp:find-class name)
+	  (lisp::make-standard-class :name name))))
 
 (defun make-class-eq-predicate (class)
   (when (symbolp class) (setq class (find-class class)))
@@ -272,23 +271,6 @@
 	      (t
 	       (subtypep (convert-to-system-type type1)
 			 (convert-to-system-type type2))))))))
-
-(defun do-satisfies-deftype (name predicate)
-  (declare (ignore name predicate)))
-
-(defun make-type-predicate-name (name &optional kind)
-  (if (symbol-package name)
-      (intern (format nil
-		      "~@[~A ~]TYPE-PREDICATE ~A ~A"
-		      kind
-		      (package-name (symbol-package name))
-		      (symbol-name name))
-	      *the-pcl-package*)
-      (make-symbol (format nil
-			   "~@[~A ~]TYPE-PREDICATE ~A"
-			   kind
-			   (symbol-name name)))))
-
 
 
 (defvar *built-in-class-symbols* ())
@@ -459,7 +441,7 @@
 (defclass kernel:funcallable-instance (function) ()
   (:metaclass built-in-class))
 
-(defclass stream (t) ()
+(defclass stream (kernel:instance) ()
   (:metaclass built-in-class))
 
 (defclass slot-object (t) ()
