@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/pcl/defsys.lisp,v 1.21 1999/05/30 23:13:56 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/pcl/defsys.lisp,v 1.24 2001/04/10 22:37:21 pw Exp $")
 ;;;
 ;;; Some support stuff for compiling and loading PCL.  It would be nice if
 ;;; there was some portable make-system we could all agree to share for a
@@ -468,7 +468,7 @@ and load your system with:
            *pcl-directory*
   ;;
   ;; file         load           compile      files which       port
-  ;;              environment    environment  force the of
+  ;;              environment    environment  force the
   ;;                                          recompilation
   ;;                                          of this file
   ;;                                          
@@ -479,8 +479,6 @@ and load your system with:
    (macros          t            t            ())
    (low             (pkg macros) t            (macros))
    
-   (cmu-low        (low)         (low)        (low)            CMU)
-   
    (fin         t                                   t (low))
    (defclass    t                                   t (low))
    (defs        t                                   t (defclass macros iterate))
@@ -514,19 +512,42 @@ and load your system with:
    (precom2     (dlisp)                             t (defs low cache fin dfun))
    ))
 
+(defsystem gray-streams
+           *pcl-directory*
+  ;;
+  ;; file         load           compile      files which       port
+  ;;              environment    environment  force the
+  ;;                                          recompilation
+  ;;                                          of this file
+  ;;                                          
+  (
+   (gray-streams-class  t            t            ()                   CMU)
+   (gray-streams        t            t            (gray-streams-class) CMU)
+   ))
+
 (defun compile-pcl (&optional m)
-  (cond ((null m)        (operate-on-system 'pcl :compile))
-	((eq m :print)   (operate-on-system 'pcl :compile () t))
-	((eq m :query)   (operate-on-system 'pcl :query-compile))
-	((eq m :confirm) (operate-on-system 'pcl :confirm-compile))
-	((eq m 't)       (operate-on-system 'pcl :recompile))        
-	((listp m)       (operate-on-system 'pcl :compile-from m))
-	((symbolp m)     (operate-on-system 'pcl :recompile-some `(,m)))))
+  (cond ((null m)
+	 (operate-on-system 'pcl :compile)
+	 (operate-on-system 'gray-streams :compile))
+	((eq m :print)
+	 (operate-on-system 'pcl :compile () t))
+	((eq m :query)
+	 (operate-on-system 'pcl :query-compile))
+	((eq m :confirm)
+	 (operate-on-system 'pcl :confirm-compile))
+	((eq m 't)
+	 (operate-on-system 'pcl :recompile)
+	 (operate-on-system 'gray-streams :recompile))
+	((listp m)
+	 (operate-on-system 'pcl :compile-from m))
+	((symbolp m)
+	 (operate-on-system 'pcl :recompile-some `(,m)))))
 
 (defun load-pcl (&optional m)
-  (cond ((null m)      (operate-on-system 'pcl :load))
-	((eq m :query) (operate-on-system 'pcl :query-load))))
-
+  (cond ((null m)
+	 (operate-on-system 'pcl :load))
+	((eq m :query)
+	 (operate-on-system 'pcl :query-load))))
 
 (defun bug-report-info (&optional (stream *standard-output*))
   (format stream "~&PCL system date: ~A~
