@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/pcl/env.lisp,v 1.12 1999/05/30 23:13:58 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/pcl/env.lisp,v 1.13 2001/03/14 23:25:16 pw Exp $")
 ;;;
 ;;; Basic environmental stuff.
 ;;;
@@ -37,6 +37,10 @@
 ;;;
 ;;;
 
+;;; ANSI compliance wants default structure printer to use #S(...) format.
+(defmethod print-object ((object structure-object) stream)
+  (lisp::default-structure-print object stream 0))
+
 (defgeneric describe-object (object stream))
 
 (defmethod describe-object (object stream)
