@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/pcl/std-class.lisp,v 1.25 1999/05/30 23:14:08 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/pcl/std-class.lisp,v 1.28 2001/11/29 03:50:35 pmai Exp $")
 ;;;
 
 (in-package :pcl)
@@ -363,7 +363,6 @@
 	 (unsupplied (list 1))
 	 (supplied-meta   (getf initargs :metaclass unsupplied))
 	 (supplied-supers (getf initargs :direct-superclasses unsupplied))
-	 (supplied-slots  (getf initargs :direct-slots unsupplied))
 	 (meta
 	   (cond ((neq supplied-meta unsupplied)
 		  (find-class supplied-meta))
@@ -383,14 +382,13 @@
 					     :name s)))))))      
       (loop (unless (remf initargs :metaclass) (return)))
       (loop (unless (remf initargs :direct-superclasses) (return)))
-      (loop (unless (remf initargs :direct-slots) (return)))
       (values meta
-	      (list* :direct-superclasses
-		     (and (neq supplied-supers unsupplied)
-			  (mapcar #'fix-super supplied-supers))
-		     :direct-slots
-		     (and (neq supplied-slots unsupplied) supplied-slots)
-		     initargs)))))
+	      (nconc
+	       (when (neq supplied-supers unsupplied)
+		 (list 
+		  :direct-superclasses
+		  (mapcar #'fix-super supplied-supers)))
+	       initargs)))))
 
 
 ;;;
@@ -413,7 +411,8 @@
 		 (direct-default-initargs nil direct-default-initargs-p)
 	         (predicate-name nil predicate-name-p))
   (declare (ignore slot-names))
-  (cond (direct-superclasses-p
+  (cond ((or direct-superclasses-p
+	     (null (slot-value class 'direct-superclasses)))
 	 (setq direct-superclasses
 	       (or direct-superclasses
 		   (list (if (funcallable-standard-class-p class)
@@ -550,8 +549,7 @@
 	     (defstruct-form
 	       `(progn
 		  ,defstruct
-		  ,@readers-init ,@writers-init
-		  (declare-structure ',name nil nil))))
+		  ,@readers-init ,@writers-init)))
 	(unless (structure-type-p name) (eval defstruct-form))
 	(mapc #'(lambda (dslotd reader-name writer-name)
 		  (let* ((reader (gdefinition reader-name))
@@ -992,9 +990,8 @@
 
 
 ;;;; inform-type-system-about-class
-;;;; make-type-predicate
 ;;;
-;;; These are NOT part of the standard protocol.  They are internal mechanism
+;;; This is NOT part of the standard protocol.  It is an internal mechanism
 ;;; which PCL uses to *try* and tell the type system about class definitions.
 ;;; In a more fully integrated implementation of CLOS, the type system would
 ;;; know about class objects and class names in a more fundamental way and
@@ -1002,7 +999,16 @@
 ;;; different.
 ;;;
 (defmethod inform-type-system-about-class ((class std-class) name)
+  ;; Maybe add skeleton lisp:standard-class to avoid undefined-function
+  ;; compiler warnings. Not otherwise needed in this implementation.
   (inform-type-system-about-std-class name))
+
+(defmethod inform-type-system-about-class
+    ((class funcallable-standard-class) name)
+  (declare (ignore name))
+  ;; Avoid load-time warning of changing metaclass.
+  )
+
 
 
 (defmethod compatible-meta-class-change-p (class proto-new-class)
