@@ -20,8 +20,8 @@
 ;;;
 
 (ext:file-comment
-  "$Header$")
-
+  "$Header: /home/CVS-cmucl/src/pcl/user-instances.lisp,v 1.3 1999/05/30 23:14:09 pw Exp $")
+;;;
 
 (in-package 'pcl)
 
@@ -129,7 +129,7 @@
 (defmacro get-user-instance-slots (x)
   `(user-vector-instance-slots ,x))
 
-(eval-when (eval #+cmu load)
+(eval-when (eval load)
   (force-compile 'user-instance-p)
   (force-compile 'user-instance-slots)
   (force-compile 'user-instance-wrapper)
