@@ -1,4 +1,4 @@
-;;;-*- Mode:LISP; Package:(WALKER LISP 1000); Base:10; Syntax:Common-lisp -*-
+;;;-*- Mode:LISP; Package:WALKER -*-
 ;;;
 ;;; *************************************************************************
 ;;; Copyright (c) 1985, 1986, 1987, 1988, 1989, 1990 Xerox Corporation.
@@ -26,7 +26,7 @@
 ;;;
 
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/pcl/walk.lisp,v 1.16 1999/05/30 23:14:10 pw Exp $")
+  "$Header: /home/CVS-cmucl/src/pcl/walk.lisp,v 1.21 2001/05/05 23:34:47 pw Exp $")
 ;;;
 ;;; A simple code walker, based IN PART on: (roll the credits)
 ;;;   Larry Masinter's Masterscope
@@ -134,11 +134,25 @@
 
 ;;; In CMU Common Lisp, the environment is represented with a structure
 ;;; that holds alists for the functional things, variables, blocks, etc.
-;;; Only the c::lexenv-functions slot is relevent.  It holds:
+;;; Except for symbol-macrolet, only the c::lexenv-functions slot is
+;;; relevent.  It holds:
 ;;; Alist (name . what), where What is either a Functional (a local function)
 ;;; or a list (MACRO . <function>) (a local macro, with the specifier
 ;;; expander.)    Note that Name may be a (SETF <name>) function.
 
+;;; If WITH-AUGMENTED-ENVIRONMENT is called from WALKER-ENVIRONMENT-BIND
+;;; this code hides the WALKER version of an environment
+;;; inside the C::LEXENV structure. It makes a list of lists of form
+;;; (<gensym-name> MACRO . #<interpreted-function>) which makes up
+;;; the :functions slot in a c::lexenv. This seems to be a form that
+;;; the compiler guts will accept as valid and otherwise ignore it.
+;;; The <interpreted-function> is used as a structure to hold the
+;;; bits of interest, {function, form, declarations, lexical-variables}.
+;;; and is otherwise not a valid interpreted-function, eg describe will
+;;; barf on it. Accessors are defined below, eg (env-walk-function env)
+;;;
+;;; MACROEXPAND-1 is the only cmucl function that gets called with the
+;;; constructed environment argument.
 
 (defmacro with-augmented-environment
 	  ((new-env old-env &key functions macros) &body body)
@@ -147,6 +161,8 @@
 							,macros)))
      ,@body))
 
+(defvar *key-to-walker-environment*) ; Initialized below.
+
 (defun with-augmented-environment-internal (env functions macros)
   ;; Note: In order to record the correct function definition, we would
   ;; have to create an interpreted closure, but the with-new-definition
@@ -154,9 +170,17 @@
   ;; we have no idea what to use for the environment.  So we just blow it
   ;; off, 'cause anything real we do would be wrong.  We still have to
   ;; make an entry so we can tell functions from macros.
-  (let ((env (or env (c::make-null-environment))))
+
+  (let* ((env (or env (c::make-null-environment)))
+	 (tem (first macros))
+	 ;; A symbol-macro spec suitable for macroexpand-1
+	 (variables 
+	  (and (eql (first tem) *key-to-walker-environment*)
+	       (loop for item in (fourth (cadr tem))
+		 collect (substitute 'c::macro :macro item)))))
     (c::make-lexenv 
       :default env
+      :variables variables
       :functions
       (append (mapcar #'(lambda (f)
 			  (cons (car f) (c::make-functional :lexenv env)))
@@ -178,8 +202,9 @@
     (let ((entry (assoc macro (c::lexenv-functions env) :test #'eq)))
       (and entry 
 	   (eq (cadr entry) 'c::macro)
-	   (function-lambda-expression (cddr entry))))))
+	   (values (function-lambda-expression (cddr entry)))))))
 
+;;; End of CMUCL specific environment hacking.
 
 
 (defmacro with-new-definition-in-environment
@@ -392,7 +417,7 @@
 
 (define-walker-template BLOCK                (NIL NIL REPEAT (EVAL)))
 (define-walker-template CATCH                (NIL EVAL REPEAT (EVAL)))
-(define-walker-template COMPILER-LET         walk-compiler-let)
+(define-walker-template ext:COMPILER-LET     walk-compiler-let)
 (define-walker-template DECLARE              walk-unexpected-declare)
 (define-walker-template EVAL-WHEN            (NIL QUOTE REPEAT (EVAL)))
 (define-walker-template FLET                 walk-flet)
@@ -427,12 +452,13 @@
 ;;;
 ;;; And the extra templates...
 ;;;
+#|| These are macros in CMUCL
 (define-walker-template DO      walk-do)
 (define-walker-template DO*     walk-do*)
 (define-walker-template PROG    walk-prog)
 (define-walker-template PROG*   walk-prog*)
 (define-walker-template COND    (NIL REPEAT ((TEST REPEAT (EVAL)))))
-
+||#
 
 
 (defvar walk-form-expand-macros-p nil)
