@@ -3,13 +3,14 @@
 ;;; **********************************************************************
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/tools/pclcom.lisp,v 1.19 1999/01/09 11:05:20 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/tools/pclcom.lisp,v 1.22 2001/03/15 18:01:39 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
 (in-package "USER")
 
 (when (find-package "PCL")
+
   ;; Load the lisp:documentation functions.
   (load "target:code/misc")
 
@@ -18,6 +19,11 @@
   ;; bootstrapping.
   (setf (compiler-macro-function 'make-instance) nil)
   ;;
+  ;; Blow away other compiler-macros exported from Lisp so that bootstrapping
+  ;; doesn't get confused.
+  (setf (compiler-macro-function 'slot-value) nil)
+  (setf (compiler-macro-function 'slot-boundp) nil)
+  ;;
   ;; Undefine all generic functions exported from Lisp so that bootstrapping
   ;; doesn't get confused.
   (let ((class (find-class 'generic-function nil)))
@@ -64,6 +70,9 @@
 (when (find-package "CLOS-MOP")
   (rename-package "CLOS-MOP" "OLD-CLOS-MOP"))
 
+;;; Inhibit ANSI :print-function and :print-object defstruct options.
+(setq kernel::*ansi-defstruct-options-p* nil)
+
 (setf c:*suppress-values-declaration* t)
 (pushnew :setf *features*)
 
@@ -99,3 +108,9 @@
        ((:or :macro (:match "$EARLY-") (:match "$BOOT-"))
 	(declare (optimize (speed 0))))))
  (pcl::compile-pcl))
+
+(cat-if-anything-changed
+ "pcl:gray-streams-library"
+ "pcl:gray-streams-class"
+ "pcl:gray-streams")
+
