@@ -3,15 +3,21 @@
 ;;; **********************************************************************
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/tools/setup.lisp,v 1.29 1999/01/09 11:05:20 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/tools/setup.lisp,v 1.31 2001/03/15 18:01:39 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
 ;;;    Set up package environment and search lists for compiler.  Also some
 ;;; compilation utilities.
 ;;;
+
+;;; Ensure pre-ANSI defstruct processing occurs during system builds.
+(in-package "KERNEL")
+(defparameter *ansi-defstruct-options-p* nil)
+
 (in-package "USER")
 
+
 
 ;;; DUMP-PACKAGE-STATE  --  Public
 ;;;
@@ -221,6 +227,7 @@
 	  ((c:target-featurep :x86) "x86/")
 	  ((c:target-featurep :alpha) "alpha/")
 	  ((c:target-featurep :sgi) "mips/")
+	  ((c:target-featurep :ppc) "ppc/")
 	  (t
 	   (error "What machine is this?")))
     (make-pathname :directory (pathname-directory f)))))
