@@ -7,7 +7,7 @@
 ;;; Scott Fahlman or slisp-group@cs.cmu.edu.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/tools/worldbuild.lisp,v 1.41 1998/05/01 01:21:42 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/tools/worldbuild.lisp,v 1.43 2001/12/06 19:15:48 pmai Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -55,6 +55,13 @@
 	  "target:assembly/mips/array.assem"
 	  "target:assembly/mips/arith.assem"
 	  "target:assembly/mips/alloc.assem"))
+    ,@(when (c:backend-featurep :ppc)
+	'("target:assembly/ppc/assem-rtns.assem"
+	  "target:assembly/ppc/array.assem"
+	  "target:assembly/ppc/arith.assem"
+	  "target:assembly/ppc/alloc.assem"))
+    
+    
 
     "target:code/type-boot"
     "target:code/fdefinition"
@@ -128,7 +135,7 @@
 	'("target:code/osf1-os"))
     ,@(when (c:backend-featurep :irix)
 	'("target:code/irix-os"))
-    ,@(when (c:backend-featurep :FreeBSD)
+    ,@(when (c:backend-featurep :BSD)
 	'("target:code/bsd-os"))
     ,@(when (c:backend-featurep :Linux)
 	'("target:code/linux-os"))
@@ -157,6 +164,8 @@
 	'("target:code/alpha-vm"))
     ,@(when (c:backend-featurep :sgi)
 	'("target:code/sgi-vm"))
+    ,@(when (c:backend-featurep :ppc)
+	'("target:code/ppc-vm"))
 
     "target:code/signal"
     "target:code/interr"
