@@ -7,7 +7,7 @@
 ;;; Scott Fahlman or slisp-group@cs.cmu.edu.
 ;;;
 (ext:file-comment
-  "$Header: /home/CVS-cmucl/src/tools/worldcom.lisp,v 1.78 1998/08/30 04:55:03 dtc Exp $")
+  "$Header: /home/CVS-cmucl/src/tools/worldcom.lisp,v 1.81 2001/03/03 15:16:07 pw Exp $")
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -17,6 +17,7 @@
 (in-package "USER")
 
 (defvar *byte-compile* #+small t #-small :maybe)
+(defvar *original-%deftype* #'lisp::%deftype)
 
 (with-compiler-log-file
         ("target:compile-lisp.log"
@@ -109,6 +110,13 @@
   (comf "target:assembly/mips/arith" :assem t)
   (comf "target:assembly/mips/alloc" :assem t))
 
+(when (c:backend-featurep :ppc)
+  (comf "target:assembly/ppc/assem-rtns" :assem t)
+  (comf "target:assembly/ppc/array" :assem t)
+  (comf "target:assembly/ppc/arith" :assem t)
+  (comf "target:assembly/ppc/alloc" :assem t))
+
+
 ;;; these guys can supposedly come in any order, but not really.
 ;;; some are put at the end so macros don't run interpreted and stuff.
 
@@ -124,7 +132,6 @@
 ;;; prevent deftypes from taking effect at compile time so that we don't
 ;;; install interpreted type expanders causing the compiler to infinitely
 ;;; recurse.
-(defvar *original-%deftype* #'lisp::%deftype)
 (setf (fdefinition 'lisp::%deftype) #'list)
 (comf "target:code/typedefs")
 
@@ -195,6 +202,8 @@
   (comf "target:code/alpha-vm"))
 (when (c:backend-featurep :sgi)
   (comf "target:code/sgi-vm"))
+(when (c:backend-featurep :ppc)
+  (comf "target:code/ppc-vm"))
 
 (comf "target:code/symbol")
 (comf "target:code/bignum")
@@ -275,7 +284,7 @@
 (comf "target:code/parse-time")
 #-no-runtime (comf "target:code/parse-time")
 
-(comf "target:code/run-program")
+(comf "target:code/run-program" :proceed t)
 
 (comf "target:code/loop" :byte-compile *byte-compile*)
 
