@@ -6,7 +6,7 @@
 ;;; If you want to use this code or any part of CMU Common Lisp, please contact
 ;;; Scott Fahlman or slisp-group@cs.cmu.edu.
 ;;;
-;;; $Header: /home/CVS-cmucl/src/tools/worldload.lisp,v 1.88 1998/01/16 07:22:25 dtc Exp $
+;;; $Header: /home/CVS-cmucl/src/tools/worldload.lisp,v 1.94 2001/12/11 00:27:31 pmai Exp $
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -63,6 +63,7 @@
 	#+hppa "c:hppa/"
 	#+x86 "c:x86/"
 	#+alpha "c:alpha/"
+	#+ppc "c:ppc/"
 	"c:generic/"))
 (setf (ext:search-list "assem:")
       '(#+(or pmax sgi) "target:assembly/mips/"
@@ -71,6 +72,7 @@
 	#+hppa "target:assembly/hppa/"
 	#+x86 "target:assembly/x86/"
 	#+alpha "target:assembly/alpha/"
+	#+ppc "target:assembly/ppc/"
 	"target:assembly/"))
 (setf (ext:search-list "hem:") '("target:hemlock/"))
 (setf (ext:search-list "clx:") '("target:clx/"))
@@ -149,8 +151,10 @@
 
 ;;; PCL.
 ;;;
-#-(or no-pcl runtime) (maybe-byte-load "pcl:pclload")
-
+#-(or no-pcl runtime) 
+(progn
+ (maybe-byte-load "pcl:pclload")
+)
 ;;; CLX.
 ;;;
 #-(or no-clx runtime)
@@ -166,7 +170,8 @@
 #-(or no-clm runtime)
 (maybe-byte-load "target:interface/clm-library")
 
-(defvar *target-sl*)
+#+no-compiler (proclaim '(special *target-sl*))
+#-no-compiler (defvar *target-sl*)
 (setq *target-sl* (search-list "target:"))
 
 ;;; Don't include the search lists used for loading in the resultant core.
@@ -214,10 +219,20 @@
   ;;; Reset the counter of the number of native code fixups.
   #+x86 (setf x86::*num-fixups* 0)
 
+  ;; Maybe enable ANSI defstruct :print-function/:print-object processing
+  #-NO-PCL
+  (setq ext:*ansi-defstruct-options-p* t)
   ;;
   ;; Save the lisp.  If RUNTIME, there is nothing new to purify, so don't.
+  ;; the following features are only used to control the build
+  ;; process, so we remove them from the generated image
+  (setq *features*
+	(nreverse
+	 (set-difference
+	  *features* 
+	  '(:runtime :no-compiler :no-pcl :no-clx :no-clm :no-hemlock))))
   (save-lisp "lisp.core"
-	     :root-structures
-	     #-(or runtime no-hemlock) `(ed ,hi::*global-command-table*)
-	     #+(or runtime no-hemlock) ()
-	     :purify #+runtime nil #-runtime t))
+             :root-structures
+             #-(or runtime no-hemlock) `(ed ,hi::*global-command-table*)
+             #+(or runtime no-hemlock) ()
+             :purify #+runtime nil #-runtime t))
